(function () {
	'use strict';

	class Sprite extends Laya.Sprite {
	    constructor() {
	        super();
	        this._touchable = false;
	        this.isDestroyed = false;
	        this.isChanged = true;
	        this.isFirstDestroy = true;
	    }
	    addAutoDestroyValue(val) {
	        if (this.customVariable == null)
	            this.customVariable = [];
	        this.customVariable.push(val);
	    }
	    get touchable() { return this._touchable; }
	    set touchable(can) {
	        this._touchable = can;
	        this.mouseEnabled = can;
	    }
	    removeFromParent(dispose = false) {
	        this.removeSelf();
	    }
	    set color(col) {
	        var fil;
	        if (col == 0xffffff) {
	            if (this.filters == null)
	                return;
	            for (let i = this.filters.length - 1; i >= 0; i--) {
	                if (this.filters[i] instanceof Laya.ColorFilter) {
	                    fil = this.filters[i];
	                    this.filters.splice(i, 1);
	                    break;
	                }
	            }
	            this.filters = this.filters;
	            return;
	        }
	        var matrix = [
	            this.onColor_getRed(col) / 255, 0, 0, 0, 0,
	            0, this.onColor_getGreen(col) / 255, 0, 0, 0,
	            0, 0, this.onColor_getBlue(col) / 255, 0, 0,
	            0, 0, 0, 1, 0,
	        ];
	        fil = new Laya.ColorFilter(matrix);
	        if (this.filters == null)
	            this.filters = [fil];
	        else
	            this.filters.push(fil);
	    }
	    onColor_getRed(color) { return (color >> 16) & 0xff; }
	    onColor_getGreen(color) { return (color >> 8) & 0xff; }
	    onColor_getBlue(color) { return color & 0xff; }
	    get width() {
	        var rec = this.getBounds();
	        return rec.width;
	    }
	    set width(value) {
	        var w0 = this.width / this.scaleX;
	        this.scaleX = value / w0;
	    }
	    get height() {
	        var rec = this.getBounds();
	        return rec.height;
	    }
	    set height(value) {
	        var w0 = this.height / this.scaleY;
	        this.scaleY = value / w0;
	    }
	    addChildWithChild(child, tarName, xy, scale = false) {
	        let tar = this.getChildByName(tarName);
	        if (tar == null) {
	            this.addChild(child);
	        }
	        else {
	            this.addChildAt(child, this.getChildIndex(tar) + 1);
	            if (xy) {
	                child.x = tar.x;
	                child.y = tar.y;
	            }
	            if (scale) {
	                child.scaleX *= tar.scaleX;
	                child.scaleY *= tar.scaleY;
	            }
	        }
	    }
	    setSpriteData(data) {
	        this.spriteData = data;
	        if (this.spriteData && (this.spriteData[0] instanceof Array) == false && this.spriteData.length > 0 && this.spriteData[0]["url"] == null) {
	            this.setMetaData(this.spriteData[0]);
	        }
	    }
	    setMetaData(data) {
	        this.metaData = data;
	        if (this.metaData["鼠标区域"]) {
	            if (this.metaData["鼠标区域"]["形状"] == "圆") {
	                this.hit_半径 = this.metaData["鼠标区域"]["半径"];
	                this.hit_半径half = this.hit_半径 / 2;
	                this.hit_半径2 = this.hit_半径 * this.hit_半径;
	            }
	        }
	    }
	    clearChild(dispose = false) {
	        this.removeChildren();
	    }
	    setDisplayColor(display, color) {
	        display.color = color;
	    }
	    getSelfBounds() {
	        if (this.isChanged == false && this.selfBonds != null && this.lastX == this.x && this.lastY == this.y && this.lastRo == this.rotation && this.lastSx == this.scaleX
	            && this.lastSy == this.scaleY && this.lastSkewx == this.skewX && this.lastSkewy == this.skewY && this.mask == null) {
	            return this.selfBonds;
	        }
	        this.selfBonds = super.getSelfBounds();
	        this.lastX = this.x;
	        this.lastY = this.y;
	        this.lastRo = this.rotation;
	        this.lastSx = this.scaleX;
	        this.lastSy = this.scaleY;
	        this.lastSkewx = this.skewX;
	        this.lastSkewy = this.skewY;
	        this.isChanged = false;
	        if (this.selfBonds.width == 0 || this.selfBonds.height == 0)
	            this.isChanged = true;
	        if (this.numChildren > 0) {
	            let minx = 999999;
	            let miny = 999999;
	            let maxx = -999999;
	            let maxy = -999999;
	            for (let i = 0; i < this.numChildren; i++) {
	                let c = this.getChildAt(i);
	                let b2 = c.getBounds();
	                if (b2.x < minx) {
	                    minx = b2.x;
	                }
	                if (b2.x + b2.width > maxx) {
	                    maxx = b2.x + b2.width;
	                }
	                if (b2.y < miny) {
	                    miny = b2.y;
	                }
	                if (b2.y + b2.height > maxy) {
	                    maxy = b2.y + b2.height;
	                }
	            }
	            if (this.selfBonds.width < maxx - minx) {
	                this.selfBonds.width = maxx - minx;
	            }
	            if (this.selfBonds.height < maxy - miny) {
	                this.selfBonds.height = maxy - miny;
	            }
	        }
	        return this.selfBonds;
	    }
	    getBounds() {
	        if (this.isChanged == false && this.bonds != null && this.lastX == this.x && this.lastY == this.y && this.lastRo == this.rotation && this.lastSx == this.scaleX
	            && this.lastSy == this.scaleY && this.lastSkewx == this.skewX && this.lastSkewy == this.skewY && this.mask == null) {
	            return this.bonds;
	        }
	        this.bonds = super.getBounds();
	        this.lastX = this.x;
	        this.lastY = this.y;
	        this.lastRo = this.rotation;
	        this.lastSx = this.scaleX;
	        this.lastSy = this.scaleY;
	        this.lastSkewx = this.skewX;
	        this.lastSkewy = this.skewY;
	        this.isChanged = false;
	        if (this.numChildren > 0) {
	            let sb = this.getSelfBounds();
	            if (sb.width * this.scaleX != this.bonds.width) {
	                this.bonds.width = sb.width * this.scaleX;
	            }
	            if (sb.height * this.scaleY != this.bonds.height) {
	                this.bonds.height = sb.height * this.scaleY;
	            }
	        }
	        if (this.bonds.width == 0 || this.bonds.height == 0)
	            this.isChanged = true;
	        return this.bonds;
	    }
	    addChild(node) {
	        this.isChanged = true;
	        return super.addChild(node);
	    }
	    addChildAt(node, index) {
	        this.isChanged = true;
	        return super.addChildAt(node, index);
	    }
	    removeChild(node) {
	        this.isChanged = true;
	        return super.removeChild(node);
	    }
	    removeChildAt(index) {
	        this.isChanged = true;
	        return super.removeChildAt(index);
	    }
	    removeChildByName(name) {
	        this.isChanged = true;
	        return super.removeChildByName(name);
	    }
	    removeChildren(beginIndex, endIndex) {
	        this.isChanged = true;
	        return super.removeChildren(beginIndex, endIndex);
	    }
	    destroyF() {
	        if (this.isFirstDestroy) {
	            this.offAll();
	            this.isFirstDestroy = false;
	            this.mask = null;
	            if (this.filters)
	                this.filters = null;
	            while (this.numChildren > 0) {
	                let cone = this.getChildAt(0);
	                this.removeChild(cone);
	                if (cone["destroyF"]) {
	                    cone["destroyF"]();
	                }
	                else {
	                    cone.removeSelf();
	                }
	                try {
	                    this.numChildren;
	                }
	                catch (e) {
	                    break;
	                }
	            }
	            this.removeFromParent(true);
	        }
	    }
	}

	class SwfMovieClip extends Sprite {
	    constructor(frames, labels, displayObjects, ownerSwf, _events) {
	        super();
	        this._isPlay = false;
	        this._loop = true;
	        this._complete = null;
	        this._hasCompleteListener = false;
	        this.numMC = 0;
	        this.needUpdateEveryFrame = false;
	        this.indexEnter = -1;
	        this.pastTime = 0;
	        this.touchable = false;
	        this._frames = frames;
	        this._labels = labels;
	        this._frameEvents = _events;
	        this._displayObjects = displayObjects;
	        this._startFrame = 0;
	        this._endFrame = this._frames.length - 1;
	        this._ownerSwf = ownerSwf;
	        this.fps = SwfMovieClip.FPS_default;
	        this.currentFrame = 0;
	        var k;
	        var arr;
	        var l;
	        for (k in this._displayObjects) {
	            if (k.indexOf("mc") == 0) {
	                arr = this._displayObjects[k];
	                if (arr == null)
	                    continue;
	                l = arr.length;
	                for (var i = 0; i < l; i++) {
	                    this.numMC++;
	                }
	            }
	        }
	        this.play();
	    }
	    onCashSameFrameInfo(obj) {
	        for (var f in obj) {
	            this._frames[f] = null;
	        }
	    }
	    _frameLoop() {
	        if (!this._isPlay) {
	            if (this.indexEnter > 0) {
	                ConfigTidus.GFun_MM_removeEnter(this.indexEnter);
	                this.indexEnter = -1;
	            }
	            return;
	        }
	        this.pastTime += SwfMovieClip.TimeFrame;
	        var f2 = parseInt((this.pastTime / this.NeedTime) + "");
	        while (f2 > 0) {
	            if (this._ownerSwf == null) {
	                return;
	            }
	            this.nextFrame();
	            f2--;
	            this.pastTime -= this.NeedTime;
	        }
	    }
	    nextFrame() {
	        if (this._ownerSwf == null)
	            return;
	        if (this._currentFrame >= this._endFrame) {
	            var isReturn = false;
	            if (!this._loop || this._startFrame == this._endFrame) {
	                if (this._ownerSwf)
	                    this.stop(false);
	                isReturn = true;
	            }
	            if (this._complete) {
	                let f = this._complete;
	                this._complete = null;
	                f(this);
	            }
	            if (isReturn)
	                return;
	            this._currentFrame = this._startFrame;
	        }
	        else {
	            this._currentFrame++;
	        }
	        this.currentFrame = this._currentFrame;
	        if (this._currentFrame >= this._endFrame) {
	            if (!this._loop || this._startFrame == this._endFrame) {
	                if (this._ownerSwf)
	                    this.stop(false);
	                if (this._complete) {
	                    let f = this._complete;
	                    this._complete = null;
	                    f(this);
	                }
	            }
	        }
	    }
	    removeChildFromAllFrame(_classLink) {
	        var length = this._frames.length;
	        for (var i = 0; i < length; i++) {
	            var frameInfos2 = this._frames[i];
	            if (frameInfos2 == null)
	                continue;
	            for (var j = 0; j < frameInfos2.length; j++) {
	                this._mdata = frameInfos2[j];
	                if (this._mdata instanceof Array == false) {
	                    if (this._mdata["url"] == _classLink || this._mdata["name"] == _classLink) {
	                        frameInfos2.splice(j--, 1);
	                    }
	                }
	            }
	        }
	    }
	    getChildByName_onFrame(_name, f) {
	        if (this._frames[f] == null)
	            return null;
	        var frameInfos2 = this._frames[f];
	        for (var j = 0; j < frameInfos2.length; j++) {
	            this._mdata = frameInfos2[j];
	            if (this._mdata instanceof Array == false) {
	                if (this._mdata["name"] == _name) {
	                    var display;
	                    if (this._displayObjects)
	                        display = this._displayObjects[this._mdata["url"]][this._mdata["index"]];
	                    return display;
	                }
	            }
	        }
	        return null;
	    }
	    set currentFrame(frame) {
	        this._currentFrame = frame;
	        this.__frameInfos = this._frames[this._currentFrame];
	        if (this.__frameInfos == null || this._ownerSwf == null)
	            return;
	        this.onSetCurrentFrame();
	    }
	    onSetCurrentFrame() {
	        if (this.lastFrame == this._currentFrame)
	            return;
	        this.lastFrame = this._currentFrame;
	        if (this.frameFunction)
	            this.frameFunction();
	        if (this._displayObjects)
	            this.clearChild();
	        else
	            this.clearChild(true);
	        var data;
	        var display;
	        var useIndex;
	        var length = this.__frameInfos.length;
	        for (var i = 0; i < length; i++) {
	            data = this.__frameInfos[i];
	            useIndex = data["index"];
	            if (data["type"] == "particle") {
	                var pd = this._displayObjects[data["url"]][useIndex];
	                pd.x = data.x;
	                pd.y = data.y;
	                pd.alpha = data["alpha"];
	                pd.name = data["name"];
	                pd.emitter.emit();
	                this.addChild(pd);
	                continue;
	            }
	            if (this._displayObjects) {
	                display = this._displayObjects[data["url"]][useIndex];
	            }
	            if (display == null) {
	                if (data["type"] == "text")
	                    display = this._ownerSwf.createTextField(data);
	                else
	                    display = this._ownerSwf.getObject(data["url"]);
	                if (display && this._displayObjects) {
	                    this._displayObjects[data["url"]][useIndex] = display;
	                }
	            }
	            if (display == null || (this.dicRemoved != null && this.dicRemoved[display] == true))
	                continue;
	            this._ownerSwf.onCheckDataForFilterAndColor(display, data);
	            display.alpha = data["alpha"];
	            display.name = data["name"];
	            display.x = data["x"];
	            display.y = data["y"];
	            display.skewX = -data["skewx"];
	            display.skewY = data["skewy"];
	            if (this.touchable == true) {
	                display.touchable = true;
	                if (display.mouseEnabled == false) {
	                    display.mouseEnabled = true;
	                }
	            }
	            switch (data["type"]) {
	                case "s9":
	                    display.width = data["w"];
	                    display.height = data["h"];
	                    ConfigTidus.GVal["SWF"].setBlendMode(display, data["blend"]);
	                    ConfigTidus.GVal["SWF"].changeColor(display, data["color"]);
	                    this.addChild(display);
	                    break;
	                case "text":
	                    var tx = display;
	                    if (tx.isCleared == true)
	                        break;
	                    tx.width = data["w"];
	                    tx.height = data["h"];
	                    tx.format.font = data["font"];
	                    tx.format.color = data["color"];
	                    tx.format.size = data["size"];
	                    tx.format.horizontalAlign = data["align"];
	                    tx.format.italic = data["italic"];
	                    tx.format.bold = data["bold"];
	                    if (tx.text != data["text"] && tx.text != "\r" && tx.text != "") { }
	                    else if (data["text"] && data["text"] != "\r" && data["text"] != "") {
	                        if (tx.changed == false) {
	                            tx.text = data["text"];
	                            tx.text = data["text"];
	                        }
	                    }
	                    ConfigTidus.GVal["SWF"].setBlendMode(display, data["blend"]);
	                    this.addChild(display);
	                    break;
	                case "img":
	                    display.scaleX = data["sx"];
	                    display.scaleY = data["sy"];
	                    ConfigTidus.GVal["SWF"].setBlendMode(display, data["blend"]);
	                    ConfigTidus.GVal["SWF"].changeColor(display, data["color"]);
	                    this.addChild(display);
	                    break;
	                default:
	                    display.scaleX = data["sx"];
	                    display.scaleY = data["sy"];
	                    ConfigTidus.GVal["SWF"].setBlendMode(display, data["blend"]);
	                    ConfigTidus.GVal["SWF"].changeColor(display, data["color"]);
	                    this.addChild(display);
	                    break;
	            }
	        }
	        if (this.eventFunction && this._frameEvents != null && this._frameEvents[this._currentFrame] != null) {
	            this.eventFunction(this._frameEvents[this._currentFrame]);
	        }
	    }
	    get currentFrame() {
	        return this._currentFrame;
	    }
	    play(rePlayChildMovie = true) {
	        if (this._isPlay == true)
	            return;
	        this._isPlay = true;
	        if (this._currentFrame >= this._endFrame) {
	            this._currentFrame = this._startFrame;
	        }
	        if (this.indexEnter == -1)
	            this.indexEnter = ConfigTidus.GFun_MM_addEnter(this._frameLoop.bind(this));
	        if (this.numMC == 0 || this._displayObjects == null)
	            return;
	        var k;
	        var arr;
	        var l;
	        for (k in this._displayObjects) {
	            if (k.indexOf("mc") == 0) {
	                arr = this._displayObjects[k];
	                if (arr == null)
	                    continue;
	                l = arr.length;
	                for (var i = 0; i < l; i++) {
	                    if (arr[i] == null || (arr[i] instanceof SwfMovieClip) == false)
	                        continue;
	                    if (rePlayChildMovie)
	                        arr[i].currentFrame = 0;
	                    arr[i].play(rePlayChildMovie);
	                }
	            }
	        }
	        this.needUpdateEveryFrame = rePlayChildMovie;
	    }
	    stop(stopChild = false) {
	        this._isPlay = false;
	        this.pastTime = 0;
	        if (this.indexEnter > 0) {
	            ConfigTidus.GFun_MM_removeEnter(this.indexEnter);
	            this.indexEnter = -1;
	        }
	        if (this.numMC == 0)
	            return;
	        var k;
	        var arr;
	        var l;
	        for (k in this._displayObjects) {
	            if (k.indexOf("mc") == 0) {
	                arr = this._displayObjects[k];
	                if (arr == null)
	                    continue;
	                l = arr.length;
	                for (var i = 0; i < l; i++) {
	                    if (arr[i] == null || (arr[i] instanceof SwfMovieClip) == false)
	                        continue;
	                    if (stopChild == true)
	                        arr[i].stop(stopChild);
	                    else
	                        arr[i].play();
	                }
	            }
	        }
	        this.needUpdateEveryFrame = !stopChild;
	    }
	    gotoAndStop(frame, stopChild = false) {
	        this.goTo(frame);
	        if (typeof frame == "string") {
	            var labelData = this.getLabelData(frame);
	            if (labelData == null) {
	                this._endFrame = this._frames.length - 1;
	            }
	            else {
	                this._currentLabel = labelData[0];
	                this._endFrame = labelData[2];
	            }
	        }
	        else {
	        }
	        this.onCheckLastNotNullFrame();
	        this.stop(stopChild);
	    }
	    gotoAndPlay(frame, rePlayChildMovie = false) {
	        this.goTo(frame);
	        if (typeof frame == "string") {
	            var labelData = this.getLabelData(frame);
	            if (labelData == null) {
	                this._endFrame = this._frames.length - 1;
	            }
	            else {
	                this._currentLabel = labelData[0];
	                this._endFrame = labelData[2];
	            }
	        }
	        else {
	        }
	        this.onCheckLastNotNullFrame();
	        this.play(rePlayChildMovie);
	    }
	    onCheckLastNotNullFrame() {
	        var count = 0;
	        while (this.__frameInfos == null) {
	            if (this._currentFrame - count < 0) {
	                this.__frameInfos = this._frames[0];
	                break;
	            }
	            this.__frameInfos = this._frames[this._currentFrame - count];
	            count++;
	        }
	        this.onSetCurrentFrame();
	    }
	    goTo(frame) {
	        if (typeof frame == "string") {
	            var labelData = this.getLabelData(frame);
	            if (labelData == null) {
	                frame = 0;
	                this._currentFrame = this._startFrame = frame;
	            }
	            else {
	                this._currentLabel = labelData[0];
	                this._currentFrame = this._startFrame = labelData[1];
	            }
	        }
	        else {
	            if (frame < 0)
	                frame = 0;
	            this._currentFrame = this._startFrame = frame;
	        }
	        this.currentFrame = this._currentFrame;
	    }
	    getLabelData(label) {
	        var length = this._labels.length;
	        var labelData;
	        for (var i = 0; i < length; i++) {
	            labelData = this._labels[i];
	            if (labelData[0] == label) {
	                return labelData;
	            }
	        }
	        return null;
	    }
	    get isPlay() {
	        return this._isPlay;
	    }
	    get loop() {
	        return this._loop;
	    }
	    set loop(value) {
	        this._loop = value;
	    }
	    set completeFunction(value) {
	        this._complete = value;
	    }
	    get completeFunction() {
	        return this._complete;
	    }
	    get totalFrames() {
	        return this._frames.length;
	    }
	    get currentLabel() {
	        return this._currentLabel;
	    }
	    get labels() {
	        if (this._labelStrings != null)
	            return this._labelStrings;
	        this._labelStrings = [];
	        var length = this._labels.length;
	        for (var i = 0; i < length; i++) {
	            this._labelStrings.push(this._labels[i][0]);
	        }
	        return this._labelStrings;
	    }
	    hasLabel(label) {
	        return !(this.labels.indexOf(label) == -1);
	    }
	    set startFrame(value) {
	        this._startFrame = value < 0 ? 0 : value;
	        this._startFrame = this._startFrame > this._endFrame ? this._endFrame : this._startFrame;
	    }
	    get startFrame() {
	        return this._startFrame;
	    }
	    set endFrame(value) {
	        this._endFrame = value > this._frames.length - 1 ? this._frames.length - 1 : value;
	        this._endFrame = this._endFrame < this._startFrame ? this._startFrame : this._endFrame;
	    }
	    get endFrame() {
	        return this._endFrame;
	    }
	    removeChildF(child, dispose = true) {
	        if (this.dicRemoved == null)
	            this.dicRemoved = {};
	        if (child instanceof String) {
	            child = this.getChildByName(child);
	        }
	        if (child != null) {
	            this.dicRemoved[child] = true;
	            this.removeChild(child);
	        }
	    }
	    destroyF() {
	        this.dicRemoved = null;
	        this.frameFunction = null;
	        this.eventFunction = null;
	        this.completeFunction = null;
	        if (this._displayObjects) {
	            for (var key in this._displayObjects) {
	                let array = this._displayObjects[key];
	                var len = array.length;
	                for (let i = 0; i < len; i++) {
	                    if (array[i]) {
	                        array[i].removeFromParent(true);
	                    }
	                }
	            }
	            this._displayObjects = null;
	        }
	        this._ownerSwf = null;
	        if (this._isPlay) {
	            this._isPlay = false;
	            if (this.indexEnter > 0) {
	                ConfigTidus.GFun_MM_removeEnter(this.indexEnter);
	                this.indexEnter = -1;
	            }
	        }
	        super.destroyF();
	    }
	    get fps() { return this._fps; }
	    set fps(value) {
	        this._fps = value;
	        this.NeedTime = 1000 / this._fps;
	    }
	}
	SwfMovieClip.ANGLE_TO_RADIAN = Math.PI / 180;
	SwfMovieClip.FPS_default = 30;
	SwfMovieClip.TimeFrame = 0;
	SwfMovieClip.CacheNumBegin = 9999999;

	class GameConfig {
	    constructor() {
	    }
	    static init() {
	        var reg = Laya.ClassUtils.regClass;
	    }
	}
	GameConfig.width = 1280;
	GameConfig.height = 640;
	GameConfig.scaleMode = "fixedwidth";
	GameConfig.screenMode = "none";
	GameConfig.alignV = "top";
	GameConfig.alignH = "left";
	GameConfig.startScene = "";
	GameConfig.sceneRoot = "";
	GameConfig.debug = false;
	GameConfig.stat = false;
	GameConfig.physicsDebug = false;
	GameConfig.exportSceneToJson = true;
	GameConfig.init();

	class LayerManager extends Laya.Sprite {
	    constructor() {
	        super();
	        LayerManager.instance = this;
	        this.LayerDown = new Sprite();
	        this.addChild(this.LayerDown);
	        this.LayerView = new Sprite();
	        this.addChild(this.LayerView);
	        this.LayerTop = new Sprite();
	        this.addChild(this.LayerTop);
	    }
	    static getInstence() {
	        if (LayerManager.instance == null)
	            LayerManager.instance = new LayerManager();
	        return LayerManager.instance;
	    }
	    static get LayerTop() { return this.getInstence().LayerTop; }
	    static get LayerView() { return this.getInstence().LayerView; }
	    static get LayerDown() { return this.getInstence().LayerDown; }
	    checkLayaOriSceneFit() {
	        Laya.Physics.I.worldRoot = LayerManager.LayerDown;
	        let sx = (ConfigTidus.stageRealW / ConfigTidus.stageW);
	        let sy = (ConfigTidus.stageRealH / ConfigTidus.stageH);
	        let sCombine = sx <= sy ? sx : sy;
	        this.LayerDown.scale(sCombine, sCombine);
	        this.LayerDown.x = (ConfigTidus.stageRealW - ConfigTidus.stageW * sCombine) / 2;
	        this.LayerDown.y = (ConfigTidus.stageRealH - ConfigTidus.stageH * sCombine) / 2;
	    }
	    init(_stage) {
	        _stage.addChild(this);
	    }
	    destroyF() {
	        var child;
	        while (this.LayerView.numChildren > 0) {
	            child = this.LayerView.getChildAt(0);
	            this.LayerView.removeChild(child);
	            ConfigTidus.GFun_ToolObj_destroyOne(child);
	        }
	        while (this.LayerTop.numChildren > 0) {
	            child = this.LayerTop.getChildAt(0);
	            this.LayerTop.removeChild(child);
	            ConfigTidus.GFun_ToolObj_destroyOne(child);
	        }
	    }
	}

	class MyEventListener {
	    constructor(isglobal = true) {
	        this.dicOnce = {};
	        this.index = -1;
	        if (isglobal) {
	            this.index = MyEventListener.ALL["i"]++;
	            MyEventListener.ALL[this.index] = this;
	        }
	    }
	    destroyF() {
	        if (this.index != -1)
	            delete MyEventListener.ALL[this.index];
	    }
	    addCustomEventListener(ename, cb, once = false) {
	        if (this.dicCEvent == null)
	            this.dicCEvent = {};
	        if (this.dicCEvent[ename] == null)
	            this.dicCEvent[ename] = {};
	        var keys = [];
	        for (let key in this.dicCEvent[ename]) {
	            keys.push(key);
	        }
	        let maxid = 0;
	        for (let i = 0; i < keys.length; i++) {
	            let _id = parseInt(keys[i]);
	            if (_id > maxid)
	                maxid = _id;
	        }
	        maxid++;
	        this.dicCEvent[ename][maxid] = cb;
	        if (once) {
	            if (this.dicOnce[ename] == null)
	                this.dicOnce[ename] = {};
	            this.dicOnce[ename][maxid] = true;
	        }
	        return maxid;
	    }
	    dispatchEvent(ename, val = null) {
	        for (let key in MyEventListener.ALL) {
	            if (key == "i")
	                continue;
	            let one = MyEventListener.ALL[key];
	            one.onCustomEventAct(ename, val);
	        }
	    }
	    onCustomEventAct(ename, val = null) {
	        if (this.dicCEvent == null || this.dicCEvent[ename] == null)
	            return;
	        let dic = this.dicCEvent[ename];
	        for (let key in dic) {
	            let f = dic[key];
	            if (this.dicOnce[ename] && this.dicOnce[ename][key] == true) {
	                delete dic[key];
	            }
	            if (val == null)
	                f();
	            else
	                f(val);
	        }
	    }
	    removeCustomEventListener(ename, index) {
	        if (this.dicCEvent == null || this.dicCEvent[ename] == null)
	            return;
	        delete this.dicCEvent[ename][index];
	    }
	    hasListenerOfEvent(ename) {
	        if (this.dicCEvent == null || this.dicCEvent[ename] == null)
	            return false;
	        for (let key in this.dicCEvent[ename]) {
	            return true;
	        }
	        return false;
	    }
	}
	MyEventListener.ALL = { "i": 1 };

	class MainManager {
	    constructor(staticOne = false) {
	        this.countEnterFrame = 0;
	        this.countEnterSec = 0;
	        this.countDelayCB = 0;
	        this.INDEX_enterframe = -1;
	        this.EventManager = new MyEventListener();
	        this.countFrameTotals = 0;
	        this.countSecondTotals = 0;
	        this.countFrameTemp = 0;
	        if (staticOne == true) {
	            LayerManager.getInstence().init(Laya.stage);
	            ConfigTidus.stageW = GameConfig.width;
	            ConfigTidus.stageH = GameConfig.height;
	            ConfigTidus.initF();
	            ConfigTidus.GFun_MM_removeEnter = this.removeEnterFrame.bind(this);
	            ConfigTidus.GFun_MM_addEnter = this.addEnterFrame.bind(this);
	            ConfigTidus.GFun_MM_addDelay = this.addDelayFunction.bind(this);
	            ConfigTidus.GFun_MM_removeDelay = this.removeDelayFunction.bind(this);
	            ConfigTidus.GFun_MEO_dispatch = this.dispatchEventF.bind(this);
	            ConfigTidus.GVal["MMMain"] = this;
	            Laya.timer.frameLoop(1, this, this.enterFrame, null, false);
	            ConfigTidus.GVal["舞台尺寸事件"] = () => { };
	        }
	    }
	    static initF() {
	        if (this._ins == null)
	            this._ins = new MainManager(true);
	    }
	    static get instance() {
	        if (this._ins == null)
	            this._ins = new MainManager(true);
	        return this._ins;
	    }
	    destroyF() {
	        if (MainManager.instance == this) {
	            console.error("不能清理全局MainManager!!");
	        }
	        else {
	            if (this.INDEX_enterframe >= 0) {
	                MainManager.instance.removeEnterFrame(this.INDEX_enterframe);
	            }
	        }
	    }
	    addEnterFrame(fun) {
	        if (this.Dic_enterFrame == null)
	            this.Dic_enterFrame = {};
	        this.Dic_enterFrame[this.countEnterFrame] = fun;
	        if (MainManager._ins != this && this.INDEX_enterframe == -1) {
	            this.INDEX_enterframe = MainManager.instance.addEnterFrame(this.enterFrame.bind(this));
	        }
	        return this.countEnterFrame++;
	    }
	    removeEnterFrame(index) {
	        if (this.Dic_enterFrame && this.Dic_enterFrame[index]) {
	            delete this.Dic_enterFrame[index];
	        }
	    }
	    addEnterSecondFunction(fun) {
	        if (this.Dic_enterSec == null)
	            this.Dic_enterSec = {};
	        this.Dic_enterSec[this.countEnterSec++] = fun;
	        if (MainManager._ins != this && this.INDEX_enterframe == -1) {
	            this.INDEX_enterframe = MainManager.instance.addEnterFrame(this.enterFrame.bind(this));
	        }
	        return this.countEnterSec;
	    }
	    removeEnterSecondFun(index) {
	        if (this.Dic_enterSec && this.Dic_enterSec[index]) {
	            delete this.Dic_enterSec[index];
	        }
	    }
	    addDelayFunction(frame, fun, val = null) {
	        if (this.Dic_delayFun == null)
	            this.Dic_delayFun = {};
	        this.Dic_delayFun[this.countDelayCB++] = { "f": fun, "time": frame, "val": val };
	        if (MainManager._ins != this && this.INDEX_enterframe == -1) {
	            this.INDEX_enterframe = MainManager.instance.addEnterFrame(this.enterFrame.bind(this));
	        }
	        return this.countDelayCB;
	    }
	    removeDelayFunction(i) {
	        if (this.Dic_delayFun)
	            delete this.Dic_delayFun[i];
	    }
	    dispatchEventF(ename, val = null) {
	        this.EventManager.dispatchEvent(ename, val);
	    }
	    enterFrame() {
	        this.countFrameTotals++;
	        this.countFrameTemp++;
	        if (this.countFrameTemp >= ConfigTidus.FrameRate) {
	            this.countFrameTemp -= ConfigTidus.FrameRate;
	            this.countSecondTotals++;
	            if (this.Dic_enterSec) {
	                for (let i in this.Dic_enterSec) {
	                    this.Dic_enterSec[i]();
	                }
	            }
	        }
	        if (this.Dic_enterFrame) {
	            for (let i in this.Dic_enterFrame) {
	                this.Dic_enterFrame[i]();
	            }
	        }
	        if (this.Dic_delayFun) {
	            for (let i in this.Dic_delayFun) {
	                if (--this.Dic_delayFun[i]["time"] <= 0) {
	                    let f = this.Dic_delayFun[i]["f"];
	                    let val = this.Dic_delayFun[i]["val"];
	                    this.removeDelayFunction(i);
	                    if (val)
	                        f(val);
	                    else
	                        f();
	                }
	            }
	        }
	    }
	}

	var Handler = Laya.Handler;
	var Event = Laya.Event;
	class MyZMovieClip extends Sprite {
	    constructor(url, f, vvalue) {
	        super();
	        this._loop = true;
	        this.isComplete = false;
	        this.countMoreFrame = 0;
	        ConfigTidus.GVal["MyMC"].addF(this);
	        this.Fun = f;
	        this.FunValu = vvalue;
	        let realurl;
	        if (typeof url === "string") {
	            this.URL = url;
	            realurl = url;
	            var index = this.URL.lastIndexOf("/");
	            this.Name = this.URL.substr(index + 1);
	        }
	        else {
	            this.customFileData = url;
	            this.Name = this.customFileData.name;
	            realurl = this.customFileData.sk;
	            this.URL = this.Name;
	        }
	        if (MyZMovieClip.dicTemplet == null) {
	            MyZMovieClip.dicTemplet = {};
	        }
	        if (MyZMovieClip.dicTemplet[this.URL] == null) {
	            this.zGameMc = new Laya.Templet();
	            MyZMovieClip.dicTemplet[this.URL] = { "模板": this.zGameMc, "num": 1 };
	            this.zGameMc.on(Event.COMPLETE, this, this.parseComplete);
	            this.zGameMc.on(Event.ERROR, this, this.onError);
	            if (this.customFileData) {
	                Laya.loader.load(this.customFileData.sk, Handler.create(this, this.onCompleteLocalSK), null, Laya.Loader.BUFFER);
	            }
	            else {
	                this.zGameMc.loadAni(realurl);
	            }
	        }
	        else {
	            this.zGameMc = MyZMovieClip.dicTemplet[this.URL]["模板"];
	            MyZMovieClip.dicTemplet[this.URL]["num"]++;
	            if (this.FunValu)
	                this.Fun(this, this.FunValu);
	            else
	                this.Fun(this);
	            this.Fun = null;
	        }
	    }
	    onCompleteLocalSK(content = null) {
	        if (!content) {
	            console.error("加载本地.sk文件失败");
	            this.onError(null);
	            return;
	        }
	        this.__skbuffer = content;
	        Laya.loader.load(this.customFileData.png, Laya.Handler.create(this, this.onLoadedPic), null, Laya.Loader.IMAGE);
	    }
	    onLoadedPic(data) {
	        if (data) {
	            this.zGameMc.parseData(data, this.__skbuffer);
	        }
	        else {
	            console.error("加载本地图片失败");
	            this.onError(null);
	        }
	    }
	    parseComplete(fac) {
	        this.zGameMc.offAll();
	        if (fac == null) {
	            if (this.FunValu)
	                this.Fun(null, this.FunValu);
	            else
	                this.Fun(null);
	        }
	        else {
	            if (this.FunValu)
	                this.Fun(this, this.FunValu);
	            else
	                this.Fun(this);
	        }
	        this.Fun = null;
	    }
	    onError(e) {
	        console.log("加载spine失败：" + URL);
	        this.parseComplete(null);
	    }
	    initF(type) {
	        this.mSkeleton = this.zGameMc.buildArmature(type);
	        this.mSkeleton.offAll();
	        this.addChild(this.mSkeleton);
	        this.mSkeleton.playbackRate(30 / ConfigTidus.FrameRate);
	        this.mSkeleton.on(Event.STOPPED, this, this.completeHandler);
	        this.mSkeleton.on(Event.LABEL, this, this.onEvent);
	    }
	    get labels() {
	        if (this.__labels != null)
	            return this.__labels;
	        let arr = null;
	        if (this.mSkeleton) {
	            var allLabel = this.mSkeleton.getAnimNum();
	            for (var i = 0; i < allLabel; i++) {
	                let lname = this.mSkeleton.getAniNameByIndex(i);
	                if (arr == null)
	                    arr = [];
	                arr.push(lname);
	            }
	            this.__labels = arr;
	        }
	        return arr;
	    }
	    onEvent(e) {
	        var tEventData = e;
	        if ((tEventData.stringValue != null && tEventData.stringValue.length > 0) || (tEventData.intValue > 0 || tEventData.intValue < 0)) {
	            let dic = { "name": tEventData.name, "str": tEventData.stringValue, "int": tEventData.intValue };
	            if (this.FunEvent)
	                this.FunEvent(dic);
	        }
	        else {
	            if (this.FunEvent)
	                this.FunEvent(tEventData.name);
	        }
	    }
	    completeHandler() {
	        console.info("spine complete");
	        this.isComplete = true;
	        if (this.completeFunction) {
	            this.completeFunction();
	        }
	    }
	    play(rePlayChildMovie = true) {
	        if (this.nowLabel == null) {
	            console.error(this.Name + "：spine：play失败，不应该没有label直接播放");
	            return;
	        }
	        this.nowFrame = -1;
	        this.mSkeleton.resume();
	    }
	    gotoAndPlay(frame, rePlayChildMovie = false) {
	        if (typeof frame == "string") {
	            this.gotoAndPlayLable(frame);
	        }
	        else {
	            if (this.nowLabel == null) {
	                console.error(this.Name + "：spine：play失败，不应该没有label直接播放");
	                return;
	            }
	            try {
	                this.mSkeleton.index = frame;
	            }
	            catch (e) {
	                console.log(this.Name + "：spine：gotoAndPlay失败：" + e.message);
	                return;
	            }
	            this.play();
	        }
	    }
	    gotoAndPlayLable(label) {
	        if (typeof label == "number") {
	            label = this.mSkeleton.getAniNameByIndex(label);
	        }
	        var allLabel = this.mSkeleton.getAnimNum();
	        for (var i = 0; i < allLabel; i++) {
	            if (label == this.mSkeleton.getAniNameByIndex(i)) {
	                this.nowLabel = label;
	                this.nowFrame = -1;
	                this.mSkeleton.play(label, this.loop, true);
	                return;
	            }
	        }
	        console.log("spine动画播放label：" + label + "：失败");
	    }
	    gotoAndStop(frame, stopChild = false) {
	        if (this.zGameMc) {
	            this.countMoreFrame = 0;
	            this.mSkeleton.index = frame;
	            this.nowFrame = frame;
	            while (this.mSkeleton.index < frame) {
	                this.countMoreFrame++;
	                if (frame + this.countMoreFrame < this.mSkeleton.total) {
	                    this.mSkeleton.index = frame + this.countMoreFrame;
	                    if (this.mSkeleton.index > this.nowFrame) {
	                        this.nowFrame = this.mSkeleton.index;
	                    }
	                }
	                else {
	                    break;
	                }
	            }
	        }
	    }
	    gotoAndStopLable(label) {
	        this.isComplete = false;
	        this.nowLabel = label;
	        var allLabel = this.mSkeleton.getAnimNum();
	        for (var i = 0; i < allLabel; i++) {
	            if (label == this.mSkeleton.getAniNameByIndex(i)) {
	                this.mSkeleton.play(i, this.loop);
	                this.mSkeleton.index = 0;
	                this.nowFrame = -1;
	                return;
	            }
	        }
	        console.log("spine动画播放label：" + label + "：失败");
	    }
	    stop(stopChild = false) {
	        if (this.zGameMc) {
	            this.mSkeleton.stop();
	        }
	    }
	    pauseF() {
	        this.stop();
	    }
	    resumeF() {
	        this.mSkeleton.resume();
	    }
	    get totalFrames() {
	        if (this.mSkeleton && this.mSkeleton.player.currentAnimationClipIndex != -1) {
	            return this.mSkeleton.total;
	        }
	        return 0;
	    }
	    get currentFrame() {
	        if (this.zGameMc) {
	            if (this.mSkeleton.index < this.nowFrame && this.nowFrame >= 0) {
	                return this.nowFrame;
	            }
	            return this.mSkeleton.index;
	        }
	        return 0;
	    }
	    get loop() {
	        return this._loop;
	    }
	    set loop(real) {
	        this._loop = real;
	    }
	    findBone(bName) {
	        if (this.mSkeleton == null)
	            return null;
	        return this.zGameMc.boneSlotArray;
	    }
	    replaceSlotTexture(slotName, t) {
	        if (this.mSkeleton)
	            this.mSkeleton.setSlotSkin(slotName, t);
	    }
	    destroyF() {
	        ConfigTidus.GVal["MyMC"].removeF(this);
	        this.completeFunction = ConfigTidus.GFun_ToolObj_destroyOne(this.completeFunction);
	        if (this.mSkeleton) {
	            this.mSkeleton.destroy();
	            this.mSkeleton = null;
	        }
	        if (this.zGameMc) {
	            if (MyZMovieClip.dicTemplet[this.URL]) {
	                if (--MyZMovieClip.dicTemplet[this.URL]["num"] <= 0) {
	                    this.zGameMc.destroy();
	                    MyZMovieClip.dicTemplet[this.URL] = null;
	                }
	            }
	            this.zGameMc = null;
	        }
	        super.destroyF();
	    }
	}

	class MyMC extends Sprite {
	    constructor(mc) {
	        super();
	        this.TimeFrame = 1000 / ConfigTidus.FrameRate;
	        this.pastTime = 0;
	        this.needLast = 0;
	        this.MC = mc;
	        this.fps = SwfMovieClip.FPS_default;
	        this.loop = false;
	        this.autoStopChild = false;
	        this.isPlay = false;
	        MyMC.addF(this);
	        if (this.MC != null) {
	            this.addChild(this.MC);
	            this.gotoAndStop(0);
	            if (this.MC instanceof MyZMovieClip) {
	                this.isSpine = true;
	                this.nowLabel = this.MC.nowLabel;
	            }
	        }
	    }
	    static addF(mc) {
	        this.dicAll.set(mc, true);
	    }
	    static removeF(mc) {
	        this.dicAll.delete(mc);
	    }
	    static pauseF() {
	        for (var i = 0; i < this.dicAll.keys.length; i++) {
	            var mc = this.dicAll.keys[i];
	            if (mc) {
	                mc.pauseF();
	            }
	        }
	    }
	    static resumeF() {
	        for (var i = 0; i < this.dicAll.keys.length; i++) {
	            var mc = this.dicAll.keys[i];
	            if (mc) {
	                mc.resumeF();
	            }
	        }
	    }
	    static onNewByMc(mc) {
	        if (mc == null)
	            return null;
	        return new MyMC(mc);
	    }
	    enterPlayF() {
	        this.pastTime += this.TimeFrame;
	        var f2 = parseInt("" + this.pastTime / this.NeedTime);
	        while (f2 > 0) {
	            this.nextFrame();
	            f2--;
	            this.pastTime -= this.NeedTime;
	        }
	    }
	    enterPrePlayF() {
	        this.pastTime += this.TimeFrame;
	        var f2 = parseInt("" + this.pastTime / this.NeedTime);
	        while (f2 > 0) {
	            this.preFrame();
	            f2--;
	            this.pastTime -= this.NeedTime;
	        }
	    }
	    gotoF(f) {
	        if (this.MC != null) {
	            if (f > this.endFrame) {
	                if (this.isComplete() == true)
	                    return;
	                f = this.endFrame;
	            }
	            this.MC.gotoAndStop(f, this.autoStopChild);
	            if (this.frameFunction)
	                Tool_Function.onRunFunction(this.frameFunction);
	            if (this.compFun) {
	                if (this.isComplete() == true) {
	                    Tool_Function.onRunFunction(this.compFun);
	                }
	            }
	        }
	    }
	    nextFrame() {
	        if (this.MC != null) {
	            if (this.isComplete() == true) {
	                if (this.loop == true) {
	                    this.gotoF(this.startFrame);
	                }
	            }
	            else {
	                this.gotoF(this.MC.currentFrame + 1);
	            }
	        }
	    }
	    preFrame() {
	        if (this.MC != null) {
	            if (this.MC.currentFrame == this.startFrame) {
	                if (this.loop == true)
	                    this.gotoF(this.endFrame);
	            }
	            else {
	                this.gotoF(this.MC.currentFrame - 1);
	            }
	        }
	    }
	    play() {
	        if (this.mmo) {
	            return;
	        }
	        this.mmo = new MainManager();
	        this.mmo.addEnterFrame(this.onEnterF.bind(this));
	    }
	    onEnterF() {
	        this.enterPlayF();
	        if (this.MC == null) {
	            this.mmo = ConfigTidus.GFun_ToolObj_destroyOne(this.mmo);
	            return;
	        }
	        if (this.loop == true)
	            return;
	        if (this.isComplete() == true) {
	            this.stop();
	        }
	    }
	    stop() {
	        this.pastTime = 0;
	        if (this.mmo) {
	            this.mmo = ConfigTidus.GFun_ToolObj_destroyOne(this.mmo);
	        }
	    }
	    gotoAndStop(f) {
	        if (typeof f == "string") {
	            this.nowLabel = f;
	            if (this.MC instanceof SwfMovieClip) {
	                this.MC.gotoAndStop(f);
	                this.startFrame = this.MC.startFrame;
	                this.endFrame = this.MC.endFrame;
	            }
	            else if (this.MC instanceof MyZMovieClip) {
	                this.MC.gotoAndStopLable(f);
	                this.startFrame = 0;
	                this.endFrame = this.MC.totalFrames - this.needLast;
	            }
	        }
	        else {
	            this.nowLabel = null;
	            if (this.MC instanceof SwfMovieClip) {
	                this.MC.gotoAndStop(f);
	                this.startFrame = this.MC.startFrame;
	                this.endFrame = this.MC.endFrame;
	            }
	            else if (this.MC instanceof MyZMovieClip) {
	                this.MC.gotoAndStop(f);
	                this.startFrame = 0;
	                this.endFrame = this.MC.totalFrames - this.needLast;
	            }
	            this.gotoF(f);
	        }
	        this.stop();
	    }
	    gotoAndPlay(f) {
	        if (typeof f == "string") {
	            this.nowLabel = f;
	            if (this.MC instanceof SwfMovieClip) {
	                this.MC.gotoAndStop(f);
	                this.startFrame = this.MC.startFrame;
	                this.endFrame = this.MC.endFrame;
	            }
	            else if (this.MC instanceof MyZMovieClip) {
	                this.MC.gotoAndStopLable(f);
	                this.startFrame = 0;
	                this.endFrame = this.MC.totalFrames - this.needLast;
	            }
	        }
	        else {
	            this.nowLabel = null;
	            if (this.MC instanceof SwfMovieClip) {
	                this.MC.gotoAndStop(f);
	                this.startFrame = this.MC.startFrame;
	                this.endFrame = this.MC.endFrame;
	            }
	            else if (this.MC instanceof MyZMovieClip) {
	                this.MC.gotoAndStop(f);
	                this.startFrame = 0;
	                this.endFrame = this.MC.totalFrames - this.needLast;
	            }
	            this.gotoF(f);
	        }
	        this.pastTime = 0;
	        this.play();
	    }
	    pauseF() {
	        this.stop();
	    }
	    resumeF() {
	        this.play();
	    }
	    addEvent(fun) {
	        if (this.MC instanceof MyZMovieClip) {
	            this.MC.FunEvent = fun;
	        }
	        else if (this.MC instanceof SwfMovieClip) {
	            this.MC.eventFunction = fun;
	        }
	    }
	    destroyF() {
	        MyMC.removeF(this);
	        this.frameFunction = ConfigTidus.GFun_ToolObj_destroyOne(this.frameFunction);
	        this.compFun = ConfigTidus.GFun_ToolObj_destroyOne(this.compFun);
	        this.MC = ConfigTidus.GFun_ToolObj_destroyOne(this.MC);
	        this.mmo = ConfigTidus.GFun_ToolObj_destroyOne(this.mmo);
	        super.destroyF();
	    }
	    get loop() {
	        return this._loop;
	    }
	    set loop(value) {
	        this._loop = value;
	        if (this.MC instanceof MyZMovieClip) {
	            this.MC.loop = this._loop;
	        }
	    }
	    get fps() {
	        return this._fps;
	    }
	    set fps(value) {
	        this._fps = value;
	        this.NeedTime = 1000 / this._fps;
	    }
	    get currentFrame() {
	        return this.MC.currentFrame;
	    }
	    get totalFrames() {
	        return this.MC.totalFrames;
	    }
	    isComplete() {
	        if (this.MC == null)
	            return true;
	        if (this.MC instanceof SwfMovieClip) {
	            return this.MC.currentFrame >= this.endFrame;
	        }
	        if (this.MC instanceof MyZMovieClip) {
	            return this.MC.currentFrame >= this.endFrame;
	        }
	        return this.MC.currentFrame >= this.endFrame;
	    }
	    get autoStopChild() {
	        return this._autoStopChild;
	    }
	    set autoStopChild(value) {
	        this._autoStopChild = value;
	    }
	    onWindowFit(needclearBG = true, type = 4) {
	        if (this.MC instanceof SwfMovieClip) {
	            var mc = this.MC;
	            if (needclearBG) {
	                var bimg = mc.getChildAt(0);
	                mc.removeChildFromAllFrame(bimg.classLink);
	                this.addChildAt(bimg, 0);
	            }
	            ConfigTidus.GFun_ToolView_winfit(this, type);
	        }
	    }
	}
	MyMC.dicAll = new Map();

	var AutoBitmap = Laya.AutoBitmap;
	class SwfS9Image extends Sprite {
	    constructor() {
	        super();
	        var g = new AutoBitmap();
	        this.graphics = g;
	    }
	    get sizeGrid() {
	        if (this.graphics.sizeGrid)
	            return this.graphics.sizeGrid.join(",");
	        return null;
	    }
	    set sizeGrid(value) {
	        this.graphics.sizeGrid = Laya.UIUtils.fillArray(Laya.Styles.defaultSizeGrid, value, Number);
	    }
	    get source() {
	        return this.graphics.source;
	    }
	    set source(value) {
	        if (!this.graphics)
	            return;
	        this.graphics.source = value;
	        super.event(Laya.Event.LOADED);
	        super.repaint();
	        this.bw = value.width;
	        this.bh = value.height;
	    }
	    set width(value) {
	        this.graphics.width = value == 0 ? 0.0000001 : value;
	    }
	    get width() {
	        return this.graphics.width;
	    }
	    set height(value) {
	        this.graphics.height = value == 0 ? 0.0000001 : value;
	    }
	    get height() {
	        return this.graphics.height;
	    }
	    removeFromParent(dispose = false) {
	        if (this.parent != null) {
	            this.parent.removeChild(this);
	        }
	        if (dispose) {
	            this.destroy();
	        }
	    }
	    destroy(destroyChild = true) {
	        if (this.graphics)
	            this.graphics.destroy();
	        super.destroy(true);
	    }
	}

	var SoundManager = Laya.SoundManager;
	class SoundManagerMy {
	    constructor() {
	        this.pause = 0;
	        this.mute = false;
	        this.isLoading = false;
	        this.Dic_fullName = {};
	    }
	    static get instance() {
	        if (this._instance == null)
	            this._instance = new SoundManagerMy();
	        return this._instance;
	    }
	    addSounds(arr, f) {
	        if (this.isLoading == true) {
	            if (this.Arr_waite == null)
	                this.Arr_waite = [];
	            this.Arr_waite.push({ "arr": arr, "f": f });
	            return;
	        }
	        this.Arr_now = arr;
	        this.Fun = f;
	        this.nextF(true);
	    }
	    nextF(first = false) {
	        this.isLoading = true;
	        if (first == false) {
	            this.Arr_now.shift();
	            if (this.Arr_now.length == 0) {
	                this.Fun();
	                this.Fun = null;
	                this.Arr_now = null;
	                if (this.Arr_waite == null || this.Arr_waite.length == 0) {
	                    this.isLoading = false;
	                }
	                else {
	                    var waite = this.Arr_waite.shift();
	                    this.Arr_now = waite["arr"];
	                    this.Fun = waite["f"];
	                    this.nextF(true);
	                }
	                return;
	            }
	        }
	        if (this.Arr_now[0] == null) {
	            this.nextF();
	            return;
	        }
	        var name = this.Arr_now[0][0];
	        var fullName;
	        var dir = this.Arr_now[0][1];
	        if (dir == null || dir.length == 0) {
	            fullName = name;
	        }
	        else if (dir.charAt(dir.length - 1) == "/") {
	            fullName = dir + name;
	        }
	        else {
	            fullName = dir + "/" + name;
	        }
	        fullName = SoundManagerMy.DirPath + "/" + fullName + "." + this.Arr_now[0][2];
	        this.Dic_fullName[name] = fullName;
	        this.nextF();
	    }
	    getFullName(name) {
	        return this.Dic_fullName[name];
	    }
	    playMusic(name, vol = -1, stopF = null, sTime = 0, loop = 9999, fadeTime = 0) {
	        if (vol <= 0)
	            vol = SoundManagerMy.soundVal;
	        name = this.getFullName(name);
	        if (name) {
	            SoundManager.playMusic(name, 0, stopF);
	        }
	    }
	    playSound(name, vol = -1) {
	        if (vol <= 0)
	            vol = SoundManagerMy.soundVal;
	        if (vol <= 0)
	            return;
	        name = this.getFullName(name);
	        if (name) {
	            SoundManager.playSound(name);
	        }
	    }
	    setVol(vol, name = null) {
	        SoundManager.soundVolume = vol;
	    }
	    stopMusic(name, fadeTime = 0) {
	        SoundManager.stopMusic();
	    }
	    removeSound(id) {
	    }
	    stopAll() {
	        SoundManager.stopAll();
	    }
	    haveSound(name) {
	        return true;
	    }
	    destroyF() {
	    }
	}
	SoundManagerMy.soundVal = 1;
	SoundManagerMy.DirPath = "sounds";

	class TextFormat {
	    constructor(t) {
	        this.T = t;
	    }
	    get color() { return Number("0x" + this.T.color.substr(1)); }
	    set color(value) {
	        this._color = value;
	        if (this.T) {
	            this.T.color = "#" + this._color.toString(16);
	        }
	    }
	    get font() { return this.T.font; }
	    set font(value) { this.T.font = value; }
	    get size() { return this.T.fontSize; }
	    set size(value) { this.T.fontSize = value; }
	    get horizontalAlign() { return this.T.align; }
	    set horizontalAlign(value) {
	        if (value == "左" || value == "left")
	            this.T.align = "left";
	        else if (value == "右" || value == "right")
	            this.T.align = "right";
	        else
	            this.T.align = "center";
	    }
	    get italic() { return this.T.italic; }
	    set italic(value) { this.T.italic = value; }
	    get bold() { return this.T.bold; }
	    set bold(value) { this.T.bold = value; }
	}

	class TextField extends Laya.Text {
	    constructor(_w, _h, text, formate = null) {
	        super();
	        this.isCleared = false;
	        this._autoScale = false;
	        this._touchable = false;
	        this.text = text;
	        this.lastStr = text;
	    }
	    get changed() {
	        let bol = this.lastStr != this.text;
	        this.lastStr = this.text;
	        return bol;
	    }
	    get format() {
	        if (this._format == null) {
	            this._format = new TextFormat(this);
	        }
	        else {
	            this._format.T = this;
	        }
	        return this._format;
	    }
	    set format(value) {
	        this._format = value;
	    }
	    get autoScale() { return this._autoScale; }
	    set autoScale(_x) {
	        this._autoScale = _x;
	    }
	    get touchable() { return this._touchable; }
	    set touchable(can) {
	        this._touchable = can;
	        this.mouseEnabled = can;
	    }
	    removeFromParent(dispose = false) {
	        this.isCleared = true;
	        if (this.parent != null) {
	            this.parent.removeChild(this);
	        }
	        if (dispose) {
	            this.destroy();
	        }
	    }
	}

	class Tool_Textfield {
	    static newTextfield(w, h, text, font, size, col, vUD = "中", hLR = "中", italic = false, bold = false, multiline = false) {
	        var t = new TextField(w, h, text);
	        t.font = font;
	        t.fontSize = size;
	        if (Tool_Function.isTypeOf(col, "num"))
	            t.color = Tool_Function.getColorStrFromInt(col);
	        else
	            t.color = col;
	        if (vUD == "上" || vUD == "up")
	            t.valign = "top";
	        else if (vUD == "下" || vUD == "down")
	            t.valign = "bottom";
	        else
	            t.valign = "middle";
	        if (hLR == "左" || hLR == "left")
	            t.align = "left";
	        else if (hLR == "右" || hLR == "right")
	            t.align = "right";
	        else
	            t.align = "center";
	        if (multiline)
	            t.wordWrap = true;
	        t.width = w;
	        t.height = h;
	        t.italic = italic;
	        t.bold = bold;
	        t.text = text;
	        t.autoScale = true;
	        return t;
	    }
	    static changeTextfieldColor(t, col) {
	        if (t == null)
	            return;
	        if (Tool_Function.isTypeOf(col, "num"))
	            t.color = Tool_Function.getColorStrFromInt(col);
	        else
	            t.color = col;
	    }
	    static creatTextButton(label, x, y, fun, parent = null) {
	        let t = this.newTextfield(150, 25, label, null, 20, 0xffffff, "中", "左");
	        t.x = x;
	        t.y = y;
	        if (parent)
	            parent.addChild(t);
	        t.on(Laya.Event.MOUSE_DOWN, this, (e) => { fun(e); });
	        return t;
	    }
	}

	class Color {
	    static getAlpha(color) { return (color >> 24) & 0xff; }
	    static getRed(color) { return (color >> 16) & 0xff; }
	    static getGreen(color) { return (color >> 8) & 0xff; }
	    static getBlue(color) { return color & 0xff; }
	    static setAlpha(color, alpha) {
	        return (color & 0x00ffffff) | (alpha & 0xff) << 24;
	    }
	    static setRed(color, red) {
	        return (color & 0xff00ffff) | (red & 0xff) << 16;
	    }
	    static setGreen(color, green) {
	        return (color & 0xffff00ff) | (green & 0xff) << 8;
	    }
	    static setBlue(color, blue) {
	        return (color & 0xffffff00) | (blue & 0xff);
	    }
	    static rgb(red, green, blue) {
	        return (red << 16) | (green << 8) | blue;
	    }
	    static argb(alpha, red, green, blue) {
	        return (alpha << 24) | (red << 16) | (green << 8) | blue;
	    }
	    static toVector(color, out = null) {
	        if (out == null)
	            out = new Array();
	        out[0] = ((color >> 16) & 0xff) / 255.0;
	        out[1] = ((color >> 8) & 0xff) / 255.0;
	        out[2] = (color & 0xff) / 255.0;
	        out[3] = ((color >> 24) & 0xff) / 255.0;
	        return out;
	    }
	    static multiply(color, factor) {
	        if (factor == 0.0)
	            return 0x0;
	        var alpha = ((color >> 24) & 0xff) * factor;
	        var red = ((color >> 16) & 0xff) * factor;
	        var green = ((color >> 8) & 0xff) * factor;
	        var blue = (color & 0xff) * factor;
	        if (alpha > 255)
	            alpha = 255;
	        if (red > 255)
	            red = 255;
	        if (green > 255)
	            green = 255;
	        if (blue > 255)
	            blue = 255;
	        return this.argb(alpha, red, green, blue);
	    }
	    static numbererpolate(startColor, endColor, ratio) {
	        var startA = (startColor >> 24) & 0xff;
	        var startR = (startColor >> 16) & 0xff;
	        var startG = (startColor >> 8) & 0xff;
	        var startB = (startColor) & 0xff;
	        var endA = (endColor >> 24) & 0xff;
	        var endR = (endColor >> 16) & 0xff;
	        var endG = (endColor >> 8) & 0xff;
	        var endB = (endColor) & 0xff;
	        var newA = startA + (endA - startA) * ratio;
	        var newR = startR + (endR - startR) * ratio;
	        var newG = startG + (endG - startG) * ratio;
	        var newB = startB + (endB - startB) * ratio;
	        return (newA << 24) | (newR << 16) | (newG << 8) | newB;
	    }
	    Color() { throw new Error(); }
	}
	Color.WHITE = 0xffffff;
	Color.SILVER = 0xc0c0c0;
	Color.GRAY = 0x808080;
	Color.BLACK = 0x000000;
	Color.RED = 0xff0000;
	Color.MAROON = 0x800000;
	Color.YELLOW = 0xffff00;
	Color.OLIVE = 0x808000;
	Color.LIME = 0x00ff00;
	Color.GREEN = 0x008000;
	Color.AQUA = 0x00ffff;
	Color.TEAL = 0x008080;
	Color.BLUE = 0x0000ff;
	Color.NAVY = 0x000080;
	Color.FUCHSIA = 0xff00ff;
	Color.PURPLE = 0x800080;

	class Image$1 extends Sprite {
	    constructor(t) {
	        super();
	        this.sx = 1;
	        this.sy = 1;
	        this.w0 = 0;
	        this.h0 = 0;
	        this.isMipmap = false;
	        this.texture = t;
	    }
	    get texture() {
	        return this.tmpTexture;
	    }
	    set texture(value) {
	        if (this.tmpTexture) {
	            this.graphics.clear();
	        }
	        this.tmpTexture = value;
	        if (this.tmpTexture == null)
	            return;
	        this.graphics.drawImage(this.tmpTexture, 0, 0);
	        this.w0 = this.tmpTexture.width;
	        this.h0 = this.tmpTexture.height;
	        if (this.isMipmap) {
	            this.isMipmap = false;
	            this.setMipMap(true);
	        }
	    }
	    setMipMap(real) {
	        if (this.isMipmap == real)
	            return;
	        this.isMipmap = real;
	        if (this.isMipmap == true) {
	            this.graphics.clear();
	            let mt = new Laya.Matrix();
	            mt.scale(4, 4);
	            this.graphics.drawTexture(this.tmpTexture, 0, 0, this.w0 / 4, this.h0 / 4, mt);
	        }
	        else {
	            this.graphics.clear();
	            this.graphics.drawImage(this.tmpTexture, 0, 0);
	        }
	    }
	    get height() {
	        if (this.h0 == 0) {
	            this.h0 = this.getBounds().height / this.scaleY;
	        }
	        return this.h0 * this.scaleY;
	    }
	    set height(value) {
	        this.scaleY = value / this.h0;
	    }
	    get width() {
	        if (this.w0 == 0) {
	            this.w0 = this.getBounds().width / this.scaleX;
	        }
	        return this.w0 * this.scaleX;
	    }
	    set width(value) {
	        this.scaleX = value / this.w0;
	    }
	    destroy(destroyChild = true) {
	        this.tmpTexture = null;
	        super.destroy(destroyChild);
	    }
	}

	class SwfImage extends Image$1 {
	    constructor(t) {
	        super(t);
	    }
	}

	class SWF {
	    constructor(url_Json, FLoaded, FProgress = null) {
	        this.Dic_NameFrame = {};
	        if (SwfMovieClip.TimeFrame <= 0)
	            SwfMovieClip.TimeFrame = 1000 / ConfigTidus.FrameRate;
	        this.FunLoaded = FLoaded;
	        this.FProgress = FProgress;
	        this.hanProgress = Laya.Handler.create(this, this.onAtlasProgress, null, false);
	        if (typeof url_Json === "string") {
	            this.URL_Swf = url_Json;
	            if (url_Json.indexOf("/") != -1) {
	                this.SwfName = url_Json.slice(url_Json.lastIndexOf("/") + 1, url_Json.indexOf(".json"));
	            }
	            Laya.loader.load(url_Json, Laya.Handler.create(this, this.onLoaded), this.hanProgress, Laya.Loader.ATLAS);
	        }
	        else {
	            this.customLoadData = url_Json;
	            this.SwfName = url_Json["name"];
	            Laya.loader.load(this.customLoadData.json, Laya.Handler.create(this, this.onLoadedFromData), null, Laya.Loader.JSON);
	        }
	    }
	    onLoadedFromData(data) {
	        if (this.FProgress)
	            this.FProgress(0.6);
	        var toloadPics = [];
	        if (!this._data) {
	            this._data = data;
	            if (data.meta && data.meta.image) {
	                toloadPics = [data.meta.image];
	                var changeType = null;
	                if (Laya.Browser.onAndroid && data.meta.compressTextureAndroid) {
	                    changeType = ".ktx";
	                }
	                if (Laya.Browser.onIOS && data.meta.compressTextureIOS) {
	                    changeType = ".pvr";
	                }
	                if (changeType) {
	                    toloadPics[0] = toloadPics[0].replace(".png", changeType);
	                }
	            }
	            else {
	            }
	            toloadPics.reverse();
	            data.toLoads = toloadPics;
	            data.pics = [];
	        }
	        Laya.loader.load(this.customLoadData.png, Laya.Handler.create(this, this.onLoadedAtlasPic), null, Laya.Loader.IMAGE);
	    }
	    onLoadedAtlasPic(data) {
	        this._data.pics.push(data);
	        var frames = this._data.frames;
	        var directory = this.customLoadData.name + "/";
	        var pics = this._data.pics;
	        for (let name in frames) {
	            let obj = frames[name];
	            let tPic = pics[0];
	            let url = Laya.URL.formatURL(directory + name);
	            let tTexture = Laya.Texture.create(tPic, obj.frame.x, obj.frame.y, obj.frame.w, obj.frame.h, obj.spriteSourceSize.x, obj.spriteSourceSize.y, obj.sourceSize.w, obj.sourceSize.h);
	            Laya.Loader.cacheTexture(url, tTexture);
	            tTexture.url = url;
	            this.Dic_NameFrame[name.split(".")[0]] = tTexture;
	        }
	        delete this._data.pics;
	        if (this.FProgress)
	            this.FProgress(0.9);
	        this.l = new Laya.Loader();
	        this.l.once(Laya.Event.COMPLETE, this, this.loadCompleteF);
	        this.l.once(Laya.Event.ERROR, this, this.loadErrorF);
	        this.l.on(Laya.Event.PROGRESS, null, function (per) {
	            if (per != 1) {
	                if (this.FProgress)
	                    this.FProgress(0.9 + per * 0.1);
	            }
	        });
	        this.l.load(this.customLoadData.cer, Laya.Loader.BUFFER);
	    }
	    onLoaded() {
	        this.hanProgress.clear();
	        this.hanProgress = null;
	        var atlas = Laya.Loader.getAtlas(this.URL_Swf);
	        var data = Laya.Loader.getRes(this.URL_Swf);
	        if (data == null) {
	            console.error(this.SwfName + ":加载json报错！");
	            this.onLoadEndF(false);
	            return;
	        }
	        var frames = data.frames;
	        var str;
	        var count = 0;
	        for (let name in frames) {
	            var t = Laya.loader.getRes(atlas[count++]);
	            if (name.indexOf(".") >= 0)
	                str = name.slice(0, name.lastIndexOf("."));
	            this.Dic_NameFrame[str] = t;
	        }
	        this.onLoadBytes(this.URL_Swf);
	    }
	    onAtlasProgress(per) {
	        if (this.FProgress)
	            this.FProgress(0.9 * per);
	    }
	    onLoadBytes(url_Json, fileType = null) {
	        if (fileType == null)
	            fileType = SWF.dataFileType;
	        this.url_bytes = url_Json.substr(0, url_Json.lastIndexOf(".")) + fileType;
	        var data = Laya.Loader.getRes(this.url_bytes);
	        if (data) {
	            this.initData(data);
	        }
	        else {
	            this.l = new Laya.Loader();
	            this.l.once(Laya.Event.COMPLETE, this, this.loadCompleteF);
	            this.l.once(Laya.Event.ERROR, this, this.loadErrorF);
	            this.l.on(Laya.Event.PROGRESS, null, function (per) {
	                if (per != 1) {
	                    if (this.FProgress)
	                        this.FProgress(0.9 + per * 0.1);
	                }
	            });
	            this.l.load(this.url_bytes, Laya.Loader.BUFFER);
	        }
	    }
	    loadErrorF(err) {
	        console.error(this.SwfName + ":加载bytes报错！");
	        this.loadCompleteF(null);
	    }
	    loadCompleteF(data) {
	        this.l.offAll();
	        if (data == null && this.url_bytes.includes(".cer")) {
	            this.onLoadBytes(this.URL_Swf, ".mp3");
	            return;
	        }
	        if (data == null && this.url_bytes.includes(".mp3")) {
	            this.onLoadBytes(this.URL_Swf, ".txt");
	            return;
	        }
	        this.initData(data);
	    }
	    initData(data) {
	        try {
	            var inflate = new Zlib.Inflate(new Uint8Array(data));
	            var inbuffer = inflate.decompress();
	            var bytes = new Laya.Byte(inbuffer);
	        }
	        catch (e) {
	            console.error(this.SwfName + ":解压报错！" + e.message);
	            return;
	        }
	        this.readUTFAsync(bytes, (str) => {
	            this._swfDatas = JSON.parse(str);
	            if (this._swfDatas["particle"] != null) {
	                var arrAll = [];
	                for (var key in this._swfDatas["particle"]) {
	                    if (this._swfDatas["particle"][key] != null) {
	                        arrAll.push(key);
	                    }
	                }
	                if (arrAll.length > 0) {
	                    this.onLoadParticalSource(arrAll);
	                    return;
	                }
	            }
	            this.onLoadEndF(true);
	        });
	    }
	    readUTFAsync(byte, callback, start = null) {
	        let i = 0;
	        let str = start ? start : "";
	        let len = 100000;
	        if (byte.bytesAvailable > len) {
	            str += byte.readUTFBytes(len);
	        }
	        else {
	            str += byte.readUTFBytes();
	        }
	        if (byte.bytesAvailable == 0) {
	            callback(str);
	            return;
	        }
	        setTimeout(() => {
	            this.readUTFAsync(byte, callback, str);
	        }, 3);
	    }
	    onLoadParticalSource(arr) {
	        var key = arr[0];
	        key = key.substr(9);
	        Laya.loader.load("assets/Particles/" + key + ".part", Laya.Handler.create(this, this.onParticleLoadOne, [arr]), null, Laya.Loader.JSON);
	    }
	    onParticleLoadOne(arr, settings) {
	        var key = arr.shift();
	        if (this.Dic_ParticleSetting == null) {
	            this.Dic_ParticleSetting = {};
	        }
	        this.Dic_ParticleSetting[key] = settings;
	        if (arr.length == 0) {
	            this.onLoadEndF(true);
	        }
	        else {
	            this.onLoadParticalSource(arr);
	        }
	    }
	    onLoadEndF(suc) {
	        this.FunLoaded(suc == true ? this : null);
	        this.FunLoaded = null;
	    }
	    createFuns(objData) {
	        var strName;
	        if (objData instanceof Array)
	            strName = objData[0];
	        else if (typeof objData === 'string')
	            strName = objData;
	        else
	            strName = objData["url"];
	        if (strName.indexOf(SWF.dataKey_Image) == 0)
	            return this.getImage(strName);
	        if (strName.indexOf(SWF.dataKey_Sprite) == 0)
	            return this.getSprite(strName);
	        if (strName.indexOf(SWF.dataKey_MovieClip) == 0)
	            return this.getMc(strName);
	        if (strName.indexOf(SWF.dataKey_Scale9) == 0)
	            return this.getS9Image(strName);
	        if (strName.indexOf(SWF.dataKey_TextField) == 0)
	            return this.createTextField(objData);
	        if (strName.indexOf(SWF.dataKey_Particle) == 0)
	            return this.creatParticle(objData);
	        return null;
	    }
	    ;
	    getObject(strName) {
	        if (strName.indexOf("img_") == 0)
	            return this.getImage(strName);
	        if (strName.indexOf("spr_") == 0)
	            return this.getSprite(strName);
	        if (strName.indexOf("mc_") == 0)
	            return this.getMc(strName);
	        if (strName.indexOf("s9_") == 0)
	            return this.getS9Image(strName);
	        return null;
	    }
	    getTexture(strName) {
	        if (this.Dic_NameFrame[strName] != null) {
	            var t = this.Dic_NameFrame[strName];
	            return t;
	        }
	        return null;
	    }
	    getImage(strName) {
	        if (this._swfDatas[SWF.dataKey_Image][strName] == null) {
	            return null;
	        }
	        if (this.Dic_NameFrame[strName] != null) {
	            var t = this.Dic_NameFrame[strName];
	            var image = new SwfImage(null);
	            var imageData = this._swfDatas[SWF.dataKey_Image][strName];
	            image.classLink = strName;
	            image.swfName = this.SwfName;
	            image.texture = t;
	            image.pivotX = imageData[imageData.length - 2];
	            image.pivotY = imageData[imageData.length - 1];
	            if (typeof (imageData[0]) != "number") {
	                image.metaData = imageData[0];
	                if (imageData.length >= 5) {
	                }
	            }
	            else {
	                if (imageData.length >= 4) {
	                }
	            }
	            return image;
	        }
	        return null;
	    }
	    getS9Image(strName) {
	        if (this.Dic_NameFrame[strName] != null) {
	            var scale9Data = this._swfDatas[SWF.dataKey_Scale9][strName];
	            var t = this.Dic_NameFrame[strName];
	            var image = new SwfS9Image();
	            image.classLink = strName;
	            image.swfName = this.SwfName;
	            image.source = t;
	            image.sizeGrid = scale9Data["y"] + "," + (t.width - scale9Data["x"] - scale9Data["w"]) + "," + (t.height - scale9Data["y"] - scale9Data["h"]) + "," + scale9Data["x"];
	            return image;
	        }
	        return null;
	    }
	    getSprite(strName) {
	        if (this._swfDatas[SWF.dataKey_Sprite] == null || this._swfDatas[SWF.dataKey_Sprite][strName] == null) {
	            console.log("没有Sprite:" + strName);
	            return null;
	        }
	        var sprData = this._swfDatas[SWF.dataKey_Sprite][strName];
	        var sprite = new Sprite();
	        var length = sprData.length;
	        var objData;
	        var display;
	        var i = 0;
	        if (sprData.length > 0) {
	            if (!(sprData[0] instanceof Array) && sprData[0]["url"] == null)
	                i = 1;
	            for (; i < length; i++) {
	                objData = sprData[i];
	                display = this.getOneChild(objData);
	                if (display == null)
	                    continue;
	                sprite.addChild(display);
	            }
	        }
	        sprite.setSpriteData(sprData);
	        sprite.classLink = strName;
	        sprite.swfName = this.SwfName;
	        return sprite;
	    }
	    getOneChild(objData) {
	        if (objData == null)
	            return null;
	        var display;
	        var i = 0;
	        display = this.createFuns(objData);
	        if (display == null)
	            return null;
	        display.name = objData["name"];
	        display.pos(objData["x"], objData["y"]);
	        if (objData["type"] != SWF.dataKey_Scale9) {
	            display.scaleX = objData["sx"];
	            display.scaleY = objData["sy"];
	            display.sx0 = objData["sx"];
	            display.sy0 = objData["sy"];
	        }
	        else {
	            let s9w = display.bw * objData["sx"];
	            let s9h = display.bh * objData["sy"];
	            if (objData["w"] && objData["w"] > s9w)
	                s9w = objData["w"];
	            if (objData["h"] && objData["h"] > s9h)
	                s9h = objData["h"];
	            display.width = s9w;
	            display.height = s9h;
	        }
	        display.skewX = -objData["skewx"];
	        display.skewY = objData["skewy"];
	        display.alpha = objData["alpha"];
	        return display;
	    }
	    getMc(strName) {
	        if (this._swfDatas[SWF.dataKey_MovieClip] == null || this._swfDatas[SWF.dataKey_MovieClip][strName] == null) {
	            console.log("没有MC:" + strName);
	            return null;
	        }
	        var movieClipData = this._swfDatas[SWF.dataKey_MovieClip][strName];
	        if (movieClipData == null)
	            return null;
	        var objectCountData = movieClipData["objCount"];
	        var displayObjects = {};
	        var displayObjectArray;
	        var type;
	        var count;
	        for (var objName in objectCountData) {
	            type = objectCountData[objName][0];
	            count = objectCountData[objName][1];
	            displayObjectArray = displayObjects[objName] == null ? [] : displayObjects[objName];
	            if (type == "particle") {
	                for (let i = 0; i < count; i++) {
	                    displayObjectArray.push(this.createFuns(objName));
	                }
	            }
	            else if (type == "text") {
	                for (let i = 0; i < count; i++) {
	                    displayObjectArray.push(null);
	                }
	            }
	            else {
	                for (let i = 0; i < count; i++) {
	                    if (i >= SwfMovieClip.CacheNumBegin)
	                        break;
	                    displayObjectArray.push(this.createFuns(objName));
	                }
	            }
	            displayObjects[objName] = displayObjectArray;
	        }
	        var mc = new SwfMovieClip(movieClipData["frames"], movieClipData["labels"], displayObjects, this, movieClipData["frameEvents"]);
	        if (movieClipData["重复"] != null)
	            mc.onCashSameFrameInfo(movieClipData["重复"]);
	        if (movieClipData["meta"] != null)
	            mc.setMetaData(movieClipData["meta"]);
	        mc.loop = movieClipData["loop"];
	        mc.classLink = strName;
	        mc.swfName = this.SwfName;
	        return mc;
	    }
	    createTextField(data) {
	        var textfield;
	        var txt = "";
	        if (data) {
	            if (data["text"] && data["text"] != "\r" && data["text"] != "") {
	                txt = data["text"];
	            }
	            if (data["font"] != null) {
	                if (SWF.DicFont && SWF.DicFont[data["font"]] != null) {
	                    data["font"] = SWF.DicFont[data["font"]];
	                }
	                else if (SWF.DicFont && (data["font"] == "Arial" || data["font"] == "_sans") && SWF.DicFont["default"]) {
	                    data["font"] = SWF.DicFont["default"];
	                }
	            }
	            else {
	                if (SWF.DicFont && SWF.DicFont["default"])
	                    data["font"] = SWF.DicFont["default"];
	            }
	            let halign = "上";
	            if (data["multiline"] != true)
	                halign = "中";
	            textfield = Tool_Textfield.newTextfield(data["w"], data["h"], txt, data["font"], data["size"], data["color"], halign, data["align"], data["italic"], data["bold"], data["multiline"]);
	            if (data["filter"]) {
	                var filterData;
	                for (let filterName in data["filter"]) {
	                    filterData = data["filter"][filterName];
	                    if (filterName.includes("GlowFilter")) {
	                        textfield.stroke = filterData.blurX;
	                        textfield.strokeColor = Tool_Function.getColorStrFromInt(filterData.color);
	                    }
	                    else if (filterName.includes("BlurFilter")) {
	                    }
	                }
	            }
	        }
	        return textfield;
	    }
	    creatParticle(_name) {
	        if (this.Dic_ParticleSetting[_name] == null) {
	            return null;
	        }
	        var sp = new Laya.Particle2D(this.Dic_ParticleSetting[_name]);
	        sp.emitter.start();
	        sp.play();
	        return sp;
	    }
	    destroyF() {
	        Laya.Loader.clearRes(this.URL_Swf);
	    }
	    onCheckDataForFilterAndColor(display, data) {
	        if (data == null)
	            return;
	        SWF.setBlendMode(display, data["blend"]);
	        if (data["color"] != null && (display instanceof Sprite) == true) {
	            SWF.changeColor(display, data["color"]);
	        }
	    }
	    static changeColor(tar, color) {
	        if (tar == null) {
	            return;
	        }
	        if (color == null) {
	            if (tar.filters == null)
	                return;
	            for (let i = 0; i < tar.filters.length; i++) {
	                let lastf = tar.filters[i];
	                if (lastf instanceof Laya.ColorFilter) {
	                    tar.filters.splice(i--, 1);
	                }
	            }
	            if (tar.filters.length == 0)
	                tar.filters = null;
	            return;
	        }
	        if (color instanceof Array) {
	            let red = color[0] / color[1];
	            let green = color[2] / color[3];
	            let blue = color[4] / color[5];
	            var matrix = [
	                1, 0, 0, 0, red,
	                0, 1, 0, 0, green,
	                0, 0, 1, 0, blue,
	                0, 0, 0, 1, -color[1] * 255
	            ];
	            var fil = new Laya.ColorFilter(matrix);
	            if (tar.filters == null)
	                tar.filters = [fil];
	            else {
	                for (let i = 0; i < tar.filters.length; i++) {
	                    let lastf = tar.filters[i];
	                    if (lastf instanceof Laya.ColorFilter) {
	                        tar.filters.splice(i--, 1);
	                    }
	                }
	                tar.filters.push(fil);
	                tar.filters = tar.filters;
	            }
	        }
	        else {
	            matrix = [
	                Color.getRed(color) / 255, 0, 0, 0, 0,
	                0.3086, Color.getGreen(color) / 255, 0.0820, 0, 0,
	                0.3086, 0.6094, Color.getBlue(color) / 255, 0, 0,
	                0, 0, 0, 1, 0
	            ];
	            tar.color = color;
	        }
	    }
	    static setBlendMode(display, blendMode) {
	        if (blendMode == "add") {
	            display.blendMode = "lighter";
	        }
	        else if (blendMode == "erase") {
	        }
	        else {
	            display.blendMode = null;
	        }
	    }
	}
	SWF.dataKey_Sprite = "spr";
	SWF.dataKey_Image = "img";
	SWF.dataKey_MovieClip = "mc";
	SWF.dataKey_TextField = "text";
	SWF.dataKey_Scale9 = "s9";
	SWF.dataKey_Particle = "particle";
	SWF.dataFileType = ".cer";

	class MySourceManager {
	    constructor() {
	        this.Dic_Num = {};
	        this.Dic_have = {};
	        this.Dic_swf = {};
	        this.Dic_json = {};
	        this.Dic_txt = {};
	        this.Arr_waiteArrs = [];
	    }
	    static getInstance() {
	        if (this.instance == null)
	            this.instance = new MySourceManager();
	        return this.instance;
	    }
	    addTexture(arr, f, fprogress = null) {
	        for (var i = 0; i < arr.length; i++) {
	            if (arr[i] == null)
	                continue;
	            var name = arr[i][0];
	            if (name == null)
	                continue;
	            if (this.Dic_Num[name] == null)
	                this.Dic_Num[name] = 0;
	            this.Dic_Num[name]++;
	        }
	        if (this.Arr_waite) {
	            this.Arr_waiteArrs.push([arr, f, fprogress]);
	            return;
	        }
	        this.FunLoadOver = f;
	        this.Arr_waite = [];
	        for (i = 0; i < arr.length; i++) {
	            if (arr[i] == null)
	                continue;
	            this.Arr_waite.push(arr[i]);
	        }
	        this.allNum = this.Arr_waite.length;
	        ConfigTidus.GFun_MM_addDelay(2, this.loadNextF.bind(this), true);
	    }
	    removeTexture(name) {
	        if (name == null)
	            return;
	        if (--this.Dic_Num[name] > 0)
	            return;
	        this.onRemoveF(name);
	    }
	    onRemoveF(name) {
	        var swf = this.Dic_swf[name];
	        if (swf)
	            swf.destroyF();
	        delete this.Dic_swf[name];
	        delete this.Dic_have[name];
	    }
	    removeTextures(arr) {
	        while (arr.length > 0) {
	            if (arr[0] != null)
	                this.removeTexture(arr[0][0]);
	            arr.shift();
	        }
	    }
	    loadNextF(first = false) {
	        if (first != true) {
	            if (this.Arr_waite == null || this.Arr_waite.length == 0)
	                console.error("loadNextF中Arr_waite = " + this.Arr_waite);
	            else
	                this.Arr_waite.shift();
	            if (this.FunProgress)
	                this.FunProgress((this.allNum - this.Arr_waite.length) / this.allNum);
	        }
	        if (this.Arr_waite == null || this.Arr_waite.length == 0) {
	            var tmpF = this.FunLoadOver;
	            this.FunLoadOver = null;
	            Tool_Function.onRunFunction(tmpF);
	            if (this.Arr_waiteArrs.length > 0) {
	                this.Arr_waite = this.Arr_waiteArrs[0][0];
	                this.FunLoadOver = this.Arr_waiteArrs[0][1];
	                this.FunProgress = this.Arr_waiteArrs[0][2];
	                this.Arr_waiteArrs.shift();
	                if (this.Arr_waite == null || this.Arr_waite.length == 0) {
	                    console.error("新添加的Arr_waite == null!");
	                }
	                else {
	                    this.allNum = this.Arr_waite.length;
	                }
	                this.loadNextF(true);
	            }
	            else {
	                this.Arr_waite = null;
	                this.FunLoadOver = null;
	                this.FunProgress = null;
	            }
	            return;
	        }
	        var name = this.Arr_waite[0][0];
	        if (this.Dic_have[name] != null) {
	            this.loadNextF();
	            return;
	        }
	        var Type = this.Arr_waite[0][1];
	        if (Type == "png" || Type == "jpg" || Type == "net") {
	            if (this.Arr_waite[0][0] == null) {
	                this.onWantNextF("pic");
	                return;
	            }
	            if (this.Dic_pic == null) {
	                this.Dic_pic = {};
	            }
	            var Add = this.Arr_waite[0][2];
	            if (Add == null) {
	                Add = "res/";
	            }
	            else {
	                if (Add.charAt(Add.length - 1) != "/")
	                    Add += "/";
	            }
	            if (MySourceManager.DirMain != null) {
	                Add = MySourceManager.DirMain + "/" + Add;
	            }
	            if (Add.indexOf(".") == -1) {
	                Add += this.Arr_waite[0][0] + "." + Type;
	            }
	            Laya.loader.load(Add, Laya.Handler.create(this, this.onPicEnd, [Add]));
	        }
	        else if (Type == "swf") {
	            if (this.Arr_waite[0][0] == null) {
	                this.onWantNextF("swf");
	                return;
	            }
	            Add = this.Arr_waite[0][2];
	            if (Add == null) {
	                Add = name + "/";
	            }
	            else {
	                if (Add.charAt(Add.length - 1) != "/")
	                    Add += "/";
	            }
	            if (Add.indexOf("http:") == 0) {
	            }
	            else {
	                if (Add.indexOf("res/") != 0)
	                    Add = "res/" + Add;
	                if (MySourceManager.DirMain != null) {
	                    Add = MySourceManager.DirMain + "/" + Add;
	                }
	            }
	            new SWF(Add + name + ".json", this.onSwfLoaded.bind(this), this.onSWFProgress.bind(this));
	        }
	        else if (Type == "zmc") {
	            Add = this.Arr_waite[0][2];
	            if (Add == null) {
	                Add = name + "/";
	            }
	            else {
	                if (Add.charAt(Add.length - 1) != "/")
	                    Add += "/";
	            }
	            if (Add.indexOf("res/") != 0)
	                Add = "res/" + Add;
	            if (MySourceManager.DirMain != null) {
	                Add = MySourceManager.DirMain + "/" + Add;
	            }
	            new MyZMovieClip(Add + name + ".sk", this.onZMCLoaded.bind(this), name);
	        }
	        else if (Type == "mp3" || Type == "wav" || Type == "snd") {
	            var sm = SoundManagerMy.instance;
	            Add = "";
	            if (this.Arr_waite[0].length == 3)
	                Add = this.Arr_waite[0][2];
	            if (MySourceManager.DirMain) {
	                Add = MySourceManager.DirMain + "/" + Add;
	            }
	            if (Type == "snd") {
	                if (window["conch"])
	                    Type = "wav";
	                else
	                    Type = "mp3";
	            }
	            var arr = [this.Arr_waite[0][0], Add, Type];
	            sm.addSounds([arr], this.onWantNextF.bind(this));
	        }
	        else if (Type == "json") {
	            Add = this.Arr_waite[0][2];
	            if (Add == null) {
	                Add = "";
	            }
	            if (Add.indexOf("res/") != 0)
	                Add = "res/" + Add;
	            if (MySourceManager.DirMain != null) {
	                Add = MySourceManager.DirMain + "/" + Add;
	            }
	            if (Add.indexOf(".") == -1) {
	                if (Add.charAt(Add.length - 1) != "/")
	                    Add += "/";
	                Add += this.Arr_waite[0][0] + "." + Type;
	            }
	            Laya.loader.load(Add, Laya.Handler.create(this, this.onJsonLoaded), null, Laya.Loader.TEXT);
	        }
	        else if (Type == "txt") {
	            Add = this.Arr_waite[0][2];
	            if (Add == null) {
	                Add = "";
	            }
	            if (Add.indexOf("res/") != 0)
	                Add = "res/" + Add;
	            if (MySourceManager.DirMain != null) {
	                Add = MySourceManager.DirMain + "/" + Add;
	            }
	            if (Add.indexOf(".") == -1) {
	                if (Add.charAt(Add.length - 1) != "/")
	                    Add += "/";
	                Add += this.Arr_waite[0][0] + "." + Type;
	            }
	            Laya.loader.load(Add, Laya.Handler.create(this, this.onTxtLoaded), null, Laya.Loader.TEXT);
	        }
	        else {
	            ConfigTidus.LogF("暂时不支持加载的文件" + Type);
	        }
	    }
	    onPicEnd(Add) {
	        var t = Laya.loader.getRes(Add);
	        this.Dic_pic[this.Arr_waite[0][0]] = t;
	        ConfigTidus.LogF("pic加载成功：" + this.Arr_waite[0][0]);
	        this.onWantNextF("swf");
	    }
	    onZMCLoaded(_name, zmc) {
	        if (zmc == null) {
	            ConfigTidus.LogF("加载zmc：" + _name + "失败！");
	        }
	        else {
	            if (this.Dic_zmc == null) {
	                this.Dic_zmc = {};
	            }
	            this.Dic_zmc[_name] = zmc;
	            this.Dic_have[_name] = true;
	        }
	        this.onWantNextF("zmc");
	    }
	    onSwfLoaded(swf) {
	        if (swf != null) {
	            this.Dic_have[swf.SwfName] = true;
	            this.Dic_swf[swf.SwfName] = swf;
	        }
	        this.onWantNextF("swf");
	    }
	    onJsonLoaded(tar) {
	        var Add = this.Arr_waite[0][2];
	        if (tar != null) {
	            try {
	                var data = JSON.parse(tar);
	                this.Dic_json[this.Arr_waite[0][0]] = data;
	            }
	            catch (error) {
	                console.error("加载的Json文件[" + this.Arr_waite[0][0] + "]解析失败!");
	            }
	        }
	        this.onWantNextF("json");
	    }
	    onTxtLoaded(tar) {
	        var Add = this.Arr_waite[0][2];
	        if (tar != null) {
	            this.Dic_txt[this.Arr_waite[0][0]] = tar;
	        }
	        this.onWantNextF("txt");
	    }
	    onWantNextF(type) {
	        if (this.Arr_waite == null && this.FunLoadOver == null)
	            return;
	        this.loadNextF();
	    }
	    onSWFProgress(per) {
	        if (per != 1) {
	            if (this.FunProgress)
	                this.FunProgress(per * 1 / this.allNum + (this.allNum - this.Arr_waite.length) / this.allNum);
	        }
	    }
	    getSwf(name) {
	        return this.Dic_swf[name];
	    }
	    getTexture(picname) {
	        return this.Dic_pic[picname];
	    }
	    getTextureFromSWF(swfName, name) {
	        var swf = this.Dic_swf[swfName];
	        if (swf == null) {
	            ConfigTidus.LogF("没有SWF：" + swfName);
	            return null;
	        }
	        return swf.getTexture(name);
	    }
	    getImgFromSwf(swfName, imgname) {
	        var swf = this.Dic_swf[swfName];
	        if (swf == null) {
	            ConfigTidus.LogF("没有SWF：" + swfName);
	            return null;
	        }
	        return swf.getImage(imgname);
	    }
	    getS9FromSwf(swfName, s9name) {
	        var swf = this.Dic_swf[swfName];
	        if (swf == null) {
	            return null;
	        }
	        return swf.getS9Image(s9name);
	    }
	    getSprFromSwf(swfName, sprname) {
	        var swf = this.Dic_swf[swfName];
	        if (swf == null) {
	            console.error("没有swf：" + swfName);
	            return null;
	        }
	        return swf.getSprite(sprname);
	    }
	    getMcFromSwf(swfName, mcname, canChangeClothes = false) {
	        var swf = this.Dic_swf[swfName];
	        if (swf == null) {
	            if (this.Dic_zmc != null && this.Dic_zmc[swfName]) {
	                var mc = this.Dic_zmc[swfName];
	                var nmc = new MyZMovieClip(mc.URL, null, null);
	                nmc.initF(canChangeClothes ? 1 : 0);
	                if (mcname)
	                    nmc.gotoAndPlayLable(mcname);
	                return nmc;
	            }
	            return null;
	        }
	        return swf.getMc(mcname);
	    }
	    getObjFromSwf(swfName, objName) {
	        if (swfName == null || objName == null)
	            return null;
	        if (objName.indexOf("img_") == 0)
	            return this.getImgFromSwf(swfName, objName);
	        if (objName.indexOf("spr_") == 0)
	            return this.getSprFromSwf(swfName, objName);
	        if (objName.indexOf("mc_") == 0)
	            return this.getMcFromSwf(swfName, objName);
	        if (objName.indexOf("s9_") == 0)
	            return this.getS9FromSwf(swfName, objName);
	        return null;
	    }
	    getJson(name) {
	        return this.Dic_json[name];
	    }
	    getTxt(name) {
	        return this.Dic_txt[name];
	    }
	    destroyF() {
	        for (var n in this.Dic_Num) {
	            this.removeTexture(n);
	        }
	        MySourceManager.instance = null;
	    }
	}

	class SwfImage$1 extends Image$1 {
	    constructor(t) {
	        super(t);
	    }
	}

	class ImageNum extends Sprite {
	    constructor(fname, _swf) {
	        super();
	        this.W = -1;
	        this.Arr_Save = [];
	        this.MidL = 0;
	        this.vAlin = "左";
	        this.MinL = -1;
	        this.Col = -1;
	        this.tmpCol = -1;
	        this.OnlyNumberColor = false;
	        this.neetW_to_10000 = -1;
	        this.Arr数字 = ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "+", "-", "."];
	        this.indexRun = -1;
	        this.SwfName = _swf;
	        if (this.SwfName == null)
	            this.SwfName = ImageNum.SWFDefault;
	        this.FrontName = fname;
	    }
	    addSource(arr = null) {
	        if (arr == null)
	            arr = [];
	        arr.push([this.SwfName, "swf", "FrontSource/" + this.SwfName]);
	        return arr;
	    }
	    setValue(str, val = null) {
	        switch (str) {
	            case "间隔":
	                this.MidL = val;
	                break;
	            case "对齐":
	                this.vAlin = val;
	                break;
	            case "最少位数":
	                this.MinL = val;
	                break;
	            case "颜色":
	                if (this.Col == val)
	                    return;
	                if (val == "红")
	                    val = 0xff0000;
	                else if (val == "黑")
	                    val = 0;
	                else if (val == "黄")
	                    val = 0xffff00;
	                else if (val == "蓝")
	                    val = 0x3366ff;
	                else if (val == "绿")
	                    val = 0x00ff00;
	                this.Col = val;
	                for (var i = 0; i < this.Arr_Save.length; i++) {
	                    if (this.OnlyNumberColor && this.Arr_Save[i][2] == true)
	                        continue;
	                    if (this.Arr_Save[i][1])
	                        this.Arr_Save[i][1].color = this.Col;
	                }
	                break;
	            case "局部颜色":
	                tmpCol = val;
	                break;
	            case "颜色仅数字":
	                this.OnlyNumberColor = val;
	                if (this.Col != -1) {
	                    var tmpCol = this.Col;
	                    this.Col = -1;
	                    this.setValue("颜色", tmpCol);
	                }
	                break;
	            case "父对象":
	                if (val instanceof SwfImage$1) {
	                    val.parent.addChildAt(this, val.parent.getChildIndex(val));
	                    val.parent.removeChild(val);
	                    this.x = val.x;
	                    this.y = val.y;
	                    this.TypeOfParent = "img";
	                }
	                else if (val instanceof Sprite) {
	                    val.removeChildren();
	                    val.addChild(this);
	                    this.TypeOfParent = "spr";
	                }
	                break;
	            case "x":
	                if (this.TypeOfParent == "spr")
	                    this.parent.x = val;
	                else
	                    this.x = val;
	                break;
	            case "y":
	                if (this.TypeOfParent == "spr")
	                    this.parent.y = val;
	                else
	                    this.y = val;
	                break;
	            case "stage监听":
	                this.once(Laya.Event.REMOVED, this, this.onRemoveF);
	                break;
	            case "万":
	                this.neetW_to_10000 = val;
	                break;
	        }
	    }
	    showF(val) {
	        this.L = 0;
	        if (this.Layer == null) {
	            this.Layer = new Sprite();
	            this.addChild(this.Layer);
	            this.tm = MySourceManager.getInstance();
	        }
	        else {
	            this.Layer.removeChildren();
	            this.clearSaveF();
	        }
	        if (val == null)
	            val = "";
	        else if (typeof (val) === "number" && this.neetW_to_10000 > 0 && val >= this.neetW_to_10000) {
	            val = Tool_Function.onChangeInstance((val / 10000).toFixed(1)) + "万";
	        }
	        this.appendShowF(val);
	    }
	    appendShowF(val) {
	        this.NowStr = Tool_Function.onChangeInstance(val, "str");
	        var x0 = this.L == 0 ? 0 : (this.L * this.W + (this.L - 1) * this.MidL);
	        this.L = this.NowStr.length;
	        if (this.L == 0)
	            return;
	        var more0 = -1;
	        if (this.L < this.MinL) {
	            more0 = this.MinL - this.L;
	            this.L = this.MinL;
	        }
	        for (var i = 0; i < this.L; i++) {
	            var char = this.NowStr.charAt(i);
	            if (more0-- >= 0)
	                char = "0";
	            if (char == ".")
	                char = "小数点";
	            var is数字 = true;
	            var img;
	            img = ImageNum.getCache(this.FrontName + char, this.SwfName);
	            if (img == null) {
	                img = this.tm.getImgFromSwf(this.SwfName, "img_" + this.FrontName + "Num_" + char);
	                if (img == null && this.SwfName != ImageNum.SWFDefault)
	                    img = this.tm.getImgFromSwf(ImageNum.SWFDefault, "img_" + this.FrontName + "Num_" + char);
	                if (img == null && char == "-") {
	                    img = ImageNum.getCache(this.FrontName + "减", this.SwfName);
	                    if (img == null && this.SwfName != ImageNum.SWFDefault)
	                        img = this.tm.getImgFromSwf(ImageNum.SWFDefault, "img_" + this.FrontName + "Num_" + "减");
	                }
	                if (img == null)
	                    console.log("找不到数字：" + this.SwfName + "：" + this.FrontName + "：" + char);
	            }
	            if (this.Arr数字.indexOf(char) == -1)
	                is数字 = false;
	            if (is数字 == false)
	                this.Arr_Save.push([char, img]);
	            else
	                this.Arr_Save.push([char, img, true]);
	            if (img) {
	                if (this.W == -1) {
	                    var imgW = this.tm.getImgFromSwf(this.SwfName, "img_" + this.FrontName + "FrontWidth");
	                    if (imgW == null) {
	                        if (img) {
	                            this.W = img.width + img.pivotX;
	                        }
	                    }
	                    else {
	                        this.W = imgW.width;
	                    }
	                }
	                img.x = x0 + i * this.W + this.MidL;
	                if (this.tmpCol != -1) {
	                    if (this.OnlyNumberColor == false || is数字 == true)
	                        img.color = this.tmpCol;
	                }
	                else if (this.Col != -1) {
	                    if (this.OnlyNumberColor == false || is数字 == true)
	                        img.color = this.Col;
	                }
	                this.Layer.addChild(img);
	            }
	        }
	        if (this.vAlin == "左")
	            this.Layer.x = 0;
	        else if (this.vAlin == "中")
	            this.Layer.x = -(this.L * this.W + (this.L - 1) * this.MidL) / 2 + this.W / 2;
	        else if (this.vAlin == "右")
	            this.Layer.x = -(this.L * this.W + (this.L - 1) * this.MidL) + this.W;
	        this.tmpCol = -1;
	    }
	    runF(fend, s, e, t, head, tail) {
	        this.FunRunover = fend;
	        this.NowNum = s;
	        this.EndNum = e;
	        this.RunTime = t;
	        this.STR_head = head;
	        if (this.STR_head == null)
	            this.STR_head = "";
	        this.STR_tail = tail;
	        if (this.STR_tail == null)
	            this.STR_tail = "";
	        this.SpdNum = (this.EndNum - this.NowNum) / t;
	        this.showF(this.STR_head + Tool_Function.onChangeInstance(this.NowNum) + this.STR_tail);
	        this.indexRun = ConfigTidus.GFun_MM_addEnter(this.runCheck.bind(this));
	    }
	    runCheck() {
	        if (this.NowNum == this.EndNum) {
	            if (this.indexRun > 0) {
	                ConfigTidus.GFun_MM_removeEnter(this.indexRun);
	                this.indexRun = -1;
	            }
	            if (this.FunRunover)
	                this.FunRunover.run();
	            return;
	        }
	        this.NowNum += this.SpdNum;
	        if (this.SpdNum > 0 && this.NowNum > this.EndNum)
	            this.NowNum = this.EndNum;
	        else if (this.SpdNum < 0 && this.NowNum < this.EndNum)
	            this.NowNum = this.EndNum;
	        if (--this.RunTime <= 0)
	            this.NowNum = this.EndNum;
	        this.showF(this.STR_head + Tool_Function.onChangeInstance(this.NowNum) + this.STR_tail);
	    }
	    getValue(want) {
	        switch (want) {
	            case "右边x":
	                var x0;
	                if (this.vAlin == "左")
	                    x0 = this.x + (this.L * this.W + (this.L - 1) * this.MidL);
	                else if (this.vAlin == "中")
	                    x0 = this.x - this.Layer.x + this.W;
	                else if (this.vAlin == "右")
	                    x0 = this.x;
	                if (this.TypeOfParent == "spr")
	                    x0 += this.parent.x;
	                return x0;
	            default: return this[want];
	        }
	    }
	    onRemoveF(e) {
	        this.destroyF();
	    }
	    destroyF() {
	        if (this.Layer) {
	            this.removeChild(this.Layer);
	            this.Layer.destroyF();
	            this.Layer = null;
	        }
	        this.tm = null;
	        this.clearSaveF();
	        super.destroyF();
	    }
	    clearSaveF() {
	        for (var i = 0; i < this.Arr_Save.length; i++) {
	            if (this.Arr_Save[i][1]) {
	                if (this.Col != -1)
	                    this.Arr_Save[i][1].color = Color.WHITE;
	                ImageNum.addCache(this.FrontName + this.Arr_Save[i][0], this.SwfName, this.Arr_Save[i][1]);
	            }
	            this.Arr_Save.splice(i--, 1);
	        }
	    }
	    static addNeedCache(fName, real) {
	        this.NeedCache[fName] = real;
	    }
	    static removeNeedCache(fName) {
	        if (fName == null) {
	            this.NeedCache = {};
	            this.clearF();
	        }
	        else
	            this.NeedCache[fName] = false;
	    }
	    static isNeedCache(fName) {
	        if (this.NeedCache[fName] != null)
	            return this.NeedCache[fName];
	        return false;
	    }
	    static getCache(str, swf) {
	        if (this.isNeedCache(str) == false)
	            return null;
	        if (this.DicCache == null)
	            this.DicCache = {};
	        if (this.DicCache[swf] == null)
	            this.DicCache[swf] = {};
	        if (this.DicCache[swf][str] == null)
	            this.DicCache[swf][str] = [];
	        var img;
	        if (this.DicCache[swf][str].length > 0) {
	            img = this.DicCache[swf][str][0];
	            this.DicCache[swf][str].shift();
	        }
	        return img;
	    }
	    static addCache(str, swf, obj) {
	        if (this.isNeedCache(str) == false)
	            return;
	        if (this.DicCache == null)
	            this.DicCache = {};
	        if (this.DicCache[swf] == null)
	            this.DicCache[swf] = {};
	        if (this.DicCache[swf][str] == null)
	            this.DicCache[swf][str] = [];
	        this.DicCache[swf][str].push(obj);
	    }
	    static clearF() {
	        if (this.DicCache == null)
	            return;
	        for (var swf in this.DicCache) {
	            var dic = this.DicCache[swf];
	            for (var key in dic) {
	                var arr = dic[key];
	                for (var i = 0; i < arr.length; i++) {
	                    arr[i].removeFromParent(true);
	                }
	                delete dic[key];
	            }
	        }
	        this.DicCache = null;
	    }
	    static onCacheAndSaveF(fName, swf) {
	        this.addNeedCache(fName, true);
	        for (var i = 0; i < 10; i++) {
	            var count = 5;
	            while (count-- > 0) {
	                this.addCache(fName + i, swf, MySourceManager.instance.getImgFromSwf(swf, "img_" + fName + "Num_" + i));
	            }
	        }
	    }
	}
	ImageNum.SWFDefault = "SWF_Default";
	ImageNum.NeedCache = {};

	class Tool_ImgUtils {
	    static onNoSmooth(obj, maxS = 3) {
	    }
	    static onAddSmooth(obj) {
	    }
	    static onCreateImageNum(fName, spr, _url, _swf = null) {
	        var tmp = spr.getChildByName(_url);
	        if (tmp == null)
	            return null;
	        var num = new ImageNum(fName, _swf);
	        num.setValue("父对象", tmp);
	        return num;
	    }
	    static cloneImage(target) {
	        var image;
	        if (target.swfName)
	            image = MySourceManager.getInstance().getImgFromSwf(target.swfName, target.classLink);
	        else
	            image = new Image$1(target.texture);
	        image.x = target.x;
	        image.y = target.y;
	        image.scaleX = target.scaleX;
	        image.scaleY = target.scaleY;
	        image.skewX = target.skewX;
	        image.skewY = target.skewY;
	        image.alpha = target.alpha;
	        image.classLink = target.classLink;
	        image.swfName = target.swfName;
	        return image;
	    }
	    static onNewImageFromTexture(t) {
	        if (t == null) {
	            return null;
	        }
	        var image = new Image$1(t);
	        return image;
	    }
	    static setMipMap(tar, value) {
	        if (tar == null)
	            return;
	        if (tar instanceof Image$1) {
	            tar.setMipMap(value);
	        }
	        else if (tar instanceof MyMC) {
	            this.setMipMap(tar.MC, value);
	        }
	        else if (tar instanceof SwfMovieClip) {
	            for (let k in tar._displayObjects) {
	                let arr = tar._displayObjects[k];
	                if (arr == null)
	                    continue;
	                let l = arr.length;
	                for (let i = 0; i < l; i++) {
	                    this.setMipMap(arr[i], value);
	                }
	            }
	        }
	        else if (tar instanceof Sprite) {
	            for (let i = 0; i < tar.numChildren; i++) {
	                this.setMipMap(tar.getChildAt(i), value);
	            }
	        }
	    }
	}

	class Tool_SpriteUtils {
	    static onGetImageNum(fName, spr, _url, _Parentswf = null) {
	        var tmp = spr.getChildByName(_url);
	        if (tmp == null)
	            return null;
	        var classLink = tmp.classLink;
	        var arr = classLink.split("_");
	        if (fName == null)
	            fName = arr[1];
	        if (_Parentswf == null) {
	            for (var i = 2; i < arr.length; i++) {
	                if (arr[i].indexOf("swf") == 0) {
	                    _Parentswf = arr[i].slice(3);
	                    if (_Parentswf == "SWF")
	                        _Parentswf += "_" + arr[i + 1];
	                    break;
	                }
	            }
	        }
	        var num = new ImageNum(fName, _Parentswf);
	        num.setValue("父对象", tmp);
	        if (arr.length > 2) {
	            var info = {};
	            for (var j = 2; j < arr.length; j++) {
	                var one = arr[j];
	                if (one.indexOf("颜色") == 0) {
	                    one = one.slice(2);
	                    if (one.indexOf("0x") != 0)
	                        one = "0x" + one;
	                    num.setValue("颜色", Tool_Function.onChangeInstance(one));
	                }
	                else if (one.indexOf("对齐") == 0) {
	                    num.setValue("对齐", one.slice(2));
	                }
	            }
	        }
	        return num;
	    }
	    static onGetAll_ImageNum(spr, fName = null, _Parentswf = null) {
	        var dic = {};
	        var arrChild = [];
	        for (var i = 0; i < spr.numChildren; i++) {
	            var c = spr.getChildAt(i);
	            var str = c.name;
	            if (str == null || str.indexOf("num_") != 0)
	                continue;
	            arrChild.push(str);
	        }
	        for (i = 0; i < arrChild.length; i++) {
	            str = arrChild[i];
	            var tmp = str.split("_");
	            var name = tmp[0] + "_" + tmp[1];
	            var info = { "fName": fName, "swf": _Parentswf };
	            for (var j = 2; j < tmp.length; j++) {
	                var one = tmp[j];
	                if (one.indexOf("字体") == 0)
	                    info["fName"] = one.slice(2);
	                else if (one.indexOf("swf") == 0) {
	                    var strSWF = "";
	                    j++;
	                    while (j < tmp.length) {
	                        if (strSWF.length > 0)
	                            strSWF += "_";
	                        strSWF += tmp[j++];
	                    }
	                    info["swf"] = strSWF;
	                    break;
	                }
	                else if (one.indexOf("颜色") == 0) {
	                    one = one.slice(2);
	                    if (one.indexOf("0x") != 0)
	                        one = "0x" + one;
	                    info["颜色"] = Tool_Function.onChangeInstance(one);
	                }
	                else if (one.indexOf("对齐") == 0)
	                    info["对齐"] = one.slice(2);
	            }
	            var num = this.onGetImageNum(info["fName"], spr, str, info["swf"]);
	            if (info["颜色"] != null)
	                num.setValue("颜色", info["颜色"]);
	            if (info["对齐"] != null)
	                num.setValue("对齐", info["对齐"]);
	            dic[name] = num;
	        }
	        return dic;
	    }
	    static cloneSprite(target) {
	        var numChilds = target.numChildren;
	        var sprite = new Sprite();
	        var child;
	        for (var i = 0; i < numChilds; i++) {
	            child = target.getChildAt(i);
	            if (child instanceof Image$1) {
	                sprite.addChild(Tool_ImgUtils.cloneImage(child));
	            }
	            else if (child instanceof Sprite) {
	                sprite.addChild(Tool_SpriteUtils.cloneSprite(child));
	            }
	        }
	        sprite.x = target.x;
	        sprite.y = target.y;
	        sprite.scaleX = target.scaleX;
	        sprite.scaleY = target.scaleY;
	        sprite.skewX = target.skewX;
	        sprite.skewY = target.skewY;
	        sprite.alpha = target.alpha;
	        sprite.classLink = target.classLink;
	        sprite.swfName = target.swfName;
	        return sprite;
	    }
	    static getMetaData_FromSPR(spr) {
	        var meta;
	        if (spr instanceof Sprite && spr.spriteData[0] instanceof Array == false)
	            meta = spr.spriteData[0];
	        return meta;
	    }
	    static onAddchild_ChangeParent(spr, child) {
	        spr.x = child.x;
	        spr.y = child.y;
	        if (child.parent) {
	            child.parent.addChildAt(spr, child.parent.getChildIndex(child));
	        }
	        spr.addChild(child);
	        child.x = child.y = 0;
	        spr.name = child.name;
	    }
	    static getBounds(tar, parent) {
	        if (tar instanceof SwfS9Image || tar instanceof Image$1) {
	            let rec = new Laya.Rectangle();
	            rec.x = tar.x;
	            rec.y = tar.y;
	            rec.width = tar.width;
	            rec.height = tar.height;
	            return rec;
	        }
	        if (tar instanceof Sprite) {
	            var x0 = 9999999;
	            var y0 = 9999999;
	            var x1 = -9999999;
	            var y1 = -9999999;
	            for (let i = 0; i < tar.numChildren; i++) {
	                var rec = Tool_SpriteUtils.getBounds(tar.getChildAt(i), null);
	                if (rec.x < x0) {
	                    x0 = rec.x;
	                }
	                if (rec.x + rec.width > x1) {
	                    x1 = rec.x + rec.width;
	                }
	                if (rec.y < y0) {
	                    y0 = rec.y;
	                }
	                if (rec.y + rec.height > y1) {
	                    y1 = rec.y + rec.height;
	                }
	            }
	            if (rec == null) {
	                rec = new Laya.Rectangle();
	                rec.x = tar.x;
	                rec.y = tar.y;
	                return rec;
	            }
	            rec.x = x0 + tar.x;
	            rec.y = y0 + tar.y;
	            rec.width = x1 - x0;
	            rec.height = y1 - y0;
	            return rec;
	        }
	        return tar.getBounds();
	    }
	    static removeChildren(spr, index = 0, end = -1, dispose = false) {
	        var count = end <= index ? 99999999 : end - index;
	        for (; count > 0 && spr.numChildren > index; count--) {
	            var one = spr.getChildAt(index);
	            if (dispose == true)
	                ConfigTidus.GFun_ToolObj_destroyOne(one);
	            else
	                spr.removeChild(one);
	        }
	    }
	    static replaceSprite(layer, nameTar, newChild) {
	        let tar = layer.getChildByName(nameTar);
	        if (tar && newChild) {
	            newChild.x = tar.x;
	            newChild.y = tar.y;
	            if (tar.classLink.indexOf("透明") == -1) {
	                newChild.scaleX = tar.scaleX;
	                newChild.scaleY = tar.scaleY;
	            }
	            layer.addChildAt(newChild, layer.getChildIndex(tar));
	            layer.removeChild(tar);
	        }
	    }
	    static sortChildren(tar, fun) {
	        let arr = [];
	        for (let i = 0; i < tar.numChildren; i++) {
	            arr[i] = tar.getChildAt(i);
	        }
	        tar.removeChildren();
	        arr.sort(fun);
	        for (let i = 0; i < arr.length; i++) {
	            tar.addChild(arr[i]);
	        }
	    }
	    static sortChildren_byY(tar) {
	        this.sortChildren(tar, (a, b) => {
	            if (a.y > b.y)
	                return 1;
	            if (a.y < b.y)
	                return -1;
	            if (isNaN(a.sortIndex)) {
	                if (isNaN(b.sortIndex)) {
	                    if (a.x <= b.x)
	                        return 1;
	                    return -1;
	                }
	            }
	            else if (isNaN(b.sortIndex)) {
	                return 1;
	            }
	            else {
	                if (a.sortIndex > b.sortIndex)
	                    return 1;
	                if (a.sortIndex < b.sortIndex)
	                    return -1;
	                if (a.x <= b.x)
	                    return 1;
	                return -1;
	            }
	        });
	    }
	    static fitSpriteToRec(spr, rec, scale, typex, typey) {
	        if (spr == null || rec == null)
	            return;
	        let w = spr.width;
	        let h = spr.height;
	        if (scale == true) {
	            let sx = rec.width / w;
	            let sy = rec.height / h;
	            let s = sx <= sy ? sx : sy;
	            if (s > 1)
	                s = 1;
	            w = w * s;
	            h = h * s;
	            spr.scaleX *= s;
	            spr.scaleY *= s;
	        }
	        if (typex == "中") {
	            spr.x = rec.x + (rec.width - w) / 2;
	        }
	        else if (typex == "左") {
	            spr.x = rec.x;
	        }
	        else {
	            spr.x = rec.x + rec.width - w;
	        }
	        if (typey == "中") {
	            spr.y = rec.y + (rec.height - h) / 2;
	        }
	        else if (typey == "上") {
	            spr.y = rec.y;
	        }
	        else {
	            spr.y = rec.y + (rec.height - h);
	        }
	    }
	    static addRecMask(tar, showRec) {
	        tar.scrollRect = new Laya.Rectangle(showRec.x, showRec.y, showRec.width, showRec.height);
	    }
	    static onMoveToPointWithAnyPoint(tar, selfPoint, tarPoint) {
	        let p = tar.localToGlobal(new Laya.Point(selfPoint.x, selfPoint.y), false, tar.parent);
	        let needx = tarPoint.x - p.x;
	        let needy = tarPoint.y - p.y;
	        tar.x += needx;
	        tar.y += needy;
	    }
	    static getOneChildInfo(tar, _name, remove) {
	        let one = tar.getChildByName(_name);
	        if (one == null)
	            return null;
	        let dic = { "x": one.x, "y": one.y };
	        if (one instanceof Sprite) {
	            dic["url"] = one.classLink;
	            dic["swf"] = one.swfName;
	        }
	        if (remove)
	            ConfigTidus.GFun_ToolObj_destroyOne(one);
	        return dic;
	    }
	    static onReplaceChild(mc, class0, FnewTar) {
	        if (mc == null) {
	            return;
	        }
	        var _displayObjects = mc._displayObjects;
	        var arr;
	        var l;
	        arr = _displayObjects[class0];
	        if (arr == null)
	            return;
	        l = arr.length;
	        for (let i = 0; i < l; i++) {
	            var old = arr[i];
	            if (FnewTar)
	                arr[i] = FnewTar();
	            else
	                arr[i] = null;
	            if (old)
	                old.removeFromParent(true);
	        }
	        var f = mc.currentFrame;
	        if (mc.isPlay == false)
	            mc.gotoAndStop(f);
	        else
	            mc.gotoAndPlay(f);
	    }
	}

	class MyMouseManager {
	    constructor(tar) {
	        this.pointIndex = {};
	        this.startXY = {};
	        this.nowLocalXY = new Laya.Point();
	        this.worldXY = {};
	        this.isDown = false;
	        this.pausing = false;
	        this.stopEventMop = null;
	        this.LStartMove = 10;
	        this.indexEnter = -1;
	        this.indexLongTouch = -1;
	        this.Target = tar;
	        tar.addAutoDestroyValue(this);
	        tar.on(Laya.Event.MOUSE_DOWN, this, this.mouseHandler);
	        tar.on(Laya.Event.MOUSE_UP, this, this.mouseHandler);
	        tar.on(Laya.Event.MOUSE_MOVE, this, this.mouseHandler);
	        tar.on(Laya.Event.MOUSE_OUT, this, this.mouseHandler);
	    }
	    destroyF() {
	        if (this.Target) {
	            this.Target.off(Laya.Event.MOUSE_DOWN, this, this.mouseHandler);
	            this.Target.off(Laya.Event.MOUSE_UP, this, this.mouseHandler);
	            this.Target.off(Laya.Event.MOUSE_MOVE, this, this.mouseHandler);
	            this.Target.off(Laya.Event.MOUSE_OUT, this, this.mouseHandler);
	            this.Target.off(Laya.Event.RIGHT_CLICK, this, this.mouseHandler);
	            this.Target = null;
	        }
	        if (this.indexEnter > 0) {
	            ConfigTidus.GFun_MM_removeEnter(this.indexEnter);
	            this.indexEnter = -1;
	        }
	        this.clearLongTouch();
	        this.Dic_Slide = null;
	        this.Dic_Click = null;
	        this.FunRightClick = null;
	        if (this.Dic_roll) {
	            Laya.stage.off(Laya.Event.MOUSE_WHEEL, this, this.onMouseWheel);
	            this.Dic_roll = null;
	        }
	    }
	    mouseHandler(e) {
	        var touches = e.touches;
	        if (touches && touches.length > 0) {
	            for (let i = 0; i < touches.length; i++) {
	                let t = touches[i];
	                let nameIndex = e.touchId + "_" + i;
	                if (this.pointIndex[nameIndex] == null)
	                    this.pointIndex[nameIndex] = new Laya.Point();
	                if (t == null) {
	                    this.upF(nameIndex, this.pointIndex[nameIndex]);
	                    continue;
	                }
	                this.pointIndex[nameIndex].x = t.stageX;
	                this.pointIndex[nameIndex].y = t.stageY;
	                switch (e.type) {
	                    case Laya.Event.MOUSE_DOWN:
	                        this.downF(nameIndex, this.pointIndex[nameIndex]);
	                        break;
	                    case Laya.Event.MOUSE_UP:
	                        this.upF(nameIndex, this.pointIndex[nameIndex]);
	                        break;
	                    case Laya.Event.MOUSE_MOVE:
	                        this.moveF(nameIndex, this.pointIndex[nameIndex]);
	                        break;
	                    case Laya.Event.MOUSE_OUT:
	                        this.upF(nameIndex, null);
	                        break;
	                    case Laya.Event.RIGHT_CLICK:
	                        if (this.FunRightClick)
	                            this.FunRightClick(nameIndex, this.pointIndex[nameIndex]);
	                        break;
	                }
	            }
	        }
	        else {
	            let i = "999";
	            let nameIndex = "_" + i;
	            if (this.pointIndex[nameIndex] == null)
	                this.pointIndex[nameIndex] = new Laya.Point();
	            this.pointIndex[nameIndex].x = Laya.stage.mouseX;
	            this.pointIndex[nameIndex].y = Laya.stage.mouseY;
	            switch (e.type) {
	                case Laya.Event.MOUSE_DOWN:
	                    this.downF(i, this.pointIndex[nameIndex]);
	                    break;
	                case Laya.Event.MOUSE_UP:
	                    this.upF(i, this.pointIndex[nameIndex]);
	                    break;
	                case Laya.Event.MOUSE_MOVE:
	                    this.moveF(i, this.pointIndex[nameIndex]);
	                    break;
	                case Laya.Event.MOUSE_OUT:
	                    this.upF(i, null);
	                    break;
	                case Laya.Event.RIGHT_CLICK:
	                    if (this.FunRightClick)
	                        this.FunRightClick(i, this.pointIndex[nameIndex]);
	                    break;
	            }
	        }
	        if (this.stopEventMop != null) {
	            e.stopPropagation();
	            if (this.stopEventMop == "本次")
	                this.stopEventMop = null;
	        }
	    }
	    downF(index, p) {
	        this.worldXY.x = p.x;
	        this.worldXY.y = p.y;
	        this.nowLocalXY = this.Target.globalToLocal(p, true);
	        if (this.pausing == true)
	            return;
	        if (this.isDown == true)
	            return;
	        this.isDown = true;
	        if (this.Dic_Click != null) {
	            this.Dic_Click["down"] = true;
	            this.Dic_Click["起点x"] = p.x;
	            this.Dic_Click["起点y"] = p.y;
	            if (this.Dic_Click["fun_down"] != null) {
	                Tool_Function.onRunFunction(this.Dic_Click["fun_down"], p);
	            }
	        }
	        if (this.Dic_Slide != null) {
	            this.Dic_Slide["起点x"] = p.x;
	            this.Dic_Slide["起点y"] = p.y;
	            this.Dic_Slide["开始移动"] = false;
	            this.Dic_Slide["x速度"] = 0;
	            this.Dic_Slide["y速度"] = 0;
	            this.Dic_Slide["计时"] = 0;
	            if (this.indexEnter == -1)
	                this.indexEnter = ConfigTidus.GFun_MM_addEnter(this.enterFrameF.bind(this));
	        }
	        if (this.Dic_LongTouch) {
	            this.Dic_LongTouch["start"] = ConfigTidus.GVal["MMMain"].countFrameTotals;
	            if (this.indexLongTouch == -1)
	                this.indexLongTouch = ConfigTidus.GFun_MM_addEnter(this.enterFrameF2.bind(this));
	        }
	    }
	    upF(index, p) {
	        if (this.pausing == true)
	            return;
	        if (this.isDown == false)
	            return;
	        if (this.indexLongTouch > 0)
	            this.clearLongTouch();
	        if (this.Dic_Click != null) {
	            if (p != null && (this.Dic_Slide == null || this.Dic_Slide["开始移动"] == false) && this.Dic_Click["down"] == true && this.Dic_Click["开始移动"] != true) {
	                if (this.Dic_Click && this.Dic_Click["fun_click"] != null) {
	                    Tool_Function.onRunFunction(this.Dic_Click["fun_click"], p);
	                }
	            }
	            if (this.Dic_Click) {
	                this.Dic_Click["down"] = false;
	                this.Dic_Click["开始移动"] = false;
	                if (this.Dic_Click["fun_up"] != null) {
	                    Tool_Function.onRunFunction(this.Dic_Click["fun_up"], p);
	                }
	            }
	        }
	        if (this.Dic_Slide != null && this.Dic_Slide["开始移动"] == true && p != null) {
	            var dic = {};
	            dic["类型"] = "滑动";
	            if (Math.abs(this.Dic_Slide["x速度"]) >= this.Dic_Slide["满足滑动速度"])
	                dic["x"] = this.Dic_Slide["x速度"];
	            else
	                dic["x"] = 0;
	            if (Math.abs(this.Dic_Slide["y速度"]) >= this.Dic_Slide["满足滑动速度"])
	                dic["y"] = this.Dic_Slide["y速度"];
	            else
	                dic["y"] = 0;
	            dic["gx"] = this.worldXY.x;
	            dic["gy"] = this.worldXY.y;
	            Tool_Function.onRunFunction(this.Dic_Slide["fun"], dic);
	            if (this.indexEnter > 0) {
	                ConfigTidus.GFun_MM_removeEnter(this.indexEnter);
	                this.indexEnter = -1;
	            }
	        }
	        else if (this.Dic_Slide != null) {
	            dic = {};
	            dic["类型"] = "滑动";
	            dic["x"] = 0;
	            dic["y"] = 0;
	            Tool_Function.onRunFunction(this.Dic_Slide["fun"], dic);
	            if (this.indexEnter > 0) {
	                ConfigTidus.GFun_MM_removeEnter(this.indexEnter);
	                this.indexEnter = -1;
	            }
	        }
	        this.isDown = false;
	    }
	    moveF(index, p) {
	        if (this.pausing == true || p == null)
	            return;
	        if (this.isDown == false)
	            return;
	        var gx = p.x;
	        var gy = p.y;
	        this.worldXY.x = gx;
	        this.worldXY.y = gy;
	        this.nowLocalXY.x = p.x;
	        this.nowLocalXY.y = p.y;
	        if (this.Dic_Slide == null) {
	            if (this.Dic_Click && this.Dic_Click["开始移动"] != true) {
	                var L = (gx - this.Dic_Click["起点x"]) * (gx - this.Dic_Click["起点x"]) + (gy - this.Dic_Click["起点y"]) * (gy - this.Dic_Click["起点y"]);
	                L = Math.sqrt(L);
	                if (L > this.LStartMove)
	                    this.Dic_Click["开始移动"] = true;
	            }
	            return;
	        }
	        this.Dic_Slide["计时"] = 3;
	        if (this.dicSlide == null)
	            this.dicSlide = {};
	        if (this.Dic_Slide["开始移动"] == false) {
	            L = (gx - this.Dic_Slide["起点x"]) * (gx - this.Dic_Slide["起点x"]) + (gy - this.Dic_Slide["起点y"]) * (gy - this.Dic_Slide["起点y"]);
	            L = Math.sqrt(L);
	            if (L > this.Dic_Slide["开始移动距离"]) {
	                this.Dic_Slide["开始移动"] = true;
	                this.Dic_Slide["上次x"] = gx;
	                this.Dic_Slide["上次y"] = gy;
	                if (this.indexLongTouch > 0) {
	                    this.clearLongTouch();
	                }
	            }
	        }
	        if (this.Dic_Slide["开始移动"] == true) {
	            if (this.Dic_Click != null && this.Dic_Click["down"] == true) {
	                L = (gx - this.Dic_Slide["起点x"]) * (gx - this.Dic_Slide["起点x"]) + (gy - this.Dic_Slide["起点y"]) * (gy - this.Dic_Slide["起点y"]);
	                L = Math.sqrt(L);
	                if (L > this.Dic_Click["关闭点击距离"]) {
	                    this.Dic_Click["down"] = false;
	                }
	            }
	            this.dicSlide["类型"] = "移动";
	            if (gx != 0)
	                this.dicSlide["x"] = gx - this.Dic_Slide["上次x"];
	            else
	                this.dicSlide["x"] = 0;
	            if (gy != 0)
	                this.dicSlide["y"] = gy - this.Dic_Slide["上次y"];
	            else
	                this.dicSlide["y"] = 0;
	            if (this.dicSlide["x"] == 0)
	                this.Dic_Slide["x速度"] = 0;
	            else if ((this.Dic_Slide["x速度"] > 0 && this.dicSlide["x"] > 0) || (this.Dic_Slide["x速度"] < 0 && this.dicSlide["x"] < 0))
	                this.Dic_Slide["x速度"] = (this.Dic_Slide["x速度"] + this.dicSlide["x"]) / 2;
	            else
	                this.Dic_Slide["x速度"] = this.dicSlide["x"] / 2;
	            if (this.dicSlide["y"] == 0)
	                this.Dic_Slide["y速度"] = 0;
	            else if ((this.Dic_Slide["y速度"] > 0 && this.dicSlide["y"] > 0) || (this.Dic_Slide["y速度"] < 0 && this.dicSlide["y"] < 0))
	                this.Dic_Slide["y速度"] = (this.Dic_Slide["y速度"] + this.dicSlide["y"]) / 2;
	            else
	                this.Dic_Slide["y速度"] = this.dicSlide["y"] / 2;
	            this.Dic_Slide["上次x"] = gx;
	            this.Dic_Slide["上次y"] = gy;
	            this.dicSlide["gx"] = gx;
	            this.dicSlide["gy"] = gy;
	            Tool_Function.onRunFunction(this.Dic_Slide["fun"], this.dicSlide);
	        }
	    }
	    addLongTouchnHandler(val) {
	        this.Dic_LongTouch = { "fun": val, "need": ConfigTidus.FrameRate / 2 };
	    }
	    addDownHandler(val) {
	        if (this.Dic_Click == null) {
	            this.Dic_Click = { "关闭点击距离": 20, "down": false };
	        }
	        this.Dic_Click["fun_down"] = val;
	    }
	    addClickHandler(val) {
	        if (this.Dic_Click == null) {
	            this.Dic_Click = { "关闭点击距离": 20, "down": false };
	        }
	        this.Dic_Click["fun_click"] = val;
	    }
	    addUpHandler(val) {
	        if (this.Dic_Click == null) {
	            this.Dic_Click = { "关闭点击距离": 20, "down": false };
	        }
	        this.Dic_Click["fun_up"] = val;
	    }
	    addRollHandler(val) {
	        if (this.Dic_roll == null) {
	            this.Dic_roll = {};
	            Laya.stage.on(Laya.Event.MOUSE_WHEEL, this, this.onMouseWheel);
	        }
	        this.Dic_roll["fun"] = val;
	    }
	    addRightClickHandler(val) {
	        this.FunRightClick = val;
	        this.Target.on(Laya.Event.RIGHT_CLICK, this, this.mouseHandler);
	    }
	    addSlideHandler(val) {
	        this.Dic_Slide = {};
	        this.Dic_Slide["初始x"] = this.Target.x;
	        this.Dic_Slide["初始y"] = this.Target.y;
	        this.Dic_Slide["上次x"] = this.Target.x;
	        this.Dic_Slide["上次y"] = this.Target.y;
	        this.Dic_Slide["开始移动"] = false;
	        this.Dic_Slide["开始移动距离"] = this.LStartMove;
	        this.Dic_Slide["满足滑动速度"] = 1;
	        this.Dic_Slide["fun"] = val;
	    }
	    onMouseWheel(e) {
	        if (this.Dic_roll && this.Dic_roll["fun"]) {
	            var delta = e.delta;
	            if (delta > 0) {
	            }
	            Tool_Function.onRunFunction(this.Dic_roll["fun"], delta);
	        }
	        else {
	            Laya.stage.off(Laya.Event.MOUSE_WHEEL, this, this.onMouseWheel);
	            this.Dic_roll = null;
	        }
	    }
	    enterFrameF() {
	        if (this.pausing == true)
	            return;
	        if (this.isDown == false)
	            return;
	        this.Dic_Slide["计时"] -= 1;
	        if (this.Dic_Slide["计时"] <= 0) {
	            this.Dic_Slide["计时"] = 0;
	            this.Dic_Slide["x速度"] = 0;
	            this.Dic_Slide["y速度"] = 0;
	        }
	    }
	    enterFrameF2() {
	        if (this.pausing == true)
	            return;
	        if (this.isDown == false) {
	            return;
	        }
	        if (ConfigTidus.GVal["MMMain"].countFrameTotals - this.Dic_LongTouch["start"] > this.Dic_LongTouch["need"]) {
	            Tool_Function.onRunFunction(this.Dic_LongTouch["fun"]);
	            this.clearLongTouch();
	            return;
	        }
	    }
	    clearLongTouch() {
	        if (this.indexLongTouch > 0) {
	            ConfigTidus.GFun_MM_removeEnter(this.indexLongTouch);
	            this.indexLongTouch = -1;
	        }
	        this.Dic_LongTouch = null;
	    }
	}

	class TmpMovieClip extends Sprite {
	    constructor(mc, f = null, type = "nor") {
	        super();
	        this.Arr = [];
	        this.End = false;
	        this.Type = type;
	        if (this.Type == "循环点击") {
	            if (mc instanceof SwfMovieClip) {
	                mc.loop = true;
	                mc.play();
	            }
	        }
	        else if (mc instanceof SwfMovieClip) {
	            mc.completeFunction = this.overF.bind(this);
	            mc.play();
	        }
	        else if (mc instanceof MyZMovieClip) {
	            mc.completeFunction = this.overF.bind(this);
	            mc.play();
	        }
	        else if (mc instanceof Sprite) {
	        }
	        this.Fun = f;
	        if (mc == null) {
	            this.overF();
	        }
	        else {
	            this.addMC(mc);
	        }
	    }
	    addMC(mc) {
	        if (mc == null)
	            return;
	        Tool_SpriteUtils.onAddchild_ChangeParent(this, mc);
	        this.Arr.push(mc);
	    }
	    overF(val = null) {
	        this.stopF();
	        if (this.Type == "nor") {
	            this.onRunFunction();
	            this.destroyF();
	        }
	        else if (this.Type == "点击") {
	            this.addClickF();
	        }
	        else if (this.Type == "屏幕点击") {
	            this.addScreenClickF();
	        }
	        else if (this.Type == "手动") {
	            this.onRunFunction();
	        }
	    }
	    stageF(e) {
	        this.addClickF();
	    }
	    addClickF() {
	        if (this.mme == null) {
	            this.mme = new MyMouseManager(this);
	            this.mme.addClickHandler(this.clickF.bind(this));
	        }
	    }
	    clickF(e) {
	        this.onRunFunction();
	        this.destroyF();
	    }
	    addScreenClickF() {
	        function clickF(e) {
	            this.onRunFunction();
	            this.destroyF();
	        }
	    }
	    stopF() {
	        for (var i = 0; i < this.Arr.length; i++) {
	            var mc = this.Arr[i];
	            if (mc)
	                mc.stop();
	        }
	    }
	    playF() {
	        for (var i = 0; i < this.Arr.length; i++) {
	            var mc = this.Arr[i];
	            if (mc)
	                mc.play();
	        }
	    }
	    addStageRemoveListener() {
	    }
	    onRemoveF(e) {
	        this.destroyF();
	    }
	    onRunFunction() {
	        if (this.Fun != null) {
	            var f = this.Fun;
	            this.Fun = null;
	            Tool_Function.onRunFunction(f);
	        }
	    }
	    onWindowFit(needclearBG = true, type = 4) {
	        if (this.Arr[0] instanceof SwfMovieClip) {
	            var mc = this.Arr[0];
	            if (needclearBG) {
	                var bimg = mc.getChildAt(0);
	                mc.removeChildFromAllFrame(bimg.classLink);
	                this.addChildAt(bimg, 0);
	            }
	            ConfigTidus.GFun_ToolView_winfit(this, type);
	        }
	    }
	    destroyF() {
	        this.Fun = null;
	        this.End = true;
	        this.mme = ConfigTidus.GFun_ToolObj_destroyOne(this.mme);
	        this.Arr = ConfigTidus.GFun_ToolObj_destroyOne(this.Arr);
	        super.destroyF();
	    }
	}

	class Tool_ViewFit {
	    static get isFat() {
	        var isFat;
	        if (ConfigTidus.stageRealW / ConfigTidus.stageRealH >= ConfigTidus.stageW / ConfigTidus.stageH) {
	            isFat = true;
	        }
	        else {
	            isFat = false;
	        }
	        return isFat;
	    }
	    static onViewFitF(tar) {
	        if (tar == null) {
	            return;
	        }
	        var w0 = 50;
	        var h0 = 50;
	        var infoFit;
	        var i;
	        var child;
	        var rec;
	        var rec2;
	        var dicNo = {};
	        var typex;
	        var typey;
	        var info;
	        var dicStatic = {};
	        var dicSuc = {};
	        var s0;
	        var meta;
	        if (tar instanceof Sprite)
	            meta = tar.metaData;
	        if (meta == null || meta["适配"] == null) {
	            infoFit = {};
	        }
	        else
	            infoFit = meta["适配"];
	        var arrWaiteRelationship = [];
	        for (var key in infoFit) {
	            if (infoFit[key]["类型"] == "关联") {
	                arrWaiteRelationship.push(key);
	                child = tar.getChildByName(key);
	                this.onSaveAssociated(infoFit[key], tar, child);
	            }
	        }
	        for (key in infoFit) {
	            if (key.indexOf("合并") != 0) {
	                continue;
	            }
	            var arrChilds = infoFit[key]["包含"];
	            var recCombo = null;
	            for (i = 0; i < arrChilds.length; i++) {
	                dicNo[arrChilds[i]] = true;
	                child = tar.getChildByName(arrChilds[i]);
	                rec = Tool_SpriteUtils.getBounds(child, tar);
	                if (recCombo == null) {
	                    recCombo = rec;
	                }
	                else {
	                    if (rec.x < recCombo.x) {
	                        recCombo.x = rec.x;
	                    }
	                    if (rec.y < recCombo.y) {
	                        recCombo.y = rec.y;
	                    }
	                    if (rec.right > recCombo.right) {
	                        recCombo.width += rec.right - recCombo.right;
	                    }
	                    if (rec.bottom > recCombo.bottom) {
	                        recCombo.height += rec.bottom - recCombo.bottom;
	                    }
	                }
	            }
	            info["sx"] = info["sy"] = null;
	            info = this.onFitOne(recCombo, infoFit[key], null);
	            let sx = info["sx"] != null ? info["sx"] : info["s"];
	            let sy = info["sy"] != null ? info["sy"] : info["s"];
	            for (i = 0; i < arrChilds.length; i++) {
	                child = tar.getChildByName(arrChilds[i]);
	                rec2 = Tool_SpriteUtils.getBounds(child, tar);
	                child.x = info.x + (rec2.x - recCombo.x) * sx;
	                child.y = info.y + (rec2.y - recCombo.y) * sy;
	                child.scaleX *= sx;
	                child.scaleY *= sy;
	                dicStatic[arrChilds[i]] = info.s;
	                dicSuc[arrChilds[i]] = true;
	            }
	        }
	        for (i = 0; i < tar.numChildren; i++) {
	            child = tar.getChildAt(i);
	            if (child.name && (dicNo[child.name] == true || arrWaiteRelationship.indexOf(child.name) != -1)) {
	                continue;
	            }
	            rec = Tool_SpriteUtils.getBounds(child, tar);
	            if ((rec.width > 0) == false)
	                rec.width = 10;
	            if ((rec.height > 0) == false)
	                rec.height = 10;
	            info = this.onFitOne(rec, infoFit[child.name], child.classLink);
	            if (info["sx"] != null) {
	                child.scaleX *= info.sx;
	                child.scaleY *= info.sy;
	                child.x = info.x + (child.x - rec.x) * info.sx;
	                child.y = info.y + (child.y - rec.y) * info.sy;
	            }
	            else {
	                child.scaleX *= info.s;
	                child.scaleY *= info.s;
	                child.x = info.x + (child.x - rec.x) * info.s;
	                child.y = info.y + (child.y - rec.y) * info.s;
	            }
	            dicStatic[child.name] = info.s;
	            dicSuc[child.name] = true;
	        }
	        var L = arrWaiteRelationship.length;
	        while (true) {
	            for (i = 0; i < arrWaiteRelationship.length; i++) {
	                key = arrWaiteRelationship[i];
	                if (dicSuc[infoFit[key]["tar"]] != true) {
	                    continue;
	                }
	                dicSuc[key] = true;
	                child = tar.getChildByName(key);
	                if (child == null)
	                    continue;
	                var childTar = tar.getChildByName(infoFit[key]["tar"]);
	                var s = 1;
	                if (infoFit[key]["ss"] != "无" && dicStatic[infoFit[key]["tar"]] != null) {
	                    s = dicStatic[infoFit[key]["tar"]];
	                }
	                child.scaleX *= s;
	                child.scaleY *= s;
	                if (infoFit[key]["关联方式"] == null) {
	                    child.x = childTar.x + infoFit[key]["x"] * s;
	                    child.y = childTar.y + infoFit[key]["y"] * s;
	                }
	                else {
	                    rec = Tool_SpriteUtils.getBounds(child, tar);
	                    rec2 = Tool_SpriteUtils.getBounds(childTar, tar);
	                    if (infoFit[key]["关联方式"].charAt(0) == "左") {
	                        child.x = childTar.x + infoFit[key]["x"] * s;
	                    }
	                    else if (infoFit[key]["关联方式"].charAt(0) == "右") {
	                        child.x += rec2.right + infoFit[key]["x"] * s - rec.right;
	                    }
	                    else if (infoFit[key]["关联方式"].charAt(0) == "中") {
	                        child.x = rec2.x + rec2.width / 2 - rec.width / 2 + infoFit[key]["x"] * s;
	                    }
	                    if (infoFit[key]["关联方式"].charAt(1) == "上") {
	                        child.y = childTar.y + infoFit[key]["y"] * s;
	                    }
	                    else if (infoFit[key]["关联方式"].charAt(1) == "下") {
	                        child.y += rec2.bottom + infoFit[key]["y"] * s - rec.bottom;
	                    }
	                    else if (infoFit[key]["关联方式"].charAt(1) == "中") {
	                        child.y = rec2.y + rec2.height / 2 - rec.height / 2 + infoFit[key]["y"] * s;
	                    }
	                }
	                arrWaiteRelationship.splice(i--, 1);
	            }
	            if (L == arrWaiteRelationship.length || arrWaiteRelationship.length == 0) {
	                break;
	            }
	            L = arrWaiteRelationship.length;
	        }
	    }
	    static onSaveAssociated(infoSelf, tar, child) {
	        var childTar = tar.getChildByName(infoSelf["tar"]);
	        if (childTar == null || child == null) {
	            return;
	        }
	        if (infoSelf["关联方式"] == null) {
	            infoSelf["x"] = child.x - childTar.x;
	            infoSelf["y"] = child.y - childTar.y;
	        }
	        else {
	            var rec = Tool_SpriteUtils.getBounds(child, tar);
	            var rec2 = Tool_SpriteUtils.getBounds(childTar, tar);
	            if (infoSelf["关联方式"].charAt(0) == "左") {
	                infoSelf["x"] = child.x - childTar.x;
	            }
	            else if (infoSelf["关联方式"].charAt(0) == "右") {
	                infoSelf["x"] = rec.right - rec2.right;
	            }
	            else if (infoSelf["关联方式"].charAt(0) == "中") {
	                infoSelf["x"] = rec.x + rec.width / 2 - (rec2.x + rec2.width / 2);
	            }
	            if (infoSelf["关联方式"].charAt(1) == "上") {
	                infoSelf["y"] = child.y - childTar.y;
	            }
	            else if (infoSelf["关联方式"].charAt(1) == "下") {
	                infoSelf["y"] = rec.bottom - rec2.bottom;
	            }
	            else if (infoSelf["关联方式"].charAt(1) == "中") {
	                infoSelf["y"] = rec.y + rec.height / 2 - (rec2.y + rec2.height / 2);
	            }
	        }
	    }
	    static onFitOne(recCombo, infoSelf, _name) {
	        var w0 = 20;
	        var h0 = 20;
	        var typex;
	        var typey;
	        var info = {};
	        var realW = ConfigTidus.stageRealW;
	        var realH = ConfigTidus.stageRealH;
	        if (ConfigTidus.iSWakongping) {
	            if (recCombo.width >= ConfigTidus.stageW && recCombo.height >= ConfigTidus.stageH) {
	            }
	            else {
	                if (ConfigTidus.stageW > ConfigTidus.stageH)
	                    realW -= 100;
	                else
	                    realH -= 100;
	            }
	        }
	        if (infoSelf && infoSelf["强制rec"] != null) {
	            if (infoSelf["强制rec"] instanceof Array) {
	                if (infoSelf["强制rec"][0] != null) {
	                    recCombo.x = infoSelf["强制rec"][0];
	                }
	                if (infoSelf["强制rec"][1] != null) {
	                    recCombo.y = infoSelf["强制rec"][1];
	                }
	                if (infoSelf["强制rec"][2] != null) {
	                    recCombo.width = infoSelf["强制rec"][2];
	                }
	                if (infoSelf["强制rec"][3] != null) {
	                    recCombo.height = infoSelf["强制rec"][3];
	                }
	            }
	            else {
	                recCombo = infoSelf["强制rec"];
	            }
	        }
	        if (recCombo.x < w0) {
	            info["左"] = true;
	        }
	        if (recCombo.x + recCombo.width > ConfigTidus.stageW - w0) {
	            info["右"] = true;
	        }
	        if (recCombo.y < h0) {
	            info["上"] = true;
	        }
	        if (recCombo.y + recCombo.height > ConfigTidus.stageH - h0) {
	            info["下"] = true;
	        }
	        if (info["左"] == true && info["右"] == true) {
	            typex = "中";
	        }
	        else if (info["左"] == true) {
	            typex = "左";
	        }
	        else if (info["右"] == true) {
	            typex = "右";
	        }
	        else {
	            typex = null;
	        }
	        if (info["上"] == true && info["下"] == true) {
	            typey = "中";
	        }
	        else if (info["上"] == true) {
	            typey = "上";
	        }
	        else if (info["下"] == true) {
	            typey = "下";
	        }
	        else {
	            typey = null;
	        }
	        if (infoSelf && infoSelf["x"] != null) {
	            typex = infoSelf["x"];
	        }
	        if (infoSelf && infoSelf["y"] != null) {
	            typey = infoSelf["y"];
	        }
	        if (infoSelf && infoSelf["类型"] != null) {
	            typex = infoSelf["类型"].charAt(0);
	            typey = infoSelf["类型"].charAt(1);
	        }
	        var sCombine = 1;
	        var rec = {};
	        if (infoSelf && infoSelf["缩放"] && infoSelf["缩放"] == "xy") {
	            let sx = (realW / ConfigTidus.stageW);
	            let sy = (realH / ConfigTidus.stageH);
	            if (recCombo.width >= ConfigTidus.stageW && recCombo.height >= ConfigTidus.stageH) {
	                sCombine = sx >= sy ? sx : sy;
	            }
	            else {
	                sCombine = sx <= sy ? sx : sy;
	            }
	        }
	        else if (infoSelf && infoSelf["缩放"] && infoSelf["缩放"] == "x") {
	            sCombine = (realW / ConfigTidus.stageW);
	        }
	        else if (infoSelf && infoSelf["缩放"] && infoSelf["缩放"] == "y") {
	            sCombine = (realH / ConfigTidus.stageH);
	        }
	        else {
	            let sx = (realW / ConfigTidus.stageW);
	            let sy = (realH / ConfigTidus.stageH);
	            if (recCombo.width >= ConfigTidus.stageW - 1 && recCombo.height >= ConfigTidus.stageH - 1) {
	                sCombine = sx >= sy ? sx : sy;
	            }
	            else {
	                sCombine = sx <= sy ? sx : sy;
	            }
	        }
	        rec["s"] = sCombine;
	        if (typex == "左") {
	            rec.x = (recCombo.x / ConfigTidus.stageW) * realW;
	        }
	        else if (typex == "右") {
	            rec.x = realW - ((ConfigTidus.stageW - recCombo.right) / ConfigTidus.stageW) * realW - recCombo.width * sCombine;
	        }
	        else if (typex == "中") {
	            rec.x = realW / 2 + ((recCombo.x + recCombo.width / 2 - ConfigTidus.stageW / 2) / ConfigTidus.stageW) * realW - recCombo.width * sCombine / 2;
	        }
	        else {
	            rec.x = ((recCombo.x + recCombo.width / 2) / ConfigTidus.stageW) * realW - recCombo.width * sCombine / 2;
	        }
	        if (typey == "上") {
	            rec.y = (recCombo.y / ConfigTidus.stageH) * realH;
	        }
	        else if (typey == "下") {
	            rec.y = realH - ((ConfigTidus.stageH - recCombo.bottom) / ConfigTidus.stageH) * realH - recCombo.height * sCombine;
	        }
	        else if (typey == "中") {
	            rec.y = realH / 2 + ((recCombo.y + recCombo.height / 2 - ConfigTidus.stageH / 2) / ConfigTidus.stageH) * realH - recCombo.height * sCombine / 2;
	        }
	        else {
	            rec.y = ((recCombo.y + recCombo.height / 2) / ConfigTidus.stageH) * realH - recCombo.height * sCombine / 2;
	        }
	        return rec;
	    }
	    static onWindowFitScreen(sprWin, scaleType = 0) {
	        if (scaleType == 0) {
	            var rec = Tool_SpriteUtils.getBounds(sprWin, sprWin.parent);
	            var sw = rec.width / ConfigTidus.stageW;
	            var sh = rec.height / ConfigTidus.stageH;
	            var wNeed = ConfigTidus.stageRealW * sw;
	            var hNeed = ConfigTidus.stageRealH * sh;
	            sprWin.scaleX *= wNeed / rec.width;
	            sprWin.scaleY *= hNeed / rec.height;
	        }
	        else if (scaleType == 1) {
	            sw = ConfigTidus.stageRealW / ConfigTidus.stageW;
	            sprWin.scaleX *= sw;
	            sprWin.scaleY *= sw;
	            sprWin.y = (ConfigTidus.stageH * sw - ConfigTidus.stageRealH) / 2;
	        }
	        else if (scaleType == 1.1) {
	            sw = ConfigTidus.stageRealH / ConfigTidus.stageH;
	            sprWin.scaleX *= sw;
	            sprWin.scaleY *= sw;
	            sprWin.x = (ConfigTidus.stageRealW - ConfigTidus.stageW * sw) / 2;
	        }
	        else if (scaleType == 5) {
	            sw = ConfigTidus.stageRealW / ConfigTidus.stageW;
	            sprWin.scaleX *= sw;
	            sprWin.scaleY *= sw;
	        }
	        else if (scaleType == 5.1) {
	            let oldBottom = ConfigTidus.stageH - sprWin.y - sprWin.height;
	            sw = ConfigTidus.stageRealW / ConfigTidus.stageW;
	            sprWin.scaleX *= sw;
	            sprWin.scaleY *= sw;
	            sprWin.y = ConfigTidus.stageRealH - sprWin.height - oldBottom * sw;
	        }
	        else if (scaleType == 6) {
	            sw = ConfigTidus.stageRealH / ConfigTidus.stageH;
	            sprWin.scaleX *= sw;
	            sprWin.scaleY *= sw;
	        }
	        else if (scaleType == 2) {
	            sprWin.x = ConfigTidus.stageRealW / 2;
	            sprWin.y = ConfigTidus.stageRealH / 2;
	        }
	        else if (scaleType == 3) {
	            sprWin.x = ConfigTidus.stageRealW / 2;
	            sprWin.y = ConfigTidus.stageRealH / 2;
	            rec = Tool_SpriteUtils.getBounds(sprWin, sprWin.parent);
	            sw = rec.width / ConfigTidus.stageW;
	            sh = rec.height / ConfigTidus.stageH;
	            wNeed = ConfigTidus.stageRealW * sw;
	            hNeed = ConfigTidus.stageRealH * sh;
	            var smin = wNeed / rec.width;
	            if (hNeed / rec.height < smin) {
	                smin = hNeed / rec.height;
	            }
	            sprWin.scaleX *= smin;
	            sprWin.scaleY *= smin;
	        }
	        else if (scaleType >= 4 && scaleType < 5) {
	            wNeed = ConfigTidus.stageW - ConfigTidus.stageRealW;
	            hNeed = ConfigTidus.stageH - ConfigTidus.stageRealH;
	            if (wNeed == hNeed) {
	                return;
	            }
	            if (wNeed > hNeed) {
	                sprWin.scaleX = (ConfigTidus.stageW - wNeed) / ConfigTidus.stageW;
	                sprWin.scaleY = sprWin.scaleX;
	                sprWin.y = (ConfigTidus.stageH * (1 - sprWin.scaleY)) / 2;
	            }
	            else {
	                sprWin.scaleY = (ConfigTidus.stageH - hNeed) / ConfigTidus.stageH;
	                sprWin.scaleX = sprWin.scaleY;
	                sprWin.x = (ConfigTidus.stageW * (1 - sprWin.scaleX)) / 2;
	            }
	            if (scaleType != 4.1) {
	                let b = sprWin.getChildAt(0);
	                if (wNeed > hNeed) {
	                    b.y = -sprWin.y / sprWin.scaleY;
	                    b.scaleY /= sprWin.scaleX;
	                }
	                else {
	                    b.x = -sprWin.x / sprWin.scaleX;
	                    b.scaleX /= sprWin.scaleY;
	                    if (scaleType == 4.2) {
	                        b.y -= (b.height - ConfigTidus.stageRealH) / (2 * sprWin.scaleY);
	                    }
	                }
	            }
	        }
	    }
	    static onMcFit(tar, needclearBG = true, type = 4) {
	        if (tar instanceof TmpMovieClip) {
	            var mc = tar.Arr[0];
	            if (needclearBG) {
	                let bimg = mc.getChildAt(0);
	                mc.removeChildFromAllFrame(bimg.classLink);
	                tar.addChildAt(bimg, 0);
	                if (type == 4)
	                    type = 4.1;
	            }
	            Tool_ViewFit.onWindowFitScreen(tar, type);
	        }
	        else if (tar instanceof SwfMovieClip) {
	            Tool_ViewFit.onWindowFitScreen(tar, type);
	        }
	        else if (tar instanceof MyMC) {
	            this.onMcFit(tar.MC, needclearBG, 4);
	        }
	    }
	}

	class SwfMovieClip$1 extends Sprite {
	    constructor(frames, labels, displayObjects, ownerSwf, _events) {
	        super();
	        this._isPlay = false;
	        this._loop = true;
	        this._complete = null;
	        this._hasCompleteListener = false;
	        this.numMC = 0;
	        this.needUpdateEveryFrame = false;
	        this.indexEnter = -1;
	        this.pastTime = 0;
	        this.touchable = false;
	        this._frames = frames;
	        this._labels = labels;
	        this._frameEvents = _events;
	        this._displayObjects = displayObjects;
	        this._startFrame = 0;
	        this._endFrame = this._frames.length - 1;
	        this._ownerSwf = ownerSwf;
	        this.fps = SwfMovieClip$1.FPS_default;
	        this.currentFrame = 0;
	        var k;
	        var arr;
	        var l;
	        for (k in this._displayObjects) {
	            if (k.indexOf("mc") == 0) {
	                arr = this._displayObjects[k];
	                if (arr == null)
	                    continue;
	                l = arr.length;
	                for (var i = 0; i < l; i++) {
	                    this.numMC++;
	                }
	            }
	        }
	        this.play();
	    }
	    onCashSameFrameInfo(obj) {
	        for (var f in obj) {
	            this._frames[f] = null;
	        }
	    }
	    _frameLoop() {
	        if (!this._isPlay) {
	            if (this.indexEnter > 0) {
	                ConfigTidus.GFun_MM_removeEnter(this.indexEnter);
	                this.indexEnter = -1;
	            }
	            return;
	        }
	        this.pastTime += SwfMovieClip$1.TimeFrame;
	        var f2 = parseInt((this.pastTime / this.NeedTime) + "");
	        while (f2 > 0) {
	            if (this._ownerSwf == null) {
	                return;
	            }
	            this.nextFrame();
	            f2--;
	            this.pastTime -= this.NeedTime;
	        }
	    }
	    nextFrame() {
	        if (this._ownerSwf == null)
	            return;
	        if (this._currentFrame >= this._endFrame) {
	            var isReturn = false;
	            if (!this._loop || this._startFrame == this._endFrame) {
	                if (this._ownerSwf)
	                    this.stop(false);
	                isReturn = true;
	            }
	            if (this._complete) {
	                let f = this._complete;
	                this._complete = null;
	                f(this);
	            }
	            if (isReturn)
	                return;
	            this._currentFrame = this._startFrame;
	        }
	        else {
	            this._currentFrame++;
	        }
	        this.currentFrame = this._currentFrame;
	        if (this._currentFrame >= this._endFrame) {
	            if (!this._loop || this._startFrame == this._endFrame) {
	                if (this._ownerSwf)
	                    this.stop(false);
	                if (this._complete) {
	                    let f = this._complete;
	                    this._complete = null;
	                    f(this);
	                }
	            }
	        }
	    }
	    removeChildFromAllFrame(_classLink) {
	        var length = this._frames.length;
	        for (var i = 0; i < length; i++) {
	            var frameInfos2 = this._frames[i];
	            if (frameInfos2 == null)
	                continue;
	            for (var j = 0; j < frameInfos2.length; j++) {
	                this._mdata = frameInfos2[j];
	                if (this._mdata instanceof Array == false) {
	                    if (this._mdata["url"] == _classLink || this._mdata["name"] == _classLink) {
	                        frameInfos2.splice(j--, 1);
	                    }
	                }
	            }
	        }
	    }
	    getChildByName_onFrame(_name, f) {
	        if (this._frames[f] == null)
	            return null;
	        var frameInfos2 = this._frames[f];
	        for (var j = 0; j < frameInfos2.length; j++) {
	            this._mdata = frameInfos2[j];
	            if (this._mdata instanceof Array == false) {
	                if (this._mdata["name"] == _name) {
	                    var display;
	                    if (this._displayObjects)
	                        display = this._displayObjects[this._mdata["url"]][this._mdata["index"]];
	                    return display;
	                }
	            }
	        }
	        return null;
	    }
	    set currentFrame(frame) {
	        this._currentFrame = frame;
	        this.__frameInfos = this._frames[this._currentFrame];
	        if (this.__frameInfos == null || this._ownerSwf == null)
	            return;
	        this.onSetCurrentFrame();
	    }
	    onSetCurrentFrame() {
	        if (this.lastFrame == this._currentFrame)
	            return;
	        this.lastFrame = this._currentFrame;
	        if (this.frameFunction)
	            this.frameFunction();
	        if (this._displayObjects)
	            this.clearChild();
	        else
	            this.clearChild(true);
	        var data;
	        var display;
	        var useIndex;
	        var length = this.__frameInfos.length;
	        for (var i = 0; i < length; i++) {
	            data = this.__frameInfos[i];
	            useIndex = data["index"];
	            if (data["type"] == "particle") {
	                var pd = this._displayObjects[data["url"]][useIndex];
	                pd.x = data.x;
	                pd.y = data.y;
	                pd.alpha = data["alpha"];
	                pd.name = data["name"];
	                pd.emitter.emit();
	                this.addChild(pd);
	                continue;
	            }
	            if (this._displayObjects) {
	                display = this._displayObjects[data["url"]][useIndex];
	            }
	            if (display == null) {
	                if (data["type"] == "text")
	                    display = this._ownerSwf.createTextField(data);
	                else
	                    display = this._ownerSwf.getObject(data["url"]);
	                if (display && this._displayObjects) {
	                    this._displayObjects[data["url"]][useIndex] = display;
	                }
	            }
	            if (display == null || (this.dicRemoved != null && this.dicRemoved[display] == true))
	                continue;
	            this._ownerSwf.onCheckDataForFilterAndColor(display, data);
	            display.alpha = data["alpha"];
	            display.name = data["name"];
	            display.x = data["x"];
	            display.y = data["y"];
	            display.skewX = -data["skewx"];
	            display.skewY = data["skewy"];
	            if (this.touchable == true) {
	                display.touchable = true;
	                if (display.mouseEnabled == false) {
	                    display.mouseEnabled = true;
	                }
	            }
	            switch (data["type"]) {
	                case "s9":
	                    display.width = data["w"];
	                    display.height = data["h"];
	                    ConfigTidus.GVal["SWF"].setBlendMode(display, data["blend"]);
	                    ConfigTidus.GVal["SWF"].changeColor(display, data["color"]);
	                    this.addChild(display);
	                    break;
	                case "text":
	                    var tx = display;
	                    if (tx.isCleared == true)
	                        break;
	                    tx.width = data["w"];
	                    tx.height = data["h"];
	                    tx.format.font = data["font"];
	                    tx.format.color = data["color"];
	                    tx.format.size = data["size"];
	                    tx.format.horizontalAlign = data["align"];
	                    tx.format.italic = data["italic"];
	                    tx.format.bold = data["bold"];
	                    if (tx.text != data["text"] && tx.text != "\r" && tx.text != "") { }
	                    else if (data["text"] && data["text"] != "\r" && data["text"] != "") {
	                        if (tx.changed == false) {
	                            tx.text = data["text"];
	                            tx.text = data["text"];
	                        }
	                    }
	                    ConfigTidus.GVal["SWF"].setBlendMode(display, data["blend"]);
	                    this.addChild(display);
	                    break;
	                case "img":
	                    display.scaleX = data["sx"];
	                    display.scaleY = data["sy"];
	                    ConfigTidus.GVal["SWF"].setBlendMode(display, data["blend"]);
	                    ConfigTidus.GVal["SWF"].changeColor(display, data["color"]);
	                    this.addChild(display);
	                    break;
	                default:
	                    display.scaleX = data["sx"];
	                    display.scaleY = data["sy"];
	                    ConfigTidus.GVal["SWF"].setBlendMode(display, data["blend"]);
	                    ConfigTidus.GVal["SWF"].changeColor(display, data["color"]);
	                    this.addChild(display);
	                    break;
	            }
	        }
	        if (this.eventFunction && this._frameEvents != null && this._frameEvents[this._currentFrame] != null) {
	            this.eventFunction(this._frameEvents[this._currentFrame]);
	        }
	    }
	    get currentFrame() {
	        return this._currentFrame;
	    }
	    play(rePlayChildMovie = true) {
	        if (this._isPlay == true)
	            return;
	        this._isPlay = true;
	        if (this._currentFrame >= this._endFrame) {
	            this._currentFrame = this._startFrame;
	        }
	        if (this.indexEnter == -1)
	            this.indexEnter = ConfigTidus.GFun_MM_addEnter(this._frameLoop.bind(this));
	        if (this.numMC == 0 || this._displayObjects == null)
	            return;
	        var k;
	        var arr;
	        var l;
	        for (k in this._displayObjects) {
	            if (k.indexOf("mc") == 0) {
	                arr = this._displayObjects[k];
	                if (arr == null)
	                    continue;
	                l = arr.length;
	                for (var i = 0; i < l; i++) {
	                    if (arr[i] == null || (arr[i] instanceof SwfMovieClip$1) == false)
	                        continue;
	                    if (rePlayChildMovie)
	                        arr[i].currentFrame = 0;
	                    arr[i].play(rePlayChildMovie);
	                }
	            }
	        }
	        this.needUpdateEveryFrame = rePlayChildMovie;
	    }
	    stop(stopChild = false) {
	        this._isPlay = false;
	        this.pastTime = 0;
	        if (this.indexEnter > 0) {
	            ConfigTidus.GFun_MM_removeEnter(this.indexEnter);
	            this.indexEnter = -1;
	        }
	        if (this.numMC == 0)
	            return;
	        var k;
	        var arr;
	        var l;
	        for (k in this._displayObjects) {
	            if (k.indexOf("mc") == 0) {
	                arr = this._displayObjects[k];
	                if (arr == null)
	                    continue;
	                l = arr.length;
	                for (var i = 0; i < l; i++) {
	                    if (arr[i] == null || (arr[i] instanceof SwfMovieClip$1) == false)
	                        continue;
	                    if (stopChild == true)
	                        arr[i].stop(stopChild);
	                    else
	                        arr[i].play();
	                }
	            }
	        }
	        this.needUpdateEveryFrame = !stopChild;
	    }
	    gotoAndStop(frame, stopChild = false) {
	        this.goTo(frame);
	        if (typeof frame == "string") {
	            var labelData = this.getLabelData(frame);
	            if (labelData == null) {
	                this._endFrame = this._frames.length - 1;
	            }
	            else {
	                this._currentLabel = labelData[0];
	                this._endFrame = labelData[2];
	            }
	        }
	        else {
	        }
	        this.onCheckLastNotNullFrame();
	        this.stop(stopChild);
	    }
	    gotoAndPlay(frame, rePlayChildMovie = false) {
	        this.goTo(frame);
	        if (typeof frame == "string") {
	            var labelData = this.getLabelData(frame);
	            if (labelData == null) {
	                this._endFrame = this._frames.length - 1;
	            }
	            else {
	                this._currentLabel = labelData[0];
	                this._endFrame = labelData[2];
	            }
	        }
	        else {
	        }
	        this.onCheckLastNotNullFrame();
	        this.play(rePlayChildMovie);
	    }
	    onCheckLastNotNullFrame() {
	        var count = 0;
	        while (this.__frameInfos == null) {
	            if (this._currentFrame - count < 0) {
	                this.__frameInfos = this._frames[0];
	                break;
	            }
	            this.__frameInfos = this._frames[this._currentFrame - count];
	            count++;
	        }
	        this.onSetCurrentFrame();
	    }
	    goTo(frame) {
	        if (typeof frame == "string") {
	            var labelData = this.getLabelData(frame);
	            if (labelData == null) {
	                frame = 0;
	                this._currentFrame = this._startFrame = frame;
	            }
	            else {
	                this._currentLabel = labelData[0];
	                this._currentFrame = this._startFrame = labelData[1];
	            }
	        }
	        else {
	            if (frame < 0)
	                frame = 0;
	            this._currentFrame = this._startFrame = frame;
	        }
	        this.currentFrame = this._currentFrame;
	    }
	    getLabelData(label) {
	        var length = this._labels.length;
	        var labelData;
	        for (var i = 0; i < length; i++) {
	            labelData = this._labels[i];
	            if (labelData[0] == label) {
	                return labelData;
	            }
	        }
	        return null;
	    }
	    get isPlay() {
	        return this._isPlay;
	    }
	    get loop() {
	        return this._loop;
	    }
	    set loop(value) {
	        this._loop = value;
	    }
	    set completeFunction(value) {
	        this._complete = value;
	    }
	    get completeFunction() {
	        return this._complete;
	    }
	    get totalFrames() {
	        return this._frames.length;
	    }
	    get currentLabel() {
	        return this._currentLabel;
	    }
	    get labels() {
	        if (this._labelStrings != null)
	            return this._labelStrings;
	        this._labelStrings = [];
	        var length = this._labels.length;
	        for (var i = 0; i < length; i++) {
	            this._labelStrings.push(this._labels[i][0]);
	        }
	        return this._labelStrings;
	    }
	    hasLabel(label) {
	        return !(this.labels.indexOf(label) == -1);
	    }
	    set startFrame(value) {
	        this._startFrame = value < 0 ? 0 : value;
	        this._startFrame = this._startFrame > this._endFrame ? this._endFrame : this._startFrame;
	    }
	    get startFrame() {
	        return this._startFrame;
	    }
	    set endFrame(value) {
	        this._endFrame = value > this._frames.length - 1 ? this._frames.length - 1 : value;
	        this._endFrame = this._endFrame < this._startFrame ? this._startFrame : this._endFrame;
	    }
	    get endFrame() {
	        return this._endFrame;
	    }
	    removeChildF(child, dispose = true) {
	        if (this.dicRemoved == null)
	            this.dicRemoved = {};
	        if (child instanceof String) {
	            child = this.getChildByName(child);
	        }
	        if (child != null) {
	            this.dicRemoved[child] = true;
	            this.removeChild(child);
	        }
	    }
	    destroyF() {
	        this.dicRemoved = null;
	        this.frameFunction = null;
	        this.eventFunction = null;
	        this.completeFunction = null;
	        if (this._displayObjects) {
	            for (var key in this._displayObjects) {
	                let array = this._displayObjects[key];
	                var len = array.length;
	                for (let i = 0; i < len; i++) {
	                    if (array[i]) {
	                        array[i].removeFromParent(true);
	                    }
	                }
	            }
	            this._displayObjects = null;
	        }
	        this._ownerSwf = null;
	        if (this._isPlay) {
	            this._isPlay = false;
	            if (this.indexEnter > 0) {
	                ConfigTidus.GFun_MM_removeEnter(this.indexEnter);
	                this.indexEnter = -1;
	            }
	        }
	        super.destroyF();
	    }
	    get fps() { return this._fps; }
	    set fps(value) {
	        this._fps = value;
	        this.NeedTime = 1000 / this._fps;
	    }
	}
	SwfMovieClip$1.ANGLE_TO_RADIAN = Math.PI / 180;
	SwfMovieClip$1.FPS_default = 30;
	SwfMovieClip$1.TimeFrame = 0;
	SwfMovieClip$1.CacheNumBegin = 9999999;

	var Input = Laya.Input;
	class MyTextInput extends Sprite {
	    constructor(_width, _height, _size = -1, multiline = false) {
	        super();
	        this.visibleCount = 1;
	        this.isPasswork = false;
	        this.isFocus = false;
	        this.isLinsening = false;
	        this.pvisible = true;
	        if (_size <= 0)
	            _size = 20;
	        if (_size >= _height)
	            _size = _height - 2;
	        this.ID = MyTextInput.Count++;
	        MyTextInput.Dic.set(this.ID, this);
	        this.inputText = new Input();
	        this.inputText.size(_width, _height);
	        this.inputText.multiline = multiline;
	        this.inputText.bold = true;
	        this.inputText.color = "#ffffff";
	        this.inputText.fontSize = _size;
	        if (multiline == false)
	            this.inputText.valign = "middle";
	        else
	            this.inputText.valign = "top";
	    }
	    static onHideF(real) {
	        for (var i = 0; i < this.Dic.keys.length; i++) {
	            var id = this.Dic.keys[i];
	            this.Dic.get(id).setValue("隐藏", real);
	        }
	    }
	    static DestroyALL() {
	        for (var i = 0; i < this.Dic.keys.length; i++) {
	            var id = this.Dic.keys[i];
	            this.Dic.get(id).destroyF();
	            this.Dic.delete(id);
	            i--;
	        }
	    }
	    static getNewOne(spr, url, size = 0, col = "白", multiline = false) {
	        var tmpTX = spr.getChildByName(url);
	        var p0 = spr.localToGlobal(new Laya.Point(tmpTX.x, tmpTX.y));
	        var p1 = spr.localToGlobal(new Laya.Point(tmpTX.x + tmpTX.width, tmpTX.y + tmpTX.height));
	        if (size <= 0 || size > (p1.y - p0.y) - 5)
	            size = (p1.y - p0.y) - 5;
	        var TX = new MyTextInput(p1.x - p0.x, p1.y - p0.y, size, multiline);
	        var arr = url.split("_");
	        TX.Name = arr[0] + "_" + arr[1];
	        TX.name = TX.Name;
	        if (arr.length > 2) {
	            var info = {};
	            for (var j = 2; j < arr.length; j++) {
	                var one = arr[j];
	                if (one.indexOf("颜色") == 0) {
	                    one = one.slice(2);
	                    if (one.length == 1)
	                        col = one;
	                    else if (one.indexOf("0x") != 0)
	                        col = Tool_Function.onForceConvertType("0x" + one);
	                }
	                else if (one.indexOf("对齐") == 0) {
	                    TX.setValue("对齐", one.slice(2));
	                }
	                else if (one.indexOf("多行") == 0) {
	                    var num = Tool_Function.onForceConvertType(one.slice(2));
	                    size = size / (num + 1);
	                    TX.setValue("字号", size);
	                }
	                else if (one.indexOf("密码") == 0) {
	                    TX.setValue("密码", true);
	                }
	                else if (one.indexOf("默认") == 0) {
	                    TX.setValue("默认文字", one.slice(2));
	                }
	                else if (one == "仅数字") {
	                    TX.setValue("仅数字", true);
	                }
	                else if (one.indexOf("Length") == 0) {
	                    TX.setValue("最大长度", Tool_Function.onForceConvertType(one.slice(6)));
	                }
	            }
	        }
	        TX.x = tmpTX.x;
	        TX.y = tmpTX.y;
	        TX.setValue("颜色", col);
	        spr.addChildAt(TX, spr.getChildIndex(tmpTX));
	        TX.initF();
	        return TX;
	    }
	    get text() {
	        return this.inputText.text;
	    }
	    set text(value) {
	        if (this.isPasswork == true && this.inputText.type != "password") {
	            this.inputText.type = "password";
	        }
	        else if (this.isPasswork == false && this.inputText.type == "password") {
	            this.inputText.type = "text";
	        }
	        this.inputText.text = value;
	        if (this.visibleCount != 1)
	            return;
	        this.inputText.visible = true;
	    }
	    initF() {
	        Laya.stage.addChild(this.inputText);
	        if (this.mmo == null) {
	            this.mmo = new MainManager();
	            this.mmo.addEnterFrame(this.enterF.bind(this));
	        }
	    }
	    enterF() {
	        if (this.inputText == null)
	            return;
	        let tar = this;
	        while (tar != Laya.stage) {
	            if (tar == null || tar.visible == false) {
	                if (this.pvisible == true) {
	                    this.setValue("隐藏", true);
	                    this.pvisible = false;
	                }
	                return;
	            }
	            tar = tar.parent;
	        }
	        if (this.pvisible == false) {
	            this.pvisible = true;
	            this.setValue("隐藏", false);
	        }
	        var p = this.localToGlobal(new Laya.Point(0, 0));
	        this.inputText.x = p.x;
	        this.inputText.y = p.y;
	    }
	    setValue(type, val) {
	        if (this.inputText == null)
	            return;
	        switch (type) {
	            case "颜色":
	                if (typeof val === "string") {
	                    if (val == "红")
	                        this.inputText.color = "#ff0000";
	                    else if (val == "白")
	                        this.inputText.color = "#ffffff";
	                    else if (val == "黑")
	                        this.inputText.color = "#000000";
	                    else if (val == "黄")
	                        this.inputText.color = "#ffff00";
	                    else {
	                        this.inputText.color = val;
	                    }
	                }
	                else {
	                    this.inputText.color = Tool_Function.getColorStrFromInt(val);
	                }
	                this.setValue("默认文字颜色", val);
	                break;
	            case "对齐":
	                if (val == "左")
	                    this.inputText.align = "left";
	                else if (val == "中")
	                    this.inputText.align = "center";
	                else if (val == "右")
	                    this.inputText.align = "right";
	                else
	                    this.inputText.align = val;
	                break;
	            case "v对齐":
	                if (val == "上")
	                    this.inputText.valign = "top";
	                else if (val == "中")
	                    this.inputText.valign = "middle";
	                else if (val == "下")
	                    this.inputText.valign = "bottom";
	                else
	                    this.inputText.valign = val;
	                break;
	            case "隐藏":
	                if (val == true)
	                    this.visibleCount--;
	                else {
	                    if (this.visibleCount >= 1)
	                        return;
	                    this.visibleCount++;
	                }
	                if (this.visibleCount == 1) {
	                    this.inputText.visible = true;
	                }
	                else if (this.visibleCount == 0) {
	                    this.inputText.visible = false;
	                }
	                break;
	            case "密码":
	                this.isPasswork = true;
	                this.inputText.type = "password";
	                break;
	            case "仅数字":
	                this.inputText.restrict = "0-9";
	                break;
	            case "仅英文":
	                this.inputText.restrict = "A-Z a-z";
	                break;
	            case "仅数字英文":
	                this.inputText.restrict = "A-Z a-z 0-9";
	                break;
	            case "输入事件":
	                this.ChangedFun = val;
	                if (this.ChangedFun != null) {
	                    this.onAddFocusF();
	                }
	                else {
	                    this.onClearFocusF();
	                }
	                break;
	            case "最大长度":
	                this.maxLength = val;
	                this.inputText.maxChars = val;
	                this.onAddFocusF();
	                break;
	            case "字号":
	                this.inputText.fontSize = val;
	                break;
	            case "默认文字":
	                this.inputText.prompt = val;
	                break;
	            case "默认文字颜色":
	                if (typeof val === "string") {
	                    if (val == "红")
	                        this.inputText.promptColor = "#ff0000";
	                    else if (val == "白")
	                        this.inputText.promptColor = "#ffffff";
	                    else if (val == "黑")
	                        this.inputText.promptColor = "#000000";
	                    else if (val == "黄")
	                        this.inputText.promptColor = "#ffff00";
	                    else {
	                        this.inputText.promptColor = val;
	                    }
	                }
	                else {
	                    this.inputText.promptColor = Tool_Function.getColorStrFromInt(val);
	                }
	                break;
	            case "可编辑":
	                this.inputText.editable = val;
	                break;
	        }
	    }
	    setFocus(real) {
	        if (this.inputText) {
	            this.inputText.focus = real;
	        }
	    }
	    get isFocusIn() { return this.inputText.focus; }
	    onAddFocusF() {
	        if (this.inputText == null)
	            return;
	        if (this.isLinsening == false) {
	            this.isLinsening = true;
	            this.inputText.on(Laya.Event.FOCUS, this, this.onFocusIn);
	            this.inputText.on(Laya.Event.BLUR, this, this.onFocusOut);
	        }
	        this.onFocusOut(null);
	    }
	    onClearFocusF() {
	        if (this.inputText == null)
	            return;
	        if (this.isLinsening == true) {
	            this.isLinsening = false;
	            this.inputText.offAll();
	        }
	    }
	    onFocusIn(e) {
	        this.isFocus = true;
	    }
	    onFocusOut(e) {
	        if (this.ChangedFun && this.isFocus) {
	            Tool_Function.onRunFunction(this.ChangedFun, this.Name);
	        }
	        this.isFocus = false;
	    }
	    destroyF() {
	        MyTextInput.Dic.delete(this.ID);
	        if (this.inputText) {
	            this.inputText.destroy();
	            this.inputText = null;
	        }
	        this.ChangedFun = null;
	        this.mmo = ConfigTidus.GFun_ToolObj_destroyOne(this.mmo);
	        super.destroyF();
	    }
	}
	MyTextInput.Count = 0;
	MyTextInput.Dic = new Map();

	class BTN_Starling extends Sprite {
	    constructor(tar) {
	        super();
	        this.isDown = false;
	        this.autoChangeFrame = true;
	        this.MC = tar;
	        Tool_SpriteUtils.onAddchild_ChangeParent(this, this.MC);
	        this.isMc = this.MC instanceof SwfMovieClip;
	        this.onChangeFrame(BTN_Starling.Frame_Nor);
	    }
	    destroyF() {
	        this.removeFromParent();
	        this.mme = ConfigTidus.GFun_ToolObj_destroyOne(this.mme);
	        this.MC = null;
	        this.FClick = null;
	        this.FDown = null;
	        this.FUp = null;
	    }
	    getChildByName(_name) {
	        let one = super.getChildByName(_name);
	        if (one == null) {
	            one = this.MC.getChildByName(_name);
	        }
	        return one;
	    }
	    initTouch(fd, fu, fc, val = null) {
	        this.FClick = fc;
	        this.FDown = fd;
	        this.FUp = fu;
	        this.valargs = val;
	        if (this.mme == null) {
	            this.mme = new MyMouseManager(this);
	            this.mme.addDownHandler(this.onDownF.bind(this));
	            this.mme.addUpHandler(this.onUpF.bind(this));
	            this.mme.addClickHandler(this.onClickF.bind(this));
	            this.mme.stopEventMop = "永久";
	        }
	    }
	    onDownF(p) {
	        if (this.pause)
	            return;
	        if (this.FDown) {
	            if (this.valargs != null)
	                this.FDown(this.valargs);
	            else
	                this.FDown();
	        }
	        if (this.autoChangeFrame == true)
	            this.onChangeFrame(BTN_Starling.Frame_Down);
	    }
	    onUpF(p) {
	        if (this.autoChangeFrame == true)
	            this.onChangeFrame(BTN_Starling.Frame_Nor);
	        if (this.pause)
	            return;
	        if (this.FUp) {
	            if (p == null)
	                this.FUp(null);
	            else
	                this.FUp(this.valargs);
	        }
	    }
	    onClickF(p) {
	        if (this.FClick) {
	            if (this.valargs != null)
	                this.FClick(this.valargs);
	            else
	                this.FClick();
	        }
	    }
	    onChangeFrame(f) {
	        if (this.isMc == false || this.MC == null)
	            return;
	        if (this.MC.totalFrames <= f)
	            return;
	        this.MC.gotoAndStop(f, false);
	    }
	    checkIn(mx, my) {
	        if (this._pause || this.visible == false)
	            return false;
	        var lx = mx;
	        var ly = my;
	        let bounse = this.getBounds();
	        if (lx >= bounse.x && lx <= bounse.right && ly >= bounse.y && ly <= bounse.bottom)
	            return true;
	        return false;
	    }
	    get pause() {
	        return this._pause;
	    }
	    set pause(value) {
	        this._pause = value;
	        if (this.pause == true)
	            this.setFrame(BTN_Starling.FramePause);
	        else
	            this.setFrame(BTN_Starling.Frame_Nor);
	    }
	    setFrame(n) {
	        if (this.isMc == false)
	            return;
	        if (n == this.nowFrame)
	            return;
	        this.nowFrame = n;
	        if (this.MC != null) {
	            if (this.MC.totalFrames > n)
	                this.MC.gotoAndStop(n, false);
	        }
	    }
	}
	BTN_Starling.Frame_Nor = 0;
	BTN_Starling.Frame_Down = 1;
	BTN_Starling.Frame_Selected = 2;
	BTN_Starling.FramePause = 3;

	class MyEffect_WindowAppear_Scale {
	    constructor(mc, f) {
	        this.S = 0.2;
	        this.time = 0.2;
	        this.MC = mc;
	        this.Fun = f;
	        this.layer = new Sprite();
	        if (this.MC.parent) {
	            this.MC.parent.addChildAt(this.layer, this.MC.parent.getChildIndex(this.MC));
	        }
	        this.layer.addChild(this.MC);
	        this.W = ConfigTidus.stageW;
	        this.H = ConfigTidus.stageH;
	        this.layer.scaleX = this.layer.scaleY = this.S;
	        this.layer.x += this.W * (1 - this.S) / 2;
	        this.layer.y += this.H * (1 - this.S) / 2;
	        this.layer.alpha = 0.1;
	        Laya.Tween.to(this.layer, { x: 0, y: 0, scaleX: 1, scaleY: 1, alpha: 1 }, this.time * 1000, null, Laya.Handler.create(this, this.Tover));
	    }
	    Tover() {
	        this.layer.parent.addChildAt(this.MC, this.layer.parent.getChildIndex(this.layer));
	        this.layer = ConfigTidus.GFun_ToolObj_destroyOne(this.layer);
	        this.Fun();
	    }
	}

	class AlertWindow {
	    constructor(str, viewText, haveok, fok = null, haveno = false, fno = null, noAni = false, okvalue = null, novalue = null) {
	        this.Busy = false;
	        this.simple = false;
	        MyTextInput.onHideF(true);
	        this.FOK = fok;
	        this.FNO = fno;
	        this.Vok = okvalue;
	        this.Vno = novalue;
	        this.HaveNO = haveno;
	        this.HaveOK = haveok;
	        if (this.HaveNO == false && this.HaveOK == false)
	            this.HaveOK = true;
	        this._strMessage = str;
	        this.initStarlingVer(str, viewText, noAni);
	    }
	    static showF(str, viewText, haveok = true, fok = null, haveno = false, fno = null, noAni = false, okvalue = null, novalue = null) {
	        if (this.Instance != null) {
	            this.Arr_Waite.push([str, viewText, haveok, fok, haveno, fno, noAni, okvalue, novalue]);
	            return;
	        }
	        this.Instance = new AlertWindow(str, viewText, haveok, fok, haveno, fno, noAni, okvalue, novalue);
	    }
	    static showSimpleF(str, viewText, haveok, fok = null, haveno = false, fno = null) {
	        var a = new AlertWindow(str, viewText, haveok, fok, haveno, fno, true);
	        a.simple = true;
	        return a;
	    }
	    static clearF() {
	        this.Arr_Waite = [];
	        if (this.Instance) {
	            this.Instance.clickF(-1);
	            this.Instance = null;
	        }
	    }
	    get strMessage() {
	        return this._strMessage;
	    }
	    set strMessage(value) {
	        this._strMessage = value;
	        if (this.Spr == null)
	            return;
	        var TX;
	        TX = this.Spr.getChildByName("tx_内容");
	        TX.text = value;
	    }
	    tover() {
	        this.Busy = false;
	    }
	    onKeyboardF(c) {
	        if (c == Laya.Keyboard.ENTER) {
	            if (this.HaveOK || (this.HaveNO == false && this.HaveOK == false)) {
	                this.clickF(0);
	            }
	        }
	        else if (c == Laya.Keyboard.BACKSPACE) {
	            if (this.HaveNO) {
	                this.clickF(1);
	            }
	        }
	    }
	    clickF(n) {
	        if (this.Busy)
	            return;
	        MyTextInput.onHideF(false);
	        this.Layer2 = ConfigTidus.GFun_ToolObj_destroyOne(this.Layer2);
	        this.Spr = ConfigTidus.GFun_ToolObj_destroyOne(this.Spr);
	        this.BtnOK = ConfigTidus.GFun_ToolObj_destroyOne(this.BtnOK);
	        this.BtnNO = ConfigTidus.GFun_ToolObj_destroyOne(this.BtnNO);
	        if (n == 0 && this.FOK != null) {
	            if (this.Vok)
	                this.FOK(this.Vok);
	            else
	                this.FOK();
	        }
	        else if (n == 1 && this.FNO != null) {
	            if (this.Vno)
	                this.FNO(this.Vno);
	            else
	                this.FNO();
	        }
	        this.FOK = null;
	        this.FNO = null;
	        this.nextF();
	    }
	    nextF() {
	        if (this.simple == false) {
	            AlertWindow.Instance = null;
	            if (AlertWindow.Arr_Waite.length > 0) {
	                var arr = AlertWindow.Arr_Waite[0];
	                AlertWindow.Arr_Waite.splice(0, 1);
	                AlertWindow.showF(arr[0], arr[1], arr[2], arr[3], arr[4], arr[5], arr[6], arr[7], arr[8]);
	            }
	        }
	    }
	    initStarlingVer(str, viewText, noAni) {
	        console.log("Alert::" + str);
	        if (ConfigTidus.stageW > ConfigTidus.stageH) {
	            if (this.HaveOK == true && this.HaveNO == true)
	                this.Spr = ConfigTidus.GVal["MySourceManager"].instance.getSprFromSwf("SWF_Default", "spr_AlertWin");
	            else if (this.HaveOK == true)
	                this.Spr = ConfigTidus.GVal["MySourceManager"].instance.getSprFromSwf("SWF_Default", "spr_AlertWin2");
	            else
	                this.Spr = ConfigTidus.GVal["MySourceManager"].instance.getSprFromSwf("SWF_Default", "spr_AlertWin3");
	            if (this.Spr == null)
	                this.Spr = ConfigTidus.GVal["MySourceManager"].instance.getSprFromSwf("SWF_Default", "spr_AlertWin");
	        }
	        else {
	            if (this.HaveOK == true && this.HaveNO == true)
	                this.Spr = ConfigTidus.GVal["MySourceManager"].instance.getSprFromSwf("SWF_Default", "spr_AlertWin竖");
	            else if (this.HaveOK == true)
	                this.Spr = ConfigTidus.GVal["MySourceManager"].instance.getSprFromSwf("SWF_Default", "spr_AlertWin2竖");
	            else
	                this.Spr = ConfigTidus.GVal["MySourceManager"].instance.getSprFromSwf("SWF_Default", "spr_AlertWin3竖");
	            if (this.Spr == null)
	                this.Spr = ConfigTidus.GVal["MySourceManager"].instance.getSprFromSwf("SWF_Default", "spr_AlertWin竖");
	        }
	        if (this.Spr == null) {
	            this.clickF(0);
	            return;
	        }
	        this.Layer2 = new Sprite();
	        LayerManager.instance.LayerTop.addChild(this.Layer2);
	        if (this.Spr.getChildAt(0).width >= ConfigTidus.stageW - 1)
	            Tool_ViewFit.onWindowFitScreen(this.Spr, 4);
	        else
	            Tool_ViewFit.onWindowFitScreen(this.Spr, 4.1);
	        this.Layer2.addChild(this.Spr);
	        var TX;
	        TX = this.Spr.getChildByName("tx_内容");
	        TX.text = str;
	        if (viewText && viewText["title"]) {
	            TX = this.Spr.getChildByName("tx_title");
	            TX.text = viewText["title"];
	        }
	        this.BtnOK = this.Spr.getChildByName("btn_确定");
	        this.BtnNO = this.Spr.getChildByName("btn_取消");
	        var dicImg = {};
	        var i = 0;
	        if (this.BtnOK && this.BtnOK instanceof SwfMovieClip$1) {
	            for (var j = 0; j < 2; j++) {
	                this.BtnOK.gotoAndStop(j);
	                i = 0;
	                while (i < this.BtnOK.numChildren) {
	                    var cone = this.BtnOK.getChildAt(i);
	                    if (cone.name && cone.name.indexOf("_img") == 0) {
	                        if (dicImg["ok"] == null)
	                            dicImg["ok"] = [];
	                        dicImg["ok"].push(cone);
	                    }
	                    i++;
	                }
	            }
	        }
	        else if (this.BtnOK) {
	            while (i < this.BtnOK.numChildren) {
	                cone = this.BtnOK.getChildAt(i);
	                if (cone.name && cone.name.indexOf("_img") == 0) {
	                    if (dicImg["ok"] == null)
	                        dicImg["ok"] = [];
	                    dicImg["ok"].push(cone);
	                }
	                i++;
	            }
	        }
	        if (this.BtnNO && this.BtnNO instanceof SwfMovieClip$1) {
	            for (j = 0; j < 2; j++) {
	                this.BtnNO.gotoAndStop(j);
	                i = 0;
	                while (i < this.BtnNO.numChildren) {
	                    cone = this.BtnNO.getChildAt(i);
	                    if (cone.name && cone.name.indexOf("_img") == 0) {
	                        if (dicImg["no"] == null)
	                            dicImg["no"] = [];
	                        dicImg["no"].push(cone);
	                    }
	                    i++;
	                }
	            }
	        }
	        else if (this.BtnNO) {
	            while (i < this.BtnNO.numChildren) {
	                cone = this.BtnNO.getChildAt(i);
	                if (cone.name && cone.name.indexOf("_img") == 0) {
	                    if (dicImg["no"] == null)
	                        dicImg["no"] = [];
	                    dicImg["no"].push(cone);
	                }
	                i++;
	            }
	        }
	        if (this.HaveOK) {
	            TX = this.BtnOK.getChildByName("tx_name");
	            if (TX) {
	                if (viewText && viewText["ok"]) {
	                    TX.text = viewText["ok"];
	                    if (dicImg["ok"]) {
	                        for (i = 0; i < dicImg["ok"].length; i++) {
	                            var childarr = dicImg["ok"];
	                            while (childarr.length > 0) {
	                                var childone = childarr.shift();
	                                if (this.BtnOK instanceof SwfMovieClip$1) {
	                                    this.BtnOK.removeChildFromAllFrame(childone.classLink);
	                                }
	                                else {
	                                    childone = ConfigTidus.GFun_ToolObj_destroyOne(childone);
	                                }
	                            }
	                        }
	                    }
	                }
	                else if (dicImg["ok"] != null) {
	                    if (this.BtnOK instanceof SwfMovieClip$1) {
	                        this.BtnOK.removeChildFromAllFrame("tx_name");
	                    }
	                    else {
	                        TX = ConfigTidus.GFun_ToolObj_destroyOne(TX);
	                    }
	                }
	                else
	                    TX.text = "确定";
	            }
	        }
	        if (this.HaveNO == true) {
	            TX = this.BtnNO.getChildByName("tx_name");
	            if (TX) {
	                if (viewText && viewText["no"]) {
	                    TX.text = viewText["no"];
	                    if (dicImg["no"]) {
	                        for (i = 0; i < dicImg["no"].length; i++) {
	                            childarr = dicImg["no"];
	                            while (childarr.length > 0) {
	                                childone = childarr.shift();
	                                if (this.BtnNO instanceof SwfMovieClip$1) {
	                                    this.BtnNO.removeChildFromAllFrame(childone.classLink);
	                                }
	                                else {
	                                    childone = ConfigTidus.GFun_ToolObj_destroyOne(childone);
	                                }
	                            }
	                        }
	                    }
	                }
	                else if (dicImg["no"] != null) {
	                    if (this.BtnNO instanceof SwfMovieClip$1) {
	                        this.BtnNO.removeChildFromAllFrame("tx_name");
	                    }
	                    else {
	                        TX = ConfigTidus.GFun_ToolObj_destroyOne(TX);
	                    }
	                }
	                else
	                    TX.text = "取消";
	            }
	        }
	        else if (this.BtnNO) {
	            this.BtnNO = ConfigTidus.GFun_ToolObj_destroyOne(this.BtnNO);
	        }
	        if (this.BtnOK) {
	            this.BtnOK = new BTN_Starling(this.BtnOK);
	            this.BtnOK.initTouch(null, null, this.clickF.bind(this), 0);
	        }
	        if (this.BtnNO) {
	            this.BtnNO = new BTN_Starling(this.BtnNO);
	            this.BtnNO.initTouch(null, null, this.clickF.bind(this), 1);
	        }
	        if (AlertWindow.FunInitWin) {
	            AlertWindow.FunInitWin(this);
	        }
	        if (noAni)
	            return;
	        var eff = new MyEffect_WindowAppear_Scale(this.Spr, this.tover.bind(this));
	        this.Busy = false;
	    }
	}
	AlertWindow.Arr_Waite = [];

	class MyViewTXController {
	    constructor(spr, useMyTx = false) {
	        this.Dic_TX = {};
	        this.Dic_TXI = {};
	        this.Dic_Hide = {};
	        for (var i = 0; i < spr.numChildren; i++) {
	            var obj = spr.getChildAt(i);
	            if (obj.name == null)
	                continue;
	            var strName = obj.name;
	            if (strName.indexOf("tx_") == 0 && obj instanceof TextField) {
	                obj.autoScale = true;
	                this.Dic_TX[strName] = obj;
	            }
	            else if (strName.indexOf("txi_") == 0 && (obj instanceof MyTextInput) == false) {
	                var col = "白";
	                var size = 0;
	                var multiline = false;
	                if (obj instanceof TextField) {
	                    col = obj.format.color;
	                    size = obj.format.size;
	                    multiline = obj.wordWrap;
	                }
	                var tx = MyTextInput.getNewOne(spr, strName, size, col, multiline);
	                if (obj instanceof TextField) {
	                    tx.setValue("对齐", obj.align);
	                }
	                this.Dic_TXI[tx.Name] = tx;
	                if (obj instanceof TextField) {
	                    tx.setValue("默认文字", obj.text);
	                }
	                ConfigTidus.GFun_ToolObj_destroyOne(obj);
	                i--;
	            }
	        }
	    }
	    getTextField(strName) {
	        if (this.Dic_TX == null)
	            return null;
	        return this.Dic_TX[strName];
	    }
	    getInput(strName) {
	        if (this.Dic_TXI == null)
	            return null;
	        return this.Dic_TXI[strName];
	    }
	    getText(strName) {
	        if (this.Dic_TX == null)
	            return null;
	        if (this.Dic_TX[strName])
	            return this.Dic_TX[strName].text;
	        if (this.Dic_TXI[strName])
	            return this.Dic_TXI[strName].text;
	        var n;
	        for (n in this.Dic_TX) {
	            if (n.indexOf(strName) == 0)
	                return this.Dic_TX[n].text;
	        }
	        for (n in this.Dic_TXI) {
	            if (n.indexOf(strName) == 0)
	                return this.Dic_TXI[n].text;
	        }
	        return null;
	    }
	    setText(strName, txt) {
	        if (this.Dic_TX == null)
	            return;
	        if (this.Dic_TX[strName])
	            this.Dic_TX[strName].text = txt;
	        if (this.Dic_TXI[strName])
	            this.Dic_TXI[strName].text = txt;
	        var n;
	        if (strName == "tx_") {
	            for (n in this.Dic_TX) {
	                this.Dic_TX[n].text = txt;
	            }
	        }
	        else if (strName == "txi_") {
	            for (n in this.Dic_TXI) {
	                this.Dic_TXI[n].text = txt;
	            }
	        }
	    }
	    changeTextfieldColor(tfname, col) {
	        let tf = this.getTextField(tfname);
	        if (tf) {
	            Tool_Textfield.changeTextfieldColor(tf, col);
	        }
	    }
	    addChangeEventListener(strName, f) {
	        if (strName == null || f == null || this.Dic_TXI[strName] == null)
	            return;
	        if (this.Dic_Listener == null)
	            this.Dic_Listener = {};
	        if (this.Dic_Listener[strName] == null) {
	            this.Dic_TXI[strName].setValue("输入事件", this.onChangeEvent.bind(this));
	            this.Dic_Listener[strName] = [];
	        }
	        if (this.Dic_Listener[strName].indexOf(f) == -1)
	            this.Dic_Listener[strName].push(f);
	    }
	    onChangeEvent(tar) {
	        if (this.Dic_TXI[tar] == null || this.Dic_Listener == null || this.Dic_Listener[tar] == null)
	            return;
	        for (var i = 0; i < this.Dic_Listener[tar].length; i++) {
	            if (this.Dic_Listener[tar][i]) {
	                var f = this.Dic_Listener[tar][i];
	                if (f)
	                    f(tar);
	            }
	        }
	    }
	    onHideInput(real) {
	        if (this.Dic_TXI == null)
	            return;
	        var n;
	        for (n in this.Dic_TXI) {
	            if (this.Dic_TXI[n] == null)
	                continue;
	            if (real == false) {
	                if (this.Dic_Hide[n] != true)
	                    continue;
	            }
	            else {
	                if (this.Dic_Hide[n] == true)
	                    continue;
	            }
	            this.Dic_Hide[n] = real;
	            this.Dic_TXI[n].setValue("隐藏", real);
	        }
	    }
	    onStopInput(real) {
	        if (this.Dic_TXI == null)
	            return;
	        var n;
	        for (n in this.Dic_TXI) {
	            if (this.Dic_TXI[n] == null)
	                continue;
	            this.Dic_TXI[n].setValue("可编辑", !real);
	        }
	    }
	    destroyF() {
	        this.Dic_Listener = null;
	        this.Dic_TX = ConfigTidus.GFun_ToolObj_destroyOne(this.Dic_TX);
	        this.Dic_TXI = ConfigTidus.GFun_ToolObj_destroyOne(this.Dic_TXI);
	    }
	}

	class LoadingSmall {
	    constructor(str) {
	        MyTextInput.onHideF(true);
	        if (ConfigTidus.stageW > ConfigTidus.stageH)
	            this.spr = ConfigTidus.GVal["MySourceManager"].getInstance().getObjFromSwf("SWF_Default", "spr_Loading");
	        else
	            this.spr = ConfigTidus.GVal["MySourceManager"].getInstance().getObjFromSwf("SWF_Default", "spr_Loading竖");
	        if (this.spr == null) {
	            if (ConfigTidus.stageW > ConfigTidus.stageH)
	                this.spr = ConfigTidus.GVal["MySourceManager"].getInstance().getObjFromSwf("SWF_Default", "spr_LoadSmall");
	            else
	                this.spr = ConfigTidus.GVal["MySourceManager"].getInstance().getObjFromSwf("SWF_Default", "spr_LoadSmall竖");
	        }
	        if (this.spr) {
	            LayerManager.instance.LayerTop.addChild(this.spr);
	            this.mt = new MyViewTXController(this.spr);
	            this.changeSTR(str);
	            Tool_ViewFit.onWindowFitScreen(this.spr, 4);
	        }
	    }
	    static showF(str = null) {
	        if (this.One == null) {
	            this.One = new LoadingSmall(str);
	        }
	        else {
	            this.One.changeSTR(str);
	        }
	    }
	    static removeF(e = null) {
	        if (this.One != null) {
	            this.One.destroyF();
	            this.One = null;
	        }
	    }
	    changeSTR(str) {
	        if (str == null)
	            str = "";
	        if (this.mt && this.mt.Dic_TX) {
	            for (let key in this.mt.Dic_TX) {
	                this.mt.setText(key, str);
	            }
	        }
	    }
	    destroyF() {
	        MyTextInput.onHideF(false);
	        this.mt = ConfigTidus.GFun_ToolObj_destroyOne(this.mt);
	        this.spr = ConfigTidus.GFun_ToolObj_destroyOne(this.spr);
	    }
	}

	class Tool_Function {
	    static onRunFunction(f, ...arg) {
	        if (f == null) {
	            return;
	        }
	        if (f instanceof Function) {
	            return f.apply(null, arg);
	        }
	        else {
	            return f.runWith(arg);
	        }
	    }
	    static getLastClassName(tar) {
	        var className = tar.constructor.toString();
	        if (className.indexOf("(") != -1) {
	            className = className.slice(0, className.indexOf("("));
	            className = className.substr(className.indexOf(" ") + 1);
	        }
	        return className;
	    }
	    static navigateToURL(url) {
	        window.open(url);
	    }
	    static isTypeOf(obj, type) {
	        if (obj == null)
	            return false;
	        if (type == "number" || type == "num") {
	            if (typeof obj === "number")
	                return true;
	            return false;
	        }
	        if (type == "int") {
	            if (typeof obj === "number" && Tool_Function.onForceConvertType(obj) == obj) {
	                return true;
	            }
	            return false;
	        }
	        if (type == "string" || type == "str") {
	            if (typeof obj === "string")
	                return true;
	            return false;
	        }
	        if (type == "obj") {
	            if (obj instanceof Object)
	                return true;
	            return false;
	        }
	        if (type == "arr") {
	            if (obj instanceof Array)
	                return true;
	            return false;
	        }
	        return false;
	    }
	    static deg2rad(deg) {
	        return deg / 180.0 * Math.PI;
	    }
	    static rad2deg(rad) {
	        return rad * 180 / Math.PI;
	    }
	    static onForceConvertType(tar, type = "int") {
	        return Tool_Function.onChangeInstance(tar, type);
	    }
	    static onForceConvertType_string(tar) {
	        return Tool_Function.onChangeInstance(tar, "str");
	    }
	    static onGetLengthBetweenPoint(point1, point2) {
	        var lx = point1.x - point2.x;
	        var ly = point1.y - point2.y;
	        var l = lx * lx + ly * ly;
	        return Math.sqrt(l);
	    }
	    static onGetXLengthBetweenPoint(point1, point2) {
	        let l = Tool_Function.onGetLengthBetweenPoint(point1, point2);
	        if (l == 0) {
	            return 0;
	        }
	        let ang = Tool_Function.onGetAngleBetweenPoint(point1, point2);
	        let sx;
	        let sy;
	        if (ang <= 90) {
	            sx = l * Math.sin(Tool_Function.deg2rad(ang));
	        }
	        else if (ang <= 180) {
	            sx = l * Math.cos(Tool_Function.deg2rad(ang - 90));
	        }
	        else if (ang <= 270) {
	            sx = -l * Math.sin(Tool_Function.deg2rad(ang - 180));
	        }
	        else {
	            sx = -l * Math.sin(Tool_Function.deg2rad(360 - ang));
	        }
	        if (sx)
	            sy = Math.sqrt(l * l - sx * sx);
	        else if (sy)
	            sx = Math.sqrt(l * l - sy * sy);
	        if (point1.y > point2.y)
	            sy = -sy;
	        return sx;
	    }
	    static onGetYLengthBetweenPoint(point1, point2) {
	        let l = Tool_Function.onGetLengthBetweenPoint(point1, point2);
	        if (l == 0) {
	            return 0;
	        }
	        let ang = Tool_Function.onGetAngleBetweenPoint(point1, point2);
	        let sx;
	        let sy;
	        if (ang <= 90) {
	            sx = l * Math.sin(Tool_Function.deg2rad(ang));
	        }
	        else if (ang <= 180) {
	            sx = l * Math.cos(Tool_Function.deg2rad(ang - 90));
	        }
	        else if (ang <= 270) {
	            sx = -l * Math.sin(Tool_Function.deg2rad(ang - 180));
	        }
	        else {
	            sx = -l * Math.sin(Tool_Function.deg2rad(360 - ang));
	        }
	        if (sx)
	            sy = Math.sqrt(l * l - sx * sx);
	        else if (sy)
	            sx = Math.sqrt(l * l - sy * sy);
	        if (point1.y > point2.y)
	            sy = -sy;
	        return sy;
	    }
	    static onGetAngleBetweenPoint(point1, point2) {
	        var ang = -1;
	        var lx = point2.x - point1.x;
	        var ly = point2.y - point1.y;
	        if (lx == 0 && ly == 0) {
	            return -1;
	        }
	        if (ly == 0) {
	            if (lx >= 0) {
	                ang = 90;
	            }
	            else {
	                ang = 270;
	            }
	        }
	        else if (ly < 0) {
	            if (lx >= 0) {
	                ang = this.rad2deg(Math.atan(lx / -ly));
	            }
	            else {
	                ang = 360 - this.rad2deg(Math.atan(-lx / -ly));
	            }
	        }
	        else {
	            if (lx >= 0) {
	                ang = 90 + 90 - this.rad2deg(Math.atan(lx / ly));
	            }
	            else {
	                ang = 270 - (90 - this.rad2deg(Math.atan(-lx / ly)));
	            }
	        }
	        return ang;
	    }
	    static onGetSpdXY_bySpd_and_Ang(spd, ang) {
	        var spdx;
	        var spdy;
	        if (ang == -1) {
	            spdx = spdy = spd;
	        }
	        else if (ang == 0) {
	            spdx = 0;
	            spdy = -spd;
	        }
	        else if (ang < 90) {
	            spdx = spd * Math.tan(this.deg2rad(ang));
	            spdy = -Math.sqrt(spd * spd - spdx * spdx);
	        }
	        else if (ang == 90) {
	            spdx = spd;
	            spdy = 0;
	        }
	        else if (ang < 180) {
	            spdy = spd * Math.tan(this.deg2rad(ang - 90));
	            spdx = Math.sqrt(spd * spd - spdy * spdy);
	        }
	        else if (ang == 180) {
	            spdx = 0;
	            spdy = spd;
	        }
	        else if (ang < 270) {
	            spdx = -spd * Math.tan(this.deg2rad(ang - 180));
	            spdy = Math.sqrt(spd * spd - spdx * spdx);
	        }
	        else if (ang == 270) {
	            spdx = -spd;
	            spdy = 0;
	        }
	        else {
	            spdx = -spd * Math.tan(this.deg2rad(360 - ang));
	            spdy = Math.sqrt(spd * spd - spdx * spdx);
	        }
	        return { "x": spdx, "y": spdy };
	    }
	    static onChangeInstance(tar, type = "int") {
	        var tarType = typeof (tar);
	        if (type == "int") {
	            if (tar == null || isNaN(tar))
	                return 0;
	            if (tarType == "number") {
	                return parseInt(tar + "");
	            }
	            else if (tarType === "string") {
	                if (tar == "true")
	                    return 1;
	                if (tar == "false")
	                    return 0;
	                return parseInt(tar);
	            }
	            else {
	                return 0;
	            }
	        }
	        else if (type == "num") {
	            if (tar == null || isNaN(tar))
	                return 0;
	            if (tarType == "number") {
	                return tar;
	            }
	            else if (tarType === "string") {
	                if (tar == "true")
	                    return 1;
	                if (tar == "false")
	                    return 0;
	                return parseFloat(tar);
	            }
	            else {
	                return 0;
	            }
	        }
	        if (type == "str") {
	            return String(tar);
	        }
	        return null;
	    }
	    static validateIdCard(card) {
	        if (card === '') {
	            return false;
	        }
	        if (isCardNo(card) === false) {
	            return false;
	        }
	        if (checkProvince(card) === false) {
	            return false;
	        }
	        if (checkBirthday(card) === false) {
	            return false;
	        }
	        if (checkParity(card) === false) {
	            return false;
	        }
	        return true;
	        function isCardNo(card) {
	            var reg = /(^\d{15}$)|(^\d{17}(\d|X)$)/;
	            if (reg.test(card) === false) {
	                return false;
	            }
	            return true;
	        }
	        function checkProvince(card) {
	            var vcity = { 11: "北京", 12: "天津", 13: "河北", 14: "山西", 15: "内蒙古",
	                21: "辽宁", 22: "吉林", 23: "黑龙江", 31: "上海", 32: "江苏",
	                33: "浙江", 34: "安徽", 35: "福建", 36: "江西", 37: "山东", 41: "河南",
	                42: "湖北", 43: "湖南", 44: "广东", 45: "广西", 46: "海南", 50: "重庆",
	                51: "四川", 52: "贵州", 53: "云南", 54: "西藏", 61: "陕西", 62: "甘肃",
	                63: "青海", 64: "宁夏", 65: "新疆", 71: "台湾", 81: "香港", 82: "澳门", 91: "国外"
	            };
	            var province = card.substr(0, 2);
	            if (vcity[province] == undefined) {
	                return false;
	            }
	            return true;
	        }
	        function checkBirthday(card) {
	            var len = card.length;
	            if (len == '15') {
	                var re_fifteen = /^(\d{6})(\d{2})(\d{2})(\d{2})(\d{3})$/;
	                var arr_data = card.match(re_fifteen);
	                var year = arr_data[2];
	                var month = arr_data[3];
	                var day = arr_data[4];
	                var birthday = new Date('19' + year + '/' + month + '/' + day);
	                return verifyBirthday('19' + year, month, day, birthday);
	            }
	            if (len == '18') {
	                var re_eighteen = /^(\d{6})(\d{4})(\d{2})(\d{2})(\d{3})([0-9]|X)$/;
	                var arr_data = card.match(re_eighteen);
	                var year = arr_data[2];
	                var month = arr_data[3];
	                var day = arr_data[4];
	                var birthday = new Date(year + '/' + month + '/' + day);
	                return verifyBirthday(year, month, day, birthday);
	            }
	            return false;
	        }
	        function verifyBirthday(year, month, day, birthday) {
	            var now = new Date();
	            var now_year = now.getFullYear();
	            if (birthday.getFullYear() == year && (birthday.getMonth() + 1) == month && birthday.getDate() == day) {
	                var time = now_year - year;
	                if (time >= 3 && time <= 100) {
	                    return true;
	                }
	                return false;
	            }
	            return false;
	        }
	        function checkParity(card) {
	            card = changeFivteenToEighteen(card);
	            var len = card.length;
	            if (len == '18') {
	                var arrInt = new Array(7, 9, 10, 5, 8, 4, 2, 1, 6, 3, 7, 9, 10, 5, 8, 4, 2);
	                var arrCh = new Array('1', '0', 'X', '9', '8', '7', '6', '5', '4', '3', '2');
	                var cardTemp = 0, i, valnum;
	                for (i = 0; i < 17; i++) {
	                    cardTemp += card.substr(i, 1) * arrInt[i];
	                }
	                valnum = arrCh[cardTemp % 11];
	                if (valnum == card.substr(17, 1)) {
	                    return true;
	                }
	                return false;
	            }
	            return false;
	        }
	        function changeFivteenToEighteen(card) {
	            if (card.length == '15') {
	                var arrInt = new Array(7, 9, 10, 5, 8, 4, 2, 1, 6, 3, 7, 9, 10, 5, 8, 4, 2);
	                var arrCh = new Array('1', '0', 'X', '9', '8', '7', '6', '5', '4', '3', '2');
	                var cardTemp = 0, i;
	                card = card.substr(0, 6) + '19' + card.substr(6, card.length - 6);
	                for (i = 0; i < 17; i++) {
	                    cardTemp += card.substr(i, 1) * arrInt[i];
	                }
	                card += arrCh[cardTemp % 11];
	                return card;
	            }
	            return card;
	        }
	    }
	    static onDate_int_to_Date(ms, sec) {
	        var d = new Date(sec > 0 ? sec * 1000 : ms);
	        console.log(this.onDate_Data_to_String(d, true));
	        return d;
	    }
	    static onDate_String_to_Date(str) {
	        if (str == null)
	            return null;
	        var year;
	        var month;
	        var day;
	        var hour = null;
	        var minute = null;
	        var second = null;
	        var typeMiddle = -1;
	        var index;
	        var index2 = -1;
	        index = str.indexOf("年");
	        if (index > 0) {
	            typeMiddle = 0;
	        }
	        else {
	            index = str.indexOf("/");
	            if (index > 0) {
	                typeMiddle = 1;
	            }
	            else {
	                index = str.indexOf(".");
	                if (index > 0) {
	                    typeMiddle = 2;
	                }
	                else {
	                    index = str.indexOf("-");
	                    if (index > 0) {
	                        typeMiddle = 3;
	                    }
	                }
	            }
	        }
	        if (typeMiddle == -1)
	            return null;
	        year = this.onForceConvertType(str.slice(0, index));
	        if (typeMiddle == 0)
	            index2 = str.indexOf("月");
	        else if (typeMiddle == 1)
	            index2 = str.indexOf("/", index + 1);
	        else if (typeMiddle == 2)
	            index2 = str.indexOf(".", index + 1);
	        else if (typeMiddle == 3)
	            index2 = str.indexOf("-", index + 1);
	        if (index2 == -1)
	            return null;
	        month = this.onForceConvertType(str.slice(index + 1, index2));
	        index = index2;
	        index2 = -1;
	        if (typeMiddle == 0)
	            index2 = str.indexOf("日");
	        else
	            index2 = str.indexOf(" ", index + 1);
	        if (index2 == -1)
	            return null;
	        day = this.onForceConvertType(str.slice(index + 1, index2));
	        index = index2;
	        index2 = -1;
	        index2 = str.indexOf("时");
	        if (index2 > 0) {
	            hour = this.onForceConvertType(str.slice(index + 1, index2));
	            index = index2;
	            index2 = -1;
	            index2 = str.indexOf("分");
	            if (index2 == -1)
	                return null;
	            minute = this.onForceConvertType(str.slice(index + 1, index2));
	            index = index2;
	            index2 = -1;
	            index2 = str.indexOf("秒");
	            if (index2 != -1) {
	                second = this.onForceConvertType(str.slice(index + 1, index2));
	            }
	        }
	        else {
	            var str2 = str.substr(index + 1);
	            var arr = str2.split(":");
	            if (arr.length >= 2) {
	                hour = this.onForceConvertType(arr[0]);
	                minute = this.onForceConvertType(arr[1]);
	                if (arr.length == 3)
	                    second = this.onForceConvertType(arr[2]);
	            }
	        }
	        var d = new Date(year, month - 1, day, hour, minute, second);
	        return d;
	    }
	    static onDate_Data_to_String(d, needUTC = false, useChinese = true) {
	        if (d == null)
	            return null;
	        var str;
	        if (needUTC)
	            str = d.getUTCFullYear() + "年" + (d.getUTCMonth() + 1) + "月" + d.getUTCDate() + "日" + d.getUTCHours() + ":" + d.getUTCMinutes() + ":" + d.getUTCSeconds();
	        str = d.getFullYear() + "年" + (d.getMonth() + 1) + "月" + d.getDate() + "日" + d.getHours() + ":" + d.getMinutes() + ":" + d.getSeconds();
	        if (useChinese == false) {
	            str = str.replace("年", "-");
	            str = str.replace("月", "-");
	            str = str.replace("日", "");
	        }
	        return str;
	    }
	    static onDate_getBetween(d1, d2, timeName = "时") {
	        var day = this.getBetweenDays(d2, d1);
	        if (timeName == "天") {
	            return day;
	        }
	        var min = Tool_Function.onForceConvertType(d2.getMinutes() - d1.getMinutes() + (day * 24 + d2.getHours() - d1.getHours()) * 60);
	        if (timeName == "分") {
	            return min;
	        }
	        if (timeName == "时") {
	            return Tool_Function.onForceConvertType(min / 60);
	        }
	        if (timeName == "秒") {
	            var sec = Tool_Function.onForceConvertType(d2.getSeconds() - d1.getSeconds() + min * 60);
	            return sec;
	        }
	        return 0;
	    }
	    static onDate_getAllDays(d) {
	        var ms = d.getTime();
	        var day = (ms - (-8 * 60 * 60 * 1000)) / (24 * 60 * 60 * 1000);
	        return Tool_Function.onForceConvertType(day);
	    }
	    static getBetweenDays(d1, d2) {
	        return this.onDate_getAllDays(d1) - this.onDate_getAllDays(d2);
	    }
	    static getColorStrFromInt(col) {
	        let str = col.toString(16);
	        while (str.length < 6) {
	            str = "0" + str;
	        }
	        return "#" + str;
	    }
	    static onColor_getRed(color) { return (color >> 16) & 0xff; }
	    static onColor_getGreen(color) { return (color >> 8) & 0xff; }
	    static onColor_getBlue(color) { return color & 0xff; }
	    static onColor_rgb(red, green, blue) {
	        return (red << 16) | (green << 8) | blue;
	    }
	    static onColor_argb(alpha, red, green, blue) {
	        return (alpha << 24) | (red << 16) | (green << 8) | blue;
	    }
	    static getColorV4FromInt(col, a = 1) {
	        let r = this.onColor_getRed(col) / 255;
	        let g = this.onColor_getGreen(col) / 255;
	        let b = this.onColor_getBlue(col) / 255;
	        return new Laya.Vector4(r, g, b, a);
	    }
	    static getPerValueInArray(val, arr) {
	        let per = 0;
	        if (arr == null || arr.length < 4) {
	            console.error("Tool_Function.getPerValueInArray：arr数据格式错误！");
	            return per;
	        }
	        if (val <= arr[0])
	            return arr[1];
	        if (val >= arr[arr.length - 2])
	            return arr[arr.length - 1];
	        for (let i = 0; i < arr.length; i += 2) {
	            if (i == arr.length - 1 || val <= arr[i]) {
	                if (arr[i] - arr[i - 2] == 0)
	                    per = 0;
	                else
	                    per = arr[i - 1] + (val - arr[i - 2]) * (arr[i + 1] - arr[i - 1]) / (arr[i] - arr[i - 2]);
	            }
	        }
	        return per;
	    }
	    static getRandomInt(max, min, seed) {
	        max = max || 1;
	        min = min || 0;
	        seed = (seed * 9301 + 49297) % 233280;
	        let rnd = seed / 233280.0;
	        return parseInt(Math.floor(min + rnd * (max - min)) + "");
	    }
	    static math_limiteNumber(now, min, max) {
	        if (now < min)
	            return min;
	        if (now > max)
	            return max;
	        return now;
	    }
	    static science_to_numberStr(x) {
	        if (Math.abs(x) < 1.0) {
	            var e = parseInt(x.toString().split('e-')[1]);
	            if (e) {
	                x *= Math.pow(10, e - 1);
	                x = '0.' + (new Array(e)).join('0') + x.toString().substring(2);
	            }
	        }
	        else {
	            var e = parseInt(x.toString().split('+')[1]);
	            if (e > 20) {
	                e -= 20;
	                x /= Math.pow(10, e);
	                x += (new Array(e + 1)).join('0');
	            }
	        }
	        return x;
	    }
	    static getBezierPoints(anchorpoints, pointsAmount) {
	        var points = [];
	        for (var i = 0; i < pointsAmount; i++) {
	            var point = MultiPointBezier(anchorpoints, i / pointsAmount);
	            points.push(point);
	        }
	        return points;
	        function MultiPointBezier(points, t) {
	            var len = points.length;
	            var x = 0, y = 0;
	            var erxiangshi = function (start, end) {
	                var cs = 1, bcs = 1;
	                while (end > 0) {
	                    cs *= start;
	                    bcs *= end;
	                    start--;
	                    end--;
	                }
	                return (cs / bcs);
	            };
	            for (var i = 0; i < len; i++) {
	                var point = points[i];
	                x += point.x * Math.pow((1 - t), (len - 1 - i)) * Math.pow(t, i) * (erxiangshi(len - 1, i));
	                y += point.y * Math.pow((1 - t), (len - 1 - i)) * Math.pow(t, i) * (erxiangshi(len - 1, i));
	            }
	            return { x: x, y: y };
	        }
	    }
	    static isEqualNumber(a, b) {
	        if (Math.abs(a - b) < 0.01)
	            return true;
	        return false;
	    }
	    static onJSONParse(str) {
	        var out;
	        try {
	            out = JSON.parse(str);
	        }
	        catch (e) {
	            console.error("解析JSON失败！");
	        }
	        return out;
	    }
	    static handleNetBack(dic, badcb = null) {
	        LoadingSmall.removeF();
	        if (typeof dic === "string") {
	            if (dic.charAt(0) != "{") {
	                if (dic == "ok")
	                    dic = { "结果": true };
	                else
	                    dic = { "结果": false, "失败原因": dic };
	            }
	            else {
	                dic = this.onJSONParse(dic);
	            }
	        }
	        if (dic == null)
	            dic = { "结果": false, "失败原因": "联网失败" };
	        if (dic["结果"] == false) {
	            AlertWindow.showF(dic["失败原因"], null, badcb != null, badcb);
	            return null;
	        }
	        return dic;
	    }
	}

	class MyCheaterNum {
	    constructor() {
	        this.Dic = {};
	        this.mySeedNum = Math.random() * 50000 - 100000;
	    }
	    addNoCheckKey(key) {
	        if (this.dicNoCheck == null) {
	            this.dicNoCheck = {};
	        }
	        this.dicNoCheck[key] = true;
	    }
	    initByDic(dic) {
	        for (var key in dic) {
	            this.setValue(key, dic[key]);
	        }
	    }
	    changeValue(_name, val) {
	        if (_name == null)
	            return;
	        if (this.dicNoCheck && this.dicNoCheck[_name] == true)
	            return;
	        if (Tool_Function.isTypeOf(val, "num")) {
	            let now = this.getValue(_name);
	            this.setValue(_name, now + val);
	        }
	        else {
	            this.setValue(_name, val);
	        }
	    }
	    setValue(_name, val) {
	        if (_name == null)
	            return;
	        if (this.dicNoCheck && this.dicNoCheck[_name] == true)
	            return;
	        if (Tool_Function.isTypeOf(val, "num")) {
	            this.Dic[_name] = val + this.mySeedNum;
	        }
	        else {
	            this.Dic[_name] = ConfigTidus.GFun_ToolObj_copy(val);
	        }
	    }
	    getValue(name) {
	        if (this.Dic[name] == null)
	            return null;
	        if (Tool_Function.isTypeOf(this.Dic[name], "num") == true) {
	            return this.Dic[name] - this.mySeedNum;
	        }
	        else {
	            return this.Dic[name];
	        }
	    }
	    checkF(name, val2) {
	        if (val2 == null || isNaN(val2) || this.Dic[name] == null)
	            return true;
	        if (this.dicNoCheck && this.dicNoCheck[name] == true)
	            return true;
	        var val = this.getValue(name);
	        var suc = ConfigTidus.GFun_ToolObj_isEqual(val, val2);
	        if (suc == false) {
	            console.log("!");
	        }
	        return suc;
	    }
	    getNowObject() {
	        let dic = {};
	        for (let key in this.Dic) {
	            dic[key] = this.getValue(key);
	        }
	        return dic;
	    }
	    destroyF() {
	        this.Dic = ConfigTidus.GFun_ToolObj_destroyOne(this.Dic);
	    }
	}

	class TidusScript_VM {
	    constructor() {
	        this.optimized = false;
	        this.runCoroutineMode = false;
	        this.registeredDynamicScopeVars = [];
	        this.externalSuspend = false;
	        this.returnSuspendTarget = -1;
	        this.initialize();
	    }
	    initialize() {
	        this.programCounter = 1;
	        this.byteCode = new Array();
	        this.stack = new Array();
	        this.global = new Object();
	        this.global.__scope = null;
	        this.localObject = this.global;
	        this.thisObject = this.global;
	    }
	    rewind() {
	        this.programCounter = 1;
	    }
	    setProgramCounter(p) {
	        this.programCounter = p;
	    }
	    runCoroutine(cname, parms = null) {
	        this.runCoroutineMode = true;
	        var passedparms;
	        if (parms == null)
	            passedparms = [];
	        else
	            passedparms = parms;
	        this.thisObject = new Object();
	        this.localObject = { arguments: passedparms, __scope: this.global[cname].__scope, __parentenvironment: this.localObject };
	        this.manageDynamicScopeVars(this.localObject);
	        this.programCounter = this.global[cname].__entryPoint;
	        return this.execute();
	    }
	    manageDynamicScopeVars(lo) {
	        for (var i = 0; i < this.registeredDynamicScopeVars.length; i++) {
	            if (lo.__parentenvironment[this.registeredDynamicScopeVars[i]] != undefined) {
	                var parentvalue = lo.__parentenvironment[this.registeredDynamicScopeVars[i]];
	                if (parentvalue instanceof Array) {
	                    var newarray = [];
	                    for (var a = 0; a < parentvalue.length; a++) {
	                        newarray.push(parentvalue[a]);
	                    }
	                    lo[this.registeredDynamicScopeVars[i]] = newarray;
	                }
	                else
	                    lo[this.registeredDynamicScopeVars[i]] = parentvalue;
	            }
	            else {
	                lo[this.registeredDynamicScopeVars[i]] = null;
	            }
	        }
	    }
	    getGlobalObject() {
	        return this.global;
	    }
	    getLocalObject() {
	        return this.localObject;
	    }
	    setByteCode(byteCode) {
	        this.byteCode = byteCode;
	        this.byteCodeLength = byteCode.length;
	    }
	    getByteCode() {
	        return this.byteCode;
	    }
	    getByteCodeLength() {
	        return this.byteCodeLength;
	    }
	    execute(returningValue = undefined) {
	        var _byteCode = this.byteCode;
	        var _programCounter = this.programCounter;
	        var _byteCodeLength = this.byteCodeLength;
	        var next;
	        if ((this.returnSuspendTarget != -1)) {
	            _byteCode[_byteCode[this.returnSuspendTarget]] = returningValue;
	            this.returnSuspendTarget = -1;
	        }
	        if ((this.optimized == false) && (this.runCoroutineMode == false)) {
	            for (; _programCounter < _byteCodeLength;) {
	                var fun = this[_byteCode[_programCounter]];
	                if ((next = fun(_byteCode, _programCounter)) == null) {
	                    this.programCounter = _programCounter + 1;
	                    delete _byteCode[0];
	                    return true;
	                }
	                _programCounter = next;
	            }
	        }
	        else if ((this.optimized == true) && (this.runCoroutineMode == false)) {
	            for (; _programCounter < _byteCodeLength;) {
	                if ((next = _byteCode[_programCounter](_byteCode, _programCounter)) == null) {
	                    this.programCounter = _programCounter + 1;
	                    delete _byteCode[0];
	                    return true;
	                }
	                _programCounter = next;
	            }
	        }
	        else if ((this.optimized == false) && (this.runCoroutineMode == true)) {
	            for (; _programCounter < _byteCodeLength;) {
	                if (_programCounter == 0)
	                    break;
	                if ((next = this[_byteCode[_programCounter]](_byteCode, _programCounter)) == null) {
	                    if (this.externalSuspend == true)
	                        this.programCounter = this.returnSuspendTarget + 1;
	                    else
	                        this.programCounter = _programCounter + 1;
	                    delete _byteCode[0];
	                    this.externalSuspend = false;
	                    return true;
	                }
	                _programCounter = next;
	            }
	        }
	        else {
	            for (; _programCounter < _byteCodeLength;) {
	                if (_programCounter == 0)
	                    break;
	                if ((next = _byteCode[_programCounter](_byteCode, _programCounter)) == null) {
	                    if (this.externalSuspend == true)
	                        this.programCounter = this.returnSuspendTarget + 1;
	                    else
	                        this.programCounter = _programCounter + 1;
	                    delete _byteCode[0];
	                    this.externalSuspend = false;
	                    return true;
	                }
	                _programCounter = next;
	            }
	        }
	        this.programCounter = _programCounter;
	        delete _byteCode[0];
	        this.runCoroutineMode = false;
	        return false;
	    }
	    suspend() {
	        this.externalSuspend = true;
	    }
	    executeFunction(thisObject, args, entryPoint, scope) {
	        var _thisObject = this.thisObject;
	        var _localObject = this.localObject;
	        var _programCounter = this.programCounter;
	        this.thisObject = thisObject;
	        this.localObject = { arguments: args, __scope: scope };
	        this.programCounter = entryPoint;
	        while (this.execute()) { }
	        this.programCounter = _programCounter;
	        this.localObject = _localObject;
	        this.thisObject = _thisObject;
	        return this.returnValue;
	    }
	    __resolve(name) {
	        throw new Error('VirtualMachine [UnknownOperation] : ' + name + ' at pc' + arguments[1]);
	        return 0;
	    }
	    NOP(code, pc) {
	        return pc + 1;
	    }
	    SPD(code, pc) {
	        return null;
	    }
	    LIT(code, pc) {
	        code[code[pc + 2]] = code[pc + 1];
	        return pc + 3;
	    }
	    CALL(code, pc) {
	        var id = code[pc + 1];
	        var func = null;
	        for (var o = this.localObject; o != null; o = o.__scope) {
	            if (o.hasOwnProperty(id)) {
	                func = o[id];
	                break;
	            }
	        }
	        var numOfArguments = code[pc + 2] + 1;
	        var _stack = this.stack;
	        var args = [];
	        while (--numOfArguments) {
	            args.push(_stack.pop());
	        }
	        args.reverse();
	        if (func.hasOwnProperty("__entryPoint")) {
	            _stack.push(pc + 4);
	            _stack.push(this.thisObject);
	            _stack.push(this.localObject);
	            _stack.push(code[pc + 3]);
	            this.thisObject = this.global;
	            this.localObject = { arguments: args, __scope: func.__scope, __parentenvironment: this.localObject };
	            this.manageDynamicScopeVars(this.localObject);
	            return func.__entryPoint;
	            5;
	        }
	        var result = func.apply(this.global, args);
	        if (this.externalSuspend == false)
	            code[code[pc + 3]] = result;
	        else {
	            this.returnSuspendTarget = (pc + 3);
	            return null;
	        }
	        return pc + 4;
	    }
	    CALLL(code, pc) {
	        var func = this.localObject[code[pc + 1]];
	        var numOfArguments = code[pc + 2] + 1;
	        var _stack = this.stack;
	        var args = [];
	        while (--numOfArguments) {
	            args.push(_stack.pop());
	        }
	        args.reverse();
	        if (func.hasOwnProperty("__entryPoint")) {
	            _stack.push(pc + 4);
	            _stack.push(this.thisObject);
	            _stack.push(this.localObject);
	            _stack.push(code[pc + 3]);
	            this.thisObject = this.global;
	            this.localObject = { arguments: args, __scope: func.__scope, __parentenvironment: this.localObject };
	            this.manageDynamicScopeVars(this.localObject);
	            return func.__entryPoint;
	        }
	        var result = func.apply(this.global, args);
	        if (this.externalSuspend == false)
	            code[code[pc + 3]] = result;
	        else {
	            this.returnSuspendTarget = (pc + 3);
	            return null;
	        }
	        return pc + 4;
	    }
	    CALLM(code, pc) {
	        var o = code[pc + 1];
	        var func = o[code[pc + 2]];
	        var isInGlobal = false;
	        if (func == null) {
	            isInGlobal = true;
	            func = this.global[code[pc + 2]];
	        }
	        var numOfArguments = code[pc + 3] + 1;
	        var _stack = this.stack;
	        var args = [];
	        while (--numOfArguments) {
	            args.push(_stack.pop());
	        }
	        args.reverse();
	        if (func.hasOwnProperty("__entryPoint")) {
	            _stack.push(pc + 5);
	            _stack.push(this.thisObject);
	            _stack.push(this.localObject);
	            _stack.push(code[pc + 4]);
	            this.thisObject = o;
	            this.localObject = { arguments: args, __scope: func.__scope, __parentenvironment: this.localObject };
	            this.manageDynamicScopeVars(this.localObject);
	            return func.__entryPoint;
	        }
	        var result;
	        if (isInGlobal == true)
	            result = func.apply(this.global, args);
	        else
	            result = func.apply(o, args);
	        if (this.externalSuspend == false)
	            code[code[pc + 4]] = result;
	        else {
	            this.returnSuspendTarget = (pc + 4);
	            return null;
	        }
	        return pc + 5;
	    }
	    CALLF(code, pc) {
	        var func = code[pc + 1];
	        var numOfArguments = code[pc + 2] + 1;
	        var _stack = this.stack;
	        var args = [];
	        while (--numOfArguments) {
	            args.push(_stack.pop());
	        }
	        args.reverse();
	        if (func.hasOwnProperty("__entryPoint")) {
	            _stack.push(pc + 4);
	            _stack.push(this.thisObject);
	            _stack.push(this.localObject);
	            _stack.push(code[pc + 3]);
	            this.thisObject = this.global;
	            this.localObject = { arguments: args, __scope: func.__scope, __parentenvironment: this.localObject };
	            this.manageDynamicScopeVars(this.localObject);
	            return func.__entryPoint;
	        }
	        var result = func.apply(this.global, args);
	        if (this.externalSuspend == false)
	            code[code[pc + 3]] = result;
	        else {
	            this.returnSuspendTarget = (pc + 3);
	            return null;
	        }
	        return pc + 4;
	    }
	    RET(code, pc) {
	        this.returnValue = code[pc + 1];
	        return this.byteCodeLength;
	    }
	    CRET(code, pc) {
	        var _stack = this.stack;
	        code[_stack.pop()] = code[pc + 1];
	        this.localObject = _stack.pop();
	        this.thisObject = _stack.pop();
	        return Number(_stack.pop());
	    }
	    FUNC(code, pc) {
	        var vm = this;
	        var entryPoint = pc + 3;
	        var scope = this.localObject;
	        code[code[pc + 2]] = (...arg) => {
	            return vm.executeFunction(this, arg, entryPoint, scope);
	        };
	        return code[pc + 1];
	    }
	    COR(code, pc) {
	        code[code[pc + 2]] = { __entryPoint: pc + 3, __scope: this.localObject };
	        return code[pc + 1];
	    }
	    ARG(code, pc) {
	        this.localObject[code[pc + 2]] = this.localObject["arguments"][code[pc + 1]];
	        if (this.localObject["arguments"] == undefined)
	            this.localObject["arguments"] = [];
	        this.localObject["arguments"].push(code[pc + 2]);
	        return pc + 3;
	    }
	    JMP(code, pc) {
	        return code[pc + 1];
	    }
	    IF(code, pc) {
	        if (code[pc + 1]) {
	            return pc + 3;
	        }
	        return code[pc + 2];
	    }
	    NIF(code, pc) {
	        if (code[pc + 1]) {
	            return code[pc + 2];
	        }
	        return pc + 3;
	    }
	    ADD(code, pc) {
	        code[code[pc + 3]] = code[pc + 1] + code[pc + 2];
	        return pc + 4;
	    }
	    SUB(code, pc) {
	        code[code[pc + 3]] = code[pc + 1] - code[pc + 2];
	        return pc + 4;
	    }
	    MUL(code, pc) {
	        code[code[pc + 3]] = code[pc + 1] * code[pc + 2];
	        return pc + 4;
	    }
	    DIV(code, pc) {
	        code[code[pc + 3]] = code[pc + 1] / code[pc + 2];
	        return pc + 4;
	    }
	    MOD(code, pc) {
	        code[code[pc + 3]] = code[pc + 1] % code[pc + 2];
	        return pc + 4;
	    }
	    AND(code, pc) {
	        code[code[pc + 3]] = code[pc + 1] & code[pc + 2];
	        return pc + 4;
	    }
	    OR(code, pc) {
	        code[code[pc + 3]] = code[pc + 1] | code[pc + 2];
	        return pc + 4;
	    }
	    XOR(code, pc) {
	        code[code[pc + 3]] = code[pc + 1] ^ code[pc + 2];
	        return pc + 4;
	    }
	    NOT(code, pc) {
	        code[code[pc + 2]] = ~code[pc + 1];
	        return pc + 3;
	    }
	    LNOT(code, pc) {
	        code[code[pc + 2]] = !code[pc + 1];
	        return pc + 3;
	    }
	    LSH(code, pc) {
	        code[code[pc + 3]] = code[pc + 1] << code[pc + 2];
	        return pc + 4;
	    }
	    RSH(code, pc) {
	        code[code[pc + 3]] = code[pc + 1] >> code[pc + 2];
	        return pc + 4;
	    }
	    URSH(code, pc) {
	        code[code[pc + 3]] = code[pc + 1] >>> code[pc + 2];
	        return pc + 4;
	    }
	    INC(code, pc) {
	        code[code[pc + 2]] = code[pc + 1] + 1;
	        return pc + 3;
	    }
	    DEC(code, pc) {
	        code[code[pc + 2]] = code[pc + 1] - 1;
	        return pc + 3;
	    }
	    CEQ(code, pc) {
	        code[code[pc + 3]] = code[pc + 1] == code[pc + 2];
	        return pc + 4;
	    }
	    CSEQ(code, pc) {
	        code[code[pc + 3]] = code[pc + 1] === code[pc + 2];
	        return pc + 4;
	    }
	    CNE(code, pc) {
	        code[code[pc + 3]] = code[pc + 1] != code[pc + 2];
	        return pc + 4;
	    }
	    CSNE(code, pc) {
	        code[code[pc + 3]] = code[pc + 1] !== code[pc + 2];
	        return pc + 4;
	    }
	    CLT(code, pc) {
	        code[code[pc + 3]] = code[pc + 1] < code[pc + 2];
	        return pc + 4;
	    }
	    CGT(code, pc) {
	        code[code[pc + 3]] = code[pc + 1] > code[pc + 2];
	        return pc + 4;
	    }
	    CLE(code, pc) {
	        code[code[pc + 3]] = code[pc + 1] <= code[pc + 2];
	        return pc + 4;
	    }
	    CGE(code, pc) {
	        code[code[pc + 3]] = code[pc + 1] >= code[pc + 2];
	        return pc + 4;
	    }
	    DUP(code, pc) {
	        var value = code[pc + 1];
	        code[code[pc + 2]] = value;
	        code[code[pc + 3]] = value;
	        return pc + 4;
	    }
	    THIS(code, pc) {
	        code[code[pc + 1]] = this.thisObject;
	        return pc + 2;
	    }
	    ARRAY(code, pc) {
	        var elements = code[pc + 1];
	        var instance = new Array(elements);
	        var _stack = this.stack;
	        for (var i = 0; i < elements; ++i) {
	            instance[i] = _stack.pop();
	        }
	        instance.reverse();
	        code[code[pc + 2]] = instance;
	        return pc + 3;
	    }
	    OBJ(code, pc) {
	        var properties = code[pc + 1];
	        var instance = new Object();
	        var _stack = this.stack;
	        for (var i = 0; i < properties; ++i) {
	            var value = _stack.pop();
	            instance[_stack.pop()] = value;
	        }
	        code[code[pc + 2]] = instance;
	        return pc + 3;
	    }
	    SETL(code, pc) {
	        this.localObject[code[pc + 1]] = code[code[pc + 3]] = code[pc + 2];
	        return pc + 4;
	    }
	    GETL(code, pc) {
	        code[code[pc + 2]] = this.localObject[code[pc + 1]];
	        return pc + 3;
	    }
	    SET(code, pc) {
	        var id = code[pc + 1];
	        for (var o = this.localObject; o != null; o = o.__scope) {
	            if (o.hasOwnProperty(id)) {
	                o[id] = code[code[pc + 3]] = code[pc + 2];
	                return pc + 4;
	            }
	        }
	        this.global[id] = code[code[pc + 3]] = code[pc + 2];
	        return pc + 4;
	    }
	    GET(code, pc) {
	        var id = code[pc + 1];
	        for (var o = this.localObject; o != null; o = o.__scope) {
	            if (o.hasOwnProperty(id)) {
	                code[code[pc + 2]] = o[id];
	                return pc + 3;
	            }
	        }
	        code[code[pc + 2]] = undefined;
	        return pc + 3;
	    }
	    SETM(code, pc) {
	        code[pc + 1][code[pc + 2]] = code[code[pc + 4]] = code[pc + 3];
	        return pc + 5;
	    }
	    GETM(code, pc) {
	        code[code[pc + 3]] = code[pc + 1][code[pc + 2]];
	        return pc + 4;
	    }
	    GETMV(code, pc) {
	        var thefield;
	        if (code[pc + 2] == 'GETL') {
	            thefield = this.localObject[code[pc + 3]];
	        }
	        else {
	            for (var o = this.localObject; o != null; o = o.__scope) {
	                if (o.hasOwnProperty(code[pc + 3])) {
	                    thefield = o[code[pc + 3]];
	                }
	            }
	        }
	        code[code[pc + 4]] = code[pc + 1][thefield];
	        return pc + 5;
	    }
	    NEW(code, pc) {
	        var constructor = code[pc + 1];
	        this._constructor.prototype = constructor.prototype;
	        var instance = constructor;
	        var argLen = code[pc + 2] + 1;
	        var _stack = this.stack;
	        var args = [];
	        while (--argLen) {
	            args.push(_stack.pop());
	        }
	        code[code[pc + 3]] = constructor.apply(instance, args) || instance;
	        return pc + 4;
	    }
	    _constructor() {
	    }
	    DEL(code, pc) {
	        var id = code[pc + 1];
	        for (var o = this.localObject; o != null; o = o.__scope) {
	            if (o.hasOwnProperty(id)) {
	                code[code[pc + 2]] = delete o[id];
	                return pc + 3;
	            }
	        }
	        code[code[pc + 2]] = false;
	        return pc + 3;
	    }
	    DELL(code, pc) {
	        code[code[pc + 2]] = delete this.localObject[code[pc + 1]];
	        return pc + 3;
	    }
	    DELM(code, pc) {
	        code[code[pc + 3]] = delete code[pc + 1][code[pc + 2]];
	        return pc + 4;
	    }
	    TYPEOF(code, pc) {
	        code[code[pc + 2]] = typeof code[pc + 1];
	        return pc + 3;
	    }
	    INSOF(code, pc) {
	        code[code[pc + 3]] = code[pc + 1] instanceof code[pc + 2];
	        return pc + 4;
	    }
	    NUM(code, pc) {
	        code[code[pc + 2]] = Number(code[pc + 1]);
	        return pc + 3;
	    }
	    STR(code, pc) {
	        code[code[pc + 2]] = code[pc + 1] + "";
	        return pc + 3;
	    }
	    WITH(code, pc) {
	        var o = code[pc + 1];
	        o.__scope = this.localObject;
	        this.localObject = o;
	        return pc + 2;
	    }
	    EWITH(code, pc) {
	        this.localObject = this.localObject["__scope"];
	        return pc + 1;
	    }
	    PUSH(code, pc) {
	        this.stack.push(code[pc + 1]);
	        return pc + 2;
	    }
	    POP(code, pc) {
	        code[code[pc + 1]] = this.stack.pop();
	        return pc + 2;
	    }
	}

	class TidusScript_Label {
	    constructor() {
	        this.initialize();
	    }
	    initialize() {
	        this.address = null;
	        this.isExists = false;
	    }
	    commitAddress(address) {
	        this.address = address;
	        this.isExists = true;
	    }
	}

	class TidusScript_CodeGenerator {
	    constructor() {
	        this.initialize();
	    }
	    initialize() {
	        this.code = new Array();
	        this.stackList = new Array();
	        this.localVariableList = new Array();
	        this.put(null);
	        this.beginNewScope();
	    }
	    error(message) {
	        throw new Error('CodeGenerator [error] ' + message);
	    }
	    getCode() {
	        return this.code;
	    }
	    put(element) {
	        return this.code.push(element) - 1;
	    }
	    putStoreStack() {
	        this.stackList.push(this.put(null));
	    }
	    putLoadStack() {
	        var _code = this.code;
	        var current = Number(this.stackList.pop());
	        var next;
	        var address = this.put(null);
	        for (;;) {
	            next = _code[current];
	            _code[current] = address;
	            if (next == null)
	                break;
	            current = next;
	        }
	    }
	    popStack() {
	        return parseFloat(this.stackList.pop() + "");
	    }
	    pushStack(address) {
	        this.stackList.push(address);
	    }
	    setStackPatch(address) {
	        if (this.code[this.stackList[this.stackList.length - 1]] != null) {
	            this.setStackPatchRecursive(this.code[this.stackList[this.stackList.length - 1]], address);
	        }
	        else
	            this.code[this.stackList[this.stackList.length - 1]] = address;
	    }
	    setStackPatchRecursive(cd, address) {
	        if (this.code[cd] == null)
	            this.code[cd] = address;
	        else
	            this.setStackPatchRecursive(this.code[cd], address);
	    }
	    putCrossLoadStack() {
	        this.swapStack(undefined, undefined);
	        this.putLoadStack();
	        this.putLoadStack();
	    }
	    swapStack(a, b) {
	        a = this.stackList.length - (a !== undefined ? a : 0) - 1;
	        b = this.stackList.length - (b !== undefined ? b : 1) - 1;
	        var n = this.stackList[a];
	        this.stackList[a] = this.stackList[b];
	        this.stackList[b] = n;
	    }
	    getStackLength() {
	        return this.stackList.length;
	    }
	    cleanUpStack(begin) {
	        if (begin === undefined)
	            begin = 0;
	        var _stackList = this.stackList;
	        for (; _stackList.length > begin;) {
	            this.popAndDestroyStack();
	        }
	    }
	    popAndDestroyStack() {
	        var _code = this.code;
	        var current = Number(this.stackList.pop());
	        var next;
	        for (;;) {
	            next = _code[current];
	            _code[current] = 0;
	            if (next == null)
	                break;
	            current = next;
	        }
	    }
	    putLabel(label) {
	        if (label.isExists) {
	            this.put(label.address);
	        }
	        else {
	            label.address = this.put(label.address);
	        }
	    }
	    setLabel(label) {
	        var address = this.code.length;
	        this.setLabelAddress(label, address);
	        label.commitAddress(address);
	    }
	    setLabelAddress(label, address) {
	        var _code = this.code;
	        var current = label.address;
	        var next;
	        for (;;) {
	            if (current == null)
	                break;
	            next = _code[current];
	            _code[current] = address;
	            current = next;
	        }
	    }
	    beginNewScope() {
	        this.localVariableList.unshift(new Object());
	    }
	    closeScope() {
	        this.localVariableList.shift();
	    }
	    isLocalVariable(identifier) {
	        return this.localVariableList[0].hasOwnProperty(identifier + "");
	    }
	    addLocalVariable(identifier) {
	        this.localVariableList[0][identifier] = true;
	    }
	    putExpressionResult(expressionResult) {
	        switch (expressionResult.type) {
	            case 'variable':
	                {
	                    this.putGetVariable(expressionResult.value + "");
	                    expressionResult.setType('stack');
	                }
	                break;
	            case 'member':
	                {
	                    this.putGetMember(expressionResult.getObjectExpression(), expressionResult.getMemberExpression());
	                    expressionResult.setType('stack');
	                }
	                break;
	        }
	    }
	    putValue(expressionResult) {
	        switch (expressionResult.type) {
	            case 'literal':
	                {
	                    this.put(expressionResult.value);
	                }
	                break;
	            case 'stack':
	                {
	                    this.putLoadStack();
	                }
	                break;
	            default: {
	                this.error('putValueError');
	            }
	        }
	    }
	    putBinaryValue(left, right) {
	        if (left.isType('literal') && right.isType('literal')) {
	            this.put(left.value);
	            this.put(right.value);
	        }
	        else if (left.isType('stack') && right.isType('stack')) {
	            this.putCrossLoadStack();
	        }
	        else if (left.isType('stack')) {
	            this.putLoadStack();
	            this.put(right.value);
	        }
	        else if (right.isType('stack')) {
	            this.put(left.value);
	            this.putLoadStack();
	        }
	        else {
	            this.error('putBinaryValueError');
	        }
	    }
	    putSuspend() {
	        this.put((this.vmtarget == null) ? 'SPD' : (this.vmtarget.SPD.bind(this.vmtarget)));
	    }
	    putLiteral(expressionResult) {
	        this.put((this.vmtarget == null) ? 'LIT' : (this.vmtarget.LIT.bind(this.vmtarget)));
	        this.put(expressionResult.value);
	        this.putStoreStack();
	    }
	    putCall(identifier, numOfArguments) {
	        if (this.isLocalVariable(identifier.value)) {
	            this.put((this.vmtarget == null) ? 'CALLL' : this.vmtarget.CALLL.bind(this.vmtarget));
	        }
	        else {
	            this.put((this.vmtarget == null) ? 'CALL' : this.vmtarget.CALL.bind(this.vmtarget));
	        }
	        this.put(identifier.value);
	        this.put(numOfArguments);
	        this.putStoreStack();
	    }
	    putCallMember(objectExpression, memberExpression, numOfArguments) {
	        this.put((this.vmtarget == null) ? 'CALLM' : this.vmtarget.CALLM.bind(this.vmtarget));
	        this.putBinaryValue(objectExpression, memberExpression);
	        this.put(numOfArguments);
	        this.putStoreStack();
	    }
	    putCallFunctor(numOfArguments) {
	        this.put((this.vmtarget == null) ? 'CALLF' : this.vmtarget.CALLF.bind(this.vmtarget));
	        this.putLoadStack();
	        this.put(numOfArguments);
	        this.putStoreStack();
	    }
	    putReturnFunction(returnValue) {
	        this.put((this.vmtarget == null) ? 'RET' : this.vmtarget.RET.bind(this.vmtarget));
	        this.putValue(returnValue);
	    }
	    putReturnCoroutine(returnValue) {
	        this.put((this.vmtarget == null) ? 'CRET' : this.vmtarget.CRET.bind(this.vmtarget));
	        this.putValue(returnValue);
	    }
	    putFunction() {
	        var label = new TidusScript_Label();
	        this.put((this.vmtarget == null) ? 'FUNC' : this.vmtarget.FUNC.bind(this.vmtarget));
	        this.putLabel(label);
	        this.putStoreStack();
	        return label;
	    }
	    putCoroutine() {
	        var label = new TidusScript_Label();
	        this.put((this.vmtarget == null) ? 'COR' : this.vmtarget.COR.bind(this.vmtarget));
	        this.putLabel(label);
	        this.putStoreStack();
	        return label;
	    }
	    putArgument(argumentIndex, identifier) {
	        this.put((this.vmtarget == null) ? 'ARG' : this.vmtarget.ARG.bind(this.vmtarget));
	        this.put(argumentIndex);
	        this.put(identifier);
	        this.addLocalVariable(identifier);
	    }
	    putJump(label) {
	        this.put((this.vmtarget == null) ? 'JMP' : this.vmtarget.JMP.bind(this.vmtarget));
	        this.putLabel(label);
	    }
	    putIf(expressionResult, label) {
	        this.put((this.vmtarget == null) ? 'IF' : this.vmtarget.IF.bind(this.vmtarget));
	        this.putValue(expressionResult);
	        this.putLabel(label);
	    }
	    putNif(expressionResult, label) {
	        this.put((this.vmtarget == null) ? 'NIF' : this.vmtarget.NIF.bind(this.vmtarget));
	        this.putValue(expressionResult);
	        this.putLabel(label);
	    }
	    putBinaryOperation(operation, left, right) {
	        this.put(operation);
	        this.putBinaryValue(left, right);
	        this.putStoreStack();
	    }
	    putUnaryOperation(operation, expression) {
	        this.put(operation);
	        this.putValue(expression);
	        this.putStoreStack();
	    }
	    putIncrement(expressionResult) {
	        this.putIncDec((this.vmtarget == null) ? 'INC' : this.vmtarget.INC.bind(this.vmtarget), false, expressionResult);
	    }
	    putDecrement(expressionResult) {
	        this.putIncDec((this.vmtarget == null) ? 'DEC' : this.vmtarget.DEC.bind(this.vmtarget), false, expressionResult);
	    }
	    putPostfixIncrement(expressionResult) {
	        this.putIncDec((this.vmtarget == null) ? 'INC' : this.vmtarget.INC.bind(this.vmtarget), true, expressionResult);
	    }
	    putPostfixDecrement(expressionResult) {
	        this.putIncDec((this.vmtarget == null) ? 'DEC' : this.vmtarget.DEC.bind(this.vmtarget), true, expressionResult);
	    }
	    putIncDec(operation, isPostfix, expressionResult) {
	        var s;
	        switch (expressionResult.type) {
	            case 'member':
	                {
	                    var objectExpression = expressionResult.getObjectExpression();
	                    var memberExpression = expressionResult.getMemberExpression();
	                    if (!memberExpression.isLiteral()) {
	                        this.putExpressionResult(memberExpression);
	                        s = this.popStack();
	                        this.putDuplicate(memberExpression);
	                        this.pushStack(s);
	                        this.putDuplicate(objectExpression);
	                        this.swapStack(1, 2);
	                    }
	                    else {
	                        this.putDuplicate(objectExpression);
	                    }
	                    this.putGetMember(objectExpression, memberExpression);
	                    expressionResult.setTypeStack();
	                    if (isPostfix) {
	                        this.putDuplicate(expressionResult);
	                        s = this.popStack();
	                        this.putUnaryOperation(operation, expressionResult);
	                        this.putSetMember(objectExpression, memberExpression, expressionResult);
	                        this.popAndDestroyStack();
	                        this.pushStack(s);
	                    }
	                    else {
	                        this.putUnaryOperation(operation, expressionResult);
	                        this.putSetMember(objectExpression, memberExpression, expressionResult);
	                    }
	                }
	                break;
	            case 'variable':
	                {
	                    var identifier = expressionResult.value + "";
	                    this.putGetVariable(identifier);
	                    expressionResult.setTypeStack();
	                    if (isPostfix) {
	                        this.putDuplicate(expressionResult);
	                    }
	                    this.putUnaryOperation(operation, expressionResult);
	                    this.putSetVariable(identifier, expressionResult);
	                    if (isPostfix) {
	                        this.popAndDestroyStack();
	                    }
	                }
	                break;
	            default: {
	                this.error('putIncDecError');
	            }
	        }
	    }
	    putWith(objectExpression) {
	        this.put((this.vmtarget == null) ? 'WITH' : this.vmtarget.WITH.bind(this.vmtarget));
	        this.putValue(objectExpression);
	    }
	    putEndWith() {
	        this.put((this.vmtarget == null) ? 'EWITH' : this.vmtarget.EWITH.bind(this.vmtarget));
	    }
	    putPush(expressionResult) {
	        this.put((this.vmtarget == null) ? 'PUSH' : this.vmtarget.PUSH.bind(this.vmtarget));
	        this.putValue(expressionResult);
	    }
	    putPop() {
	        this.put((this.vmtarget == null) ? 'POP' : this.vmtarget.POP.bind(this.vmtarget));
	        this.putStoreStack();
	    }
	    putDuplicate(expressionResult) {
	        this.put((this.vmtarget == null) ? 'DUP' : this.vmtarget.DUP.bind(this.vmtarget));
	        this.putValue(expressionResult);
	        this.putStoreStack();
	        this.putStoreStack();
	    }
	    putThis() {
	        this.put((this.vmtarget == null) ? 'THIS' : this.vmtarget.THIS.bind(this.vmtarget));
	        this.putStoreStack();
	    }
	    putArrayLiteral(numOfElements) {
	        this.put((this.vmtarget == null) ? 'ARRAY' : this.vmtarget.ARRAY.bind(this.vmtarget));
	        this.put(numOfElements);
	        this.putStoreStack();
	    }
	    putObjectLiteral(numOfProperties) {
	        this.put((this.vmtarget == null) ? 'OBJ' : this.vmtarget.OBJ.bind(this.vmtarget));
	        this.put(numOfProperties);
	        this.putStoreStack();
	    }
	    putGetVariable(identifier) {
	        if (this.isLocalVariable(identifier)) {
	            this.put((this.vmtarget == null) ? 'GETL' : this.vmtarget.GETL.bind(this.vmtarget));
	        }
	        else {
	            this.put((this.vmtarget == null) ? 'GET' : this.vmtarget.GET.bind(this.vmtarget));
	        }
	        this.put(identifier);
	        this.putStoreStack();
	    }
	    putSetVariable(identifier, right) {
	        if (this.isLocalVariable(identifier)) {
	            this.put((this.vmtarget == null) ? 'SETL' : this.vmtarget.SETL.bind(this.vmtarget));
	        }
	        else {
	            this.put((this.vmtarget == null) ? 'SET' : this.vmtarget.SET.bind(this.vmtarget));
	        }
	        this.put(identifier);
	        this.putValue(right);
	        this.putStoreStack();
	    }
	    putSetLocalVariable(identifier, right) {
	        this.put((this.vmtarget == null) ? 'SETL' : this.vmtarget.SETL.bind(this.vmtarget));
	        this.put(identifier);
	        this.putValue(right);
	        this.putStoreStack();
	        this.addLocalVariable(identifier);
	    }
	    putGetMember(objectExpression, memberExpression) {
	        if (memberExpression.isType('variable')) {
	            this.put((this.vmtarget == null) ? 'GETMV' : this.vmtarget.GETMV.bind(this.vmtarget));
	            if (objectExpression.isType('literal')) {
	                this.put(objectExpression.value);
	            }
	            else {
	                this.putLoadStack();
	            }
	            if (this.isLocalVariable(memberExpression.value)) {
	                this.put((this.vmtarget == null) ? 'GETL' : this.vmtarget.GETL.bind(this.vmtarget));
	            }
	            else {
	                this.put((this.vmtarget == null) ? 'GET' : this.vmtarget.GET.bind(this.vmtarget));
	            }
	            this.put(memberExpression.value);
	            this.putStoreStack();
	        }
	        else {
	            this.put((this.vmtarget == null) ? 'GETM' : this.vmtarget.GETM.bind(this.vmtarget));
	            this.putBinaryValue(objectExpression, memberExpression);
	            this.putStoreStack();
	        }
	    }
	    putSetMember(objectExpression, memberExpression, right) {
	        this.put((this.vmtarget == null) ? 'SETM' : this.vmtarget.SETM.bind(this.vmtarget));
	        if (objectExpression.isLiteral()) {
	            if (memberExpression.isLiteral()) {
	                if (right.isLiteral()) {
	                    this.put(objectExpression.value);
	                    this.put(memberExpression.value);
	                    this.put(right.value);
	                }
	                else {
	                    this.put(objectExpression.value);
	                    this.put(memberExpression.value);
	                    this.putLoadStack();
	                }
	            }
	            else {
	                if (right.isLiteral()) {
	                    this.put(objectExpression.value);
	                    this.putLoadStack();
	                    this.put(right.value);
	                }
	                else {
	                    this.put(objectExpression.value);
	                    this.putCrossLoadStack();
	                }
	            }
	        }
	        else {
	            if (memberExpression.isLiteral()) {
	                if (right.isLiteral()) {
	                    this.putLoadStack();
	                    this.put(memberExpression.value);
	                    this.put(right.value);
	                }
	                else {
	                    this.swapStack(undefined, undefined);
	                    this.putLoadStack();
	                    this.put(memberExpression.value);
	                    this.putLoadStack();
	                }
	            }
	            else {
	                if (right.isLiteral()) {
	                    this.putCrossLoadStack();
	                    this.put(right.value);
	                }
	                else {
	                    this.swapStack(0, 2);
	                    this.putLoadStack();
	                    this.putLoadStack();
	                    this.putLoadStack();
	                }
	            }
	        }
	        this.putStoreStack();
	    }
	    putNew(numOfArguments) {
	        this.put((this.vmtarget == null) ? 'NEW' : this.vmtarget.NEW);
	        this.putLoadStack();
	        this.put(numOfArguments);
	        this.putStoreStack();
	    }
	    putDelete(expressionResult) {
	        if (expressionResult.isType('variable') || expressionResult.isType('literal')) {
	            if (this.isLocalVariable(expressionResult.value)) {
	                this.put((this.vmtarget == null) ? 'DELL' : this.vmtarget.DELL.bind(this.vmtarget));
	            }
	            else {
	                this.put((this.vmtarget == null) ? 'DEL' : this.vmtarget.DEL.bind(this.vmtarget));
	            }
	            this.put(expressionResult.value);
	            this.putStoreStack();
	        }
	        else {
	            this.put((this.vmtarget == null) ? 'DEL' : this.vmtarget.DEL.bind(this.vmtarget));
	            this.putLoadStack();
	            this.putStoreStack();
	        }
	    }
	    putDeleteMember(objectExpression, expressionResult) {
	        this.put((this.vmtarget == null) ? 'DELM' : this.vmtarget.DELM.bind(this.vmtarget));
	        this.putBinaryValue(objectExpression, expressionResult);
	        this.putStoreStack();
	    }
	}

	class TidusScript_VMSyntaxError extends Error {
	    constructor(message) {
	        super(message);
	        this.name = 'VMSyntaxError';
	    }
	}

	class TidusScript_ExpressionResult {
	    constructor() {
	        this.isLeftHandSide = false;
	        this.initialize();
	    }
	    static createLiteral(value) {
	        var instance = new TidusScript_ExpressionResult();
	        instance.setTypeLiteral(value);
	        return instance;
	    }
	    static createStack() {
	        var instance = new TidusScript_ExpressionResult();
	        instance.setTypeStack();
	        return instance;
	    }
	    clone() {
	        var instance = new TidusScript_ExpressionResult();
	        instance.setTypeAndValue(this.type, this.value);
	        return instance;
	    }
	    initialize() {
	        this.type = 'empty';
	        this.value = null;
	    }
	    setType(type) {
	        this.type = type;
	    }
	    isType(type) {
	        return (this.type == type);
	    }
	    setValue(value) {
	        this.value = value;
	    }
	    setTypeAndValue(type, value) {
	        this.type = type;
	        this.value = value;
	    }
	    isLiteral() {
	        return this.isType('literal');
	    }
	    isVariableOrMember() {
	        return this.isVariable() || this.isMember();
	    }
	    isVariable() {
	        return this.isType('variable');
	    }
	    isMember() {
	        return this.isType('member');
	    }
	    setTypeStack() {
	        this.setType('stack');
	    }
	    setTypeLiteral(value) {
	        this.setTypeAndValue('literal', value);
	    }
	    setTypeMember(objectExpression, memberExpression) {
	        this.setTypeAndValue('member', { object: objectExpression, member: memberExpression });
	    }
	    getObjectExpression() {
	        if (!this.isMember())
	            return null;
	        return this.value.object;
	    }
	    getMemberExpression() {
	        if (!this.isMember())
	            return null;
	        return this.value.member;
	    }
	}

	class TidusScript_Parser {
	    constructor(scanner) {
	        this.parseForceCoroutine = false;
	        this.vmtarget = null;
	        this.lastReturnCaution = 0;
	        this.scanner = scanner;
	    }
	    getToken() {
	        return this.token;
	    }
	    nextToken() {
	        return (this.token = this.scanner.getToken());
	    }
	    isToken(type) {
	        if (this.token == null)
	            return false;
	        else
	            return (this.token.type == type);
	    }
	    isNextToken(type) {
	        return (this.nextToken().type == type);
	    }
	    initialize() {
	        this.scanner.rewind();
	        this.generator = new TidusScript_CodeGenerator();
	        this.breakLabelList = new Array();
	        this.continueLabelList = new Array();
	        this.functionStack = new Array();
	        this.nextToken();
	    }
	    setForceCoroutine(fc) {
	        this.parseForceCoroutine = fc;
	    }
	    parse(ovm = null) {
	        this.initialize();
	        if (ovm != null) {
	            this.generator.vmtarget = ovm;
	            this.generator.vmtarget.optimized = true;
	            this.vmtarget = ovm;
	        }
	        this.parse_program();
	        return this.generator.getCode();
	    }
	    causeSyntaxError(message) {
	        throw new TidusScript_VMSyntaxError('Parser [causeSyntaxError] on line ' + this.scanner.getLineNumber() + ' ' + message + ' (' + this.getToken().type + ')' + ' ' + this.scanner.getLine());
	    }
	    pushBreakLabel(label) {
	        this.breakLabelList.unshift(label);
	    }
	    popBreakLabel() {
	        return this.breakLabelList.shift();
	    }
	    getBreakLabel() {
	        if (this.breakLabelList.length < 1) {
	            this.causeSyntaxError('break cannot be used here');
	        }
	        return this.breakLabelList[0];
	    }
	    pushContinueLabel(label) {
	        this.continueLabelList.unshift(label);
	    }
	    popContinueLabel() {
	        return this.continueLabelList.shift();
	    }
	    getContinueLabel() {
	        if (this.continueLabelList.length < 1) {
	            this.causeSyntaxError('continue cannot be used here');
	        }
	        return this.continueLabelList[0];
	    }
	    beginFunction() {
	        this.functionStack.unshift(true);
	    }
	    endFunction() {
	        this.functionStack.shift();
	    }
	    beginCoroutine() {
	        this.functionStack.unshift(false);
	    }
	    endCoroutine() {
	        this.functionStack.shift();
	    }
	    isAllowReturn() {
	        return this.functionStack.length > 0;
	    }
	    isInFunction() {
	        return (this.functionStack.length > 0 && this.functionStack[0]);
	    }
	    parse_program() {
	        this.parse_sourceElements();
	    }
	    parse_sourceElements() {
	        this.parse_sourceElement();
	        while (this.getToken() != null) {
	            if (this.isToken('}')) {
	                return;
	            }
	            this.parse_sourceElement();
	        }
	    }
	    parse_sourceElement() {
	        if (this.isToken('function')) {
	            if (this.parseForceCoroutine == true) {
	                this.token.type = 'coroutine';
	                this.parse_coroutineDeclaration();
	            }
	            else
	                this.parse_functionDeclaration();
	        }
	        else if (this.isToken('coroutine')) {
	            this.parse_coroutineDeclaration();
	        }
	        else if (this.isStatementFirst(this.getToken() == null ? null : this.getToken().type)) {
	            this.parse_statement();
	        }
	        else {
	            this.causeSyntaxError('SourceElement found an unexpected token');
	        }
	    }
	    parse_functionDeclaration() {
	        if (!this.isToken('function')) {
	            this.causeSyntaxError("'function' not found in function declaration");
	        }
	        if (!this.isNextToken('identifier')) {
	            this.causeSyntaxError('function name not found in function declaration');
	        }
	        var funcName = this.getToken().value + "";
	        var label = this.generator.putFunction();
	        this.beginFunction();
	        this.generator.beginNewScope();
	        if (!this.isNextToken('(')) {
	            this.causeSyntaxError("'(' not found in function declaration");
	        }
	        if (this.isNextToken('identifier')) {
	            this.parse_formalParameterList();
	        }
	        if (!this.isToken(')')) {
	            this.causeSyntaxError("')' not found in function declaration");
	        }
	        if (!this.isNextToken('{')) {
	            this.causeSyntaxError("'{' not found in function declaration");
	        }
	        if (!this.isNextToken('}')) {
	            this.parse_functionBody();
	        }
	        if (!this.hasLastReturn) {
	            this.generator.putReturnFunction(TidusScript_ExpressionResult.createLiteral(undefined));
	        }
	        if (!this.isToken('}')) {
	            this.causeSyntaxError("'}' not found in function declaration");
	        }
	        this.endFunction();
	        this.generator.closeScope();
	        this.generator.setLabel(label);
	        this.generator.putSetLocalVariable(funcName, TidusScript_ExpressionResult.createStack());
	        this.generator.popAndDestroyStack();
	        this.nextToken();
	    }
	    parse_functionExpression(expressionResult) {
	        if (!this.isToken('function')) {
	            this.causeSyntaxError("'function' not found in function expression");
	        }
	        var funcName = null;
	        if (this.isNextToken('identifier')) {
	            funcName = this.getToken().value + "";
	            this.nextToken();
	        }
	        var label = this.generator.putFunction();
	        this.beginFunction();
	        this.generator.beginNewScope();
	        if (!this.isToken('(')) {
	            this.causeSyntaxError("'(' not found in function expression");
	        }
	        if (this.isNextToken('identifier')) {
	            this.parse_formalParameterList();
	        }
	        if (!this.isToken(')')) {
	            this.causeSyntaxError("')' not found in function expression");
	        }
	        if (!this.isNextToken('{')) {
	            this.causeSyntaxError("'{' not found in function expression");
	        }
	        if (!this.isNextToken('}')) {
	            this.parse_functionBody();
	        }
	        if (!this.hasLastReturn) {
	            this.generator.putReturnFunction(TidusScript_ExpressionResult.createLiteral(undefined));
	        }
	        if (!this.isToken('}')) {
	            this.causeSyntaxError("'}' not found in function expression");
	        }
	        this.generator.closeScope();
	        this.endFunction();
	        this.generator.setLabel(label);
	        if (funcName != null) {
	            this.generator.putSetLocalVariable(funcName, TidusScript_ExpressionResult.createStack());
	        }
	        expressionResult.setTypeStack();
	        this.nextToken();
	    }
	    parse_coroutineDeclaration() {
	        if (!this.isToken('coroutine')) {
	            this.causeSyntaxError("'coroutine' not found in coroutine declaration");
	        }
	        if (!this.isNextToken('identifier')) {
	            this.causeSyntaxError("coroutine name not found in coroutine declaration");
	        }
	        var funcName = this.getToken().value + "";
	        var label = this.generator.putCoroutine();
	        this.beginCoroutine();
	        this.generator.beginNewScope();
	        if (!this.isNextToken('(')) {
	            this.causeSyntaxError("'(' not found in coroutine declaration");
	        }
	        if (this.isNextToken('identifier')) {
	            this.parse_formalParameterList();
	        }
	        if (!this.isToken(')')) {
	            this.causeSyntaxError("')' not found in coroutine declaration");
	        }
	        if (!this.isNextToken('{')) {
	            this.causeSyntaxError("'{' not found in coroutine declaration");
	        }
	        if (!this.isNextToken('}')) {
	            this.parse_functionBody();
	        }
	        if (!this.hasLastReturn) {
	            this.generator.putReturnCoroutine(TidusScript_ExpressionResult.createLiteral(undefined));
	        }
	        if (!this.isToken('}')) {
	            this.causeSyntaxError("'}' not found in coroutine declaration");
	        }
	        this.generator.closeScope();
	        this.endCoroutine();
	        this.generator.setLabel(label);
	        this.generator.putSetLocalVariable(funcName, TidusScript_ExpressionResult.createStack());
	        this.generator.popAndDestroyStack();
	        this.nextToken();
	    }
	    parse_coroutineExpression(expressionResult) {
	        if (!this.isToken('coroutine')) {
	            this.causeSyntaxError("'coroutine' not found in coroutine expression");
	        }
	        var funcName = null;
	        if (this.isNextToken('identifier')) {
	            funcName = this.getToken().value + "";
	            this.nextToken();
	        }
	        var label = this.generator.putCoroutine();
	        this.beginCoroutine();
	        this.generator.beginNewScope();
	        if (!this.isToken('(')) {
	            this.causeSyntaxError("'(' not found in coroutine expression");
	        }
	        if (this.isNextToken('identifier')) {
	            this.parse_formalParameterList();
	        }
	        if (!this.isToken(')')) {
	            this.causeSyntaxError("')' not found in coroutine expression");
	        }
	        if (!this.isNextToken('{')) {
	            this.causeSyntaxError("'{' not found in coroutine expression");
	        }
	        if (!this.isNextToken('}')) {
	            this.parse_functionBody();
	        }
	        if (!this.hasLastReturn) {
	            this.generator.putReturnCoroutine(TidusScript_ExpressionResult.createLiteral(undefined));
	        }
	        if (!this.isToken('}')) {
	            this.causeSyntaxError("'}' not found in coroutine expression");
	        }
	        this.generator.closeScope();
	        this.endCoroutine();
	        this.generator.setLabel(label);
	        if (funcName != null) {
	            this.generator.putSetLocalVariable(funcName, TidusScript_ExpressionResult.createStack());
	        }
	        expressionResult.setTypeStack();
	        this.nextToken();
	    }
	    parse_formalParameterList() {
	        var argumentIndex = 0;
	        for (;;) {
	            if (!this.isToken('identifier')) {
	                this.causeSyntaxError('Parameter name is required');
	            }
	            this.generator.putArgument(argumentIndex, this.getToken().value + "");
	            argumentIndex++;
	            if (this.isNextToken(',')) {
	                this.nextToken();
	                continue;
	            }
	            break;
	        }
	    }
	    parse_functionBody() {
	        this.parse_sourceElements();
	    }
	    parse_statement() {
	        this.hasLastReturn = false;
	        var stackBegin = this.generator.getStackLength();
	        switch (this.getToken().type) {
	            case '{':
	                this.parse_block();
	                break;
	            case 'var':
	                this.parse_variableStatement();
	                break;
	            case ';':
	                this.parse_emptyStatement();
	                break;
	            case 'if':
	                this.parse_ifStatement();
	                break;
	            case 'do':
	            case 'while':
	            case 'for':
	                this.parse_iterationStatement();
	                break;
	            case 'continue':
	                this.parse_continueStatement();
	                break;
	            case 'break':
	                this.parse_breakStatement();
	                break;
	            case 'return':
	                this.parse_returnStatement();
	                break;
	            case 'with':
	                this.parse_withStatement();
	                break;
	            case 'switch':
	                this.parse_switchStatement();
	                break;
	            case 'yield':
	                this.parse_yieldStatement();
	                break;
	            case 'suspend':
	                this.parse_suspendStatement();
	                break;
	            case 'loop':
	                this.parse_loopStatement();
	                break;
	            default:
	                {
	                    if (this.isExpressionFirst(this.getToken().type)) {
	                        this.parse_expressionStatement();
	                        break;
	                    }
	                    this.causeSyntaxError('Unexpected statement token');
	                }
	            case 'function':
	                {
	                    this.causeSyntaxError('Functions are not defined in statements');
	                    break;
	                }
	        }
	        this.generator.cleanUpStack(stackBegin);
	    }
	    isStatementFirst(type) {
	        return (type == '{' || type == 'var' || type == ';' || type == 'if' || type == 'do' ||
	            type == 'while' || type == 'for' || type == 'for' || type == 'continue' ||
	            type == 'break' || type == 'return' || type == 'with' || type == 'switch' ||
	            type == 'yield' || type == 'suspend' || type == 'loop' ||
	            (type != 'function' && type != 'coroutine' && this.isExpressionFirst(type)));
	    }
	    parse_block() {
	        if (!this.isToken('{')) {
	            this.causeSyntaxError("'{' not found in block");
	        }
	        if (!this.isNextToken('}')) {
	            this.parse_statementList();
	        }
	        if (!this.isToken('}')) {
	            this.causeSyntaxError("'}' not found in block");
	        }
	        this.nextToken();
	    }
	    parse_statementList() {
	        this.parse_statement();
	        for (;;) {
	            if (this.isToken('}')) {
	                return;
	            }
	            if (this.isStatementFirst(this.getToken().type)) {
	                this.parse_statement();
	                continue;
	            }
	            break;
	        }
	    }
	    parse_variableStatement() {
	        if (!this.isToken('var')) {
	            this.causeSyntaxError("'var' not found in variable declaration");
	        }
	        this.nextToken();
	        this.parse_variableDeclarationList();
	        if (!this.isToken(';')) {
	            this.causeSyntaxError('Variable declaration must end with ;');
	        }
	        this.nextToken();
	    }
	    parse_variableDeclarationList() {
	        this.parse_variableDeclaration();
	        for (;;) {
	            if (this.isToken(',')) {
	                this.nextToken();
	                this.parse_variableDeclaration();
	                continue;
	            }
	            break;
	        }
	    }
	    parse_variableDeclaration() {
	        if (!this.isToken('identifier')) {
	            this.causeSyntaxError('Variable name not found in variable declaration');
	        }
	        var identifier = this.getToken().value + "";
	        if (this.isNextToken('=')) {
	            var rightExpression = new TidusScript_ExpressionResult();
	            this.parse_initialiser(rightExpression);
	            this.generator.putExpressionResult(rightExpression);
	            this.generator.putSetLocalVariable(identifier, rightExpression);
	            this.generator.popAndDestroyStack();
	        }
	        else {
	            this.generator.putSetLocalVariable(identifier, TidusScript_ExpressionResult.createLiteral(undefined));
	            this.generator.popAndDestroyStack();
	        }
	    }
	    parse_initialiser(expressionResult) {
	        if (!this.isToken('=')) {
	            this.causeSyntaxError("'=' not found in variable initialization");
	        }
	        this.nextToken();
	        this.parse_assignmentExpression(expressionResult);
	    }
	    parse_emptyStatement() {
	        if (!this.isToken(';')) {
	            this.causeSyntaxError("';' not found in empty statement");
	        }
	        this.nextToken();
	    }
	    parse_expressionStatement() {
	        if (this.isToken('{') || this.isToken('function')) {
	            this.causeSyntaxError('Ambiguity found in ExpressionStatement');
	        }
	        var expressionResult = new TidusScript_ExpressionResult();
	        this.parse_expression(expressionResult);
	        this.generator.putExpressionResult(expressionResult);
	        if (!this.isToken(';')) {
	            this.causeSyntaxError("';' not found in expression statement");
	        }
	        this.nextToken();
	    }
	    parse_ifStatement() {
	        this.lastReturnCaution++;
	        if (!this.isToken('if')) {
	            this.causeSyntaxError("'if' not found in if statement");
	        }
	        if (!this.isNextToken('(')) {
	            this.causeSyntaxError("'(' not found in if statement");
	        }
	        this.nextToken();
	        var condition = new TidusScript_ExpressionResult();
	        this.parse_expression(condition);
	        if (!this.isToken(')')) {
	            this.causeSyntaxError("')' not found in if statement");
	        }
	        this.generator.putExpressionResult(condition);
	        var endIfLabel = new TidusScript_Label();
	        this.generator.putIf(condition, endIfLabel);
	        this.nextToken();
	        this.parse_statement();
	        if (this.isToken('else')) {
	            var endElseLabel = new TidusScript_Label();
	            this.generator.putJump(endElseLabel);
	            this.generator.setLabel(endIfLabel);
	            this.nextToken();
	            this.parse_statement();
	            this.generator.setLabel(endElseLabel);
	        }
	        else {
	            this.generator.setLabel(endIfLabel);
	        }
	        this.lastReturnCaution--;
	    }
	    parse_iterationStatement() {
	        this.lastReturnCaution++;
	        if (this.isToken('for')) {
	            this.parse_forStatement();
	        }
	        else if (this.isToken('while')) {
	            this.parse_whileStatement();
	        }
	        else if (this.isToken('do')) {
	            this.parse_doStatement();
	        }
	        else {
	            this.causeSyntaxError('unexpected token found in loop statement');
	        }
	        this.lastReturnCaution--;
	    }
	    parse_forStatement() {
	        if (!this.isNextToken('(')) {
	            this.causeSyntaxError("'(' not found in for statement");
	        }
	        var conditionLabel = new TidusScript_Label();
	        var continueLabel = new TidusScript_Label();
	        var startLabel = new TidusScript_Label();
	        var endLabel = new TidusScript_Label();
	        this.pushBreakLabel(endLabel);
	        this.pushContinueLabel(continueLabel);
	        var stackBegin;
	        if (this.isNextToken('var')) {
	            this.nextToken();
	            this.parse_variableDeclaration();
	        }
	        else {
	            if (!this.isToken(';')) {
	                stackBegin = this.generator.getStackLength();
	                var initializeExpression = new TidusScript_ExpressionResult();
	                this.parse_expression(initializeExpression);
	                this.generator.putExpressionResult(initializeExpression);
	                this.generator.cleanUpStack(stackBegin);
	            }
	        }
	        if (!this.isToken(';')) {
	            this.causeSyntaxError("';' not found in for statement");
	        }
	        this.generator.setLabel(conditionLabel);
	        if (!this.isNextToken(';')) {
	            stackBegin = this.generator.getStackLength();
	            var conditionExpression = new TidusScript_ExpressionResult();
	            this.parse_expression(conditionExpression);
	            this.generator.putExpressionResult(conditionExpression);
	            this.generator.putIf(conditionExpression, endLabel);
	            this.generator.cleanUpStack(stackBegin);
	        }
	        if (!this.isToken(';')) {
	            this.causeSyntaxError("';' not found in for statement");
	        }
	        this.generator.putJump(startLabel);
	        this.generator.setLabel(continueLabel);
	        if (!this.isNextToken(')')) {
	            stackBegin = this.generator.getStackLength();
	            var continueExpression = new TidusScript_ExpressionResult();
	            this.parse_expression(continueExpression);
	            this.generator.putExpressionResult(continueExpression);
	            this.generator.cleanUpStack(stackBegin);
	        }
	        this.generator.putJump(conditionLabel);
	        if (!this.isToken(')')) {
	            this.causeSyntaxError("')' not found in for statement");
	        }
	        this.nextToken();
	        this.generator.setLabel(startLabel);
	        this.parse_statement();
	        this.generator.putJump(continueLabel);
	        this.generator.setLabel(endLabel);
	        this.popContinueLabel();
	        this.popBreakLabel();
	    }
	    parse_whileStatement() {
	        if (!this.isNextToken('(')) {
	            this.causeSyntaxError("'(' not found in while statement");
	        }
	        this.nextToken();
	        var startLabel = new TidusScript_Label();
	        var endLabel = new TidusScript_Label();
	        this.pushBreakLabel(endLabel);
	        this.pushContinueLabel(startLabel);
	        this.generator.setLabel(startLabel);
	        var condition = new TidusScript_ExpressionResult();
	        this.parse_expression(condition);
	        this.generator.putExpressionResult(condition);
	        this.generator.putIf(condition, endLabel);
	        if (!this.isToken(')')) {
	            this.causeSyntaxError("')' not found in while statement");
	        }
	        this.nextToken();
	        this.parse_statement();
	        this.generator.putJump(startLabel);
	        this.generator.setLabel(endLabel);
	        this.popContinueLabel();
	        this.popBreakLabel();
	    }
	    parse_doStatement() {
	        this.nextToken();
	        var startLabel = new TidusScript_Label();
	        var continueLabel = new TidusScript_Label();
	        var endLabel = new TidusScript_Label();
	        this.pushBreakLabel(endLabel);
	        this.pushContinueLabel(continueLabel);
	        this.generator.setLabel(startLabel);
	        this.parse_statement();
	        if (!this.isToken('while')) {
	            this.causeSyntaxError("'while' not found in do statement");
	        }
	        if (!this.isNextToken('(')) {
	            this.causeSyntaxError("'(' not found in do-while statement");
	        }
	        this.nextToken();
	        this.generator.setLabel(continueLabel);
	        var condition = new TidusScript_ExpressionResult();
	        this.parse_expression(condition);
	        this.generator.putExpressionResult(condition);
	        this.generator.putIf(condition, endLabel);
	        this.generator.putJump(startLabel);
	        if (!this.isToken(')')) {
	            this.causeSyntaxError("')' not found in do-while statement");
	        }
	        this.generator.setLabel(endLabel);
	        this.popContinueLabel();
	        this.popBreakLabel();
	        this.nextToken();
	    }
	    parse_continueStatement() {
	        if (!this.isToken('continue')) {
	            this.causeSyntaxError("'continue' not found in continue statement");
	        }
	        if (!this.isNextToken(';')) {
	            this.causeSyntaxError("';' not found in continue statement");
	        }
	        this.generator.putJump(this.getContinueLabel());
	        this.nextToken();
	    }
	    parse_breakStatement() {
	        if (!this.isToken('break')) {
	            this.causeSyntaxError("'break' not found in break statement");
	        }
	        if (!this.isNextToken(';')) {
	            this.causeSyntaxError("';' not found in break statement");
	        }
	        this.generator.putJump(this.getBreakLabel());
	        this.nextToken();
	    }
	    parse_returnStatement() {
	        if (!this.isToken('return')) {
	            this.causeSyntaxError("'return' not found in return statement");
	        }
	        if (!this.isAllowReturn()) {
	            this.causeSyntaxError('return is only used in functions or coroutines');
	        }
	        if (this.lastReturnCaution == 0)
	            this.hasLastReturn = true;
	        var returnValue;
	        if (this.isExpressionFirst(this.nextToken().type)) {
	            returnValue = new TidusScript_ExpressionResult();
	            this.parse_expression(returnValue);
	            this.generator.putExpressionResult(returnValue);
	        }
	        else {
	            returnValue = TidusScript_ExpressionResult.createLiteral(undefined);
	        }
	        if (this.isInFunction()) {
	            this.generator.putReturnFunction(returnValue);
	        }
	        else {
	            this.generator.putReturnCoroutine(returnValue);
	        }
	        if (!this.isToken(';')) {
	            this.causeSyntaxError("';' not found in return statement");
	        }
	        this.nextToken();
	    }
	    parse_withStatement() {
	        if (!this.isToken('with')) {
	            this.causeSyntaxError("'with' not found in with statement");
	        }
	        if (!this.isNextToken('(')) {
	            this.causeSyntaxError("'(' not found in with statement");
	        }
	        this.nextToken();
	        var objectExpression = new TidusScript_ExpressionResult();
	        this.parse_expression(objectExpression);
	        this.generator.putExpressionResult(objectExpression);
	        this.generator.putWith(objectExpression);
	        this.generator.beginNewScope();
	        if (!this.isToken(')')) {
	            this.causeSyntaxError("')' not found in with statement");
	        }
	        this.nextToken();
	        this.parse_statement();
	        this.generator.closeScope();
	        this.generator.putEndWith();
	    }
	    parse_switchStatement() {
	        if (!this.isToken('switch')) {
	            this.causeSyntaxError("'switch' not found in switch statement");
	        }
	        if (!this.isNextToken('(')) {
	            this.causeSyntaxError("'(' not found in switch statement");
	        }
	        this.nextToken();
	        var condition = new TidusScript_ExpressionResult();
	        this.parse_expression(condition);
	        this.generator.putExpressionResult(condition);
	        if (!this.isToken(')')) {
	            this.causeSyntaxError("')' not found in switch statement");
	        }
	        var endLabel = new TidusScript_Label();
	        this.pushBreakLabel(endLabel);
	        this.nextToken();
	        this.parse_caseBlock(condition);
	        this.generator.setLabel(endLabel);
	        this.popBreakLabel();
	    }
	    parse_caseBlock(condition) {
	        if (!this.isToken('{')) {
	            this.causeSyntaxError("'{' not found in switch-case statement");
	        }
	        var caseLabel = new TidusScript_Label();
	        var bodyLabel = new TidusScript_Label();
	        if (this.isNextToken('case')) {
	            this.parse_caseClauses(condition, caseLabel, bodyLabel);
	        }
	        if (this.isToken('default')) {
	            var defaultLabel = new TidusScript_Label();
	            this.generator.setLabel(defaultLabel);
	            this.parse_defaultClause(bodyLabel);
	            if (this.isToken('case')) {
	                this.parse_caseClauses(condition, caseLabel, bodyLabel);
	            }
	            this.generator.setLabelAddress(caseLabel, defaultLabel.address);
	            this.generator.setLabel(bodyLabel);
	        }
	        else {
	            this.generator.setLabel(caseLabel);
	            this.generator.setLabel(bodyLabel);
	        }
	        if (!this.isToken('}')) {
	            this.causeSyntaxError("'}' not found in switch-case statement");
	        }
	        this.nextToken();
	    }
	    parse_caseClauses(condition, caseLabel, bodyLabel) {
	        while (this.isToken('case')) {
	            this.parse_caseClause(condition, caseLabel, bodyLabel);
	        }
	    }
	    parse_caseClause(condition, caseLabel, bodyLabel) {
	        if (!this.isToken('case')) {
	            this.causeSyntaxError("'case' not found in case statement");
	        }
	        this.generator.setLabel(caseLabel);
	        caseLabel.initialize();
	        if (!condition.isLiteral()) {
	            this.generator.putDuplicate(condition);
	        }
	        this.nextToken();
	        var caseCondition = new TidusScript_ExpressionResult();
	        this.parse_expression(caseCondition);
	        this.generator.putExpressionResult(caseCondition);
	        this.generator.putBinaryOperation((this.vmtarget == null) ? 'CSEQ' : this.vmtarget.CSEQ.bind(this.vmtarget), condition, caseCondition);
	        this.generator.putIf(TidusScript_ExpressionResult.createStack(), caseLabel);
	        if (!this.isToken(':')) {
	            this.causeSyntaxError("':' not found in case statement");
	        }
	        this.generator.setLabel(bodyLabel);
	        bodyLabel.initialize();
	        if (this.isStatementFirst(this.nextToken().type)) {
	            this.parse_statementList();
	        }
	        this.generator.putJump(bodyLabel);
	    }
	    parse_defaultClause(bodyLabel) {
	        if (!this.isToken('default')) {
	            this.causeSyntaxError("'default' not found in default statement");
	        }
	        if (!this.isNextToken(':')) {
	            this.causeSyntaxError("':' not found in default statement");
	        }
	        this.generator.setLabel(bodyLabel);
	        bodyLabel.initialize();
	        if (this.isStatementFirst(this.nextToken().type)) {
	            this.parse_statementList();
	        }
	        this.generator.putJump(bodyLabel);
	    }
	    parse_yieldStatement() {
	        if (!this.isToken('yield')) {
	            this.causeSyntaxError("'yield' not found in yield statement");
	        }
	        if (!this.isNextToken(';')) {
	            this.causeSyntaxError("';' not found in yield statement");
	        }
	        if (this.isInFunction()) {
	            this.causeSyntaxError('yield statement can only be used in a coroutine');
	        }
	        this.generator.putSuspend();
	        this.nextToken();
	    }
	    parse_suspendStatement() {
	        if (!this.isToken('suspend')) {
	            this.causeSyntaxError("'suspend' not found in suspend statement");
	        }
	        if (!this.isNextToken(';')) {
	            this.causeSyntaxError("';' not found in suspend statement");
	        }
	        if (this.isInFunction()) {
	            this.causeSyntaxError('suspend statement can only be used in a coroutine');
	        }
	        this.generator.putSuspend();
	        this.nextToken();
	    }
	    parse_loopStatement() {
	        if (!this.isToken('loop')) {
	            this.causeSyntaxError("'loop' not found in loop statement");
	        }
	        this.nextToken();
	        var startLabel = new TidusScript_Label();
	        var endLabel = new TidusScript_Label();
	        this.pushBreakLabel(endLabel);
	        this.pushContinueLabel(startLabel);
	        this.generator.setLabel(startLabel);
	        this.parse_statement();
	        this.generator.putJump(startLabel);
	        this.generator.setLabel(endLabel);
	        this.popContinueLabel();
	        this.popBreakLabel();
	    }
	    parse_expression(expressionResult) {
	        this.parse_assignmentExpression(expressionResult);
	        while (this.isToken(',')) {
	            this.nextToken();
	            this.generator.putExpressionResult(expressionResult);
	            this.generator.popAndDestroyStack();
	            expressionResult.initialize();
	            this.parse_assignmentExpression(expressionResult);
	        }
	    }
	    isExpressionFirst(type) {
	        return this.isUnaryExpressionFirst(type);
	    }
	    areBothLiteral(a, b) {
	        return (a.isType('literal') && b.isType('literal'));
	    }
	    parse_assignmentExpression(expressionResult) {
	        this.parse_conditionalExpression(expressionResult);
	        var rightExpression;
	        var objectExpression;
	        var memberExpression;
	        switch (this.getToken().type) {
	            case '=':
	                {
	                    this.nextToken();
	                    switch (expressionResult.type) {
	                        case 'member':
	                            {
	                                objectExpression = expressionResult.getObjectExpression();
	                                memberExpression = expressionResult.getMemberExpression();
	                                this.generator.putExpressionResult(memberExpression);
	                                rightExpression = new TidusScript_ExpressionResult();
	                                this.parse_assignmentExpression(rightExpression);
	                                this.generator.putExpressionResult(rightExpression);
	                                this.generator.putSetMember(objectExpression, memberExpression, rightExpression);
	                            }
	                            break;
	                        case 'variable':
	                            {
	                                rightExpression = new TidusScript_ExpressionResult();
	                                this.parse_assignmentExpression(rightExpression);
	                                this.generator.putExpressionResult(rightExpression);
	                                this.generator.putSetVariable(String(expressionResult.value), rightExpression);
	                            }
	                            break;
	                        default: {
	                            this.causeSyntaxError('L-value must be a variable or property');
	                        }
	                    }
	                    expressionResult.setTypeStack();
	                }
	                break;
	            case '*=':
	            case '/=':
	            case '%=':
	            case '+=':
	            case '-=':
	            case '<<=':
	            case '>>=':
	            case '>>>=':
	            case '&=':
	            case '^=':
	            case '|=':
	                {
	                    var operation;
	                    switch (this.getToken().type) {
	                        case '*=':
	                            operation = (this.vmtarget == null) ? 'MUL' : this.vmtarget.MUL.bind(this.vmtarget);
	                            break;
	                        case '/=':
	                            operation = (this.vmtarget == null) ? 'DIV' : this.vmtarget.DIV.bind(this.vmtarget);
	                            break;
	                        case '%=':
	                            operation = (this.vmtarget == null) ? 'MOD' : this.vmtarget.MOD.bind(this.vmtarget);
	                            break;
	                        case '+=':
	                            operation = (this.vmtarget == null) ? 'ADD' : this.vmtarget.ADD.bind(this.vmtarget);
	                            break;
	                        case '-=':
	                            operation = (this.vmtarget == null) ? 'SUB' : this.vmtarget.SUB.bind(this.vmtarget);
	                            break;
	                        case '<<=':
	                            operation = (this.vmtarget == null) ? 'LSH' : this.vmtarget.LSH.bind(this.vmtarget);
	                            break;
	                        case '>>=':
	                            operation = (this.vmtarget == null) ? 'RSH' : this.vmtarget.RSH.bind(this.vmtarget);
	                            break;
	                        case '>>>=':
	                            operation = (this.vmtarget == null) ? 'URSH' : this.vmtarget.URSH.bind(this.vmtarget);
	                            break;
	                        case '&=':
	                            operation = (this.vmtarget == null) ? 'AND' : this.vmtarget.AND.bind(this.vmtarget);
	                            break;
	                        case '^=':
	                            operation = (this.vmtarget == null) ? 'XOR' : this.vmtarget.XOR.bind(this.vmtarget);
	                            break;
	                        case '|=':
	                            operation = (this.vmtarget == null) ? 'OR' : this.vmtarget.OR.bind(this.vmtarget);
	                            break;
	                    }
	                    this.nextToken();
	                    switch (expressionResult.type) {
	                        case 'member':
	                            {
	                                objectExpression = expressionResult.getObjectExpression();
	                                memberExpression = expressionResult.getMemberExpression();
	                                if (!memberExpression.isLiteral()) {
	                                    this.generator.putExpressionResult(memberExpression);
	                                    var s = this.generator.popStack();
	                                    this.generator.putDuplicate(memberExpression);
	                                    this.generator.pushStack(s);
	                                    this.generator.putDuplicate(objectExpression);
	                                    this.generator.swapStack(1, 2);
	                                }
	                                else {
	                                    this.generator.putDuplicate(objectExpression);
	                                }
	                                this.generator.putGetMember(objectExpression, memberExpression);
	                                expressionResult.setTypeStack();
	                                rightExpression = new TidusScript_ExpressionResult();
	                                this.parse_assignmentExpression(rightExpression);
	                                this.generator.putExpressionResult(rightExpression);
	                                this.generator.putBinaryOperation(operation, expressionResult, rightExpression);
	                                this.generator.putSetMember(objectExpression, memberExpression, expressionResult);
	                            }
	                            break;
	                        case 'variable':
	                            {
	                                var identifier = String(expressionResult.value);
	                                this.generator.putGetVariable(identifier);
	                                expressionResult.setTypeStack();
	                                rightExpression = new TidusScript_ExpressionResult();
	                                this.parse_assignmentExpression(rightExpression);
	                                this.generator.putExpressionResult(rightExpression);
	                                this.generator.putBinaryOperation(operation, expressionResult, rightExpression);
	                                this.generator.putSetVariable(identifier, expressionResult);
	                            }
	                            break;
	                        default: {
	                            this.causeSyntaxError('L-value must be a variable or property');
	                        }
	                    }
	                    expressionResult.setTypeStack();
	                }
	                break;
	            default:
	                break;
	        }
	    }
	    parse_conditionalExpression(expressionResult) {
	        this.parse_logicalORExpression(expressionResult);
	        if (this.isToken('?')) {
	            this.generator.putExpressionResult(expressionResult);
	            var label = new TidusScript_Label();
	            this.generator.putIf(expressionResult, label);
	            this.nextToken();
	            expressionResult.initialize();
	            this.parse_assignmentExpression(expressionResult);
	            if (expressionResult.isType('literal')) {
	                this.generator.putLiteral(expressionResult);
	            }
	            else {
	                this.generator.putExpressionResult(expressionResult);
	            }
	            var patch = this.generator.popStack();
	            var label2 = new TidusScript_Label();
	            this.generator.putJump(label2);
	            this.generator.setLabel(label);
	            if (!this.isToken(':')) {
	                this.causeSyntaxError("':' not found in ?: statement");
	            }
	            this.nextToken();
	            expressionResult.initialize();
	            this.parse_assignmentExpression(expressionResult);
	            if (expressionResult.isType('literal')) {
	                this.generator.putLiteral(expressionResult);
	            }
	            else {
	                this.generator.putExpressionResult(expressionResult);
	            }
	            expressionResult.setType('stack');
	            this.generator.setStackPatch(patch);
	            this.generator.setLabel(label2);
	        }
	    }
	    parse_logicalORExpression(expressionResult) {
	        this.parse_logicalANDExpression(expressionResult);
	        while (this.isToken('||')) {
	            this.nextToken();
	            this.generator.putExpressionResult(expressionResult);
	            this.generator.putDuplicate(expressionResult);
	            var patch = this.generator.popStack();
	            expressionResult.setType('stack');
	            var label = new TidusScript_Label();
	            this.generator.putNif(expressionResult, label);
	            expressionResult.initialize();
	            this.parse_logicalANDExpression(expressionResult);
	            if (expressionResult.isType('literal')) {
	                this.generator.putLiteral(expressionResult);
	            }
	            else {
	                this.generator.putExpressionResult(expressionResult);
	            }
	            this.generator.setStackPatch(patch);
	            expressionResult.setType('stack');
	            this.generator.setLabel(label);
	        }
	    }
	    parse_logicalANDExpression(expressionResult) {
	        this.parse_bitwiseORExpression(expressionResult);
	        while (this.isToken('&&')) {
	            this.nextToken();
	            this.generator.putExpressionResult(expressionResult);
	            this.generator.putDuplicate(expressionResult);
	            var patch = this.generator.popStack();
	            expressionResult.setType('stack');
	            var label = new TidusScript_Label();
	            this.generator.putIf(expressionResult, label);
	            expressionResult.initialize();
	            this.parse_bitwiseORExpression(expressionResult);
	            if (expressionResult.isType('literal')) {
	                this.generator.putLiteral(expressionResult);
	            }
	            else {
	                this.generator.putExpressionResult(expressionResult);
	            }
	            this.generator.setStackPatch(patch);
	            expressionResult.setType('stack');
	            this.generator.setLabel(label);
	        }
	    }
	    parse_bitwiseORExpression(expressionResult) {
	        this.parse_bitwiseXORExpression(expressionResult);
	        while (this.isToken('|')) {
	            var operation = this.getToken().type;
	            this.nextToken();
	            this.generator.putExpressionResult(expressionResult);
	            var rightExpressionResult = new TidusScript_ExpressionResult();
	            this.parse_bitwiseXORExpression(rightExpressionResult);
	            this.generator.putExpressionResult(rightExpressionResult);
	            if (this.areBothLiteral(expressionResult, rightExpressionResult)) {
	                expressionResult.setValue(expressionResult.value | rightExpressionResult.value);
	            }
	            else {
	                this.generator.putBinaryOperation((this.vmtarget == null) ? 'OR' : this.vmtarget.OR.bind(this.vmtarget), expressionResult, rightExpressionResult);
	                expressionResult.setType('stack');
	            }
	        }
	    }
	    parse_bitwiseXORExpression(expressionResult) {
	        this.parse_bitwiseANDExpression(expressionResult);
	        while (this.isToken('^')) {
	            var operation = this.getToken().type;
	            this.nextToken();
	            this.generator.putExpressionResult(expressionResult);
	            var rightExpressionResult = new TidusScript_ExpressionResult();
	            this.parse_bitwiseANDExpression(rightExpressionResult);
	            this.generator.putExpressionResult(rightExpressionResult);
	            if (this.areBothLiteral(expressionResult, rightExpressionResult)) {
	                expressionResult.setValue(expressionResult.value ^ rightExpressionResult.value);
	            }
	            else {
	                this.generator.putBinaryOperation((this.vmtarget == null) ? 'XOR' : this.vmtarget.XOR.bind(this.vmtarget), expressionResult, rightExpressionResult);
	                expressionResult.setType('stack');
	            }
	        }
	    }
	    parse_bitwiseANDExpression(expressionResult) {
	        this.parse_equalityExpression(expressionResult);
	        while (this.isToken('&')) {
	            var operation = this.getToken().type;
	            this.nextToken();
	            this.generator.putExpressionResult(expressionResult);
	            var rightExpressionResult = new TidusScript_ExpressionResult();
	            this.parse_equalityExpression(rightExpressionResult);
	            this.generator.putExpressionResult(rightExpressionResult);
	            if (this.areBothLiteral(expressionResult, rightExpressionResult)) {
	                expressionResult.setValue(expressionResult.value & rightExpressionResult.value);
	            }
	            else {
	                this.generator.putBinaryOperation((this.vmtarget == null) ? 'AND' : this.vmtarget.AND.bind(this.vmtarget), expressionResult, rightExpressionResult);
	                expressionResult.setType('stack');
	            }
	        }
	    }
	    parse_equalityExpression(expressionResult) {
	        this.parse_relationalExpression(expressionResult);
	        while (this.isToken('==') || this.isToken('!=') || this.isToken('===') || this.isToken('!==')) {
	            var operation = this.getToken().type;
	            this.nextToken();
	            this.generator.putExpressionResult(expressionResult);
	            var rightExpressionResult = new TidusScript_ExpressionResult();
	            this.parse_relationalExpression(rightExpressionResult);
	            this.generator.putExpressionResult(rightExpressionResult);
	            if (this.areBothLiteral(expressionResult, rightExpressionResult)) {
	                switch (operation) {
	                    case '==':
	                        expressionResult.setValue(expressionResult.value == rightExpressionResult.value);
	                        break;
	                    case '!=':
	                        expressionResult.setValue(expressionResult.value != rightExpressionResult.value);
	                        break;
	                    case '===':
	                        expressionResult.setValue(expressionResult.value === rightExpressionResult.value);
	                        break;
	                    case '!==':
	                        expressionResult.setValue(expressionResult.value !== rightExpressionResult.value);
	                        break;
	                }
	            }
	            else {
	                switch (operation) {
	                    case '==':
	                        this.generator.putBinaryOperation((this.vmtarget == null) ? 'CEQ' : this.vmtarget.CEQ.bind(this.vmtarget), expressionResult, rightExpressionResult);
	                        break;
	                    case '!=':
	                        this.generator.putBinaryOperation((this.vmtarget == null) ? 'CNE' : this.vmtarget.CNE.bind(this.vmtarget), expressionResult, rightExpressionResult);
	                        break;
	                    case '===':
	                        this.generator.putBinaryOperation((this.vmtarget == null) ? 'CSEQ' : this.vmtarget.CSEQ.bind(this.vmtarget), expressionResult, rightExpressionResult);
	                        break;
	                    case '!==':
	                        this.generator.putBinaryOperation((this.vmtarget == null) ? 'CSNE' : this.vmtarget.CSNE.bind(this.vmtarget), expressionResult, rightExpressionResult);
	                        break;
	                }
	                expressionResult.setType('stack');
	            }
	        }
	    }
	    parse_relationalExpression(expressionResult) {
	        this.parse_shiftExpression(expressionResult);
	        while (this.isToken('<') || this.isToken('>') || this.isToken('<=') || this.isToken('>=') || this.isToken('instanceof')) {
	            var operation = this.getToken().type;
	            this.nextToken();
	            this.generator.putExpressionResult(expressionResult);
	            var rightExpressionResult = new TidusScript_ExpressionResult();
	            this.parse_shiftExpression(rightExpressionResult);
	            this.generator.putExpressionResult(rightExpressionResult);
	            if (this.areBothLiteral(expressionResult, rightExpressionResult)) {
	                switch (operation) {
	                    case '<':
	                        expressionResult.setValue(expressionResult.value < rightExpressionResult.value);
	                        break;
	                    case '>':
	                        expressionResult.setValue(expressionResult.value > rightExpressionResult.value);
	                        break;
	                    case '<=':
	                        expressionResult.setValue(expressionResult.value <= rightExpressionResult.value);
	                        break;
	                    case '>=':
	                        expressionResult.setValue(expressionResult.value >= rightExpressionResult.value);
	                        break;
	                    case 'instanceof':
	                        expressionResult.setValue(expressionResult.value instanceof rightExpressionResult.value);
	                        break;
	                }
	            }
	            else {
	                switch (operation) {
	                    case '<':
	                        this.generator.putBinaryOperation((this.vmtarget == null) ? 'CLT' : this.vmtarget.CLT.bind(this.vmtarget), expressionResult, rightExpressionResult);
	                        break;
	                    case '>':
	                        this.generator.putBinaryOperation((this.vmtarget == null) ? 'CGT' : this.vmtarget.CGT.bind(this.vmtarget), expressionResult, rightExpressionResult);
	                        break;
	                    case '<=':
	                        this.generator.putBinaryOperation((this.vmtarget == null) ? 'CLE' : this.vmtarget.CLE.bind(this.vmtarget), expressionResult, rightExpressionResult);
	                        break;
	                    case '>=':
	                        this.generator.putBinaryOperation((this.vmtarget == null) ? 'CGE' : this.vmtarget.CGE.bind(this.vmtarget), expressionResult, rightExpressionResult);
	                        break;
	                    case 'instanceof':
	                        this.generator.putBinaryOperation((this.vmtarget == null) ? 'INSOF' : this.vmtarget.INSOF.bind(this.vmtarget), expressionResult, rightExpressionResult);
	                        break;
	                }
	                expressionResult.setType('stack');
	            }
	        }
	    }
	    parse_shiftExpression(expressionResult) {
	        this.parse_additiveExpression(expressionResult);
	        while (this.isToken('<<') || this.isToken('>>') || this.isToken('>>>')) {
	            var operation = this.getToken().type;
	            this.nextToken();
	            this.generator.putExpressionResult(expressionResult);
	            var rightExpressionResult = new TidusScript_ExpressionResult();
	            this.parse_additiveExpression(rightExpressionResult);
	            this.generator.putExpressionResult(rightExpressionResult);
	            if (this.areBothLiteral(expressionResult, rightExpressionResult)) {
	                switch (operation) {
	                    case '<<':
	                        expressionResult.setValue(expressionResult.value << rightExpressionResult.value);
	                        break;
	                    case '>>':
	                        expressionResult.setValue(expressionResult.value >> rightExpressionResult.value);
	                        break;
	                    case '>>>':
	                        expressionResult.setValue(expressionResult.value >>> rightExpressionResult.value);
	                        break;
	                }
	            }
	            else {
	                switch (operation) {
	                    case '<<':
	                        this.generator.putBinaryOperation((this.vmtarget == null) ? 'LSH' : this.vmtarget.LSH.bind(this.vmtarget), expressionResult, rightExpressionResult);
	                        break;
	                    case '>>':
	                        this.generator.putBinaryOperation((this.vmtarget == null) ? 'RSH' : this.vmtarget.RSH.bind(this.vmtarget), expressionResult, rightExpressionResult);
	                        break;
	                    case '>>>':
	                        this.generator.putBinaryOperation((this.vmtarget == null) ? 'URSH' : this.vmtarget.URSH.bind(this.vmtarget), expressionResult, rightExpressionResult);
	                        break;
	                }
	                expressionResult.setType('stack');
	            }
	        }
	    }
	    parse_additiveExpression(expressionResult) {
	        this.parse_multiplicativeExpression(expressionResult);
	        while (this.isToken('+') || this.isToken('-')) {
	            var operation = this.getToken().type;
	            this.nextToken();
	            this.generator.putExpressionResult(expressionResult);
	            var rightExpressionResult = new TidusScript_ExpressionResult();
	            this.parse_multiplicativeExpression(rightExpressionResult);
	            this.generator.putExpressionResult(rightExpressionResult);
	            if (this.areBothLiteral(expressionResult, rightExpressionResult)) {
	                switch (operation) {
	                    case '+':
	                        expressionResult.setValue(expressionResult.value + rightExpressionResult.value);
	                        break;
	                    case '-':
	                        expressionResult.setValue(expressionResult.value - rightExpressionResult.value);
	                        break;
	                }
	            }
	            else {
	                switch (operation) {
	                    case '+':
	                        this.generator.putBinaryOperation((this.vmtarget == null) ? 'ADD' : this.vmtarget.ADD.bind(this.vmtarget), expressionResult, rightExpressionResult);
	                        break;
	                    case '-':
	                        this.generator.putBinaryOperation((this.vmtarget == null) ? 'SUB' : this.vmtarget.SUB.bind(this.vmtarget), expressionResult, rightExpressionResult);
	                        break;
	                }
	                expressionResult.setType('stack');
	            }
	        }
	    }
	    parse_multiplicativeExpression(expressionResult) {
	        this.parse_unaryExpression(expressionResult);
	        while (this.isToken('*') || this.isToken('/') || this.isToken('%')) {
	            var operation = this.getToken().type;
	            this.nextToken();
	            this.generator.putExpressionResult(expressionResult);
	            var rightExpressionResult = new TidusScript_ExpressionResult();
	            this.parse_unaryExpression(rightExpressionResult);
	            this.generator.putExpressionResult(rightExpressionResult);
	            if (this.areBothLiteral(expressionResult, rightExpressionResult)) {
	                switch (operation) {
	                    case '*':
	                        expressionResult.setValue(expressionResult.value * rightExpressionResult.value);
	                        break;
	                    case '/':
	                        expressionResult.setValue(expressionResult.value / rightExpressionResult.value);
	                        break;
	                    case '%':
	                        expressionResult.setValue(expressionResult.value % rightExpressionResult.value);
	                        break;
	                }
	            }
	            else {
	                switch (operation) {
	                    case '*':
	                        this.generator.putBinaryOperation((this.vmtarget == null) ? 'MUL' : this.vmtarget.MUL.bind(this.vmtarget), expressionResult, rightExpressionResult);
	                        break;
	                    case '/':
	                        this.generator.putBinaryOperation((this.vmtarget == null) ? 'DIV' : this.vmtarget.DIV.bind(this.vmtarget), expressionResult, rightExpressionResult);
	                        break;
	                    case '%':
	                        this.generator.putBinaryOperation((this.vmtarget == null) ? 'MOD' : this.vmtarget.MOD.bind(this.vmtarget), expressionResult, rightExpressionResult);
	                        break;
	                }
	                expressionResult.setType('stack');
	            }
	        }
	    }
	    parse_unaryExpression(expressionResult) {
	        switch (this.getToken().type) {
	            case 'delete':
	                {
	                    this.nextToken();
	                    this.parse_unaryExpression(expressionResult);
	                    switch (expressionResult.type) {
	                        case 'member':
	                            {
	                                var objectExpression = expressionResult.getObjectExpression();
	                                var memberExpression = expressionResult.getMemberExpression();
	                                this.generator.putExpressionResult(memberExpression);
	                                this.generator.putDeleteMember(objectExpression, memberExpression);
	                            }
	                            break;
	                        case 'variable':
	                            {
	                                this.generator.putDelete(expressionResult);
	                            }
	                            break;
	                        default:
	                            {
	                                this.generator.putExpressionResult(expressionResult);
	                                this.generator.putDelete(expressionResult);
	                            }
	                            break;
	                    }
	                    expressionResult.setTypeStack();
	                }
	                break;
	            case 'void':
	                {
	                    this.nextToken();
	                    this.parse_unaryExpression(expressionResult);
	                }
	                break;
	            case 'typeof':
	                {
	                    this.nextToken();
	                    this.parse_unaryExpression(expressionResult);
	                    this.generator.putExpressionResult(expressionResult);
	                    if (expressionResult.isType('literal')) {
	                        expressionResult.setValue(typeof expressionResult.value);
	                    }
	                    else {
	                        this.generator.putUnaryOperation((this.vmtarget == null) ? 'TYPEOF' : this.vmtarget.TYPEOF.bind(this.vmtarget), expressionResult);
	                        expressionResult.setTypeStack();
	                    }
	                }
	                break;
	            case '++':
	                {
	                    this.nextToken();
	                    this.parse_unaryExpression(expressionResult);
	                    this.generator.putIncrement(expressionResult);
	                    expressionResult.setTypeStack();
	                }
	                break;
	            case '--':
	                {
	                    this.nextToken();
	                    this.parse_unaryExpression(expressionResult);
	                    this.generator.putDecrement(expressionResult);
	                    expressionResult.setTypeStack();
	                }
	                break;
	            case '+':
	                {
	                    this.nextToken();
	                    this.parse_unaryExpression(expressionResult);
	                }
	                break;
	            case '-':
	                {
	                    this.nextToken();
	                    this.parse_unaryExpression(expressionResult);
	                    this.generator.putExpressionResult(expressionResult);
	                    if (expressionResult.isType('literal')) {
	                        expressionResult.setValue(-expressionResult.value);
	                    }
	                    else {
	                        var zeroLiteral = new TidusScript_ExpressionResult();
	                        zeroLiteral.setTypeAndValue('literal', 0);
	                        this.generator.putBinaryOperation((this.vmtarget == null) ? 'SUB' : this.vmtarget.SUB.bind(this.vmtarget), zeroLiteral, expressionResult);
	                        expressionResult.setType('stack');
	                    }
	                }
	                break;
	            case '~':
	                {
	                    this.nextToken();
	                    this.parse_unaryExpression(expressionResult);
	                    this.generator.putExpressionResult(expressionResult);
	                    if (expressionResult.isType('literal')) {
	                        expressionResult.setValue(~expressionResult.value);
	                    }
	                    else {
	                        this.generator.putUnaryOperation((this.vmtarget == null) ? 'NOT' : this.vmtarget.NOT.bind(this.vmtarget), expressionResult);
	                        expressionResult.setType('stack');
	                    }
	                }
	                break;
	            case '!':
	                {
	                    this.nextToken();
	                    this.parse_unaryExpression(expressionResult);
	                    this.generator.putExpressionResult(expressionResult);
	                    if (expressionResult.isType('literal')) {
	                        expressionResult.setValue(!expressionResult.value);
	                    }
	                    else {
	                        this.generator.putUnaryOperation((this.vmtarget == null) ? 'LNOT' : this.vmtarget.LNOT.bind(this.vmtarget), expressionResult);
	                        expressionResult.setType('stack');
	                    }
	                }
	                break;
	            default:
	                {
	                    this.parse_postfixExpression(expressionResult);
	                }
	                break;
	        }
	    }
	    isUnaryExpressionFirst(type) {
	        return (type == 'delete' || type == 'void' || type == 'typeof' ||
	            type == '++' || type == '--' || type == '+' || type == '-' ||
	            type == '~' || type == '!' ||
	            this.isMemberExpressionFirst(type));
	    }
	    parse_postfixExpression(expressionResult) {
	        this.parse_leftHandSideExpression(expressionResult);
	        if (this.isToken('++') || this.isToken('--')) {
	            switch (this.getToken().type) {
	                case '++':
	                    this.generator.putPostfixIncrement(expressionResult);
	                    break;
	                case '--':
	                    this.generator.putPostfixDecrement(expressionResult);
	                    break;
	            }
	            expressionResult.setTypeStack();
	            this.nextToken();
	        }
	    }
	    parse_leftHandSideExpression(expressionResult) {
	        this.parse_callExpression(expressionResult);
	    }
	    parse_callExpression(expressionResult) {
	        this.parse_memberExpression(expressionResult);
	        for (;;) {
	            if (this.isToken('(')) {
	                var numOfArguments = this.parse_arguments();
	                switch (expressionResult.type) {
	                    case 'member':
	                        {
	                            var objectExpression = expressionResult.getObjectExpression();
	                            var memberExpression = expressionResult.getMemberExpression();
	                            this.generator.putExpressionResult(memberExpression);
	                            this.generator.putCallMember(objectExpression, memberExpression, numOfArguments);
	                        }
	                        break;
	                    case 'stack':
	                        {
	                            this.generator.putCallFunctor(numOfArguments);
	                        }
	                        break;
	                    default: {
	                        this.generator.putCall(expressionResult, numOfArguments);
	                    }
	                }
	                expressionResult.setType('stack');
	                continue;
	            }
	            break;
	        }
	    }
	    parse_memberExpression(expressionResult) {
	        switch (this.getToken().type) {
	            case 'function':
	                {
	                    if (this.parseForceCoroutine == false)
	                        this.parse_functionExpression(expressionResult);
	                    else {
	                        this.token.type = 'coroutine';
	                        this.parse_coroutineExpression(expressionResult);
	                    }
	                }
	                break;
	            case 'coroutine':
	                {
	                    this.parse_coroutineExpression(expressionResult);
	                }
	                break;
	            case 'new':
	                {
	                    this.nextToken();
	                    this.parse_memberExpression(expressionResult);
	                    this.generator.putExpressionResult(expressionResult);
	                    var numOfArguments = 0;
	                    if (this.isToken('(')) {
	                        numOfArguments += this.parse_arguments();
	                    }
	                    this.generator.putNew(numOfArguments);
	                    expressionResult.setType('stack');
	                }
	                break;
	            default: {
	                this.parse_primaryExpression(expressionResult);
	            }
	        }
	        for (;;) {
	            if (this.isToken('[')) {
	                this.nextToken();
	                this.generator.putExpressionResult(expressionResult);
	                var memberExpression = new TidusScript_ExpressionResult();
	                this.parse_expression(memberExpression);
	                expressionResult.setTypeMember(expressionResult.clone(), memberExpression);
	                if (!this.isToken(']')) {
	                    this.causeSyntaxError("']' not found in array access");
	                }
	                this.nextToken();
	                continue;
	            }
	            if (this.isToken('.')) {
	                this.generator.putExpressionResult(expressionResult);
	                if (!this.isNextToken('identifier')) {
	                    this.causeSyntaxError("'.' not found in property access");
	                }
	                expressionResult.setTypeMember(expressionResult.clone(), TidusScript_ExpressionResult.createLiteral(this.getToken().value));
	                this.nextToken();
	                continue;
	            }
	            break;
	        }
	    }
	    isMemberExpressionFirst(type) {
	        return (type == 'new' || type == 'function' || this.isPrimaryExpressionFirst(type));
	    }
	    parse_arguments() {
	        if (!this.isToken('(')) {
	            this.causeSyntaxError("'(' not found in argument list");
	        }
	        var numOfArguments = 0;
	        if (!this.isNextToken(')')) {
	            numOfArguments += this.parse_argumentList();
	        }
	        if (!this.isToken(')')) {
	            this.causeSyntaxError("')' not found in argument list");
	        }
	        this.nextToken();
	        return numOfArguments;
	    }
	    parse_argumentList() {
	        var numOfArguments = 0;
	        for (;;) {
	            var expressionResult = new TidusScript_ExpressionResult();
	            this.parse_assignmentExpression(expressionResult);
	            this.generator.putExpressionResult(expressionResult);
	            this.generator.putPush(expressionResult);
	            numOfArguments++;
	            if (this.isToken(',')) {
	                this.nextToken();
	                continue;
	            }
	            break;
	        }
	        return numOfArguments;
	    }
	    parse_primaryExpression(expressionResult) {
	        switch (this.getToken().type) {
	            case 'this':
	                {
	                    this.generator.putThis();
	                    expressionResult.setType('stack');
	                    this.nextToken();
	                }
	                break;
	            case 'identifier':
	                {
	                    expressionResult.setTypeAndValue('variable', this.getToken().value);
	                    this.nextToken();
	                }
	                break;
	            case 'string':
	            case 'number':
	            case 'bool':
	            case 'null':
	            case 'undefined':
	                {
	                    expressionResult.setTypeAndValue('literal', this.getToken().value);
	                    this.nextToken();
	                }
	                break;
	            case '[':
	                {
	                    var numOfElements = this.parse_arrayLiteral();
	                    this.generator.putArrayLiteral(numOfElements);
	                    expressionResult.setType('stack');
	                }
	                break;
	            case '{':
	                {
	                    var numOfProperties = this.parse_objectLiteral();
	                    this.generator.putObjectLiteral(numOfProperties);
	                    expressionResult.setType('stack');
	                }
	                break;
	            case '(':
	                {
	                    this.nextToken();
	                    this.parse_expression(expressionResult);
	                    if (!this.isToken(')')) {
	                        this.causeSyntaxError("matching ')' not found in expression");
	                    }
	                    this.nextToken();
	                }
	                break;
	            default:
	                {
	                    this.causeSyntaxError('unexpected token');
	                }
	                break;
	        }
	    }
	    isPrimaryExpressionFirst(type) {
	        return (type == 'this' || type == 'identifier' ||
	            type == 'string' || type == 'number' || type == 'bool' || type == 'undefined' ||
	            type == 'null' || type == '{' || type == '[' || type == '(');
	    }
	    parse_arrayLiteral() {
	        if (!this.isToken('[')) {
	            this.causeSyntaxError("'[' not found in array initializer");
	        }
	        var numOfElements = 0;
	        if (!this.isNextToken(']')) {
	            if (this.isToken(',')) {
	                numOfElements += this.parse_elision();
	            }
	            if (!this.isToken(']')) {
	                numOfElements += this.parse_elementList();
	            }
	            if (this.isToken(',')) {
	                numOfElements += this.parse_elision();
	            }
	        }
	        if (!this.isToken(']')) {
	            this.causeSyntaxError("']' not found in array initializer");
	        }
	        this.nextToken();
	        return numOfElements;
	    }
	    parse_elision() {
	        if (!this.isToken(',')) {
	            this.causeSyntaxError("',' not found in elision");
	        }
	        var numOfElements = 1;
	        var undefinedLiteral = TidusScript_ExpressionResult.createLiteral(undefined);
	        for (;;) {
	            this.generator.putPush(undefinedLiteral);
	            numOfElements++;
	            if (this.isNextToken(',')) {
	                continue;
	            }
	            break;
	        }
	        if (this.isToken(']')) {
	            this.generator.putPush(undefinedLiteral);
	            numOfElements++;
	        }
	        return numOfElements;
	    }
	    parse_elementList() {
	        var numOfElements = 0;
	        for (;;) {
	            if (this.isToken(',')) {
	                numOfElements += this.parse_elision();
	                continue;
	            }
	            if (this.isToken(']')) {
	                break;
	            }
	            var expressionResult = new TidusScript_ExpressionResult();
	            this.parse_assignmentExpression(expressionResult);
	            this.generator.putExpressionResult(expressionResult);
	            this.generator.putPush(expressionResult);
	            numOfElements++;
	            if (this.isToken(',')) {
	                this.nextToken();
	                continue;
	            }
	            break;
	        }
	        return numOfElements;
	    }
	    parse_objectLiteral() {
	        if (!this.isToken('{')) {
	            this.causeSyntaxError("'{' not found in object initializer");
	        }
	        var numOfProperties = 0;
	        if (!this.isNextToken('}')) {
	            numOfProperties += this.parse_propertyNameAndValueList();
	        }
	        if (!this.isToken('}')) {
	            this.causeSyntaxError("'}' not found in object initializer");
	        }
	        this.nextToken();
	        return numOfProperties;
	    }
	    parse_propertyNameAndValueList() {
	        var numOfProperties = 0;
	        for (;;) {
	            this.parse_propertyName();
	            if (!this.isToken(':')) {
	                this.causeSyntaxError("':' not found in object name-value initializer");
	            }
	            this.nextToken();
	            var expressionResult = new TidusScript_ExpressionResult();
	            this.parse_assignmentExpression(expressionResult);
	            this.generator.putExpressionResult(expressionResult);
	            this.generator.putPush(expressionResult);
	            numOfProperties++;
	            if (this.isToken(',')) {
	                this.nextToken();
	                continue;
	            }
	            break;
	        }
	        return numOfProperties;
	    }
	    parse_propertyName() {
	        switch (this.getToken().type) {
	            case 'identifier':
	            case 'string':
	            case 'number':
	                {
	                    var propertyName = TidusScript_ExpressionResult.createLiteral(this.getToken().value);
	                    this.generator.putPush(propertyName);
	                    this.nextToken();
	                }
	                break;
	            default:
	                {
	                    this.causeSyntaxError('unexpected token in property name');
	                }
	                break;
	        }
	    }
	}
	class TidusScripat_Token {
	    constructor(type, value) {
	        this.type = type;
	        this.value = value;
	    }
	}

	class TidusScript_Scanner {
	    constructor(source) {
	        this.source = source;
	        this.rewind();
	    }
	    rewind() {
	        this.index = 0;
	        this.linesCount = 0;
	    }
	    getLineNumber() {
	        return this.linesCount + 1;
	    }
	    getLine() {
	        return (this.source.split("\n"))[this.linesCount];
	    }
	    getChar() {
	        return this.source.charAt(this.index);
	    }
	    nextChar() {
	        if (this.getChar() == "\n") {
	            this.linesCount++;
	        }
	        return this.source.charAt(++this.index);
	    }
	    isSpace(c) {
	        return (c == " " || c == "\t" || c == "\r" || c == "\n");
	    }
	    isAlphabet(c) {
	        var code = c.charCodeAt(0);
	        return ((65 <= code && code <= 90) || (97 <= code && code <= 122));
	    }
	    isNumber(c) {
	        var code = c.charCodeAt(0);
	        return (48 <= code && code <= 57);
	    }
	    isAlphabetOrNumber(c) {
	        var code = c.charCodeAt(0);
	        return ((48 <= code && code <= 57) || (65 <= code && code <= 90) || (97 <= code && code <= 122));
	    }
	    isHex(c) {
	        var code = c.charCodeAt(0);
	        return ((48 <= code && code <= 57) || (65 <= code && code <= 70) || (97 <= code && code <= 102));
	    }
	    isIdentifier(c) {
	        var code = c.charCodeAt(0);
	        return (code == 36 || code == 95 || (48 <= code && code <= 57) || (65 <= code && code <= 90) || (97 <= code && code <= 122));
	    }
	    getToken() {
	        var c = this.getChar();
	        var value;
	        var type;
	        while (this.isSpace(c)) {
	            c = this.nextChar();
	        }
	        if (!c)
	            return null;
	        if (this.isAlphabet(c) || c == '$' || c == '_') {
	            value = c;
	            while ((c = this.nextChar()) && this.isIdentifier(c)) {
	                value += c;
	            }
	            type = value.toLowerCase();
	            switch (type) {
	                case 'break':
	                case 'case':
	                case 'continue':
	                case 'default':
	                case 'delete':
	                case 'do':
	                case 'else':
	                case 'for':
	                case 'function':
	                case 'if':
	                case 'instanceof':
	                case 'new':
	                case 'return':
	                case 'switch':
	                case 'this':
	                case 'typeof':
	                case 'var':
	                case 'while':
	                case 'with':
	                case 'coroutine':
	                case 'suspend':
	                case 'yield':
	                case 'loop':
	                    {
	                        return new TidusScripat_Token(type, null);
	                    }
	                case 'null': return new TidusScripat_Token('null', null);
	                case 'undefined': return new TidusScripat_Token('undefined', undefined);
	                case 'true': return new TidusScripat_Token('bool', true);
	                case 'false': return new TidusScripat_Token('bool', false);
	                default:
	                    {
	                        return new TidusScripat_Token('identifier', value);
	                    }
	            }
	        }
	        if (this.isNumber(c)) {
	            value = c;
	            if (c == '0') {
	                if ((c = this.nextChar()) && c == 'x' || c == 'X') {
	                    value += c;
	                    while ((c = this.nextChar()) && this.isHex(c)) {
	                        value += c;
	                    }
	                }
	                else if (this.isNumber(c)) {
	                    value += c;
	                    while ((c = this.nextChar()) && this.isNumber(c)) {
	                        value += c;
	                    }
	                }
	            }
	            else {
	                while ((c = this.nextChar()) && this.isNumber(c)) {
	                    value += c;
	                }
	            }
	            if (c == '.') {
	                value += c;
	                while ((c = this.nextChar()) && this.isNumber(c)) {
	                    value += c;
	                }
	                return new TidusScripat_Token('number', parseFloat(value));
	            }
	            else {
	                return new TidusScripat_Token('number', parseInt(value));
	            }
	        }
	        if (c == "'") {
	            value = '';
	            while ((c = this.nextChar()) && c != "'") {
	                if (c == '\\') {
	                    c = this.nextChar();
	                    if (c == 'n') {
	                        value += "\n";
	                        c = this.nextChar();
	                        continue;
	                    }
	                    if (c == '\\') {
	                        value += "\\";
	                        c = this.nextChar();
	                        continue;
	                    }
	                }
	                value += c;
	            }
	            if (c != "'") {
	                throw new TidusScript_VMSyntaxError('String literal is not closed.');
	            }
	            this.nextChar();
	            return new TidusScripat_Token('string', value);
	        }
	        if (c == '"') {
	            value = "";
	            while ((c = this.nextChar()) && c != '"') {
	                if (c == '\\') {
	                    c = this.nextChar();
	                    if (c == 'n') {
	                        value += "\n";
	                        c = this.nextChar();
	                        continue;
	                    }
	                    if (c == '\\') {
	                        value += "\\";
	                        c = this.nextChar();
	                        continue;
	                    }
	                }
	                value += c;
	            }
	            if (c != '"') {
	                throw new TidusScript_VMSyntaxError('String literal is not closed.');
	            }
	            this.nextChar();
	            return new TidusScripat_Token('string', value);
	        }
	        if (c == '/') {
	            if ((c = this.nextChar())) {
	                if (c == '=') {
	                    this.nextChar();
	                    return new TidusScripat_Token('/=', null);
	                }
	                else if (c == '/') {
	                    while ((c = this.nextChar()) && c != "\n") {
	                    }
	                    this.nextChar();
	                    return this.getToken();
	                }
	                else if (c == '*') {
	                    for (c = this.nextChar(); c;) {
	                        if (c == '*') {
	                            if ((c = this.nextChar()) && c == '/') {
	                                break;
	                            }
	                            continue;
	                        }
	                        c = this.nextChar();
	                    }
	                    this.nextChar();
	                    return this.getToken();
	                }
	            }
	            return new TidusScripat_Token('/', null);
	        }
	        if (c == '*' || c == '%' || c == '^') {
	            type = c;
	            if ((c = this.nextChar()) && c == '=') {
	                this.nextChar();
	                return new TidusScripat_Token(type + '=', null);
	            }
	            return new TidusScripat_Token(type, null);
	        }
	        if (c == '+' || c == '-' || c == '|' || c == '&') {
	            type = c;
	            if ((c = this.nextChar())) {
	                if (c == type) {
	                    this.nextChar();
	                    return new TidusScripat_Token(type + type, null);
	                }
	                if (c == '=') {
	                    this.nextChar();
	                    return new TidusScripat_Token(type + '=', null);
	                }
	            }
	            return new TidusScripat_Token(type, null);
	        }
	        if (c == '=' || c == '!') {
	            type = c;
	            if ((c = this.nextChar()) && c == '=') {
	                if ((c = this.nextChar()) && c == '=') {
	                    this.nextChar();
	                    return new TidusScripat_Token(type + '==', null);
	                }
	                return new TidusScripat_Token(type + '=', null);
	            }
	            return new TidusScripat_Token(type, null);
	        }
	        if (c == '>' || c == '<') {
	            type = c;
	            if ((c = this.nextChar())) {
	                if (c == '=') {
	                    this.nextChar();
	                    return new TidusScripat_Token(type + '=', null);
	                }
	                if (c == type) {
	                    if ((c = this.nextChar())) {
	                        if (type == '>' && c == '>') {
	                            if ((c = this.nextChar()) && c == '=') {
	                                this.nextChar();
	                                return new TidusScripat_Token('>>>=', null);
	                            }
	                            return new TidusScripat_Token('>>>', null);
	                        }
	                        if (c == '=') {
	                            this.nextChar();
	                            return new TidusScripat_Token(type + type + '=', null);
	                        }
	                    }
	                    return new TidusScripat_Token(type + type, null);
	                }
	            }
	            return new TidusScripat_Token(type, null);
	        }
	        switch (c) {
	            case '{':
	            case '}':
	            case '(':
	            case ')':
	            case '[':
	            case ']':
	            case '.':
	            case ';':
	            case ',':
	            case '~':
	            case '?':
	            case ':':
	                {
	                    this.nextChar();
	                    return new TidusScripat_Token(c, null);
	                }
	            default:
	                {
	                    throw new TidusScript_VMSyntaxError('Unknown character : "' + c + '" at index ' + this.index + '.');
	                }
	        }
	        return null;
	    }
	}

	class Tool_StringBuild {
	    static replaceSTR(str, str1, str2) {
	        var out = "";
	        var arr = str.split(str1);
	        for (var i = 0; i < arr.length; i++) {
	            if (i > 0)
	                out += str2;
	            out += arr[i];
	        }
	        return out;
	    }
	    static onRemoveEnterSpace(str) {
	        return str.trim();
	    }
	    static trim(str) {
	        return str.trim();
	    }
	    static rebuild_by_length(str, len, _chinese) {
	        var out = "";
	        if (_chinese == false) {
	            out = str.substr(0, len);
	        }
	        else {
	            var nowL = 0;
	            var count = 0;
	            while (count < str.length) {
	                if (nowL >= len)
	                    break;
	                var char = str.charAt(count);
	                var code = str.charCodeAt(count);
	                out += char;
	                if (code >= 10000)
	                    nowL += 2;
	                else
	                    nowL += 1;
	                count++;
	            }
	        }
	        return out;
	    }
	    static getNewSTRBySlip(str1, str2) {
	        var arr = str1.split(str2);
	        str1 = "";
	        while (arr.length > 0) {
	            str1 += arr[0];
	            arr.shift();
	        }
	        return str1;
	    }
	    static getArr_By_split(str1, str2, _changeInstance = null) {
	        var arr = str1.split(str2);
	        if (_changeInstance) {
	            var i;
	            var l = arr.length;
	            for (i = 0; i < l; i++) {
	                if (_changeInstance == "int")
	                    arr[i] = Tool_Function.onChangeInstance(arr[i]);
	                else if (_changeInstance == "Number")
	                    arr[i] = Tool_Function.onChangeInstance(arr[i], "num");
	            }
	        }
	        return arr;
	    }
	    static onInsertF(str, index, strTar) {
	        var str0 = str.slice(0, index);
	        var str1 = str.slice(index);
	        return str0 + strTar + str1;
	    }
	    static insertInto(str, newStr, index) {
	        var str0 = str.slice(0, index);
	        var str1 = str.slice(index);
	        return str0 + newStr + str1;
	    }
	    static removeStrFromStr(str, index, length, insertStr) {
	        let str0 = str.substr(0, index);
	        let str1 = str.substr(index + length);
	        if (insertStr)
	            return str0 + insertStr + str1;
	        return str0 + str1;
	    }
	    static fixNumberTOString(val, num, fillStr, isFront = true) {
	        let str = val + "";
	        while (str.length < num) {
	            if (isFront == true)
	                str = fillStr + str;
	            else
	                str += fillStr;
	        }
	        return str;
	    }
	    static getLength(str) {
	        var out = str.length;
	        var count = 0;
	        while (count < str.length) {
	            var code = str.charCodeAt(count);
	            if (code >= 10000)
	                out++;
	            count++;
	        }
	        return out;
	    }
	    static getStringBetweenTwoPlace(str, start, end, stopChar = null) {
	        var index0;
	        var index1;
	        if (Tool_Function.isTypeOf(start, "int"))
	            index0 = start;
	        else
	            index0 = str.indexOf(start) + start.length;
	        if (Tool_Function.isTypeOf(end, "int"))
	            index1 = end;
	        else
	            index1 = str.indexOf(end, index0 + 1);
	        if (index1 == -1 || index0 == -1)
	            return null;
	        var out = str.substring(index0, index1);
	        if (stopChar) {
	            for (var i = 0; i < stopChar.length; i++) {
	                if (out.indexOf(stopChar[i]) != -1)
	                    return null;
	            }
	        }
	        out = Tool_StringBuild.onRemoveEnterSpace(out);
	        return out;
	    }
	    static minIndexBefore(str, chars, start = 99999999) {
	        let max = -1;
	        for (let i = 0; i < chars.length; i++) {
	            let j = str.lastIndexOf(chars[i], start);
	            if (j != -1 && j > max)
	                max = j;
	        }
	        return max;
	    }
	    static minIndexBehind(str, chars, start = -1) {
	        let min = 9999999;
	        for (let i = 0; i < chars.length; i++) {
	            let j = str.indexOf(chars[i], start);
	            if (j != -1 && j < min)
	                min = j;
	        }
	        if (min == 9999999)
	            min = -1;
	        return min;
	    }
	}

	class TidusScriptOne {
	    constructor() {
	        this.vm = new TidusScript_VM();
	        this.vm.getGlobalObject().Math = Math;
	        this.vm.getGlobalObject().trace = console.log;
	        this.vm.getGlobalObject().console = console;
	        this.vm.getGlobalObject().JSON = JSON;
	    }
	    addToGlobalObject(key, val) {
	        this.vm.getGlobalObject()[key] = val;
	    }
	    doScriptOnce(codestring, context = null) {
	        if (context) {
	            for (var key in context) {
	                this.vm.getGlobalObject()[key] = context[key];
	            }
	        }
	        this.vm.getGlobalObject()["JSON"] = JSON;
	        this.vm.getGlobalObject()["trace"] = console.log;
	        this.vm.getGlobalObject()["console"] = console;
	        this.vm.getGlobalObject()["Tool_Object"] = ConfigTidus.GVal["Tool_Object"];
	        this.vm.getGlobalObject()["Tool_Function"] = Tool_Function;
	        this.vm.getGlobalObject()["_return_result"] = null;
	        if (codestring == null) {
	            let re;
	            if (this.vm.getGlobalObject()["a"] != null) {
	                re = this.runFunction("a", {});
	            }
	            else {
	                console.error("不应该存在没有a方法的doScriptOnce!!!!!");
	            }
	            if (context) {
	                for (var key in context) {
	                    this.vm.getGlobalObject()[key] = null;
	                }
	            }
	            return re;
	        }
	        if (codestring.indexOf("function ") == 0) {
	            console.error("ERROR！doScriptOnce不应该传入function");
	            return false;
	        }
	        if (codestring.includes("return ") == false && codestring.includes(";") == false)
	            codestring = "return " + codestring;
	        if (codestring.charAt(codestring.length - 1) != "}" && codestring.charAt(codestring.length - 1) != ";") {
	            codestring += ";";
	        }
	        codestring = Tool_StringBuild.replaceSTR(codestring, "let ", "var ");
	        let code = "function a(){" + codestring + "}";
	        this.addFunction(TidusScriptOne.FunRebuidStr(code));
	        return this.doScriptOnce(null, context);
	    }
	    addFunction(code) {
	        if (code.indexOf("function ") != 0) {
	            console.error("ERROR！addFunction应该传入function");
	            return false;
	        }
	        var s = new TidusScript_Scanner(code);
	        var p = new TidusScript_Parser(s);
	        try {
	            this.vm.setByteCode(p.parse(this.vm));
	        }
	        catch (vme) {
	            console.error(vme.message);
	            return false;
	        }
	        this.vm.execute();
	        return true;
	    }
	    runFunction(funName, context, ...arg) {
	        var global = this.vm.getGlobalObject();
	        var fun = global[funName];
	        if (fun == null) {
	            console.info("没有找到Function：" + funName);
	            return null;
	        }
	        if (context) {
	            for (var key in context) {
	                global[key] = context[key];
	            }
	        }
	        var re;
	        re = fun.apply(global, arg);
	        if (context) {
	            for (var key in context) {
	                global[key] = null;
	            }
	        }
	        return re;
	    }
	}

	class GT_GameObject {
	    constructor(info) {
	        this.info = info;
	        this.type = info["type"];
	        if (info["bid"])
	            this.baseid = info["bid"];
	        else if (info["baseid"])
	            this.baseid = info["baseid"];
	        this.num = info["num"] != null ? info["num"] : 1;
	    }
	    getIcon() {
	        if (GT_GameObject.FunGetIcon) {
	            return GT_GameObject.FunGetIcon(this);
	        }
	        return null;
	    }
	}

	class GT_Hurt {
	    static get option() {
	        if (this._option == null)
	            this._option = new gthurt_option();
	        return this._option;
	    }
	    static onRandomPercentEffect(from, tar, per, option) {
	        var p = Math.random() * 100;
	        if (option && option["属性影响"] && option["属性影响"] != "0") {
	            let v1 = from.getValue(option["属性影响"]);
	            let v2 = tar.getValue(option["属性影响"]);
	            let per2 = Tool_Function.getPerValueInArray(v1 / v2, [1, 0, 2, 0.3, 3, 0.5]);
	            p -= p * per2;
	        }
	        if (GT_Hurt.option.randomOdder == 0) {
	            if (p < per)
	                return true;
	        }
	        else if (GT_Hurt.option.randomOdder == 1) {
	            let v1 = from.getValue("幸运");
	            let v2 = tar.getValue("幸运");
	            per = Tool_Function.getPerValueInArray(v1 / v2, [0.5, 0.1, 1, 0.3, 2, 0.8, 3, 0.95]);
	            if (p < per)
	                return true;
	        }
	        else if (GT_Hurt.option.randomOdder == 2) {
	            let v1 = from.getValue("幸运");
	            let v2 = tar.getValue("幸运");
	            let per2 = Tool_Function.getPerValueInArray(v1 / v2, [1, 0, 2, 0.3, 3, 0.5]);
	            p -= p * per2;
	            if (p < per)
	                return true;
	        }
	        return false;
	    }
	    static onCritcal(attacker, tar, s) {
	        let cri = attacker.getValue(GT_Role.VName_Cri);
	        if (s && attacker.camp + tar.camp && s.customData && s.customData["暴击率"] != "正常") {
	            if (s.customData["暴击率"] == "容易")
	                cri += cri * 0.2;
	            else if (s.customData["暴击率"] == "极易")
	                cri += cri * 0.4;
	            else if (s.customData["暴击率"] == "困难")
	                cri -= cri * 0.2;
	            else if (s.customData["暴击率"] == "极难")
	                cri -= cri * 0.4;
	        }
	        let anti = tar.getValue(GT_Role.VName_AntiCri);
	        let per = cri - anti;
	        if (Math.random() * 100 < per)
	            return true;
	        return false;
	    }
	    static onMiss(attacker, tar, s, onlyHitPer = false) {
	        if (tar.buffEffectTmpObj && tar.buffEffectTmpObj["无法闪避"]) {
	            if (onlyHitPer == true)
	                return 0;
	            return false;
	        }
	        let hit = attacker.getValue(GT_Role.VName_Hit) + 100;
	        if (s && attacker.camp + tar.camp && s.customData && s.customData["攻击命中率"] != "正常") {
	            if (s.customData["攻击命中率"] == "容易")
	                hit += hit * 0.2;
	            else if (s.customData["攻击命中率"] == "极易")
	                hit += hit * 0.4;
	            else if (s.customData["攻击命中率"] == "困难")
	                hit -= hit * 0.2;
	            else if (s.customData["攻击命中率"] == "极难")
	                hit -= hit * 0.4;
	        }
	        let mis = tar.getValue(GT_Role.VName_Mis);
	        let per = hit - mis;
	        if (attacker.buffEffectObj["目盲"] == true) {
	            if (per > 0)
	                per = per * 0.5;
	            else
	                per += per * 0.5;
	        }
	        if (onlyHitPer == true)
	            return per;
	        if (Math.random() * 100 < per)
	            return false;
	        return true;
	    }
	    static onDef(atk, type, from, tar) {
	        let def = 0;
	        let perAdd = 0;
	        let perAnti = 0;
	        if (type == "物理") {
	            def = tar.getValue(GT_Role.VName_Pde);
	            if (from)
	                perAdd += from.getValue(GT_Role.VName_PhyPer);
	            perAnti += tar.getValue(GT_Role.VName_AntiPhy);
	        }
	        else if (type == "魔法") {
	            def = tar.getValue(GT_Role.VName_Mde);
	            if (from)
	                perAdd += from.getValue(GT_Role.VName_MagPer);
	            perAnti += tar.getValue(GT_Role.VName_AntiMag);
	        }
	        else {
	            def = tar.getValue(GT_Role.VName_Def);
	        }
	        if (from)
	            perAdd += from.getValue(GT_Role.VName_AtkPer);
	        perAnti += tar.getValue(GT_Role.VName_AntiAtk);
	        perAdd -= perAnti;
	        atk += atk * perAdd * 0.01;
	        var defmathtype = this.option.mathType;
	        if (defmathtype == 0) {
	            atk -= def;
	        }
	        else if (defmathtype == 1) {
	            let per = Tool_Function.getPerValueInArray(def / atk, this.option.type1Arr);
	            atk -= atk * per;
	        }
	        else {
	            console.error("还未处理的抗性计算type=" + defmathtype);
	        }
	        atk = Tool_Function.onForceConvertType(atk);
	        if (atk <= 0)
	            return 0;
	        return atk;
	    }
	    static onAttribuiltDef(atk, att, from, tar) {
	        let def = tar.getAntiAttribute(att);
	        if (this.option.antiAttType == 0) {
	            atk -= atk * def * 0.01;
	        }
	        else if (this.option.antiAttType == 1) {
	            let vfrom = from.getAttribute(att);
	            def -= vfrom;
	            atk -= atk * def * 0.01;
	        }
	        else {
	            console.error("还未处理的抗性计算type=" + this.option.antiAttType);
	        }
	        return atk;
	    }
	    static onRoleTypeDef(atk, tarType, fromType) {
	        if (this.option.roletypeDef && this.option.roletypeDef[fromType] && this.option.roletypeDef[fromType][tarType]) {
	            let per = this.option.roletypeDef[fromType][tarType];
	            atk *= per;
	        }
	        return atk;
	    }
	}
	class gthurt_option {
	    constructor() {
	        this.mathType = 0;
	        this.type1Arr = [-5, -10, 0, -0.5, 0.25, 0, 1, 0.4, 2, 0.6, 4, 0.8, 8, 1];
	        this.antiAttType = 0;
	        this.randomOdder = 0;
	    }
	}

	class LSOManager {
	    static get instance() { return this; }
	    static get(key) {
	        var str = Laya.LocalStorage.getItem(key);
	        if (str == null || str.length == 0)
	            return null;
	        var obj = Tool_Function.onJSONParse(str);
	        return obj;
	    }
	    static put(key, value) {
	        Laya.LocalStorage.setItem(key, JSON.stringify(value));
	    }
	    static del(key) {
	        Laya.LocalStorage.removeItem(key);
	    }
	    static clearAll() {
	        Laya.LocalStorage.clear();
	    }
	}

	var HttpRequest = Laya.HttpRequest;
	class Net_HttpRequest {
	    constructor(_ip, _data, f, post = false, _needBase64 = false, _needJson = false, timeout = -1) {
	        this.isOver = false;
	        this.iDelay = -1;
	        this.IP = _ip;
	        if (this.IP.indexOf("http") != 0) {
	            this.IP = "http://" + this.IP;
	        }
	        this.Data = _data;
	        this.Fun = f;
	        this.isPost = post;
	        this.needBase64 = _needBase64;
	        this.needJson = _needJson;
	        this.timeout = timeout;
	        if (this.timeout == -1)
	            this.timeout = Net_HttpRequest.timeOut;
	        if (this.timeout < 100)
	            this.timeout = this.timeout * 1000;
	        else if (this.timeout < 1000)
	            this.timeout = 1000;
	        this.connectF();
	    }
	    connectF() {
	        this.hr = new HttpRequest();
	        Net_HttpRequest.ArrHr.push(this.hr);
	        if (this.timeout == -1)
	            this.hr.http.timeout = Net_HttpRequest.timeOut;
	        else
	            this.hr.http.timeout = this.timeout;
	        this.iDelay = ConfigTidus.GFun_MM_addDelay(this.timeout * 0.001 * ConfigTidus.FrameRate, this.onTimeOut.bind(this));
	        this.hr.once(Laya.Event.COMPLETE, this, this.onHttpRequestComplete);
	        this.hr.once(Laya.Event.ERROR, this, this.onHttpRequestError);
	        if (this.isPost == false) {
	            let url = this.IP;
	            if (this.Data) {
	                if (url.indexOf("?") == -1)
	                    url += "?" + this.getDatasUrl();
	                else
	                    url += "&" + this.getDatasUrl();
	            }
	            this.hr.send(url, null, 'get', 'text');
	        }
	        else {
	            if (Tool_Function.isTypeOf(this.Data, "str")) {
	                this.hr.send(this.IP, this.Data, 'post', 'text');
	            }
	            else {
	                this.hr.send(this.IP, JSON.stringify(this.Data), 'post', 'text');
	            }
	        }
	    }
	    getDatasUrl() {
	        if (Tool_Function.isTypeOf(this.Data, "str")) {
	            return this.Data;
	        }
	        let data = this.Data;
	        let url = "";
	        var first = true;
	        for (let key in data) {
	            if (first == false)
	                url += "&";
	            url += key + "=";
	            first = false;
	            if (this.Data[key] instanceof Array || this.Data[key] instanceof Object) {
	                url += JSON.stringify(this.Data[key]);
	            }
	            else {
	                url += this.Data[key];
	            }
	        }
	        return url;
	    }
	    onTimeOut() {
	        if (this.isOver)
	            return;
	        this.isOver = true;
	        if (this.iDelay != -1)
	            ConfigTidus.GFun_MM_removeDelay(this.iDelay);
	        this.iDelay = -1;
	        Tool_Function.onRunFunction(this.Fun, null);
	    }
	    onHttpRequestError(e) {
	        if (this.isOver)
	            return;
	        this.isOver = true;
	        if (this.iDelay != -1)
	            ConfigTidus.GFun_MM_removeDelay(this.iDelay);
	        this.iDelay = -1;
	        Tool_Function.onRunFunction(this.Fun, null);
	    }
	    onHttpRequestComplete(e) {
	        if (this.isOver)
	            return;
	        this.isOver = true;
	        if (this.iDelay != -1)
	            ConfigTidus.GFun_MM_removeDelay(this.iDelay);
	        this.iDelay = -1;
	        let data = null;
	        let index = Net_HttpRequest.ArrHr.indexOf(this.hr);
	        if (index >= 0)
	            Net_HttpRequest.ArrHr.splice(index, 1);
	        if (this.hr.data == "ok") {
	            Tool_Function.onRunFunction(this.Fun, "ok");
	            return;
	        }
	        try {
	            if (this.needJson && this.hr.data) {
	                data = JSON.parse(this.hr.data);
	            }
	            else {
	                data = this.hr.data;
	            }
	        }
	        catch (error) {
	            console.error("收到网络信息报错：" + error);
	            Tool_Function.onRunFunction(this.Fun, null);
	            return;
	        }
	        Tool_Function.onRunFunction(this.Fun, data);
	    }
	}
	Net_HttpRequest.timeOut = 8000;
	Net_HttpRequest.ArrHr = [];

	class CmdRand {
	    constructor() {
	        this._seed = 0;
	        this.seed = 0;
	    }
	    static getInstance() {
	        if (this.instance == null)
	            this.instance = new CmdRand();
	        return this.instance;
	    }
	    next() {
	        if (this._seed == 0)
	            return 0;
	        this.lastSeed = this._seed;
	        do {
	            this._seed ^= (this._seed << 21);
	            this._seed ^= (this._seed >> 21);
	            this._seed ^= (this._seed << 4);
	        } while (this._seed == 0 || this.seed == this._seed);
	        this.seed = this._seed;
	        return this._seed;
	    }
	    setSeed(n) {
	        this._seed = n;
	        this.lastSeed = n;
	    }
	}

	class MyByteArray {
	    constructor(cmd = -1) {
	        this.CMD = cmd;
	    }
	    writeF(val) {
	        if (this.WriteB == null) {
	            this.WriteB = new Laya.Byte();
	            this.WriteB.endian = MyByteArray.ENDIAN;
	        }
	        if (val instanceof Array) {
	            for (var i = 0; i < val.length; i++) {
	                this.write(val[i]);
	            }
	        }
	        else
	            this.write(val);
	    }
	    write(val) {
	        if (typeof val == "number") {
	            if (val != Tool_Function.onForceConvertType(val)) {
	                this.WriteB.writeByte(5);
	                this.WriteB.writeFloat64(val);
	                return;
	            }
	            var n = Math.abs(val);
	            if (n < 127) {
	                this.WriteB.writeByte(1);
	                this.WriteB.writeByte(val);
	            }
	            else if (n < 32767) {
	                this.WriteB.writeByte(2);
	                this.WriteB.writeInt16(val);
	            }
	            else if (n < 2147483647) {
	                this.WriteB.writeByte(3);
	                this.WriteB.writeInt32(val);
	            }
	            else {
	                this.WriteB.writeByte(5);
	                this.WriteB.writeFloat64(val);
	            }
	        }
	        else if (typeof val === "string") {
	            this.WriteB.writeByte(4);
	            this.WriteB.writeUTFString(val);
	        }
	        else if (val == null || val == undefined) {
	            this.WriteB.writeByte(8);
	        }
	        else if (typeof val == "boolean") {
	            this.WriteB.writeByte(7);
	            if (val == true)
	                this.WriteB.writeByte(1);
	            else
	                this.WriteB.writeByte(0);
	        }
	        else if (val instanceof Array) {
	            this.WriteB.writeByte(6);
	            this.write(val.length);
	            for (let i = 0; i < val.length; i++) {
	                this.write(val[i]);
	            }
	            return;
	        }
	        else {
	            let arr_key = [];
	            let arr_val = [];
	            for (var key in val) {
	                arr_key.push(key);
	                arr_val.push(val[key]);
	            }
	            this.WriteB.writeByte(9);
	            this.write(arr_key.length);
	            for (let i2 = 0; i2 < arr_key.length; i2++) {
	                this.write(arr_key[i2]);
	                this.write(arr_val[i2]);
	            }
	        }
	    }
	    readF(buf) {
	        this.Arr_val = [];
	        while (buf.bytesAvailable > 0) {
	            this.Arr_val.push(readNext());
	        }
	        function readNext() {
	            var val;
	            var n = buf.readByte();
	            if (n == 1)
	                val = buf.readByte();
	            else if (n == 2)
	                val = buf.readInt16();
	            else if (n == 3)
	                val = buf.readInt32();
	            else if (n == 4) {
	                val = buf.readUTFString();
	                if (val.indexOf("\\n") != -1) {
	                    val = Tool_StringBuild.replaceSTR(val, "\\n", "\n");
	                }
	            }
	            else if (n == 5)
	                val = buf.readFloat64();
	            else if (n == 6) {
	                var lenType = buf.readByte();
	                var len;
	                if (lenType == 1)
	                    len = buf.readByte();
	                else if (lenType == 2)
	                    len = buf.readInt32();
	                var arr = [];
	                for (; len > 0; len--) {
	                    arr.push(readNext());
	                }
	                return arr;
	            }
	            else if (n == 7) {
	                val = Tool_Function.onForceConvertType(buf.readByte());
	                if (val == 0)
	                    return false;
	                else
	                    return true;
	            }
	            else if (n == 8) {
	                return null;
	            }
	            else if (n == 9) {
	                var dic = {};
	                lenType = buf.readByte();
	                if (lenType == 1)
	                    len = buf.readByte();
	                else if (lenType == 2)
	                    len = buf.readInt16();
	                var key;
	                for (; len > 0; len--) {
	                    key = readNext();
	                    dic[key] = readNext();
	                }
	                return dic;
	            }
	            else {
	                console.error("Unkownd Type In MyByteArray's readF!CMD==" + this.CMD + ",n==" + n);
	                return null;
	            }
	            return val;
	        }
	    }
	}
	MyByteArray.ENDIAN = "BIG_ENDIAN";

	class Net_SocketCMD {
	    constructor(cmd = 0, soc = null) {
	        this.isDicSendType = true;
	        this.isDicGetType = true;
	        this.DicFuns = new MyEventListener(false);
	        this.iSoc = -1;
	        this.Type = null;
	        this.CMD = cmd;
	        this.soc = soc;
	        if (this.soc == null) {
	            this.soc = MgsSocket.getInstance();
	        }
	        if (this.CMD == 0) {
	            var name = Tool_Function.getLastClassName(this);
	            var arr = name.split("CMD");
	            this.CMD = Tool_Function.onForceConvertType(arr[arr.length - 1]);
	        }
	    }
	    funAddListener(f, once) {
	        if (this.iSoc == -1) {
	            this.iSoc = this.soc.addCmdListener(this.CMD, this.recivedF.bind(this));
	        }
	        let i = this.DicFuns.addCustomEventListener("a", f, false);
	        return i;
	    }
	    funcRemoveListener(i) {
	        this.DicFuns.removeCustomEventListener("a", i);
	    }
	    recivedF(b2) {
	        if (this.isDicGetType) {
	            if (b2 instanceof MyByteArray) {
	                if (b2.Arr_val.length > 0)
	                    this.dic_Get = b2.Arr_val[0];
	            }
	            else {
	                this.dic_Get = b2;
	            }
	            this.DicFuns.onCustomEventAct("a", this.dic_Get);
	            this.dic_Get = null;
	        }
	        else {
	            if (b2 instanceof MyByteArray) {
	                this.Arr_values = b2.Arr_val;
	            }
	            else {
	                this.Arr_values = b2;
	            }
	            this.DicFuns.onCustomEventAct("a", this.Arr_values);
	        }
	    }
	    writeValue_Dic(key, val) {
	        if (this.isDicSendType == false)
	            throw new Error("不能使用该方法");
	        if (this.dic_Send == null)
	            this.dic_Send = {};
	        this.dic_Send[key] = val;
	    }
	    writeValueF_Arr(values) {
	        if (this.isDicSendType == true)
	            throw new Error("不能使用该方法");
	        this.Arr_values = values.concat();
	    }
	    sendF(needRadom = true, needback = false) {
	        if (needback == false && (this.DicFuns.hasListenerOfEvent("a") || (ConfigTidus.GVal["NetMessageModel"].instance.isListening(this.CMD)))) {
	            needback = true;
	        }
	        if (MgsSocket.websocket_useJson) {
	            if (this.dic_Send != null) {
	                if (MgsSocket.useStringMessage == true) {
	                    this.soc.sendStringMessage(this.CMD, this.dic_Send, needback);
	                    return;
	                }
	                var strJson = JSON.stringify(this.dic_Send);
	                var b = new MyByteArray();
	                b.WriteB = new Laya.Byte();
	                b.WriteB.endian = MyByteArray.ENDIAN;
	                b.WriteB.writeUTFString(strJson);
	                this.soc.sendMessage(this.CMD, b.WriteB, needRadom, this.Type, needback);
	            }
	            else {
	                if (MgsSocket.useStringMessage == true) {
	                    this.soc.sendStringMessage(this.CMD, null, needback);
	                    return;
	                }
	                this.soc.sendMessage(this.CMD, null, needRadom, this.Type, needback);
	            }
	        }
	        else {
	            if (this.dic_Send != null) {
	                var b = new MyByteArray();
	                b.writeF(this.dic_Send);
	                this.soc.sendMessage(this.CMD, b.WriteB, needRadom, this.Type, needback);
	            }
	            else {
	                this.soc.sendMessage(this.CMD, null, needRadom, this.Type, needback);
	            }
	        }
	    }
	    destroyF() {
	        if (this.iSoc != -1) {
	            this.soc.removeCmdListener(this.CMD, this.iSoc);
	            this.iSoc = -1;
	        }
	        if (this.DicFuns) {
	            this.DicFuns.destroyF();
	            this.DicFuns = null;
	        }
	        this.dic_Send = null;
	        this.dic_Get = null;
	    }
	}

	var Socket = Laya.Socket;
	var Event$1 = Laya.Event;
	class MgsSocket {
	    constructor() {
	        this.CMDLENGTH = 2;
	        this.SEEDLength = 4;
	        this.PORT = -1;
	        this.timeLimite = 8;
	        this.isListening = false;
	        this.dicListener = new MyEventListener();
	        this.SaveCMD = -1;
	        this._pause = false;
	        this.Seed = CmdRand.getInstance();
	        this.socket = new Socket();
	    }
	    static getInstance() {
	        if (this._mgsSocket == null) {
	            this._mgsSocket = new MgsSocket();
	        }
	        return this._mgsSocket;
	    }
	    static destroyF() {
	        if (this._mgsSocket)
	            this._mgsSocket.destroyF();
	        this._mgsSocket = null;
	        ConfigTidus.GVal["NetMessageModel"].destroyF();
	    }
	    getSocket(type = null) {
	        if (type == null)
	            return this.socket;
	        return null;
	    }
	    connectF(ip, port, speUrl = null) {
	        if (this.nowFlag == "正在连接" || this.nowFlag == "连接")
	            return;
	        if (ip != null)
	            this.IP = ip;
	        this.PORT = port;
	        this.addListeners();
	        this.nowFlag = "正在连接";
	        if (!this.socket.connected) {
	            console.log("webSocket开始连接：" + this.IP + "          ,端口" + this.PORT);
	            if (this.timeLimite > 0) {
	                Laya.timer.once(this.timeLimite * 1000, this, this.onConnectTimeOver);
	            }
	            if (this.PORT == -1) {
	                this.socket.connectByUrl(MgsSocket.WSType + "://" + this.IP);
	            }
	            else {
	                if (speUrl)
	                    this.socket.connectByUrl(MgsSocket.WSType + "://" + this.IP + ":" + this.PORT + "/" + speUrl);
	                else
	                    this.socket.connectByUrl(MgsSocket.WSType + "://" + this.IP + ":" + this.PORT);
	            }
	        }
	    }
	    onConnectTimeOver() {
	        this.ioErrorHandler(null);
	    }
	    onClearConnectTimer() {
	        Laya.timer.clear(this, this.onConnectTimeOver);
	    }
	    addListeners() {
	        if (this.isListening)
	            return;
	        this.isListening = true;
	        this.socket.on(Event$1.OPEN, this, this.connectHandler, null);
	        this.socket.on(Event$1.CLOSE, this, this.closeHandler, null);
	        this.socket.on(Event$1.ERROR, this, this.ioErrorHandler, null);
	        this.socket.on(Event$1.MESSAGE, this, this.socketDataHandler, null);
	    }
	    removeListeners() {
	        this.isListening = false;
	        this.socket.offAll();
	    }
	    connectHandler(event) {
	        console.log("socket连接成功！");
	        this.nowFlag = "连接";
	        this.onClearConnectTimer();
	        if (this.Fun_connect != null) {
	            Tool_Function.onRunFunction(this.Fun_connect, true);
	            this.Fun_connect = null;
	        }
	    }
	    closeHandler(event) {
	        if (this.nowFlag == null) {
	            return;
	        }
	        console.log("socket已关闭!now状态=" + this.nowFlag);
	        if (this.timeLimite > 0 && this.nowFlag == "正在连接") {
	            this.onClearConnectTimer();
	        }
	        this.isListening = false;
	        this.nowFlag = null;
	        var tmpSaveCMD = this.SaveCMD;
	        if (this.SaveCMD != -1) {
	            this.SaveCMD = -1;
	        }
	        if (MgsSocket.needCloseEvent == true) {
	            MainManager.instance.EventManager.dispatchEvent(MgsSocket.Event_Close);
	        }
	        if (tmpSaveCMD != -1 && this.pause == false) {
	            if (MgsSocket.websocket_useJson == false) {
	                var b = new MyByteArray();
	                b.writeF({ "结果": false, "失败原因": "无法连接到服务器" });
	                this.receiveCommand(tmpSaveCMD, b);
	            }
	            else {
	                this.receiveCommand(tmpSaveCMD, { "结果": false, "失败原因": "无法连接到服务器" });
	            }
	        }
	        if (this.FunClose)
	            this.FunClose();
	    }
	    closeSocket() {
	        if (this.socket != null && this.socket.connected)
	            this.socket.close();
	        else {
	            this.closeHandler(null);
	        }
	    }
	    ioErrorHandler(event) {
	        console.log("socket io错误");
	        if (this.timeLimite > 0 && this.nowFlag == "正在连接") {
	            this.onClearConnectTimer();
	        }
	        this.nowFlag = null;
	        if (this.Fun_connect != null) {
	            Tool_Function.onRunFunction(this.Fun_connect, false);
	            this.Fun_connect = null;
	        }
	    }
	    socketDataHandler(message) {
	        if (typeof message == "string") {
	            var dic;
	            try {
	                dic = JSON.parse(message);
	            }
	            catch (error) {
	                console.error("收到string消息解析json报错：" + message);
	                return;
	            }
	            let cmd = dic["cmd"];
	            delete dic["cmd"];
	            this.receiveCommand(cmd, dic);
	        }
	        else if (message instanceof ArrayBuffer) {
	            var buf = new Laya.Byte(message);
	            buf.endian = "BIG_ENDIAN";
	            buf.pos = 0;
	            this.getBytes(buf);
	        }
	        this.socket.input.clear();
	    }
	    getBytes(buf) {
	        var cmd = buf.getInt16();
	        if (MgsSocket.websocket_useJson == true) {
	            var dic = null;
	            if (buf.bytesAvailable > 0) {
	                var str = buf.readUTFBytes(buf.bytesAvailable);
	                dic = JSON.parse(str);
	            }
	            this.receiveCommand(cmd, dic);
	        }
	        else {
	            let b2 = new MyByteArray();
	            b2.readF(buf);
	            this.receiveCommand(cmd, b2);
	        }
	        buf.clear();
	    }
	    receiveCommand(cmd, dataBytes) {
	        if (cmd == MgsSocket.CMDHeart) {
	            var c = new Net_SocketCMD(MgsSocket.CMDHeart);
	            c.sendF(false);
	            return;
	        }
	        if (this.pause == true) {
	            if (this.Arr_WaiteReceiveCMD == null)
	                this.Arr_WaiteReceiveCMD = [];
	            this.Arr_WaiteReceiveCMD.push([cmd, dataBytes]);
	            return;
	        }
	        if (cmd == this.SaveCMD) {
	            this.SaveCMD = -1;
	        }
	        this.dicListener.onCustomEventAct(cmd + "", dataBytes);
	        if (-1 == this.SaveCMD) {
	            Laya.timer.clear(this, this.onTimeWrongF);
	            if (this.Arr_WaiteSendCMD != null && this.Arr_WaiteSendCMD.length > 0) {
	                var tmp = this.Arr_WaiteSendCMD.shift();
	                console.log("发送缓存指令：" + tmp[0]);
	                this.sendMessage(tmp[0], tmp[1], tmp[2], tmp[3], tmp[4]);
	            }
	            else {
	                if (this.FunGetReconnectCMD) {
	                    this.FunGetReconnectCMD();
	                    this.FunGetReconnectCMD = null;
	                }
	            }
	        }
	    }
	    addCmdListener(cmd, hander) {
	        let i = this.dicListener.addCustomEventListener(cmd + "", hander, false);
	        return i;
	    }
	    removeCmdListener(cmd, listener) {
	        this.dicListener.removeCustomEventListener(cmd + "", listener);
	    }
	    sendStringMessage(cmd, content, needBack = true) {
	        if (this.nowFlag != "连接") {
	            if (cmd == MgsSocket.CMDHeart || needBack == false) {
	                return;
	            }
	            if (MgsSocket.FunSendWhenClose && this.Type == null) {
	                Tool_Function.onRunFunction(MgsSocket.FunSendWhenClose, () => {
	                    if (this.nowFlag != "连接") {
	                        this.receiveCommand(cmd, { "结果": false, "失败原因": "无法连接到服务器" });
	                    }
	                    else {
	                        this.sendStringMessage(cmd, content, needBack);
	                    }
	                });
	                return;
	            }
	            this.receiveCommand(cmd, { "结果": false, "失败原因": "无法连接到服务器" });
	            return;
	        }
	        this.socket.send(JSON.stringify({ "cmd": cmd, "data": content }));
	        if (needBack == true) {
	            this.SaveCMD = cmd;
	            if (this.timeLimite > 0) {
	                Laya.timer.once(this.timeLimite * 1000, this, this.onTimeWrongF);
	            }
	        }
	    }
	    sendMessage(cmd, content, needRandom = true, type = null, needBack = true) {
	        if (this.SaveCMD != -1 && needBack == true) {
	            console.log("正在等待" + this.SaveCMD + "指令返回，发送的" + cmd + "指令被缓存");
	            if (this.Arr_WaiteSendCMD == null)
	                this.Arr_WaiteSendCMD = [];
	            this.Arr_WaiteSendCMD.push([cmd, content, needRandom, type, needBack]);
	            return;
	        }
	        if (this.nowFlag != "连接") {
	            if (cmd == MgsSocket.CMDHeart || needBack == false) {
	                return;
	            }
	            if (MgsSocket.FunSendWhenClose && this.Type == null) {
	                Tool_Function.onRunFunction(MgsSocket.FunSendWhenClose, () => {
	                    if (this.nowFlag != "连接") {
	                        this.receiveCommand(cmd, { "结果": false, "失败原因": "无法连接到服务器" });
	                    }
	                    else {
	                        this.sendMessage(cmd, content, needRandom, type, needBack);
	                    }
	                });
	                return;
	            }
	            this.receiveCommand(cmd, { "结果": false, "失败原因": "无法连接到服务器" });
	            return;
	        }
	        var contentLen = 0;
	        if (content != null) {
	            contentLen = content.length;
	        }
	        var SaveBuf = new Laya.Byte();
	        SaveBuf.endian = MyByteArray.ENDIAN;
	        if (needRandom) {
	            var cmdSeed = CmdRand.getInstance().next();
	            SaveBuf.writeInt32(cmdSeed);
	        }
	        else
	            SaveBuf.writeInt32(0);
	        SaveBuf.writeInt16(cmd);
	        if (content != null) {
	            SaveBuf.writeArrayBuffer(content.buffer, 0, content.length);
	        }
	        if (needBack == true) {
	            this.SaveCMD = cmd;
	            if (this.timeLimite > 0) {
	                Laya.timer.once(this.timeLimite * 1000, this, this.onTimeWrongF);
	            }
	        }
	        this.sendF(type, SaveBuf);
	    }
	    onTimeWrongF() {
	        console.log("超时关闭socket！" + this.SaveCMD);
	        if (this.SaveCMD == -1) {
	            return;
	        }
	        this.closeSocket();
	    }
	    sendF(type, SaveBuf) {
	        if (this.socket.connected == false) {
	            this.closeHandler(null);
	            return;
	        }
	        if (type == null) {
	            if (SaveBuf == null)
	                return;
	            SaveBuf.pos = 0;
	            this.socket.send(SaveBuf.buffer);
	        }
	        else {
	            console.log("暂未支持h5的mysocket");
	        }
	    }
	    destroyF() {
	        this.removeListeners();
	        if (this.socket != null && this.socket.connected)
	            this.socket.close();
	        this.dicListener = new MyEventListener();
	        if (this.Type == null)
	            MgsSocket._mgsSocket = null;
	        Laya.timer.clear(this, this.onTimeWrongF);
	    }
	    get pause() {
	        return this._pause;
	    }
	    set pause(value) {
	        this._pause = value;
	        if (value == false) {
	            if (this.Arr_WaiteReceiveCMD) {
	                while (this.Arr_WaiteReceiveCMD.length > 0) {
	                    this.receiveCommand(this.Arr_WaiteReceiveCMD[0][0], this.Arr_WaiteReceiveCMD[0][1]);
	                    this.Arr_WaiteReceiveCMD.shift();
	                }
	            }
	            this.Arr_WaiteReceiveCMD = null;
	        }
	    }
	}
	MgsSocket.CMDHeart = 2;
	MgsSocket.Event_Close = "网络断开";
	MgsSocket.WSType = "wss";
	MgsSocket.websocket_useJson = true;
	MgsSocket.needCloseEvent = false;
	MgsSocket.useStringMessage = false;

	class MyRandomIP {
	    constructor(ip, 参数, funSuc, funWrong) {
	        this.seed = 0;
	        this.count = 0;
	        this.count2 = 5;
	        this.连接数 = 0;
	        this.前缀 = ip.split("?")[0];
	        this.起始 = MyRandomIP.SEED;
	        this.后缀 = ip.split("?")[1];
	        this.data = 参数;
	        this.FunSuc = funSuc;
	        this.FunWrong = funWrong;
	        this.startLoad();
	    }
	    get url() {
	        return this.前缀 + this.seed + this.后缀;
	    }
	    get ip() {
	        var 地址 = this.url;
	        if (地址.indexOf("http") == 0) {
	            地址 = 地址.substr(地址.indexOf("//") + 2);
	        }
	        var index = 地址.lastIndexOf(":");
	        地址 = 地址.substr(0, index);
	        return 地址;
	    }
	    initLoader() {
	        this.loaders = ConfigTidus.GFun_ToolObj_destroyOne(this.loaders);
	        this.loaders = [];
	        var i = this.count2;
	        while (0 < i--) {
	            this.loaders.push(new _myrandomip_one(i, Laya.Handler.create(this, this.onLoadF)));
	        }
	    }
	    startLoad() {
	        this.initLoader();
	        var ml;
	        var url;
	        for (let key in this.loaders) {
	            ml = this.loaders[key];
	            url = this.getURL();
	            if (url == null)
	                break;
	            ml.loadUrl(this.seed, url, this.data);
	        }
	    }
	    getURL() {
	        this.seed == 0 ? this.seed = this.起始 : this.seed = this.nextSeed(this.seed);
	        var _u = this.前缀 + this.seed + this.后缀;
	        return _u;
	    }
	    onLoadF(...arg) {
	        if (arg.length == 0) {
	            this.error();
	        }
	        else {
	            this.complete(arg[0], arg[1]);
	        }
	    }
	    error() {
	        if (++this.count >= this.count2) {
	            if (this.连接数 == 0) {
	                console.log("全部失败：" + this.count);
	                this.loaders = ConfigTidus.GFun_ToolObj_destroyOne(this.loaders);
	                if (this.count2 >= 20) {
	                    this.seed = this.count = 0;
	                    this.count2 = 5;
	                    Tool_Function.onRunFunction(this.FunWrong);
	                }
	                else {
	                    this.seed = this.count = 0;
	                    this.count2 += 5;
	                    this.startLoad();
	                }
	            }
	        }
	    }
	    complete(_seed, data) {
	        if (MyRandomIP.关键词 != null && data.indexOf(MyRandomIP.关键词) == -1) {
	            this.error();
	            return;
	        }
	        this.连接数--;
	        this.seed = _seed;
	        this.回调 = data;
	        this.loaders = ConfigTidus.GFun_ToolObj_destroyOne(this.loaders);
	        Tool_Function.onRunFunction(this.FunSuc, this.ip);
	    }
	    nextSeed(seed) {
	        seed ^= seed << 21;
	        seed ^= seed >> 21;
	        seed ^= seed << 4;
	        return seed;
	    }
	}
	MyRandomIP.SEED = 0;
	MyRandomIP.关键词 = "ok";
	class _myrandomip_one {
	    constructor(i, funOk) {
	        this.loading = false;
	        this.id = i;
	        this.FunOk = funOk;
	    }
	    complete(data) {
	        this.loading = false;
	        if (data == null) {
	            Tool_Function.onRunFunction(this.FunOk);
	        }
	        else {
	            Tool_Function.onRunFunction(this.FunOk, this.seed, data);
	        }
	    }
	    error() {
	        this.complete(null);
	    }
	    loadUrl(seed, url, data) {
	        if (this.loading == true)
	            return;
	        this.seed = seed;
	        new Net_HttpRequest(url, data, this.complete.bind(this), false);
	    }
	    destroyF() {
	        this.FunOk = ConfigTidus.GFun_ToolObj_destroyOne(this.FunOk);
	    }
	}

	class CmdListenerOne {
	    constructor(soc = null) {
	        this.Dic_CMD = {};
	        if (soc == null)
	            this.soc = MgsSocket.getInstance();
	        else
	            this.soc = soc;
	    }
	    listenF(cmd, fun, once) {
	        if (this.Dic_CMD[cmd] != null) {
	            this.removeF(cmd);
	        }
	        var one = new Net_SocketCMD(cmd, this.soc);
	        one.funAddListener(fun, once);
	        this.Dic_CMD[cmd] = one;
	    }
	    isListening(cmd) {
	        if (this.Dic_CMD[cmd]) {
	            var one = this.Dic_CMD[cmd];
	            return one.iSoc != -1;
	        }
	        return false;
	    }
	    removeF(cmd) {
	        var one = this.Dic_CMD[cmd];
	        if (one) {
	            one.destroyF();
	            delete this.Dic_CMD[cmd];
	        }
	    }
	    destroyF() {
	        this.Dic_CMD = ConfigTidus.GFun_ToolObj_destroyOne(this.Dic_CMD);
	    }
	}

	class MyViewBtnController {
	    constructor(spr, fClick, fDown = null, fUp = null, fnone = null) {
	        this.Dic_Btn = {};
	        this.Busy = false;
	        this.needBtnSound = true;
	        this.sprBack = spr;
	        this.FunClick = fClick;
	        this.FunDown = fDown;
	        this.FunUp = fUp;
	        spr.touchable = true;
	        for (var i = 0; i < spr.numChildren; i++) {
	            var obj = spr.getChildAt(i);
	            var strName = obj.name;
	            if (strName && strName.indexOf("btn_") == 0) {
	                this.addOneBtn(obj, strName);
	            }
	        }
	        if (fnone)
	            this.addNoneClick(fnone);
	    }
	    addOneBtn(spr, strName) {
	        let btn = new BTN_Starling(spr);
	        this.Dic_Btn[strName] = btn;
	        btn.initTouch(this.onDownF.bind(this), this.FunUp ? this.onUpF.bind(this) : null, this.clickF.bind(this), strName);
	    }
	    addNoneClick(fnone = null) {
	        this.FunNone = fnone;
	        if (this.FunNone && this.mme == null) {
	            this.mme = new MyMouseManager(this.sprBack);
	            this.mme.addClickHandler((e) => { this.FunNone(); });
	            this.mme.stopEventMop = "本次";
	        }
	    }
	    onCanTouchThrough(real) {
	        this.sprBack.mouseThrough = real;
	        if (this.mme) {
	            if (real == true)
	                this.mme.stopEventMop = null;
	            else
	                this.mme.stopEventMop = "永久";
	        }
	    }
	    onDownF(from) {
	        if (this.Busy)
	            return;
	        if (this.needBtnSound) {
	            if (this.Dic_Sound && this.Dic_Sound[from]) {
	                SoundManagerMy.instance.playSound(this.Dic_Sound[from]);
	            }
	            else if (MyViewBtnController.Dic_BtnSound && MyViewBtnController.Dic_BtnSound[from]) {
	                SoundManagerMy.instance.playSound(MyViewBtnController.Dic_BtnSound[from]);
	            }
	            else if (MyViewBtnController.Dic_BtnSound && MyViewBtnController.Dic_BtnSound["默认"]) {
	                SoundManagerMy.instance.playSound(MyViewBtnController.Dic_BtnSound["默认"]);
	            }
	        }
	        if (this.FunDown == null)
	            return;
	        if (this.Value == null)
	            Tool_Function.onRunFunction(this.FunDown, from);
	        else
	            Tool_Function.onRunFunction(this.FunDown, this.Value, from);
	    }
	    onUpF(from) {
	        if (this.Busy)
	            return;
	        if (this.FunUp == null)
	            return;
	        if (this.Value == null)
	            Tool_Function.onRunFunction(this.FunUp, from);
	        else
	            Tool_Function.onRunFunction(this.FunUp, this.Value, from);
	    }
	    clickF(from) {
	        if (this.Busy)
	            return;
	        if (this.FunClick == null)
	            return;
	        if (this.Value == null)
	            Tool_Function.onRunFunction(this.FunClick, from);
	        else
	            Tool_Function.onRunFunction(this.FunClick, this.Value, from);
	    }
	    addBtnSound(btn, soundName) {
	        if (this.Dic_Sound == null)
	            this.Dic_Sound = {};
	        this.Dic_Sound[btn] = soundName;
	    }
	    onVisible(tar, vib) {
	        if (tar == null) {
	            for (let n in this.Dic_Btn) {
	                this.Dic_Btn[n].visible = vib;
	            }
	        }
	        else {
	            if (this.Dic_Btn[tar]) {
	                this.Dic_Btn[tar].visible = vib;
	            }
	        }
	    }
	    onPause(tar, pause) {
	        if (tar == null) {
	            for (let n in this.Dic_Btn) {
	                this.Dic_Btn[n].pause = pause;
	            }
	            if (this.mme)
	                this.mme.pausing = pause;
	        }
	        else {
	            if (this.Dic_Btn[tar]) {
	                this.Dic_Btn[tar].pause = pause;
	            }
	        }
	    }
	    onEventStop() {
	    }
	    changeFrame(tar, f) {
	        if (this.Dic_Btn[tar])
	            this.Dic_Btn[tar].setFrame(f);
	    }
	    onChangeValues(Name, vName, value) {
	        for (var n in this.Dic_Btn) {
	            if (n == Name || Name == null)
	                this.Dic_Btn[n][vName] = value;
	        }
	    }
	    getBtnByName(_name) {
	        return this.Dic_Btn[_name];
	    }
	    checkInBtn(lx, ly) {
	        for (var _name in this.Dic_Btn) {
	            var btn = this.Dic_Btn[_name];
	            if (btn && btn.checkIn(lx, ly) == true) {
	                return _name;
	            }
	        }
	        return null;
	    }
	    destroyF() {
	        this.FunDown = null;
	        this.FunUp = null;
	        this.FunClick = null;
	        this.FunNone = null;
	        this.mme = ConfigTidus.GFun_ToolObj_destroyOne(this.mme);
	        for (var n in this.Dic_Btn) {
	            this.Dic_Btn[n].destroyF();
	            delete this.Dic_Btn[n];
	        }
	        this.Dic_Btn = null;
	        this.Dic_Sound = null;
	    }
	}

	class BTNControllerStarling {
	    constructor(L, arr, Fclick, Fdown = null, Fup = null, fnone = null) {
	        this.Arr_tar = [];
	        this.now = -1;
	        this.down = 0;
	        this.pause = false;
	        this.MinSlideLength = 2;
	        this.canSlide = true;
	        this.canSlideAndSelect = true;
	        this.autoClear = false;
	        this.canMouseSlide = false;
	        this.autoChangeFrame = true;
	        this.Layer = L;
	        this.FunClick = Fclick;
	        this.FunDown = Fdown;
	        this.FunUp = Fup;
	        this.FunClickNone = fnone;
	        this.Arr_tar = arr;
	        for (let i = 0; i < arr.length; i++) {
	            if (this.Arr_tar[i] == null)
	                continue;
	            this.Arr_tar[i].initTouch(Fdown ? this.onDownF.bind(this) : null, Fup ? this.onUPF.bind(this) : null, Fclick ? this.onClickF.bind(this) : null, i);
	        }
	    }
	    onDownF(i) {
	        if (this.down > 0)
	            return;
	        this.now = i;
	        if (this.now >= 0) {
	            this.down = 1;
	            this.changeFrame(this.now, 2);
	            if (this.FunDown != null) {
	                this.FunDown(this.now);
	            }
	        }
	    }
	    onUPF(i) {
	        if (this.down > 0) {
	            if (this.now != -1) {
	                this.changeFrame(this.now, 1);
	                if (i != null) {
	                    if (this.FunUp != null)
	                        this.FunUp(this.now);
	                }
	                this.now = -1;
	            }
	            this.down = 0;
	        }
	    }
	    onClickF(i) {
	        this.onUPF(i);
	    }
	    changeFrame(num, f) {
	        if (this.autoChangeFrame == false || this.Arr_tar == null)
	            return;
	        if (num >= 0 && this.Arr_tar[num]) {
	            this.Arr_tar[num].setFrame(f);
	        }
	    }
	    pauseF(b) {
	        this.pause = b;
	        this.stopF();
	    }
	    removeIndex(n) {
	        this.Arr_tar.splice(n, 1);
	    }
	    stopF() {
	        if (this.down > 0) {
	            this.down = 0;
	            this.changeFrame(this.now, 1);
	            this.now = -1;
	        }
	    }
	    set visible(value) {
	        if (this.Arr_tar) {
	            for (let i = 0; i < this.Arr_tar.length; i++) {
	                if (this.Arr_tar[i]) {
	                    this.Arr_tar[i].visible = value;
	                }
	            }
	        }
	    }
	    destroyF() {
	        if (this.Layer) {
	            if (this.autoClear)
	                this.Layer = ConfigTidus.GFun_ToolObj_destroyOne(this.Layer);
	            else
	                this.Layer = null;
	        }
	        if (this.autoClear) {
	            this.Arr_tar = ConfigTidus.GFun_ToolObj_destroyOne(this.Arr_tar);
	        }
	        else
	            this.Arr_tar = null;
	        this.FunClick = ConfigTidus.GFun_ToolObj_destroyOne(this.FunClick);
	        this.FunDown = ConfigTidus.GFun_ToolObj_destroyOne(this.FunDown);
	        this.FunUp = ConfigTidus.GFun_ToolObj_destroyOne(this.FunUp);
	        this.FunClickNone = ConfigTidus.GFun_ToolObj_destroyOne(this.FunClickNone);
	    }
	}

	class MySlideMC_defaultOne extends Sprite {
	    constructor() {
	        super();
	    }
	    get H() { return this.height; }
	    initF(spr) {
	        this.sprBack = spr;
	        Tool_SpriteUtils.onAddchild_ChangeParent(this, this.sprBack);
	    }
	    setValue(key, value) {
	        if (this.values == null)
	            this.values = {};
	        this.values[key] = value;
	    }
	    getValue(key) {
	        if (this.values == null)
	            return null;
	        return this.values[key];
	    }
	    initAC(swfNum) {
	        if (this.sprBack == null) {
	            console.error("myslideMc_defaultOne中不存在sprback，无法初始化ac");
	            return;
	        }
	        this.ac = ConfigTidus.GVal["newAllComp"](this.sprBack, swfNum);
	        this.ac.Fun_mbc_click = this.clickBtnF.bind(this);
	        this.ac.Fun_msb_click = this.clickSBtnF.bind(this);
	    }
	    clickBtnF(btn) {
	        if (this.Fun_mbcClick) {
	            this.Fun_mbcClick(this, btn);
	        }
	    }
	    clickSBtnF(sb, btn) {
	        if (this.Fun_msbClick) {
	            this.Fun_msbClick(this, sb, btn);
	        }
	    }
	    destroyF() {
	        this.sprBack = ConfigTidus.GFun_ToolObj_destroyOne(this.sprBack);
	        this.values = ConfigTidus.GFun_ToolObj_destroyOne(this.values);
	        super.destroyF();
	    }
	}

	class Tween {
	    constructor(tar, _time, tran = "linear") {
	        this.info = {};
	        this.tarObj = tar;
	        this.time = _time;
	        this.tranType = tran;
	    }
	    moveTo(endx, endy) {
	        this.info["x"] = endx;
	        this.info["y"] = endy;
	    }
	    fadeTo(enda) {
	        this.info["alpha"] = enda;
	    }
	    scaleTo(ends) {
	        this.info["scaleX"] = ends;
	        this.info["scaleY"] = ends;
	    }
	    scaleXto(sx) {
	        this.info["scaleX"] = sx;
	    }
	    scaleYto(sy) {
	        this.info["scaleY"] = sy;
	    }
	    customTo(key, end) {
	        this.info[key] = end;
	    }
	    rotateTo(r) {
	        this.info["rotation"] = r;
	    }
	}
	Tween.LINEAR = "linear";
	Tween.BOUNCE_IN = "bounceIn";
	Tween.BOUNCE_OUT = "bounceOut";
	Tween.BOUNCE_IN_OUT = "bounceInOut";
	Tween.EASE_IN = "easeIn";
	Tween.EASE_OUT = "easeOut";
	Tween.EASE_IN_OUT = "easeInOut";
	Tween.STRONG_In = "strongOut";
	Tween.STRONG_In_Out = "strongInOut";

	class Tween3D {
	    constructor(tar, _time, tran = "linear") {
	        this.info0 = {};
	        this.info = {};
	        this.haveMove = false;
	        this.haveScaleLocal = false;
	        this.haveFade = false;
	        this.haveRotate = false;
	        this.haveRotateLocal = false;
	        this.haveCol = false;
	        this.tarSpr = tar;
	        this.time = _time;
	        this.tranType = tran;
	    }
	    moveTo(endx, endy, endz) {
	        this.info["x"] = endx != null ? endx : this.tarSpr.transform.position.x;
	        this.info["y"] = endy != null ? endy : this.tarSpr.transform.position.y;
	        this.info["z"] = endz != null ? endz : this.tarSpr.transform.position.z;
	        this.info0["x"] = this.tarSpr.transform.position.x;
	        this.info0["y"] = this.tarSpr.transform.position.y;
	        this.info0["z"] = this.tarSpr.transform.position.z;
	        this.haveMove = true;
	    }
	    fadeTo(enda) {
	        this.info["alpha"] = enda;
	        if (this.tarSpr instanceof Laya.MeshSprite3D)
	            this.info0["alpha"] = this.tarSpr.meshRenderer.material.albedoColorA;
	        else if (this.tarSpr instanceof Laya.SkinnedMeshSprite3D)
	            this.info0["alpha"] = this.tarSpr.skinnedMeshRenderer.material.albedoColorA;
	        this.haveCol = true;
	    }
	    colorTo(endr, endg, endb) {
	        this.info["r"] = endr;
	        this.info["g"] = endg;
	        this.info["b"] = endb;
	        if (this.tarSpr instanceof Laya.MeshSprite3D) {
	            this.info0["r"] = this.tarSpr.meshRenderer.material.albedoColorR;
	            this.info0["g"] = this.tarSpr.meshRenderer.material.albedoColorG;
	            this.info0["b"] = this.tarSpr.meshRenderer.material.albedoColorB;
	        }
	        else if (this.tarSpr instanceof Laya.SkinnedMeshSprite3D) {
	            this.info0["r"] = this.tarSpr.skinnedMeshRenderer.material.albedoColorR;
	            this.info0["g"] = this.tarSpr.skinnedMeshRenderer.material.albedoColorG;
	            this.info0["b"] = this.tarSpr.skinnedMeshRenderer.material.albedoColorB;
	        }
	        this.haveCol = true;
	    }
	    scaleLocalTo(ends) {
	        this.info["scaleX_l"] = ends;
	        this.info["scaleY_l"] = ends;
	        this.info["scaleZ_l"] = ends;
	        this.info0["scaleX_l"] = this.tarSpr.transform.localScale.x;
	        this.info0["scaleY_l"] = this.tarSpr.transform.localScale.y;
	        this.info0["scaleZ_l"] = this.tarSpr.transform.localScale.z;
	        this.haveScaleLocal = true;
	    }
	    rotateTo(rx, ry, rz, isRadus) {
	        if (isRadus == false) {
	            rx = rx / 180.0 * Math.PI;
	            ry = ry / 180.0 * Math.PI;
	            rz = rz / 180.0 * Math.PI;
	        }
	        this.info["rotationX"] = rx;
	        this.info["rotationY"] = ry;
	        this.info["rotationZ"] = rz;
	        this.info0["rotationX"] = this.tarSpr.transform.rotation.x;
	        this.info0["rotationY"] = this.tarSpr.transform.rotation.y;
	        this.info0["rotationZ"] = this.tarSpr.transform.rotation.z;
	        this.haveRotate = true;
	    }
	    rotateLocalTo(rx, ry, rz, isRadus) {
	        if (isRadus == true) {
	            rx = rx * 180.0 / Math.PI;
	            ry = ry * 180.0 / Math.PI;
	            rz = rz * 180.0 / Math.PI;
	        }
	        this.info["rotationX_l"] = rx;
	        this.info["rotationY_l"] = ry;
	        this.info["rotationZ_l"] = rz;
	        this.info0["rotationX_l"] = this.tarSpr.transform.localRotationEulerX;
	        this.info0["rotationY_l"] = this.tarSpr.transform.localRotationEulerY;
	        this.info0["rotationZ_l"] = this.tarSpr.transform.localRotationEulerZ;
	        this.haveRotateLocal = true;
	    }
	}

	var Handler$1 = Laya.Handler;
	class Juggler {
	    constructor() {
	        this.countID = 1;
	        this.dics = {};
	    }
	    static getInstance() {
	        if (this.instance == null)
	            this.instance = new Juggler();
	        return this.instance;
	    }
	    delayCall(f, time, ...arg) {
	        this.dics[this.countID] = { "f": f, "arg": arg };
	        Laya.timer.once(time * 1000, this, this.onTimerF, [this.countID], false);
	        return this.countID++;
	    }
	    onTimerF(id) {
	        if (this.dics[id]) {
	            let f = this.dics[id]["f"];
	            let arr = this.dics[id]["arg"];
	            delete this.dics[id];
	            if (arr.length == 0)
	                f();
	            else if (arr.length == 1)
	                f(arr[0]);
	            else if (arr.length == 2)
	                f(arr[0], arr[1]);
	            else if (arr.length == 3)
	                f(arr[0], arr[1], arr[2]);
	            else if (arr.length == 4)
	                f(arr[0], arr[1], arr[2], arr[3]);
	            else
	                f(arr[0], arr[1], arr[2], arr[3], arr[4]);
	        }
	        else {
	        }
	    }
	    removeDelayedCalls(i) {
	        delete this.dics[i];
	    }
	    add(t) {
	        var f = null;
	        if (t.tranType != null && typeof (t.tranType) != "string") {
	            f = t.tranType;
	        }
	        else if (t.tranType == null || t.tranType == "linear") {
	            f = Laya.Ease.linearNone;
	        }
	        else if (t.tranType == Tween.EASE_IN) {
	            f = Laya.Ease.backIn;
	        }
	        else if (t.tranType == Tween.EASE_OUT) {
	            f = Laya.Ease.backOut;
	        }
	        else if (t.tranType == Tween.EASE_IN_OUT) {
	            f = Laya.Ease.backInOut;
	        }
	        else if (t.tranType == Tween.BOUNCE_IN) {
	            f = Laya.Ease.bounceIn;
	        }
	        else if (t.tranType == Tween.BOUNCE_OUT) {
	            f = Laya.Ease.bounceOut;
	        }
	        else if (t.tranType == Tween.BOUNCE_IN_OUT) {
	            f = Laya.Ease.bounceInOut;
	        }
	        else if (t.tranType == Tween.STRONG_In) {
	            f = Laya.Ease.strongIn;
	        }
	        else if (t.tranType == Tween.STRONG_In_Out) {
	            f = Laya.Ease.strongInOut;
	        }
	        else {
	            console.log("没有f!!!!!!!!!!!!");
	        }
	        var comp;
	        if (t.onComplete != null) {
	            comp = Handler$1.create(null, t.onComplete, null, true);
	            if (t.onCompleteArgs != null) {
	                comp.args = t.onCompleteArgs;
	            }
	        }
	        if (t instanceof Tween)
	            t.layaTween = Laya.Tween.to(t.tarObj, t.info, t.time * 1000, f, comp);
	        else if (t instanceof Tween3D)
	            t.layaTween = Laya.Tween.to(t.info0, t.info, t.time * 1000, f, comp);
	    }
	    remove(t) {
	        if (t instanceof Tween && t.layaTween) {
	            Laya.Tween.clear(t.layaTween);
	        }
	    }
	}

	class TmpTween {
	    constructor(t, fend, endvalue = null) {
	        this.isPlay = true;
	        this.iEnter = -1;
	        this.T = t;
	        this.FunEnd = fend;
	        this.endValues = endvalue;
	        this.T.onComplete = this.onEndF.bind(this);
	        Juggler.getInstance().add(this.T);
	        if (this.T instanceof Tween3D) {
	            this.iEnter = ConfigTidus.GFun_MM_addEnter(this.enterF.bind(this));
	        }
	    }
	    enterF() {
	        let t = this.T;
	        if (t.haveMove) {
	            t.tarSpr.transform.position.x = t.info0["x"];
	            t.tarSpr.transform.position.y = t.info0["y"];
	            t.tarSpr.transform.position.z = t.info0["z"];
	            t.tarSpr.transform.position = t.tarSpr.transform.position;
	        }
	        if (t.haveFade) {
	            if (t.tarSpr instanceof Laya.MeshSprite3D)
	                t.tarSpr.meshRenderer.material.albedoColorA = t.info0["alpha"];
	            else if (t.tarSpr instanceof Laya.SkinnedMeshSprite3D)
	                t.tarSpr.skinnedMeshRenderer.material.albedoColorA = t.info0["alpha"];
	        }
	        if (t.haveCol) {
	            if (t.tarSpr instanceof Laya.MeshSprite3D) {
	                t.tarSpr.meshRenderer.material.albedoColorR = t.info0["r"];
	                t.tarSpr.meshRenderer.material.albedoColorG = t.info0["g"];
	                t.tarSpr.meshRenderer.material.albedoColorB = t.info0["b"];
	            }
	            else if (t.tarSpr instanceof Laya.SkinnedMeshSprite3D) {
	                t.tarSpr.skinnedMeshRenderer.material.albedoColorR = t.info0["r"];
	                t.tarSpr.skinnedMeshRenderer.material.albedoColorG = t.info0["g"];
	                t.tarSpr.skinnedMeshRenderer.material.albedoColorB = t.info0["b"];
	            }
	        }
	        if (t.haveScaleLocal) {
	            t.tarSpr.transform.localScale.x = t.info0["scaleX_l"];
	            t.tarSpr.transform.localScale.y = t.info0["scaleY_l"];
	            t.tarSpr.transform.localScale.z = t.info0["scaleZ_l"];
	            t.tarSpr.transform.localScale = t.tarSpr.transform.localScale;
	        }
	        if (t.haveRotate) {
	            t.tarSpr.transform.rotation.x = t.info0["rotationX"];
	            t.tarSpr.transform.rotation.y = t.info0["rotationY"];
	            t.tarSpr.transform.rotation.z = t.info0["rotationZ"];
	            t.tarSpr.transform.rotation = t.tarSpr.transform.rotation;
	        }
	        if (t.haveRotateLocal) {
	            t.tarSpr.transform.localRotationEulerX = t.info0["rotationX_l"];
	            t.tarSpr.transform.localRotationEulerY = t.info0["rotationY_l"];
	            t.tarSpr.transform.localRotationEulerZ = t.info0["rotationZ_l"];
	            t.tarSpr.transform.localRotationEuler = t.tarSpr.transform.localRotationEuler;
	        }
	    }
	    pause() {
	        if (this.isPlay == false || this.T == null)
	            return;
	        if (this.iEnter != -1) {
	            ConfigTidus.GFun_MM_removeEnter(this.iEnter);
	            this.iEnter = -1;
	        }
	        this.isPlay = false;
	        Juggler.getInstance().remove(this.T);
	    }
	    resume() {
	        if (this.isPlay == true || this.T == null)
	            return;
	        this.isPlay = true;
	        Juggler.getInstance().add(this.T);
	    }
	    onEndF() {
	        Juggler.getInstance().remove(this.T);
	        if (this.T instanceof Tween3D) {
	            this.enterF();
	        }
	        if (this.iEnter != -1) {
	            ConfigTidus.GFun_MM_removeEnter(this.iEnter);
	            this.iEnter = -1;
	        }
	        this.T = null;
	        if (this.FunEnd) {
	            if (this.endValues != null)
	                this.FunEnd(this.endValues);
	            else
	                this.FunEnd();
	            this.FunEnd = null;
	        }
	    }
	    destroyF() {
	        this.pause();
	        this.T = null;
	        this.FunEnd = null;
	    }
	}

	class MySlideMC extends Sprite {
	    constructor(spr, cx, cy, rec, showType = "mid", canout = true) {
	        super();
	        this.numOneRow = 1;
	        this.busy = false;
	        this.maxX = 0;
	        this.maxY = 0;
	        this.isDefaultOne = false;
	        this.autoClearLayer = true;
	        this.autoDestroy = false;
	        this.nowFirstIndex = 0;
	        this.noSlide = false;
	        this.orgNoSlide = false;
	        this.needSplitLine = false;
	        this.isDown = false;
	        this.Layer = spr;
	        this.canX = cx;
	        this.canY = cy;
	        this.showRec = rec;
	        this.ST = showType;
	        this.canGoOut = canout;
	        if (this.Layer.parent) {
	            Tool_SpriteUtils.onAddchild_ChangeParent(this, this.Layer);
	            this.setValue("rec", rec);
	        }
	        else {
	            this.addChild(this.Layer);
	        }
	        var downOne = this.Layer.getChildAt(0);
	        if (downOne.name == null || downOne.name == "bg") {
	            this.addChild(downOne);
	        }
	        this.MME = new MyMouseManager(this.Layer);
	        this.MME.stopEventMop = "永久";
	        this.MME.addDownHandler(this.onDownF.bind(this));
	        this.MME.addSlideHandler(this.slideF.bind(this));
	        this.MME.addClickHandler(this.clickBtn.bind(this));
	    }
	    addedToStageF() {
	        this.setValue("rec", this.showRec);
	    }
	    addClickBtns(arr, fun, val = null) {
	        if (this.Arr_one != arr)
	            this.Arr_one = ConfigTidus.GFun_ToolObj_destroyOne(this.Arr_one);
	        this.Fun_clickBtn = fun;
	        this.Fun_clickValue = val;
	        this.Arr_one = arr;
	        this.onCheckVisible();
	        if (this.needSplitLine) {
	            for (let i = 0; i < this.Arr_one.length; i++) {
	                if (this.Arr_one[i] && this.Arr_one[i] instanceof MySlideMC_defaultOne) {
	                }
	                if (i == 0 || (this.canX && this.Arr_one[i].x != this.Arr_one[i - 1].x) || (this.canY && this.Arr_one[i].y > this.Arr_one[i - 1].y)) {
	                    var line = MySourceManager.getInstance().getObjFromSwf(this.infoSplitLine["swf"], this.infoSplitLine["url"]);
	                    this.Arr_SplitLines.push(line);
	                    this.Layer.addChildAt(line, 0);
	                    if (this.canX) {
	                        line.x = this.Arr_one[i].x + (this.infoSplitLine["x"] - this.Arr_one[0].x);
	                        line.y = this.infoSplitLine["y"];
	                    }
	                    else {
	                        line.x = this.infoSplitLine["x"];
	                        line.y = this.Arr_one[i].y + (this.infoSplitLine["y"] - this.Arr_one[0].y);
	                    }
	                }
	            }
	        }
	        this.checkMINXY();
	    }
	    setValue(type, val = null) {
	        switch (type) {
	            case "rec":
	                if (val != null) {
	                    this.showRec = val;
	                    this.maxX = this.Layer.x;
	                    this.maxY = this.Layer.y;
	                }
	                this.checkMINXY();
	                break;
	            case "按钮集":
	                if (val[0] instanceof Array) {
	                    this.addClickBtns(val[0], val[1], val[2]);
	                }
	                break;
	            case "透明图":
	            case "bg":
	                if (this.imgTouming) {
	                    this.imgTouming.removeFromParent(true);
	                }
	                this.imgTouming = val;
	                this.imgTouming.touchable = true;
	                this.Layer.addChildAt(this.imgTouming, 0);
	                this.rejuseSize();
	                break;
	            case "移动事件":
	                this.Fun_move = val;
	                break;
	            case "down事件":
	                this.Fun_down = val;
	                break;
	            case "x滑动":
	                this.canX = val;
	                this.rejuseSize();
	                break;
	            case "y滑动":
	                this.canY = val;
	                this.rejuseSize();
	                break;
	            case "回弹":
	                this.canGoOut = val;
	                break;
	            case "底层":
	                this.addChildAt(val, 0);
	                break;
	            case "点击事件":
	                this.Fun_click = val;
	                break;
	            case "暂停":
	                this.busy = val;
	                break;
	            case "滑动条":
	                this.Scroll = val;
	                if (this.Scroll) {
	                    if (this.canX)
	                        this.Scroll.init(-this.minX, this.showRec.width);
	                    else
	                        this.Scroll.init(-this.minY, this.showRec.height);
	                }
	                break;
	            case "改变x":
	                if (this.canX) {
	                    if (val > this.maxX)
	                        val = this.maxX;
	                    else if (val < this.minX)
	                        val = this.minX;
	                }
	                this.Layer.x = val;
	                this.xyChange();
	                break;
	            case "改变y":
	                if (this.canY) {
	                    if (val > this.maxY)
	                        val = this.maxY;
	                    else if (val < this.minY)
	                        val = this.minY;
	                }
	                this.Layer.y = val;
	                this.xyChange();
	                break;
	            case "自动销毁":
	                this.autoDestroy = true;
	                break;
	            case "选中状态":
	                this.mcOn = val;
	                this.Layer.addChild(this.mcOn);
	                if (this.mcOn instanceof SwfMovieClip$1) {
	                    this.mcOn.loop = true;
	                    this.mcOn.play();
	                }
	                this.mcOn.visible = false;
	                break;
	            case "分割线":
	                this.needSplitLine = true;
	                this.Arr_SplitLines = [];
	                this.infoSplitLine = val;
	                break;
	        }
	    }
	    xyChange() {
	        if (this.Fun_move)
	            this.Fun_move();
	        if (this.Scroll) {
	            if (this.canY)
	                this.Scroll.changeNow(-this.Layer.y);
	            else
	                this.Scroll.changeNow(-this.Layer.x);
	        }
	        this.onCheckVisible();
	    }
	    checkMINXY() {
	        this.minX = this.maxX;
	        this.minY = this.maxY;
	        if (this.Canv) {
	            this.Canv = null;
	            this.mask = null;
	        }
	        this.scrollRect = new Laya.Rectangle(this.showRec.x, this.showRec.y, this.showRec.width, this.showRec.height);
	        if (this.ST == "mid") {
	            this.minX += (this.showRec.width >> 1) - this.Layer.width;
	            let h = this.Layer.height;
	            if (this.Arr_one && this.Arr_one[this.Arr_one.length - 1]) {
	                if (h < this.Arr_one[this.Arr_one.length - 1].y + 10) {
	                    console.error("计算Layer的高度错误，比最后一个one的高度还小！");
	                    h = this.Arr_one[this.Arr_one.length - 1].y + 50;
	                }
	            }
	            this.minY += (this.showRec.height >> 1) - h;
	        }
	        else if (this.ST == "max") {
	            this.minX -= this.Layer.width;
	            this.minY -= this.Layer.height;
	        }
	        else {
	            this.minX -= this.Layer.width - this.showRec.width;
	            this.minY -= this.Layer.height - this.showRec.height;
	        }
	        if (this.minX > this.maxX)
	            this.minX = this.maxX;
	        if (this.minY > this.maxY)
	            this.minY = this.maxY;
	        if (this.Scroll) {
	            this.setValue("滑动条", this.Scroll);
	        }
	        this.rejuseSize();
	        this.setValue("改变y", this.Layer.y);
	    }
	    rejuseSize() {
	        if (this.imgTouming) {
	            this.Layer.removeChild(this.imgTouming);
	            if (this.Arr_one == null) {
	                this.imgTouming.width = this.Layer.width;
	                this.imgTouming.height = this.Layer.height;
	            }
	            else if (this.canY == true) {
	                this.imgTouming.width = this.Layer.width;
	                this.imgTouming.height = this.Arr_one[this.Arr_one.length - 1].y + this.Arr_one[this.Arr_one.length - 1].height + 100;
	            }
	            else {
	                this.imgTouming.width = this.Arr_one[this.Arr_one.length - 1].x + this.Arr_one[this.Arr_one.length - 1].width + 100;
	                this.imgTouming.height = this.Layer.height;
	            }
	            this.Layer.addChildAt(this.imgTouming, 0);
	        }
	        this.orgNoSlide = false;
	        if (this.canY && !this.canX) {
	            if (this.Layer.height <= this.showRec.height / 2)
	                this.orgNoSlide = true;
	        }
	        else if (this.canX && !this.canY) {
	            if (this.Layer.width <= this.showRec.width / 2)
	                this.orgNoSlide = true;
	        }
	    }
	    onDownF(p) {
	        this.isDown = true;
	        if (this.tmpT) {
	            this.tmpT.destroyF();
	            this.tmpT = null;
	        }
	    }
	    slideF(dic) {
	        if (this.busy || this.isDown == false || this.noSlide == true || this.orgNoSlide == true)
	            return;
	        if (this.MME == null) {
	            console.log("MME == NULL !!!!!");
	            return;
	        }
	        if (dic["类型"] == "移动") {
	            if (this.FunStop)
	                this.onTStop();
	            if (this.FunOnBottom)
	                this.FunOnBottom("中断");
	            if (this.FunOnTop)
	                this.FunOnTop("中断");
	            if (this.canX) {
	                if (dic["x"] > 0) {
	                    if (this.Layer.x + dic["x"] >= this.maxX) {
	                        if (this.canGoOut)
	                            this.Layer.x += dic["x"];
	                        else
	                            this.Layer.x = this.maxX;
	                    }
	                    else {
	                        this.Layer.x += dic["x"];
	                    }
	                }
	                else {
	                    if (this.Layer.x + dic["x"] <= this.minX) {
	                        if (this.canGoOut)
	                            this.Layer.x += dic["x"];
	                        else
	                            this.Layer.x = this.minX;
	                    }
	                    else {
	                        this.Layer.x += dic["x"];
	                    }
	                }
	            }
	            if (this.canY) {
	                if (dic["y"] > 0) {
	                    if (this.Layer.y + dic["y"] >= this.maxY) {
	                        if (this.canGoOut)
	                            this.Layer.y += dic["y"];
	                        else
	                            this.Layer.y = this.maxY;
	                    }
	                    else {
	                        this.Layer.y += dic["y"];
	                    }
	                }
	                else {
	                    if (this.Layer.y + dic["y"] <= this.minY) {
	                        if (this.canGoOut)
	                            this.Layer.y += dic["y"];
	                        else
	                            this.Layer.y = this.minY;
	                    }
	                    else {
	                        this.Layer.y += dic["y"];
	                    }
	                }
	            }
	            this.xyChange();
	        }
	        else if (dic["类型"] == "滑动") {
	            if (this.FunOnBottom && this.canY) {
	                if (dic["y"] > 0 && this.Layer.y >= this.maxY) {
	                    if (this.FunOnBottom("开始") == true)
	                        return;
	                }
	            }
	            if (this.FunOnTop && this.canY) {
	                if (dic["y"] < 0 && this.Layer.y <= this.minY) {
	                    if (this.FunOnBottom("开始") == true)
	                        return;
	                }
	            }
	            this.isDown = false;
	            let t;
	            let endx = this.Layer.x;
	            let endy = this.Layer.y;
	            let timex = 0;
	            let timey = 0;
	            if (this.canX && dic["x"] != 0) {
	                timex = Tool_Function.getPerValueInArray(Math.abs(dic["x"]), [0, 0, 5, 0.8, 10, 1.5]);
	                endx = this.Layer.x + dic["x"] * timex * ConfigTidus.FrameRate;
	                if (endx > this.maxX)
	                    endx = this.maxX;
	                if (endx < this.minX)
	                    endx = this.minX;
	            }
	            if (this.canY && dic["y"] != 0) {
	                timey = Tool_Function.getPerValueInArray(Math.abs(dic["y"]), [0, 0, 5, 0.8, 10, 1.5]);
	                endy = this.Layer.y + dic["y"] * timey * ConfigTidus.FrameRate;
	                if (endy > this.maxY)
	                    endy = this.maxY;
	                if (endy < this.minY)
	                    endy = this.minY;
	            }
	            let time = timex >= timey ? timex : timey;
	            if (this.canGoOut == false) {
	                t = new Tween(this.Layer, time, Tween.STRONG_In_Out);
	            }
	            else {
	                t = new Tween(this.Layer, time, Tween.BOUNCE_OUT);
	            }
	            t.moveTo(endx, endy);
	            this.tmpT = new TmpTween(t, this.onTStop.bind(this));
	        }
	    }
	    onSlideToOne(n) {
	        if (this.Arr_one == null || this.Arr_one[n] == null)
	            return;
	        var one = this.Arr_one[n];
	        if (this.canX == true) {
	            this.setValue("改变x", -one.x + 20);
	        }
	        else {
	            this.setValue("改变y", -one.y + 20);
	        }
	    }
	    onCheckVisible() {
	        if (this.Arr_one == null)
	            return;
	        var i;
	        if (this.canX && this._w0 > 0) {
	            for (i = 0; i < this.Arr_one.length; i++) {
	                if (this.Arr_one[i] == null)
	                    continue;
	                if (this.Arr_one[i].x + this.Layer.x < this.showRec.x - this._w0 || this.Arr_one[i].x + this.Layer.x > this.showRec.width) {
	                    this.Arr_one[i].visible = false;
	                }
	                else {
	                    this.Arr_one[i].visible = true;
	                }
	            }
	        }
	        if (this.canY && this._h0 > 0) {
	            if (this.isDefaultOne == true) {
	                for (i = 0; i < this.Arr_one.length; i++) {
	                    if (this.Arr_one[i] == null)
	                        continue;
	                    if (this.Arr_one[i].y + this.Arr_one[i].height + this.Layer.y < this.showRec.y - this._h0 || this.Arr_one[i].y + this.Layer.y > this.showRec.height) {
	                        this.Arr_one[i].visible = false;
	                    }
	                    else {
	                        this.Arr_one[i].visible = true;
	                    }
	                }
	                return;
	            }
	            for (i = 0; i < this.Arr_one.length; i++) {
	                if (this.Arr_one[i] == null)
	                    continue;
	                if (this.Arr_one[i].y + this.Layer.y < this.showRec.y - this._h0 || this.Arr_one[i].y + this.Layer.y > this.showRec.height) {
	                    this.Arr_one[i].visible = false;
	                }
	                else {
	                    this.Arr_one[i].visible = true;
	                }
	            }
	        }
	    }
	    onStopSlide() {
	        if (this.MME) {
	        }
	    }
	    set pause(val) {
	        this.setValue("暂停", val);
	        if (val == true) {
	            this.onStopSlide();
	        }
	    }
	    onTStop() {
	        if (this.FunStop)
	            this.FunStop();
	    }
	    clickNoneF(p = null) {
	        if (this.busy || this.Fun_click == null)
	            return;
	        if (p) {
	            p.x -= this.Layer.x;
	            p.y -= this.Layer.y;
	        }
	        this.Fun_click(p);
	    }
	    clickBtn(p) {
	        if (this.busy)
	            return;
	        let screenp = { "x": p.x + this.Layer.x, "y": p.y + this.Layer.y };
	        if (screenp.x < this.showRec.x || screenp.x > this.showRec.x + this.showRec.width || screenp.y < this.showRec.y || screenp.y > this.showRec.y + this.showRec.height) {
	            this.clickNoneF(p);
	            return;
	        }
	        if (this.Fun_clickBtn == null) {
	            this.clickNoneF(p);
	            return;
	        }
	        var n = -1;
	        for (let i = 0; i < this.Arr_one.length; i++) {
	            if (this.Arr_one[i] == null || this.Arr_one[i].visible == false)
	                continue;
	            let rec = Tool_SpriteUtils.getBounds(this.Arr_one[i], this.Layer);
	            if (p.x >= rec.x && p.x <= rec.right && p.y >= rec.y && p.y <= rec.bottom) {
	                n = i;
	                break;
	            }
	        }
	        if (n == -1) {
	            this.clickNoneF(p);
	            return;
	        }
	        if (this.mcOn) {
	            this.mcOn.visible = true;
	            this.Layer.addChild(this.mcOn);
	            this.mcOn.x = this.Arr_one[n].x;
	            this.mcOn.y = this.Arr_one[n].y;
	        }
	        if (this.Fun_clickBtn) {
	            if (this.autoDestroy == null) {
	                if (this.Fun_clickValue == null)
	                    this.Fun_clickBtn(n);
	                else
	                    this.Fun_clickBtn(this.Fun_clickValue, n);
	            }
	            else {
	                if (this.Fun_clickValue == null)
	                    this.Fun_clickBtn(this.nowFirstIndex + n);
	                else
	                    this.Fun_clickBtn(this.Fun_clickValue, this.nowFirstIndex + n);
	            }
	        }
	    }
	    addMouseRollController() {
	        this.MME.addRollHandler((delta) => {
	            var old = this.canGoOut;
	            this.canGoOut = false;
	            var L = 50;
	            if (delta > 0) {
	                if (this.canX) {
	                    this.slideF({ "类型": "移动", "x": -L });
	                }
	                else {
	                    this.slideF({ "类型": "移动", "y": -L });
	                }
	            }
	            else {
	                if (this.canX) {
	                    this.slideF({ "类型": "移动", "x": L });
	                }
	                else {
	                    this.slideF({ "类型": "移动", "y": L });
	                }
	            }
	            this.canGoOut = old;
	        });
	    }
	    removeAt(n, autoDestroyF = false) {
	        if (this.isDefaultOne) {
	            var needh = this.Arr_one[n].H;
	            if (n > 0)
	                needh += this.Arr_one[n].y - this.Arr_one[n - 1].y - this.Arr_one[n - 1].H;
	            for (let i = this.Arr_one.length - 1; i > n; i--) {
	                this.Arr_one[i].y -= needh;
	            }
	        }
	        else {
	            for (let i = this.Arr_one.length - 1; i > n; i--) {
	                this.Arr_one[i].x = this.Arr_one[i - 1].x;
	                this.Arr_one[i].y = this.Arr_one[i - 1].y;
	            }
	        }
	        if (autoDestroyF) {
	            ConfigTidus.GFun_ToolObj_destroyOne(this.Arr_one[n]);
	        }
	        this.Arr_one.splice(n, 1);
	        this.checkMINXY();
	        if (this.canX) {
	            if (this.Layer.x > this.maxX) {
	                this.Layer.x = this.maxX;
	            }
	            else if (this.Layer.x < this.minX) {
	                this.Layer.x = this.minX;
	            }
	        }
	        if (this.canY) {
	            if (this.Layer.y > this.maxY) {
	                this.Layer.y = this.maxY;
	            }
	            else if (this.Layer.y < this.minY) {
	                this.Layer.y = this.minY;
	            }
	        }
	        this.onCheckVisible();
	    }
	    moveAllOnes(_x, _y, start) {
	        for (var i = start; i < this.Arr_one.length; i++) {
	            this.Arr_one[i].x += _x;
	            this.Arr_one[i].y += _y;
	        }
	        this.onCheckVisible();
	    }
	    getNowShowArea() {
	        return new Laya.Rectangle(-this.Layer.x, -this.Layer.y, this.showRec.width, this.showRec.height);
	    }
	    getNowFirstOne() {
	        var rec = this.getNowShowArea();
	        if (this.Arr_one) {
	            for (let i = 0; i < this.Arr_one.length; i++) {
	                if (this.Arr_one[i] == null)
	                    continue;
	                if (this.canX) {
	                    if (this.Arr_one[i].x >= rec.x)
	                        return this.Arr_one[i];
	                }
	                else {
	                    if (this.Arr_one[i].y >= rec.y)
	                        return this.Arr_one[i];
	                }
	            }
	        }
	        return null;
	    }
	    getValue(want) {
	    }
	    destroyF() {
	        if (this.MME) {
	            this.MME.destroyF();
	            this.MME = null;
	        }
	        if (this.Canv) {
	            this.Canv.graphics.clear();
	            this.Canv.destroy();
	            this.Canv = null;
	        }
	        if (this.Scroll) {
	            this.Scroll.destroyF();
	            this.Scroll = null;
	        }
	        this.mcOn = ConfigTidus.GFun_ToolObj_destroyOne(this.mcOn);
	        if (this.autoClearLayer) {
	            this.Arr_one = ConfigTidus.GFun_ToolObj_destroyOne(this.Arr_one);
	            if (this.Layer)
	                this.Layer.removeFromParent(true);
	        }
	        else {
	            if (this.Layer) {
	                this.Layer.removeFromParent(false);
	            }
	            this.Arr_one = null;
	        }
	        this.Layer = null;
	        this.Fun_clickBtn = ConfigTidus.GFun_ToolObj_destroyOne(this.Fun_clickBtn);
	        this.Fun_click = ConfigTidus.GFun_ToolObj_destroyOne(this.Fun_click);
	        this.Fun_move = ConfigTidus.GFun_ToolObj_destroyOne(this.Fun_move);
	        this.Fun_down = ConfigTidus.GFun_ToolObj_destroyOne(this.Fun_down);
	        if (this.FunOnBottom) {
	            this.FunOnBottom = null;
	        }
	        if (this.FunOnTop) {
	            this.FunOnTop = null;
	        }
	        if (this.FunStop) {
	            this.FunStop = null;
	        }
	        this.Arr_SplitLines = ConfigTidus.GFun_ToolObj_destroyOne(this.Arr_SplitLines);
	        super.destroyF();
	    }
	}

	class MySelectSingleBTN {
	    constructor(spr, fchange, id, canMulti = false, vtype = 0) {
	        this.Now = -1;
	        this.Busy = false;
	        this.canMulti = false;
	        this._visible = true;
	        this.ID = id;
	        this.FunChange = fchange;
	        this.FunChange.once = false;
	        this.Spr = spr;
	        this.canMulti = canMulti;
	        this.funValueType = vtype;
	        if (this.canMulti == true)
	            this.Dic_Down = {};
	    }
	    static creatBtnFromSprite(spr, fchange, id = null, strFront = "btn_", canMulti = false, vType = this.ValueType_ArrayIndex) {
	        var Arr = this.getArrFromSpr(spr, strFront, vType == this.ValueType_ArrayIndex);
	        var mbtn = new MySelectSingleBTN(spr, Laya.Handler.create(this, fchange), id, canMulti, vType);
	        mbtn.initF(Arr);
	        return mbtn;
	    }
	    static getArrFromSpr(spr, strFront = "btn_", isArrayIndex = false) {
	        var Arr = [];
	        var dicName = {};
	        for (var i = 0; i < spr.numChildren; i++) {
	            var child = spr.getChildAt(i);
	            if (child instanceof SwfMovieClip$1 && child.name.indexOf(strFront) == 0) {
	                var btn = new BTN_Starling(child);
	                btn.onChangeFrame(BTN_Starling.Frame_Nor);
	                if (isArrayIndex) {
	                    var index = Tool_Function.onForceConvertType(child.name.substr(4));
	                    Arr[index] = btn;
	                    dicName[index] = child.name;
	                }
	                else {
	                    Arr.push(btn);
	                    dicName[Arr.length - 1] = child.name;
	                }
	            }
	        }
	        return [Arr, dicName];
	    }
	    static creatBtnFromArr(spr, Arr, fchange, id = null, canMulti = false) {
	        for (var i = 0; i < Arr.length; i++) {
	            if (Arr[i] == null)
	                continue;
	            if (Arr[i] instanceof BTN_Starling)
	                continue;
	            Arr[i] = new BTN_Starling(Arr[i]);
	        }
	        var mbtn = new MySelectSingleBTN(spr, fchange, id, canMulti);
	        mbtn.initF([Arr]);
	        return mbtn;
	    }
	    initF(arr) {
	        this.Arr = arr[0];
	        this.DicName = arr[1];
	        if (this.Arr == null)
	            return;
	        if (this.Arr.length == 0) {
	            if (this.Spr instanceof SwfMovieClip$1) {
	                this.canMulti = true;
	                this.Dic_Down = {};
	                this.Btn = new BTN_Starling(this.Spr);
	                this.Btn.autoChangeFrame = false;
	                this.Btn.initTouch(null, null, this.clickF.bind(this), [0]);
	                this.Arr[0] = this.Btn;
	            }
	            return;
	        }
	        if (this.DicName) {
	            for (var n in this.DicName) {
	                this.Arr[n].name = this.DicName[n];
	                var tmp = this.Spr.getChildByName("V_" + this.DicName[n].substr(4));
	                if (tmp) {
	                    if (this.infoSecond == null) {
	                        this.infoSecond = { "index": 999 };
	                    }
	                    this.infoSecond[n] = { "x": tmp.x, "y": tmp.y, "swf": tmp.swfName, "url": tmp.classLink, "sx": tmp.scaleX, "sy": tmp.scaleY };
	                    var index = this.Spr.getChildIndex(tmp);
	                    if (index < this.infoSecond["index"]) {
	                        this.infoSecond["index"] = index;
	                    }
	                    tmp = ConfigTidus.GFun_ToolObj_destroyOne(tmp);
	                }
	            }
	        }
	        this.BC = new BTNControllerStarling(this.Spr, this.Arr, this.clickF.bind(this), null, null);
	        this.BC.autoChangeFrame = false;
	        var tmpL = this.Spr.getChildByName("_area");
	        if (tmpL == null)
	            return;
	        var meta;
	        if (this.Spr instanceof Sprite && this.Spr.spriteData[0] instanceof Array == false)
	            meta = this.Spr.spriteData[0];
	        if (meta == null)
	            return;
	        this.BC = ConfigTidus.GFun_ToolObj_destroyOne(this.BC);
	        var layer = new Sprite();
	        this.Spr.addChild(layer);
	        for (var i = 0; i < this.Arr.length; i++) {
	            if (this.Arr[i])
	                layer.addChild(this.Arr[i]);
	        }
	        this.BC = new BTNControllerStarling(layer, this.Arr, this.clickF.bind(this), null, null);
	        this.BC.autoChangeFrame = false;
	        var canx = meta["方向"] == "x";
	        var cany = !canx;
	        var slideType = meta["边界"] ? meta["边界"] : "mid";
	        this.McSlide = new MySlideMC(layer, canx, cany, new Laya.Rectangle(tmpL.x, tmpL.y, tmpL.width, tmpL.height), slideType);
	        if (meta["透明区域"])
	            this.McSlide.setValue("透明图", tmpL);
	        else
	            tmpL.removeFromParent(true);
	        if (canx == true) {
	            addCanSlide("左");
	            addCanSlide("右");
	        }
	        if (cany == true) {
	            addCanSlide("上");
	            addCanSlide("下");
	        }
	        if (this.DicCanSlideFlagmc)
	            this.McSlide.setValue("移动事件", onSlideF);
	        function addCanSlide(dir) {
	            var mc = this.Spr.getChildByName("_可滑动" + dir);
	            if (mc == null)
	                return;
	            if (this.DicCanSlideFlagmc == null)
	                this.DicCanSlideFlagmc = {};
	            mc.touchable = false;
	            this.Spr.addChild(mc);
	            this.DicCanSlideFlagmc[dir] = mc;
	        }
	        function onSlideF() {
	            if (this.McSlide.Layer.y < this.McSlide.maxY) {
	                if (this.DicCanSlideFlagmc["上"])
	                    this.DicCanSlideFlagmc["上"].visible = true;
	            }
	            else {
	                if (this.DicCanSlideFlagmc["上"])
	                    this.DicCanSlideFlagmc["上"].visible = false;
	            }
	            if (this.McSlide.Layer.y > this.McSlide.minY) {
	                if (this.DicCanSlideFlagmc["下"])
	                    this.DicCanSlideFlagmc["下"].visible = true;
	            }
	            else {
	                if (this.DicCanSlideFlagmc["下"])
	                    this.DicCanSlideFlagmc["下"].visible = false;
	            }
	        }
	    }
	    clickNameF(Name, needEvent = true) {
	        if (this.Busy)
	            return;
	        for (var n in this.DicName) {
	            if (this.DicName[n] == Name) {
	                this.clickF(Tool_Function.onForceConvertType(n), needEvent);
	                return;
	            }
	        }
	    }
	    clickF(n, needEvent = true) {
	        if (this.Busy)
	            return;
	        if (this.Arr == null) {
	            if (this.Spr)
	                this.initF(MySelectSingleBTN.getArrFromSpr(this.Spr, "btn_", this.funValueType == MySelectSingleBTN.ValueType_ArrayIndex));
	        }
	        if (this.canMulti == false) {
	            if (n == this.Now)
	                return;
	            if (this.Now >= 0)
	                this.setFrame(this.Now, BTN_Starling.Frame_Nor);
	            this.Now = n;
	            this.setFrame(this.Now, BTN_Starling.Frame_Selected);
	            this.onShowSecond();
	        }
	        else {
	            if (this.Dic_Down[n] == true)
	                this.setNow(n, false);
	            else
	                this.setNow(n, true);
	        }
	        if (this.FunChange && needEvent) {
	            var val;
	            if (this.funValueType == MySelectSingleBTN.ValueType_Name && this.DicName[n])
	                val = this.DicName[n];
	            else
	                val = n;
	            if (this.Arr.length == 1 && this.Dic_Down) {
	                val = this.Dic_Down[n];
	            }
	            if (this.ID == -1 || this.ID == null)
	                this.FunChange.runWith(val);
	            else
	                this.FunChange.runWith([this.ID, val]);
	        }
	    }
	    getAllSelected(useName = false) {
	        if (this.canMulti == false) {
	            if (this.Now <= 0)
	                return null;
	            return { Now: true };
	        }
	        var dic = {};
	        for (var po in this.Dic_Down) {
	            if (this.Dic_Down[po] == true) {
	                if (useName == false)
	                    dic[po] = true;
	                else
	                    dic[this.DicName[po]] = true;
	            }
	        }
	        return dic;
	    }
	    getNow(now = -1) {
	        if (this.canMulti == false)
	            return this.Now;
	        return this.Dic_Down[now];
	    }
	    getNowName() {
	        if (this.canMulti == false)
	            return this.DicName[this.Now];
	        if (this.Now == -1)
	            return null;
	        return this.Now + "";
	    }
	    setNow(_now, on = true) {
	        var po = -1;
	        if (typeof _now == "string") {
	            for (var n in this.DicName) {
	                if (this.DicName[n] == _now) {
	                    po = Tool_Function.onForceConvertType(n);
	                    break;
	                }
	            }
	        }
	        else {
	            po = _now;
	        }
	        if (po == -1)
	            return;
	        if (this.canMulti == false) {
	            if (po == this.Now)
	                return;
	            if (this.Now >= 0)
	                this.setFrame(this.Now, BTN_Starling.Frame_Nor);
	            this.Now = po;
	            this.setFrame(this.Now, BTN_Starling.Frame_Selected);
	            this.onShowSecond();
	        }
	        else {
	            if (this.Dic_Down[po] == on)
	                return;
	            this.Dic_Down[po] = on;
	            this.Now = on == true ? po : -1;
	            if (on == true)
	                this.setFrame(po, BTN_Starling.Frame_Selected);
	            else
	                this.setFrame(po, BTN_Starling.Frame_Nor);
	        }
	    }
	    getBTN_by_Name(str) {
	        if (this.DicName == null)
	            return null;
	        for (var po in this.DicName) {
	            if (this.DicName[po] == str)
	                return this.Arr[Tool_Function.onForceConvertType(po)];
	        }
	        return null;
	    }
	    onShowSecond() {
	        this.sprSecond = ConfigTidus.GFun_ToolObj_destroyOne(this.sprSecond);
	        if (this.infoSecond && this.infoSecond[this.Now]) {
	            this.sprSecond = MySourceManager.getInstance().getObjFromSwf(this.infoSecond[this.Now]["swf"], this.infoSecond[this.Now]["url"]);
	            if (this.sprSecond) {
	                this.sprSecond.x = this.infoSecond[this.Now].x;
	                this.sprSecond.y = this.infoSecond[this.Now].y;
	                this.sprSecond.scaleX = this.infoSecond[this.Now].sx;
	                this.sprSecond.scaleY = this.infoSecond[this.Now].sy;
	                this.Spr.addChildAt(this.sprSecond, this.infoSecond["index"]);
	            }
	        }
	    }
	    setFrame(n, f) {
	        if (this.Arr && this.Arr[n]) {
	            this.Arr[n].onChangeFrame(f);
	        }
	    }
	    get visible() { return this._visible; }
	    set visible(value) {
	        this._visible = value;
	        if (this.Btn) {
	            this.Btn.visible = value;
	        }
	        if (this.BC) {
	            this.BC.visible = value;
	        }
	    }
	    destroyF() {
	        this.Spr = ConfigTidus.GFun_ToolObj_destroyOne(this.Spr);
	        this.BC = ConfigTidus.GFun_ToolObj_destroyOne(this.BC);
	        this.Btn = ConfigTidus.GFun_ToolObj_destroyOne(this.Btn);
	        this.Arr = ConfigTidus.GFun_ToolObj_destroyOne(this.Arr);
	        this.McSlide = ConfigTidus.GFun_ToolObj_destroyOne(this.McSlide);
	        this.DicCanSlideFlagmc = ConfigTidus.GFun_ToolObj_destroyOne(this.DicCanSlideFlagmc);
	        this.sprSecond = ConfigTidus.GFun_ToolObj_destroyOne(this.sprSecond);
	        if (this.FunChange) {
	            this.FunChange.clear();
	            this.FunChange = null;
	        }
	    }
	}
	MySelectSingleBTN.ValueType_ArrayIndex = 0;
	MySelectSingleBTN.ValueType_Name = 1;

	class MyViewSelectBtnController {
	    constructor(view, f) {
	        this.DicMc = new Map();
	        this.Fun = f;
	        var l = view.numChildren;
	        for (var i = 0; i < l; i++) {
	            var obj = view.getChildAt(i);
	            if (obj instanceof Sprite && obj.name && obj.name.indexOf("SBtn_") == 0 && obj.name.length > 5) {
	                var meta = obj.metaData;
	                var Name = obj.name.substr(5);
	                var strFront = "btn_";
	                if (meta && meta["前缀"])
	                    strFront = meta["前缀"];
	                var canMulti = false;
	                if (meta && meta["多选"] != null)
	                    canMulti = meta["多选"];
	                var vType = MySelectSingleBTN.ValueType_Name;
	                if (meta && meta["参数"] != null)
	                    vType = meta["参数"];
	                var SB = MySelectSingleBTN.creatBtnFromSprite(obj, this.onClickF.bind(this), Name, strFront, canMulti, vType);
	                this.DicMc.set(Name, SB);
	                if (meta) {
	                    if (meta["初始"] != null)
	                        SB.setNow(meta["初始"]);
	                    else if (meta["初始且事件"] != null) {
	                        if (typeof meta["初始且事件"] == "number")
	                            SB.clickF(meta["初始且事件"]);
	                        else
	                            SB.clickNameF(meta["初始且事件"]);
	                    }
	                }
	            }
	        }
	    }
	    onClickF(Name, n) {
	        if (this.Fun == null || n == null || n == -1)
	            return;
	        this.Fun([Name, n]);
	    }
	    setNow(Name, now, on = true) {
	        if (Name && Name.indexOf("SBtn_") == 0)
	            Name = Name.substr(5);
	        var key;
	        for (var i = 0; i < this.DicMc.keys.length; i++) {
	            key = this.DicMc.keys[i];
	            if (Name == null || Name == key)
	                this.getSBtn(key).setNow(now, on);
	        }
	    }
	    clickF(Name, n) {
	        if (Name && Name.indexOf("SBtn_") == 0)
	            Name = Name.substr(5);
	        var SB = this.DicMc.get(Name);
	        if (SB) {
	            if (SB.Arr.length == 1) {
	                SB.clickF(0);
	            }
	            else if (typeof n == "number")
	                SB.clickF(n);
	            else
	                SB.clickNameF(n);
	        }
	    }
	    getSBtn(Name) {
	        if (Name && Name.indexOf("SBtn_") == 0)
	            Name = Name.substr(5);
	        return this.DicMc.get(Name);
	    }
	    onVisibleF(Name, visible) {
	        if (Name && Name.indexOf("SBtn_") == 0)
	            Name = Name.substr(5);
	        for (let i = 0; i < this.DicMc.keys.length; i++) {
	            let key = this.DicMc.keys[i];
	            if (Name == null || Name == key)
	                this.getSBtn(key).visible = visible;
	        }
	    }
	    onPauseF(Name, pause) {
	        if (Name && Name.indexOf("SBtn_") == 0)
	            Name = Name.substr(5);
	        var key;
	        for (var i = 0; i < this.DicMc.keys.length; i++) {
	            key = this.DicMc.keys[i];
	            if (Name == null || Name == key)
	                this.getSBtn(key).Busy = pause;
	        }
	    }
	    destroyF() {
	        if (this.Fun != null) {
	            this.Fun = null;
	        }
	        this.DicMc = ConfigTidus.GFun_ToolObj_destroyOne(this.DicMc);
	    }
	}

	class MyViewNumsController {
	    constructor(spr, fName = null, swf = null) {
	        this.Nums = Tool_SpriteUtils.onGetAll_ImageNum(spr, fName, swf);
	    }
	    getImageNum(Name) {
	        return this.Nums[Name];
	    }
	    getValueOfNum(Name) {
	        if (this.Nums[Name]) {
	            let num = this.getImageNum(Name);
	            return num.NowStr;
	        }
	        return null;
	    }
	    onShowF(txName, val) {
	        if (txName == null) {
	            for (let key in this.Nums) {
	                this.Nums[key].showF(val);
	            }
	            return;
	        }
	        if (this.Nums[txName])
	            this.Nums[txName].showF(val);
	    }
	    setValueToNum(Name, valName, val) {
	        var num = this.getImageNum(Name);
	        if (num) {
	            num.setValue(valName, val);
	        }
	    }
	    destroyF() {
	        this.Nums = ConfigTidus.GFun_ToolObj_destroyOne(this.Nums);
	    }
	}

	class MyViewPicsController {
	    constructor(spr) {
	        this.dicPics = {};
	        this.sprBack = spr;
	        for (var i = 0; i < this.sprBack.numChildren; i++) {
	            var obj = this.sprBack.getChildAt(i);
	            if (obj.name && obj.name.indexOf("pic_") == 0) {
	                let str = obj.name.substr(4);
	                let arr = str.split("__");
	                let strType = arr[arr.length - 2] + "." + arr[arr.length - 1];
	                arr.splice(arr.length - 2, 2);
	                let add = "";
	                while (arr.length > 0) {
	                    add += arr[0] + "/";
	                    arr.splice(0, 1);
	                }
	                var bg = new Laya.Image(add + strType);
	                if (obj instanceof Sprite)
	                    obj.addChild(bg);
	                else
	                    this.sprBack.addChildAt(bg, i++);
	                this.dicPics[obj.name] = bg;
	            }
	        }
	    }
	    destroyF() {
	        this.sprBack = ConfigTidus.GFun_ToolObj_destroyOne(this.sprBack);
	        this.dicPics = ConfigTidus.GFun_ToolObj_destroyOne(this.dicPics);
	    }
	}

	class MyScoll extends Sprite {
	    constructor(mc) {
	        super();
	        this.NowNum = 0;
	        this.visible = false;
	        if (mc == null) {
	            return;
	        }
	        Tool_SpriteUtils.onAddchild_ChangeParent(this, mc);
	        this.Mc_slide = mc.getChildByName("_滑块");
	        this.Img_back = mc.getChildByName("_back");
	        this.Mc_Area = mc.getChildByName("_区域");
	        this.isXside = this.Img_back.width > this.Img_back.height;
	        this.sMain = this.isXside ? mc.scaleX : mc.scaleY;
	        mc.scaleX = mc.scaleY = 1;
	        this.canScale = this.Mc_slide.classLink.indexOf("s9_") == 0;
	        if (this.isXside) {
	            this.Img_back.scaleX *= this.sMain;
	            if (this.Mc_Area)
	                this.Mc_Area.scaleX *= this.sMain;
	        }
	        else {
	            this.Img_back.scaleY *= this.sMain;
	            if (this.Mc_Area)
	                this.Mc_Area.scaleY *= this.sMain;
	        }
	    }
	    onChangeSlideArea() {
	        if (this.Mc_slide == null)
	            return;
	        if (this.canScale == false) {
	            if (this.Mc_Area) {
	                this.min = this.isXside ? this.Mc_Area.x : this.Mc_Area.y;
	                this.max = this.isXside ? this.Mc_Area.x + this.Mc_Area.width : this.Mc_Area.y + this.Mc_Area.height;
	            }
	            else {
	                this.min = this.isXside ? this.Img_back.x : this.Img_back.y;
	                this.max = this.isXside ? this.Img_back.x + this.Img_back.width : this.Img_back.y + this.Img_back.height;
	            }
	        }
	        else {
	            var per = this.CanNum / (this.AllNum + this.CanNum);
	            if (this.Mc_Area) {
	                if (this.isXside) {
	                    this.Mc_slide.width = this.Mc_Area.width * per;
	                    this.min = this.Mc_Area.x;
	                    this.max = this.min + this.Mc_Area.width - this.Mc_slide.width;
	                }
	                else {
	                    this.Mc_slide.height = this.Mc_Area.height * per;
	                    this.min = this.Mc_Area.y;
	                    this.max = this.min + this.Mc_Area.height - this.Mc_slide.height;
	                }
	            }
	            else {
	                if (this.isXside) {
	                    this.Mc_slide.width = this.Img_back.width * per;
	                    this.min = this.Img_back.x;
	                    this.max = this.min + this.Img_back.width - this.Mc_slide.width;
	                }
	                else {
	                    this.Mc_slide.height = this.Img_back.height * per;
	                    this.min = this.Img_back.y;
	                    this.max = this.min + this.Img_back.height - this.Mc_slide.height;
	                }
	            }
	        }
	    }
	    init(all, can) {
	        this.AllNum = all;
	        this.CanNum = can;
	        var per = can / (all + can);
	        if (this.isMax) {
	            this.visible = false;
	        }
	        else {
	            this.visible = true;
	            this.onChangeSlideArea();
	        }
	        this.changeNow(0);
	    }
	    changeNow(now) {
	        if (this.isMax)
	            return;
	        this.NowNum = now;
	        this.showNowF();
	    }
	    get isMax() {
	        if (this.CanNum <= 0)
	            return true;
	        var per = this.CanNum / (this.AllNum + this.CanNum);
	        if (per >= 0.9)
	            return true;
	        return false;
	    }
	    showNowF() {
	        if (this.Mc_slide == null)
	            return;
	        if (this.isMax) {
	            if (this.isXside)
	                this.Mc_slide.x = this.min;
	            else
	                this.Mc_slide.y = this.min;
	        }
	        else {
	            let per = this.NowNum / this.AllNum;
	            if (per < 0)
	                per = 0;
	            else if (per > 1)
	                per = 1;
	            if (this.isXside)
	                this.Mc_slide.x = this.min + (this.max - this.min) * per;
	            else
	                this.Mc_slide.y = this.min + (this.max - this.min) * per;
	        }
	    }
	    autoRemove() {
	        this.on(Laya.Event.REMOVED, this, this.destroyF);
	    }
	}

	class MyViewSlideMCs {
	    constructor(view) {
	        this.DicMc = {};
	        this.DicOneURL = {};
	        this.Dic_Meta = {};
	        if (view instanceof Sprite)
	            this.swfname = view.swfName;
	        var l = view.numChildren;
	        for (var i = 0; i < l; i++) {
	            var obj = view.getChildAt(i);
	            if (obj.name && obj.name.indexOf("slide_") == 0) {
	                var tmpL = obj.getChildByName("_layer");
	                if (tmpL == null) {
	                    console.log("解析MyViewSlideMCs错误，没有_layer");
	                    continue;
	                }
	                var meta;
	                if (obj instanceof Sprite && (obj.spriteData[0] instanceof Array == false && obj.spriteData[0]["url"] == null)) {
	                    meta = obj.spriteData[0];
	                }
	                if (meta == null) {
	                    meta = {};
	                }
	                var tmpOne = obj.getChildByName("one");
	                var tmpOne2 = obj.getChildByName("one2");
	                var Name = obj.name.substr(6);
	                var canx = meta["方向"] != null ? (meta["方向"] == "x") : (tmpL.width > tmpL.height ? true : false);
	                var cany = !canx;
	                var 滑出 = meta["边界"] ? meta["边界"] : "mid";
	                var Layer = new Sprite();
	                Layer.x = tmpL.x;
	                Layer.y = tmpL.y;
	                tmpL.x = tmpL.y = 0;
	                obj.addChild(Layer);
	                Layer.addChild(tmpL);
	                if (meta["w"]) {
	                    if (this.Dic_Meta[Name] == null)
	                        this.Dic_Meta[Name] = {};
	                    this.Dic_Meta[Name]["w"] = meta["w"];
	                }
	                if (meta["h"]) {
	                    if (this.Dic_Meta[Name] == null)
	                        this.Dic_Meta[Name] = {};
	                    this.Dic_Meta[Name]["h"] = meta["h"];
	                }
	                if (meta["mid"]) {
	                    if (this.Dic_Meta[Name] == null)
	                        this.Dic_Meta[Name] = {};
	                    this.Dic_Meta[Name]["mid"] = meta["mid"];
	                }
	                if (meta["每排数量"]) {
	                    if (this.Dic_Meta[Name] == null)
	                        this.Dic_Meta[Name] = {};
	                    this.Dic_Meta[Name]["每排数量"] = meta["每排数量"];
	                }
	                if (tmpOne) {
	                    obj.removeChild(tmpOne);
	                    this.DicOneURL[Name] = tmpOne.classLink;
	                    if (this.Dic_Meta[Name] == null)
	                        this.Dic_Meta[Name] = {};
	                    this.Dic_Meta[Name]["startX"] = tmpOne.x - Layer.x;
	                    this.Dic_Meta[Name]["startY"] = tmpOne.y - Layer.y;
	                    this.Dic_Meta[Name]["scale"] = tmpOne.scaleX;
	                    if (tmpOne2) {
	                        obj.removeChild(tmpOne2);
	                        if (this.Dic_Meta[Name] == null)
	                            this.Dic_Meta[Name] = {};
	                        if (tmpOne2.y != tmpOne.y && tmpOne.x != tmpOne2.x) {
	                            this.Dic_Meta[Name]["间隔排列"] = true;
	                            this.DicOneURL[Name + "_间隔"] = tmpOne2.classLink;
	                            if (canx == true) {
	                                this.Dic_Meta[Name]["h"] = tmpOne2.y - tmpOne.y;
	                            }
	                            else {
	                                this.Dic_Meta[Name]["w"] = tmpOne2.x - tmpOne.x;
	                            }
	                        }
	                        else {
	                            if (meta["方向"] == null) {
	                                if (Tool_Function.onForceConvertType(tmpOne2.x) == Tool_Function.onForceConvertType(tmpOne.x)) {
	                                    cany = true;
	                                    canx = false;
	                                }
	                                else {
	                                    canx = true;
	                                    cany = false;
	                                }
	                                var tmpone3 = obj.getChildByName("one3");
	                                if (tmpone3) {
	                                    var haveDeferentOne = false;
	                                    var countone = 3;
	                                    while (true) {
	                                        var tmpOneMore = obj.getChildByName("one" + countone);
	                                        if (tmpOneMore == null)
	                                            break;
	                                        if (tmpOneMore.x != tmpOne2.x && tmpOneMore.y != tmpOne2.y) {
	                                            haveDeferentOne = true;
	                                            break;
	                                        }
	                                        countone++;
	                                    }
	                                    if (haveDeferentOne == true) {
	                                        cany = !cany;
	                                        canx = !canx;
	                                    }
	                                }
	                            }
	                            this.Dic_Meta[Name]["w"] = tmpOne2.x - tmpOne.x;
	                            this.Dic_Meta[Name]["h"] = tmpOne2.y - tmpOne.y;
	                            if (cany && tmpOne.x != tmpOne2.x) {
	                                this.Dic_Meta[Name]["mid"] = tmpOne2.x - tmpOne.x - (tmpOne.getBounds().width + tmpOne.getBounds().x);
	                                if (this.Dic_Meta[Name]["每排数量"] == null) {
	                                    this.Dic_Meta[Name]["每排数量"] = Tool_Function.onForceConvertType(tmpL.width / (tmpOne.width + this.Dic_Meta[Name]["mid"]));
	                                    if (tmpL.width % (tmpOne.width + this.Dic_Meta[Name]["mid"]) >= tmpOne.width)
	                                        this.Dic_Meta[Name]["每排数量"] += 1;
	                                }
	                            }
	                            else if (canx && tmpOne.y != tmpOne2.y) {
	                                this.Dic_Meta[Name]["mid"] = tmpOne2.y - tmpOne.y - tmpOne.height;
	                                if (this.Dic_Meta[Name]["每排数量"] == null) {
	                                    this.Dic_Meta[Name]["每排数量"] = Tool_Function.onForceConvertType(tmpL.height / (tmpOne.height + this.Dic_Meta[Name]["mid"]));
	                                    if (tmpL.height % (tmpOne.height + this.Dic_Meta[Name]["mid"]) >= tmpOne.height)
	                                        this.Dic_Meta[Name]["每排数量"] += 1;
	                                }
	                            }
	                        }
	                        countone = 3;
	                        let loopend = false;
	                        while (true) {
	                            let tmpOneMore = obj.getChildByName("one" + countone);
	                            if (tmpOneMore == null)
	                                break;
	                            if (loopend == false) {
	                                if (canx) {
	                                    if (Tool_Function.onForceConvertType(tmpOneMore.x) == Tool_Function.onForceConvertType(tmpOne.x)) {
	                                        if (countone > this.Dic_Meta[Name]["每排数量"])
	                                            this.Dic_Meta[Name]["每排数量"] = countone;
	                                    }
	                                    else {
	                                        this.Dic_Meta[Name]["w"] = tmpOneMore.x - tmpOne.x;
	                                        loopend = true;
	                                    }
	                                }
	                                else {
	                                    if (Tool_Function.onForceConvertType(tmpOneMore.y) == Tool_Function.onForceConvertType(tmpOne.y)) {
	                                        if (countone > this.Dic_Meta[Name]["每排数量"])
	                                            this.Dic_Meta[Name]["每排数量"] = countone;
	                                    }
	                                    else {
	                                        this.Dic_Meta[Name]["h"] = tmpOneMore.y - tmpOne.y;
	                                        loopend = true;
	                                    }
	                                }
	                            }
	                            obj.removeChild(tmpOneMore);
	                            countone++;
	                        }
	                        for (let i2 = 0; i2 < obj.numChildren; i2++) {
	                            let tmpOneMore = obj.getChildAt(i2);
	                            if (tmpOneMore instanceof Sprite && tmpOneMore.classLink == this.DicOneURL[Name] && (tmpOneMore.name == null || tmpOneMore.name.length == 0)) {
	                                obj.removeChild(tmpOneMore);
	                                i2--;
	                            }
	                        }
	                    }
	                    var slide = new MySlideMC(Layer, canx, cany, new Laya.Rectangle(0, 0, tmpL.width, tmpL.height), 滑出);
	                    var tmpScroll = obj.getChildByName("_滑动条");
	                    if (tmpScroll) {
	                        var mcScroll = new MyScoll(tmpScroll);
	                        slide.setValue("滑动条", mcScroll);
	                    }
	                    var tmpBg = obj.getChildByName("bg");
	                    if (tmpBg) {
	                        slide.setValue("bg", tmpBg);
	                    }
	                    var tmpOn = obj.getChildByName("_on");
	                    if (tmpOn) {
	                        slide.setValue("选中状态", tmpOn);
	                    }
	                    var tmpLine = obj.getChildByName("_分割线");
	                    if (tmpLine) {
	                        obj.removeChild(tmpLine);
	                        slide.setValue("分割线", { "swf": tmpLine.swfName, "url": tmpLine.classLink, "x": tmpLine.x, "y": tmpLine.y });
	                    }
	                    slide.name = obj.name;
	                    if (this.Dic_Meta[Name]["每排数量"] != null)
	                        slide.numOneRow = this.Dic_Meta[Name]["每排数量"];
	                    slide.visible = false;
	                    this.DicMc[Name] = slide;
	                }
	            }
	        }
	    }
	    doName(Name) {
	        if (Name == null || Name.indexOf("slide_") != 0)
	            return Name;
	        return Name.substr(6);
	    }
	    getSlide(Name) {
	        if (Name == null) {
	            for (let key in this.DicMc) {
	                return this.DicMc[key];
	            }
	        }
	        Name = this.doName(Name);
	        return this.DicMc[Name];
	    }
	    getDefaultOne(swf, Name = null) {
	        Name = this.doName(Name);
	        let spr = this.getSlideOne_ByNameFromSwf(swf, Name);
	        var one = new MySlideMC_defaultOne();
	        if (spr) {
	            one.initF(spr);
	        }
	        return one;
	    }
	    getSlideOne_ByNameFromSwf(swf, Name) {
	        Name = this.doName(Name);
	        let url = this.getOneClass_ByName(Name);
	        if (swf == null)
	            swf = this.swfname;
	        if (url) {
	            return MySourceManager.getInstance().getSprFromSwf(swf, url);
	        }
	        return null;
	    }
	    getOneClass_ByName(Name) {
	        Name = this.doName(Name);
	        return this.DicOneURL[Name];
	    }
	    addSlideMc(Name, arr) {
	        Name = this.doName(Name);
	        var slide = this.getSlide(Name);
	        if (slide) {
	            slide.visible = true;
	            if (arr[0] instanceof Array) {
	                var Layer = slide.Layer;
	                var canX = slide.canX;
	                var midX = -1;
	                var midY = -1;
	                var startX = 0;
	                var startY = 0;
	                var everyNum = 1;
	                if (this.Dic_Meta && this.Dic_Meta[Name] && this.Dic_Meta[Name]["每排数量"])
	                    everyNum = this.Dic_Meta[Name]["每排数量"];
	                if (this.Dic_Meta && this.Dic_Meta[Name]) {
	                    startX = Tool_Function.onForceConvertType(this.Dic_Meta[Name]["startX"]);
	                    startY = Tool_Function.onForceConvertType(this.Dic_Meta[Name]["startY"]);
	                }
	                var countEvery = 0;
	                var countX = 0;
	                var countY = 0;
	                var countALL = 0;
	                if (this.Dic_Meta && this.Dic_Meta[Name]) {
	                    if (this.Dic_Meta[Name]["w"])
	                        midX = this.Dic_Meta[Name]["w"];
	                    if (this.Dic_Meta[Name]["h"])
	                        midY = this.Dic_Meta[Name]["h"];
	                }
	                var nextPo = canX ? startX : startY;
	                var isfirst = true;
	                for (var i = 0; i < arr[0].length; i++) {
	                    if (arr[0][i] == null)
	                        continue;
	                    if (isfirst == true) {
	                        isfirst = false;
	                        slide.isDefaultOne = (arr[0][i] instanceof MySlideMC_defaultOne && everyNum == 1);
	                    }
	                    Layer.addChild(arr[0][i]);
	                    if (this.Dic_Meta[Name]["scale"] != 1)
	                        arr[0][i].scaleX = arr[0][i].scaleY = this.Dic_Meta[Name]["scale"];
	                    if (midX == -1) {
	                        midX = arr[0][i].width;
	                        if (this.Dic_Meta && this.Dic_Meta[Name] && this.Dic_Meta[Name]["mid"])
	                            midX += this.Dic_Meta[Name]["mid"];
	                        else
	                            midX += midX * 0.1;
	                    }
	                    if (midY == -1) {
	                        midY = arr[0][i].height;
	                        if (this.Dic_Meta && this.Dic_Meta[Name] && this.Dic_Meta[Name]["mid"])
	                            midY += this.Dic_Meta[Name]["mid"];
	                        else
	                            midY += midY * 0.1;
	                    }
	                    countEvery++;
	                    countALL++;
	                    if (canX == true) {
	                        if (this.Dic_Meta && this.Dic_Meta[Name] && this.Dic_Meta[Name]["间隔排列"] == true) {
	                            arr[0][i].x = countX * midX + startX;
	                            if (countALL % 2 == 0) {
	                                arr[0][i].y = startY + midY;
	                            }
	                            else {
	                                arr[0][i].y = startY;
	                            }
	                            countX++;
	                        }
	                        else {
	                            if (countEvery >= everyNum) {
	                                countEvery = 0;
	                                arr[0][i].x = countX * midX + startX;
	                                arr[0][i].y = countY * midY + startY;
	                                countX++;
	                                countY = 0;
	                            }
	                            else {
	                                arr[0][i].x = countX * midX + startX;
	                                arr[0][i].y = countY++ * midY + startY;
	                            }
	                        }
	                    }
	                    else {
	                        if (this.Dic_Meta && this.Dic_Meta[Name] && this.Dic_Meta[Name]["间隔排列"] == true) {
	                            arr[0][i].y = countY * midY + startY;
	                            countY++;
	                            if (countALL % 2 == 0) {
	                                arr[0][i].x = midX + startX;
	                            }
	                            else {
	                                arr[0][i].x = startX;
	                            }
	                            countX++;
	                        }
	                        else {
	                            if (countEvery >= everyNum) {
	                                countEvery = 0;
	                                arr[0][i].x = countX * midX + startX;
	                                if (everyNum == 1 && slide.isDefaultOne) {
	                                    arr[0][i].y = nextPo;
	                                    var hone = arr[0][i].H;
	                                    if (hone <= midY)
	                                        hone = midY;
	                                    nextPo += hone;
	                                }
	                                else {
	                                    arr[0][i].y = countY * midY + startY;
	                                }
	                                countX = 0;
	                                countY++;
	                            }
	                            else {
	                                arr[0][i].y = countY * midY + startY;
	                                arr[0][i].x = countX++ * midX + startX;
	                            }
	                        }
	                    }
	                    if (this.Dic_Meta && this.Dic_Meta[Name] && this.Dic_Meta[Name]["one大小"] != null) {
	                        console.error("暂不支持异形组件");
	                    }
	                }
	            }
	            slide.addClickBtns(arr[0], arr[1]);
	            slide._w0 = midX;
	            slide._h0 = midY;
	            if (Layer) {
	                Layer.x = slide.maxX;
	                Layer.y = slide.maxY;
	            }
	            slide.onCheckVisible();
	            return true;
	        }
	        return false;
	    }
	    setPauseF(Name, busy) {
	        Name = this.doName(Name);
	        for (let key in this.DicMc) {
	            if (Name == null || Name == key) {
	                this.DicMc[key].setValue("暂停", busy);
	            }
	        }
	    }
	    destroyF() {
	        this.DicMc = ConfigTidus.GFun_ToolObj_destroyOne(this.DicMc);
	        this.DicOneURL = ConfigTidus.GFun_ToolObj_destroyOne(this.DicOneURL);
	    }
	}

	class QuadSection extends Sprite {
	    constructor(t, color = 0xffffff) {
	        super();
	        this._ratio = 1.0;
	        if (t instanceof Laya.Texture) {
	            this.imgTar = new Image$1(t);
	        }
	        else {
	            this.imgTar = t;
	        }
	        Tool_SpriteUtils.onAddchild_ChangeParent(this, this.imgTar);
	        this.mw = this.imgTar.width;
	        this.mh = this.imgTar.height;
	        this.imgMask = new Sprite();
	        this.imgMask.x = this.mw / 2;
	        this.imgMask.y = this.mh / 2;
	        this.updateVertices();
	    }
	    updateVertices() {
	        var ang = 360 * this._ratio - 90;
	        this.imgMask.graphics.clear();
	        this.imgMask.graphics.drawPie(0, 0, (this.mw + 10) / 2, -90, ang, "#FFFFFF");
	        this.imgTar.mask = this.imgMask;
	    }
	    get ratio() { return this._ratio; }
	    set ratio(value) {
	        if (this._ratio != value) {
	            this._ratio = value;
	            this.updateVertices();
	        }
	    }
	    set _width(v) {
	    }
	    get _width() {
	        return this.mw;
	    }
	    set _height(v) {
	    }
	    get _height() {
	        return this.mh;
	    }
	    static fromTexture(texture) {
	        var quadPie = new QuadSection(texture);
	        return quadPie;
	    }
	    destroyF() {
	        if (this.imgMask) {
	            this.imgMask.graphics.clear();
	            this.imgMask.removeFromParent(true);
	            this.imgMask = null;
	        }
	        this.imgTar = ConfigTidus.GFun_ToolObj_destroyOne(this.imgTar);
	        super.destroyF();
	    }
	}

	class ProgressBar extends Sprite {
	    constructor() {
	        super();
	        this.isDestroy = false;
	    }
	    init(_type, _image) {
	        this._type = _type;
	        if (_type == 1) {
	            this.progressBar = _image;
	            this.__width = _image.width;
	            this.progressBar.width = 0;
	        }
	        else if (_type == 1.1) {
	            this.progressBar = _image;
	            this.__width = _image.height;
	            this.progressBar.height = 0;
	            this.y0 = this.progressBar.y;
	        }
	        else if (_type == 2) {
	            this.progressBar = _image;
	            this._ratio = _image.ratio;
	        }
	        else if (_type == 3) {
	            this.progressBar = _image["img"];
	            this.x1 = this.progressBar.x;
	            this.x0 = _image["x"];
	            this.progressBar.scrollRect = new Laya.Rectangle(0, 0, _image["maskedObject"].width, _image["maskedObject"].height);
	        }
	        else if (_type == 4) {
	            this.totalFrame = _image["f"];
	            this.progressBar = _image["mc"];
	        }
	        this.ratio = 0;
	    }
	    initScaling(obj) {
	        if (obj == null) {
	            ConfigTidus.LogF("ProgressBar:initScaling参数错误");
	            return;
	        }
	        var _obj;
	        if (obj instanceof Sprite) {
	            _obj = obj.getChildByName("进度条");
	            if (_obj == null) {
	                _obj = obj;
	            }
	        }
	        else {
	            _obj = obj;
	        }
	        if (obj.width * 2 < obj.height)
	            this.init(1.1, _obj);
	        else
	            this.init(1, _obj);
	    }
	    initQuadSection(obj) {
	        var texture;
	        if (obj == null) {
	        }
	        else if (obj instanceof Sprite) {
	            var image = obj.getChildByName("进度条");
	            if (image != null) {
	                texture = image.texture;
	            }
	        }
	        else if (obj instanceof Image$1) {
	            texture = obj.texture;
	        }
	        else if (obj instanceof Laya.Texture) {
	            texture = obj;
	        }
	        if (texture == null) {
	            ConfigTidus.LogF("ProgressBar:initQuadSection参数错误");
	            return;
	        }
	        var q = QuadSection.fromTexture(texture);
	        q.ratio = 0;
	        if (obj instanceof Sprite) {
	            obj.addChildAt(q, obj.getChildIndex(image) + 1);
	            q.x = image.x;
	            q.y = image.y;
	            ConfigTidus.GFun_ToolObj_destroyOne(image);
	        }
	        else if (obj instanceof Image$1) {
	            this.addChild(q);
	            Tool_SpriteUtils.onAddchild_ChangeParent(this, obj);
	            ConfigTidus.GFun_ToolObj_destroyOne(obj);
	        }
	        else if (obj instanceof Laya.Texture) {
	            this.addChild(q);
	        }
	        this.init(2, q);
	    }
	    initMaskImage(image) {
	        if (image == null) {
	            ConfigTidus.LogF("ProgressBar:initMaskNoBar参数错误");
	            return;
	        }
	        var img;
	        if (image instanceof Sprite) {
	            img = Tool_SpriteUtils.cloneSprite(image);
	        }
	        if (image instanceof Image$1) {
	            img = Tool_ImgUtils.cloneImage(image);
	        }
	        image.mask = img;
	        var obj = new Object();
	        obj["img"] = img;
	        obj["maskedObject"] = img;
	        obj["x"] = -image.width;
	        this.init(3, obj);
	    }
	    initMaskSprite(sprite) {
	        if (sprite == null) {
	            ConfigTidus.LogF("ProgressBar:initMaskHasBar参数错误");
	            return;
	        }
	        var _image;
	        _image = sprite.getChildByName("进度条");
	        if (_image == null) {
	            ConfigTidus.LogF("ProgressBar:initMaskHasBar参数错误");
	            return;
	        }
	        var img = Tool_ImgUtils.cloneImage(_image);
	        img.x = 0;
	        img.y = 0;
	        _image.mask = img;
	        var obj = new Object();
	        obj["img"] = img;
	        obj["maskedObject"] = img;
	        obj["x"] = -_image.width;
	        this.init(3, obj);
	    }
	    initMc(sprite) {
	        if (sprite == null) {
	            ConfigTidus.LogF("ProgressBar:initMc参数错误");
	            return;
	        }
	        if (sprite instanceof SwfMovieClip$1) {
	            var obj = new Object();
	            obj["mc"] = sprite;
	            obj["f"] = sprite.totalFrames;
	            this.init(4, obj);
	            return;
	        }
	        var mc;
	        mc = sprite.getChildByName("进度条");
	        if (mc == null) {
	            var obj = new Object();
	            obj["mc"] = mc;
	            obj["f"] = mc.totalFrames - 1;
	            this.init(4, obj);
	            return;
	        }
	        ConfigTidus.LogF("ProgressBar:initMc参数错误");
	    }
	    get ratio() {
	        return this._ratio;
	    }
	    set ratio(value) {
	        if (this.isDestroy) {
	            return;
	        }
	        if (value < 0) {
	            value = 0;
	        }
	        else if (value > 1) {
	            value = 1;
	        }
	        if (this._type == 1) {
	            this._ratio = value;
	            this.progressBar.width = this.__width * value;
	        }
	        else if (this._type == 1.1) {
	            this._ratio = value;
	            this.progressBar.height = this.__width * value;
	            this.progressBar.y = this.y0 + this.__width * (1 - value);
	        }
	        else if (this._type == 2) {
	            this._ratio = value;
	            this.progressBar.ratio = value;
	        }
	        else if (this._type == 3) {
	            this._ratio = value;
	            this.progressBar.x = this.x0 + (this.x1 - this.x0) * value;
	        }
	        else if (this._type == 4) {
	            this._ratio = value;
	            let f = Tool_Function.onForceConvertType(value * this.totalFrame);
	            this.progressBar.gotoAndStop(f);
	        }
	    }
	    setVisible(value) {
	        this.visible = value;
	        if (this.progressBar)
	            this.progressBar.visible = value;
	    }
	    destroyF() {
	        this.isDestroy = true;
	        this.progressBar = ConfigTidus.GFun_ToolObj_destroyOne(this.progressBar);
	        this.Fun = ConfigTidus.GFun_ToolObj_destroyOne(this.Fun);
	        this.Canv = ConfigTidus.GFun_ToolObj_destroyOne(this.Canv);
	        super.destroyF();
	    }
	}

	class MyViewPBController {
	    constructor(spr) {
	        this.DicBP = {};
	        if (spr == null)
	            return;
	        for (let i = 0; i < spr.numChildren; i++) {
	            var obj = spr.getChildAt(i);
	            if (obj.name == null)
	                continue;
	            var strName = obj.name;
	            if (strName.indexOf("pb_") == 0 || strName.indexOf("PB_") == 0) {
	                let defaultType = null;
	                if (strName.indexOf("_mask") != -1) {
	                    strName = strName.replace("_mask", "");
	                }
	                var one = new __myviewPBOne(obj, defaultType);
	                this.DicBP[strName.substr(3)] = one;
	            }
	        }
	    }
	    doName(Name) {
	        if (Name == null || Name.indexOf("pb_") != 0)
	            return Name;
	        return Name.substr(3);
	    }
	    getOne(_name) {
	        _name = this.doName(_name);
	        var one = this.DicBP[_name];
	        if (one) {
	            return one;
	        }
	        if (_name.indexOf("pb_") == 0) {
	            _name = _name.substr(3);
	            one = this.DicBP[_name];
	            if (one)
	                return one;
	        }
	        return null;
	    }
	    getRatio(_name) {
	        var one = this.getOne(_name);
	        if (one) {
	            return one.ratio;
	        }
	        return -1;
	    }
	    setRatio(_name, ratio) {
	        var one = this.getOne(_name);
	        if (one) {
	            one.ratio = ratio;
	        }
	    }
	    destroyF() {
	        this.DicBP = ConfigTidus.GFun_ToolObj_destroyOne(this.DicBP);
	    }
	}
	class __myviewPBOne {
	    constructor(spr, __type = null) {
	        this.pb = new ProgressBar();
	        if (spr instanceof Image || spr instanceof SwfMovieClip$1) {
	            if (__type == null)
	                this.pb.initScaling(spr);
	            else if (__type == "mask")
	                this.pb.initMaskImage(spr);
	            return;
	        }
	        this.sprBack = spr;
	        var tarmc = this.sprBack.getChildByName("进度条");
	        if (tarmc == null) {
	            this.sprBack = null;
	            if (tarmc instanceof SwfMovieClip$1) {
	                if (__type == null)
	                    this.pb.initMc(spr);
	                else if (__type == "mask")
	                    this.pb.initMaskImage(spr);
	            }
	            else {
	                if (__type == null)
	                    this.pb.initScaling(spr);
	                else if (__type == "mask")
	                    this.pb.initMaskImage(spr);
	            }
	        }
	        else {
	            if (tarmc.scaleX != 1) {
	                this.sprBack.scale(tarmc.scaleX, tarmc.scaleY);
	                tarmc.scale(1, 1);
	            }
	            if (Math.abs(tarmc.width - tarmc.height) <= 1) {
	                this.pb.initQuadSection(this.sprBack);
	            }
	            else {
	                if (__type == null)
	                    this.pb.initScaling(spr);
	                else if (__type == "mask")
	                    this.pb.initMaskImage(spr);
	            }
	            this.mt = new MyViewTXController(this.sprBack);
	            this.mnum = new MyViewNumsController(this.sprBack);
	        }
	    }
	    get ratio() {
	        if (this.pb)
	            return this.pb.ratio;
	        return 0;
	    }
	    set ratio(val) {
	        var per;
	        var str;
	        if (Tool_Function.isTypeOf(val, "str")) {
	            str = val;
	            if (str.indexOf("/") != -1) {
	                var arr = str.split("/");
	                var num1 = this.strToNumber(arr[0]);
	                var num2 = this.strToNumber(arr[1]);
	                if (num1 <= 0)
	                    per = 0;
	                else if (num2 == 0)
	                    per = 1;
	                else
	                    per = num1 / num2;
	            }
	            else if (str.indexOf("%") != -1) {
	                per = this.strToNumber(str) / 100;
	            }
	            else {
	                per = Number(val);
	                str = Tool_Function.onForceConvertType(per * 100) + "%";
	            }
	        }
	        else {
	            per = val;
	            str = Tool_Function.onForceConvertType(per * 100) + "%";
	        }
	        if (this.pb)
	            this.pb.ratio = val;
	        if (this.mt)
	            this.mt.setText("tx_num", str);
	        if (this.mnum)
	            this.mnum.onShowF("num_num", str);
	    }
	    strToNumber(str) {
	        var num;
	        if (str.indexOf("%") != -1) {
	            num = Number(str.slice(0, str.length - 1));
	        }
	        else {
	            num = Number(str);
	        }
	        if (isNaN(num))
	            num = 0;
	        return num;
	    }
	    destroyF() {
	        this.pb = ConfigTidus.GFun_ToolObj_destroyOne(this.pb);
	        this.mt = ConfigTidus.GFun_ToolObj_destroyOne(this.mt);
	        this.mnum = ConfigTidus.GFun_ToolObj_destroyOne(this.mnum);
	        this.sprBack = ConfigTidus.GFun_ToolObj_destroyOne(this.sprBack);
	    }
	}

	class MyViewAllCompsController {
	    constructor(spr, numSwf = null) {
	        this.autoClearSpr = false;
	        this.sprBack = spr;
	        spr.addAutoDestroyValue(this);
	        for (var i = 0; i < spr.numChildren; i++) {
	            var one = spr.getChildAt(i);
	            if (one.name == null || one.name.length == 0 || one instanceof MyTextInput) {
	                continue;
	            }
	            if (one.name.indexOf("tx_") == 0 || one.name.indexOf("txi_") == 0) {
	                this.init_mt(false);
	                continue;
	            }
	            if (one.name.indexOf("btn_") == 0) {
	                this.init_mbc();
	                continue;
	            }
	            if (one.name.indexOf("SBtn_") == 0) {
	                this.init_msb();
	                continue;
	            }
	            if (one.name.indexOf("slide_") == 0) {
	                this.init_ms();
	                continue;
	            }
	            if (one.name.indexOf("num_") == 0) {
	                this.init_mnum(numSwf);
	                continue;
	            }
	            if (one.name.indexOf("pb_") == 0) {
	                this.init_mpb();
	                continue;
	            }
	            if (one.name.indexOf("pic_") == 0) {
	                this.init_mpm();
	                continue;
	            }
	            if (one.name.indexOf("combo_") == 0) {
	                if (this.DicAC == null)
	                    this.DicAC = {};
	                let newac = new MyViewAllCompsController(one, numSwf);
	                this.DicAC[one.name] = newac;
	                continue;
	            }
	        }
	    }
	    get Fun_mbc_click() { return this._Fun_mbc_click; }
	    set Fun_mbc_click(v) { this._Fun_mbc_click = v; if (this.DicAC == null)
	        return; for (let key in this.DicAC) {
	        this.DicAC[key].Fun_mbc_click = v;
	    } }
	    set Fun_mbc_none(f) {
	        this._Fun_mbc_none = f;
	        if (this.mmeBG)
	            return;
	        this.mmeBG = new MyMouseManager(this.sprBack);
	        this.mmeBG.addClickHandler((e) => { this._Fun_mbc_none(); });
	    }
	    get Fun_msb_click() { return this._Fun_msb_click; }
	    set Fun_msb_click(v) { this._Fun_msb_click = v; if (this.DicAC == null)
	        return; for (let key in this.DicAC) {
	        this.DicAC[key].Fun_msb_click = v;
	    } }
	    init_mt(needMyText) {
	        if (this.MT != null)
	            return;
	        var useMyt = false;
	        if (this.sprBack instanceof Sprite && this.sprBack.metaData && this.sprBack.metaData["vc"]
	            && this.sprBack.metaData["vc"]["mt"] && this.sprBack.metaData["vc"]["mt"]["mytext"] == true) {
	            useMyt = true;
	        }
	        this.MT = new MyViewTXController(this.sprBack, useMyt);
	    }
	    init_mbc() {
	        if (this.MBC != null)
	            return;
	        this.MBC = new MyViewBtnController(this.sprBack, this.mbc_ClickF.bind(this), this.mbc_DownF.bind(this), this.mbc_UpF.bind(this));
	    }
	    mbc_ClickF(btn) {
	        if (this.Fun_mbc_click)
	            this.Fun_mbc_click(btn);
	    }
	    mbc_DownF(btn) {
	        if (this.Fun_mbc_down)
	            this.Fun_mbc_down(btn);
	    }
	    mbc_UpF(btn) {
	        if (this.Fun_mbc_up)
	            this.Fun_mbc_up(btn);
	    }
	    init_msb() {
	        if (this.MSB != null)
	            return;
	        this.MSB = new MyViewSelectBtnController(this.sprBack, this.msb_ClickF.bind(this));
	    }
	    msb_ClickF(sbtn, btn) {
	        if (this.Fun_msb_click)
	            this.Fun_msb_click(sbtn, btn);
	    }
	    init_ms() {
	        if (this.MS != null)
	            return;
	        this.MS = new MyViewSlideMCs(this.sprBack);
	    }
	    init_mnum(numSwf) {
	        if (this.MNum != null)
	            return;
	        this.MNum = new MyViewNumsController(this.sprBack, null, numSwf);
	    }
	    init_mpb() {
	        if (this.MPB != null)
	            return;
	        this.MPB = new MyViewPBController(this.sprBack);
	    }
	    init_mpm() {
	        if (this.MPM != null)
	            return;
	        this.MPM = new MyViewPicsController(this.sprBack);
	    }
	    getText(tf) {
	        if (this.MT) {
	            let result = this.MT.getText(tf);
	            if (result)
	                return result;
	        }
	        if (this.DicAC) {
	            for (let key in this.DicAC) {
	                let ac = this.DicAC[key];
	                let result = ac.getText(tf);
	                if (result)
	                    return result;
	            }
	        }
	        return null;
	    }
	    setText(tf, text) {
	        if (text == null)
	            text = "";
	        if (this.MT) {
	            this.MT.setText(tf, text + "");
	        }
	        if (this.DicAC) {
	            for (let key in this.DicAC) {
	                let ac = this.DicAC[key];
	                ac.setText(tf, text);
	            }
	        }
	    }
	    getTextField(tf) {
	        if (this.MT) {
	            var tar = this.MT.getTextField(tf);
	            if (tar)
	                return tar;
	        }
	        if (this.DicAC) {
	            for (var key in this.DicAC) {
	                var ac = this.DicAC[key];
	                tar = ac.getTextField(tf);
	                if (tar)
	                    return tar;
	            }
	        }
	        return null;
	    }
	    getInput(tf) {
	        if (this.MT) {
	            var tar = this.MT.getInput(tf);
	            if (tar)
	                return tar;
	        }
	        if (this.DicAC) {
	            for (var key in this.DicAC) {
	                var ac = this.DicAC[key];
	                tar = ac.getInput(tf);
	                if (tar)
	                    return tar;
	            }
	        }
	        return null;
	    }
	    addChangeEventListener(tf, f) {
	        if (this.MT && this.MT.Dic_TXI) {
	            if (tf != null) {
	                this.MT.addChangeEventListener(tf, f);
	            }
	            else {
	                for (let key in this.MT.Dic_TXI) {
	                    this.MT.addChangeEventListener(key, f);
	                }
	            }
	        }
	        if (this.DicAC) {
	            for (var key in this.DicAC) {
	                var ac = this.DicAC[key];
	                ac.addChangeEventListener(tf, f);
	            }
	        }
	    }
	    setTextToChildsChild(tf, text) {
	        let arr = tf.split(".");
	        let now = this.sprBack;
	        for (let i = 0; i < arr.length; i++) {
	            try {
	                now = now.getChildByName(arr[i]);
	            }
	            catch (e) {
	                now = null;
	            }
	            if (now == null)
	                break;
	        }
	        if (now && now instanceof TextField) {
	            now.text = text;
	        }
	        else {
	            console.error("无法设置：" + tf);
	        }
	    }
	    changeTextfieldColor(tf, col) {
	        if (this.MT) {
	            this.MT.changeTextfieldColor(tf, col);
	        }
	        if (this.DicAC) {
	            for (var key in this.DicAC) {
	                var ac = this.DicAC[key];
	                ac.changeTextfieldColor(tf, col);
	            }
	        }
	    }
	    changeNumValue(name, vname, val) {
	        if (this.MNum) {
	            this.MNum.setValueToNum(name, vname, val);
	        }
	        if (this.DicAC) {
	            for (let key in this.DicAC) {
	                let ac = this.DicAC[key];
	                ac.changeNumValue(name, vname, val);
	            }
	        }
	    }
	    setNum(name, value) {
	        if (this.MNum) {
	            this.MNum.onShowF(name, value);
	        }
	        if (this.DicAC) {
	            for (let key in this.DicAC) {
	                let ac = this.DicAC[key];
	                ac.setNum(name, value);
	            }
	        }
	    }
	    getValueOfNum(name) {
	        if (this.MNum && this.MNum.Nums[name]) {
	            return this.MNum.getValueOfNum(name);
	        }
	        if (this.DicAC) {
	            for (let key in this.DicAC) {
	                let ac = this.DicAC[key];
	                let num = ac.getValueOfNum(name);
	                if (num != null)
	                    return num;
	            }
	        }
	        return null;
	    }
	    setPBRatio(name, ratio) {
	        if (this.MPB) {
	            this.MPB.setRatio(name, ratio);
	        }
	        if (this.DicAC) {
	            for (let key in this.DicAC) {
	                let ac = this.DicAC[key];
	                ac.setPBRatio(name, ratio);
	            }
	        }
	    }
	    getRatio(_name) {
	        if (this.MPB) {
	            let result = this.MPB.getRatio(_name);
	            if (result != -1)
	                return result;
	        }
	        else if (this.DicAC) {
	            for (let key in this.DicAC) {
	                let ac = this.DicAC[key];
	                let result = ac.getRatio(_name);
	                if (result != -1)
	                    return result;
	            }
	        }
	        return -1;
	    }
	    setBusy(busy) {
	        if (this.MBC) {
	            this.MBC.Busy = busy;
	        }
	        if (this.MSB) {
	            this.MSB.onPauseF(null, busy);
	        }
	        if (this.MS) {
	            this.MS.setPauseF(null, busy);
	        }
	        if (this.MT) {
	            this.MT.onStopInput(busy);
	            this.MT.onHideInput(busy);
	        }
	        if (this.DicAC) {
	            for (let key in this.DicAC) {
	                let ac = this.DicAC[key];
	                ac.setBusy(busy);
	            }
	        }
	    }
	    onVisibleOneChild(name, value) {
	        let one = this.sprBack.getChildByName(name);
	        if (one) {
	            one.visible = value;
	        }
	        else if (this.DicAC) {
	            for (let key in this.DicAC) {
	                let ac = this.DicAC[key];
	                ac.onVisibleOneChild(name, value);
	            }
	        }
	    }
	    onGotoAndStopMc(name, frame) {
	        let one = this.sprBack.getChildByName(name);
	        if (one && one instanceof SwfMovieClip$1) {
	            one.gotoAndStop(frame);
	            return true;
	        }
	        else {
	            if (this.DicAC) {
	                for (let key in this.DicAC) {
	                    let ac = this.DicAC[key];
	                    if (ac.onGotoAndStopMc(name, frame))
	                        return true;
	                }
	            }
	            return false;
	        }
	    }
	    onVisibleBtn(name, value) {
	        if (this.MBC) {
	            this.MBC.onVisible(name, value);
	        }
	        if (this.DicAC) {
	            for (let key in this.DicAC) {
	                let ac = this.DicAC[key];
	                ac.onVisibleBtn(name, value);
	            }
	        }
	    }
	    onPauseBtn(name, pause) {
	        if (this.MBC) {
	            this.MBC.onPause(name, pause);
	        }
	        if (this.DicAC) {
	            for (let key in this.DicAC) {
	                let ac = this.DicAC[key];
	                ac.onPauseBtn(name, pause);
	            }
	        }
	    }
	    getChildByName(name) {
	        var one = this.sprBack.getChildByName(name);
	        if (one)
	            return one;
	        if (this.DicAC) {
	            for (let key in this.DicAC) {
	                var ac = this.DicAC[key];
	                one = ac.getChildByName(name);
	                if (one)
	                    return one;
	            }
	        }
	        return null;
	    }
	    clickSBtn(sb, btn) {
	        if (this.MSB) {
	            this.MSB.clickF(sb, btn);
	        }
	        if (this.DicAC) {
	            for (let key in this.DicAC) {
	                let ac = this.DicAC[key];
	                ac.clickSBtn(sb, btn);
	            }
	        }
	    }
	    getSBtnNowOn(sb) {
	        if (this.MSB) {
	            if (this.MSB.getSBtn(sb) != null) {
	                return this.MSB.getSBtn(sb).getNowName();
	            }
	        }
	        if (this.DicAC) {
	            for (let key in this.DicAC) {
	                let ac = this.DicAC[key];
	                let now = ac.getSBtnNowOn(sb);
	                if (now)
	                    return now;
	            }
	        }
	        return null;
	    }
	    addSlideMc(name, arr) {
	        if (this.MS && this.MS.addSlideMc(name, arr) == true) {
	            return true;
	        }
	        if (this.DicAC) {
	            for (var key in this.DicAC) {
	                var ac = this.DicAC[key];
	                if (ac.addSlideMc(name, arr) == true)
	                    return true;
	            }
	        }
	        return false;
	    }
	    getSlideOne(name) {
	        if (this.MS && this.MS.getSlide(name) != null) {
	            return this.MS.getSlide(name);
	        }
	        if (this.DicAC) {
	            for (var key in this.DicAC) {
	                var ac = this.DicAC[key];
	                var one = ac.getSlideOne(name);
	                if (one)
	                    return one;
	            }
	        }
	        return null;
	    }
	    getSlideDefaultOne(swf, Name = null) {
	        if (this.MS) {
	            var one = this.MS.getDefaultOne(swf, Name);
	            if (one)
	                return one;
	        }
	        if (this.DicAC) {
	            for (var key in this.DicAC) {
	                var ac = this.DicAC[key];
	                var one = ac.getSlideDefaultOne(swf, Name);
	                if (one)
	                    return one;
	            }
	        }
	        return null;
	    }
	    getSlideOneSprite(swf, Name) {
	        if (this.MS) {
	            var spr = this.MS.getSlideOne_ByNameFromSwf(swf, Name);
	            if (spr)
	                return spr;
	        }
	        if (this.DicAC) {
	            for (var key in this.DicAC) {
	                var ac = this.DicAC[key];
	                var spr = ac.getSlideOneSprite(swf, Name);
	                if (spr)
	                    return spr;
	            }
	        }
	        return null;
	    }
	    destroyF() {
	        this.MT = ConfigTidus.GFun_ToolObj_destroyOne(this.MT);
	        this.MBC = ConfigTidus.GFun_ToolObj_destroyOne(this.MBC);
	        this.MSB = ConfigTidus.GFun_ToolObj_destroyOne(this.MSB);
	        this.MS = ConfigTidus.GFun_ToolObj_destroyOne(this.MS);
	        this.MNum = ConfigTidus.GFun_ToolObj_destroyOne(this.MNum);
	        this.MPM = ConfigTidus.GFun_ToolObj_destroyOne(this.MPM);
	        this.MPB = ConfigTidus.GFun_ToolObj_destroyOne(this.MPB);
	        this.Fun_mbc_click = ConfigTidus.GFun_ToolObj_destroyOne(this.Fun_mbc_click);
	        this.Fun_mbc_down = ConfigTidus.GFun_ToolObj_destroyOne(this.Fun_mbc_down);
	        this.Fun_mbc_up = ConfigTidus.GFun_ToolObj_destroyOne(this.Fun_mbc_up);
	        this.Fun_mlm_click = ConfigTidus.GFun_ToolObj_destroyOne(this.Fun_mlm_click);
	        this._Fun_mbc_none = ConfigTidus.GFun_ToolObj_destroyOne(this._Fun_mbc_none);
	        this.DicAC = ConfigTidus.GFun_ToolObj_destroyOne(this.DicAC);
	        if (this.autoClearSpr)
	            this.sprBack = ConfigTidus.GFun_ToolObj_destroyOne(this.sprBack);
	    }
	}

	class AlertNetWindow extends Sprite {
	    constructor(swf, url, script) {
	        super();
	        this.sprView = ConfigTidus.GVal["MySourceManager"].getInstance().getObjFromSwf(swf, url);
	        if (this.sprView == null)
	            return;
	        this.addChild(this.sprView);
	        Tool_ViewFit.onWindowFitScreen(this.sprView, 4);
	        this.ac = new MyViewAllCompsController(this.sprView);
	        var tso = new TidusScript_component();
	        try {
	            tso.init(this, script, { "ac": this.ac });
	        }
	        catch (error) {
	            this.destroyF();
	            return;
	        }
	        LayerManager.instance.LayerTop.addChild(this);
	    }
	    destroyF() {
	        this.ac = ConfigTidus.GFun_ToolObj_destroyOne(this.ac);
	        super.destroyF();
	    }
	}

	class NetMessageModel {
	    constructor() {
	        this.isLogined = false;
	        this.isLocal = false;
	        this.isConnecting = false;
	        this.firstOpen = true;
	        this.mmo = new MainManager();
	        this.co = new CmdListenerOne();
	        this.isLogining = false;
	        ConfigTidus.GFun_Net_SendCMD = this.sendMessage.bind(this);
	        ConfigTidus.GVal["NetMessageModel"] = NetMessageModel;
	        MgsSocket.FunSendWhenClose = (fun) => {
	            console.log("断网状态下发送指令，先开始重连和登陆");
	            if (this.dataLogin == null) {
	                console.log("没有logindata，无法自动重连");
	                Tool_Function.onRunFunction(fun);
	                return;
	            }
	            this.onStaticLogin(this.dataLogin, (dic) => {
	                Tool_Function.onRunFunction(fun, dic);
	            }, true);
	        };
	        this.mmo.EventManager.addCustomEventListener(MgsSocket.Event_Close, this.onClose.bind(this));
	        this.addMessageListener(NetMessageModel.CMD_AlertWin, this.netAlertWindow.bind(this), false);
	        this.addMessageListener(NetMessageModel.CMD_Mail, this.netMailF.bind(this), false);
	        this.mmo.addEnterSecondFunction(this.checkHeartBeat.bind(this));
	    }
	    static get instance() {
	        if (this._instance == null) {
	            this.Dic = new __netinfos();
	            this._instance = new NetMessageModel();
	        }
	        return this._instance;
	    }
	    static destroyF() {
	        if (this._instance) {
	            this._instance.destroyF();
	            this._instance = null;
	        }
	    }
	    checkHeartBeat() {
	        if (this.isLogined) {
	            NetMessageModel.instance.sendMessage(NetMessageModel.CMD_Heart, null, false);
	        }
	    }
	    netAlertWindow(dic) {
	        if (dic && dic["代码"] && dic["swf"] && dic["url"]) {
	            new AlertNetWindow(dic["swf"], dic["url"], dic["代码"]);
	        }
	        else if (dic && dic["文字"]) {
	            ConfigTidus.GFun_AlertWindow(dic["文字"], { "title": dic["标题"] });
	        }
	    }
	    netMailF(dic) {
	        GT_Mail.getCMD(dic);
	    }
	    destroyF() {
	        this.mmo = ConfigTidus.GFun_ToolObj_destroyOne(this.mmo);
	        this.co = ConfigTidus.GFun_ToolObj_destroyOne(this.co);
	    }
	    onClose() {
	        this.isLogined = false;
	    }
	    onStaticConnectF(f) {
	        if (this.isConnecting)
	            return;
	        this.isConnecting = true;
	        if (MgsSocket.getInstance().nowFlag == "连接") {
	            this.isConnecting = false;
	            Tool_Function.onRunFunction(f, true);
	            return;
	        }
	        if (MyRandomIP.NewIP == null && NetMessageModel.Dic.name == "正式" && NetMessageModel.Dic.ip.indexOf("?") != -1) {
	            var data = { "设备号": "", "code": ConfigTidus.Ver, "ver": ConfigTidus.Ver, "渠道": ConfigTidus.Platform };
	            if (PlayerDefault.player && PlayerDefault.player.DeviceInfo)
	                data["设备号"] = PlayerDefault.player.DeviceInfo["设备号"];
	            new MyRandomIP(NetMessageModel.Dic.ip + ":" + NetMessageModel.Dic.port_http + "/checkServer", JSON.stringify(data), (ip) => {
	                this.isConnecting = false;
	                ConfigTidus.LogF("得到ip：" + ip);
	                NetMessageModel.Dic.ip = ip;
	                this.onStaticConnectF(f);
	            }, () => {
	                ConfigTidus.LogF("计算失败");
	                this.isConnecting = false;
	                Tool_Function.onRunFunction(f, false);
	            });
	            return;
	        }
	        if (this.firstOpen == true) {
	            this.firstOpen = false;
	            ConfigTidus.GFun_MEO_dispatch("初次打开");
	        }
	        this.FunConnect = f;
	        MgsSocket.getInstance().Fun_connect = onConnectEff.bind(this);
	        MgsSocket.getInstance().connectF(NetMessageModel.Dic.ip, NetMessageModel.Dic.port_ws);
	        function onConnectEff(suc) {
	            this.isConnecting = false;
	            if (suc == true) {
	                Tool_Function.onRunFunction(this.FunConnect, true);
	            }
	            else {
	                Tool_Function.onRunFunction(this.FunConnect, false);
	            }
	        }
	    }
	    onStaticLogin(data, f, isReConnect = false) {
	        if (this.isLogining)
	            return;
	        this.isLogining = true;
	        this.dataLogin = data;
	        this.FunLogin = f;
	        if (MgsSocket.getInstance().nowFlag != "连接") {
	            this.onStaticConnectF((suc) => {
	                this.isLogining = false;
	                console.log("登陆前连接" + suc);
	                if (suc == true) {
	                    this.onStaticLogin(data, f, isReConnect);
	                }
	                else {
	                    this.netLogin({ "结果": false, "失败原因": "无法联网" });
	                }
	            });
	            return;
	        }
	        this.isLogined = false;
	        var c100 = new Net_SocketCMD(NetMessageModel.CMD_Login);
	        c100.funAddListener(this.netLogin.bind(this), true);
	        for (let key in data) {
	            c100.writeValue_Dic(key, data[key]);
	        }
	        if (isReConnect)
	            c100.writeValue_Dic("重连", true);
	        if (PlayerDefault.player && PlayerDefault.player.DeviceInfo) {
	            PlayerDefault.player.DeviceInfo["ver"] = ConfigTidus.Ver;
	            c100.writeValue_Dic("设备信息", PlayerDefault.player.DeviceInfo);
	        }
	        c100.sendF(false);
	    }
	    netLogin(dic) {
	        this.isLogining = false;
	        if (dic["结果"] == false)
	            console.log(dic["失败原因"]);
	        else {
	            this.isLogined = true;
	            LSOManager.put("上次账号", { "账号": this.dataLogin["账号"] });
	        }
	        Tool_Function.onRunFunction(this.FunLogin, dic);
	    }
	    addMessageListener(cmd, fun, once) {
	        this.co.listenF(cmd, fun, once);
	    }
	    isListening(cmd) {
	        return this.co.isListening(cmd);
	    }
	    sendMessage(cmd, data, needrandom) {
	        let one = new Net_SocketCMD(cmd);
	        if (data) {
	            for (let key in data) {
	                one.writeValue_Dic(key, data[key]);
	            }
	        }
	        one.sendF(needrandom);
	    }
	}
	NetMessageModel.CMD_Login = 100;
	NetMessageModel.CMD_PlayerInfo = 101;
	NetMessageModel.CMD_Heart = 2;
	NetMessageModel.CMD_AlertWin = 5;
	NetMessageModel.CMD_Mail = 9;
	class __netinfos {
	    constructor() {
	        this.useHttps = false;
	    }
	    get netbackUrl_pay() {
	        return this.urlhttp + "/1sdk_pay";
	    }
	    get netbackUrl_login() {
	        return this.urlhttp + "/1sdklogin";
	    }
	    get urlhttp() {
	        if (this.useHttps == true)
	            return "https://" + this.ip + ":" + this.port_http;
	        return "http://" + this.ip + ":" + this.port_http;
	    }
	}

	class MyPlatformManager {
	    static getLoginName() { return this.SDK + "." + this.Data["ID"]; }
	    static init(fun) {
	        if (this.SDK == null) {
	            if (Laya.Browser.onMiniGame) {
	                this.SDK = "wx";
	                Laya.MiniAdpter.window.wx.onShow(res => {
	                    console.log("小游戏回到前台", res);
	                });
	                Laya.MiniAdpter.window.wx.onHide(() => {
	                    console.log("小游戏进入后台");
	                });
	                try {
	                    const res = wx.getSystemInfoSync();
	                    console.log("判定为微信小游戏，状态栏高度：", res.statusBarHeight);
	                    if (res.statusBarHeight > 20)
	                        ConfigTidus.iSWakongping = true;
	                }
	                catch (e) {
	                }
	            }
	            else if (Laya.Browser.onBLMiniGame) {
	                this.SDK = "bili";
	            }
	            else if (Laya.Browser.onBDMiniGame) {
	            }
	            else if (Laya.Browser.onKGMiniGame) {
	            }
	            else if (Laya.Browser.onQGMiniGame) {
	            }
	            else if (Laya.Browser.onVVMiniGame) {
	            }
	            else if (Laya.Browser.onAlipayMiniGame) {
	            }
	            else if (Laya.Browser.onTTMiniGame) {
	            }
	            else if (Laya.Browser.onHWMiniGame) {
	            }
	        }
	        if (this.SDK == "4399") {
	            dynamicLoadJs_body("http://h.api.4399.com/h5mini-2.0/h5api-interface.php", fun);
	        }
	        else {
	            if (fun)
	                fun();
	            return;
	        }
	        function dynamicLoadJs_body(url, callback) {
	            var script = document.createElement('script');
	            script.type = 'text/javascript';
	            script.src = url;
	            if (script.readyState) {
	                script.onreadystatechange = function () {
	                    if (script.readyState == 'loaded' || script.readyState == 'complete') {
	                        script.onreadystatechange = null;
	                        callback();
	                    }
	                };
	            }
	            else {
	                script.onload = function () {
	                    callback();
	                };
	            }
	            document.getElementsByTagName('head')[0].appendChild(script);
	        }
	    }
	    static addLocalFileInMinigame(arrSwfName) {
	        if (this.SDK && this.SDK != "4399") {
	            let arr = ["version.json"];
	            for (let i = 0; i < arrSwfName.length; i++) {
	                let swf = arrSwfName[i];
	                this.LocalFiles[swf] = true;
	                arr.push("res/" + swf + "/" + swf + ".png");
	                arr.push("res/" + swf + "/" + swf + ".jpg");
	                arr.push("res/" + swf + "/" + swf + ".json");
	            }
	            if (this.SDK == "wx")
	                Laya.MiniAdpter.nativefiles = arr;
	        }
	    }
	    static onLogin(callback) {
	        if (this.SDK == "4399") {
	            if (('h5api' in Laya.Browser.window)) {
	                try {
	                    Laya.Browser.window.h5api.login((data) => {
	                        this.Data["uid"] = data.uId;
	                        this.Data["nick"] = data.userName;
	                        PlayerDefault.player.Nick = data.userName;
	                        PlayerDefault.player.ID = data.uId;
	                        let result = {};
	                        result.uid = data.uId;
	                        result.nick = data.userName;
	                        callback(result);
	                    });
	                }
	                catch (e) {
	                    console.error("4399登陆失败：" + e);
	                    callback(null);
	                }
	            }
	            else {
	                console.error('找不到 4399HTML5 API');
	                callback(null);
	            }
	        }
	        else if (this.SDK == "wx") {
	            if (this.WXMini_data == null) {
	                callback("还没有配置微信小游戏的所需信息");
	                return;
	            }
	            console.log("调用微信登录！");
	            var wx = window["wx"];
	            if (wx == null) {
	                console.error('找不到 wx API');
	                callback(null);
	                return;
	            }
	            wx.login({
	                success(res) {
	                    if (res.code) {
	                        console.log("微信登录成功，获得code=" + res.code);
	                        if (MyPlatformManager.WXMini_data.httplogin != null) {
	                            let netpath;
	                            if (typeof MyPlatformManager.WXMini_data.httplogin === "string")
	                                netpath = MyPlatformManager.WXMini_data.httplogin;
	                            else
	                                netpath = NetMessageModel.Dic.urlhttp + "/wxlogin";
	                            new Net_HttpRequest(netpath, { "code": res.code, "type": "game" }, (re) => {
	                                let res = typeof re == "string" ? JSON.parse(re) : re;
	                                console.log("收到服务器wx登录结果：" + JSON.stringify(res));
	                                if (res == null) {
	                                    callback("无法连接到服务器");
	                                    return;
	                                }
	                                if (res["data"])
	                                    res = res["data"];
	                                if (res["openid"] == null) {
	                                    callback("获取小程序登录验证失败:\n" + JSON.stringify(re));
	                                    return;
	                                }
	                                MyPlatformManager.Data["openid"] = res.openid;
	                                MyPlatformManager.Data["ID"] = res.openid;
	                                callback(true);
	                            }, true);
	                            return;
	                        }
	                        NetMessageModel.instance.addMessageListener(99, (dic) => {
	                            if (dic["结果"] == false) {
	                                callback(dic["失败原因"]);
	                            }
	                            else {
	                                MyPlatformManager.Data["openid"] = dic["oid"];
	                                MyPlatformManager.Data["ID"] = dic["oid"];
	                                callback(true);
	                            }
	                        }, true);
	                        NetMessageModel.instance.sendMessage(99, { "操作": "登录", "code": res.code }, false);
	                    }
	                    else {
	                        callback(res.errMsg);
	                    }
	                },
	                fail() {
	                    callback("调用登录接口失败");
	                }
	            });
	        }
	        else {
	            callback("未处理登录的平台：" + this.SDK);
	        }
	    }
	    static getUserInfo(callback, option = null) {
	        if (this.SDK == "4399") {
	            if (('h5api' in Laya.Browser.window) == false) {
	                console.error('找不到 4399HTML5 API');
	                callback(null);
	            }
	            else {
	                this.Data["头像"] = Laya.Browser.window.h5api.getUserAvatar(this.Data["uid"]);
	                this.Data["小头像"] = Laya.Browser.window.h5api.getUserSmallAvatar(this.Data["uid"]);
	                this.Data["大头像"] = Laya.Browser.window.h5api.getUserBigAvatar(this.Data["uid"]);
	                callback(true);
	            }
	        }
	        else if (this.SDK == "wx") {
	            var wx = window["wx"];
	            if (wx == null) {
	                console.error('找不到 wx API');
	                callback(null);
	                return;
	            }
	            wx.getSetting({
	                success(res) {
	                    console.log("微信小游戏的历史权限：" + JSON.stringify(res.authSetting));
	                    if (res.authSetting["scope.userInfo"] == true) {
	                        wx.getUserInfo({
	                            success: res => {
	                                var userInfo = res.userInfo;
	                                var nickName = userInfo.nickName;
	                                var avatarUrl = userInfo.avatarUrl;
	                                var gender = userInfo.gender;
	                                var province = userInfo.province;
	                                var city = userInfo.city;
	                                PlayerDefault.player.Nick = nickName;
	                                PlayerDefault.player.head = avatarUrl;
	                                PlayerDefault.player.sex = gender == 1 ? "男" : gender == 2 ? "女" : "未知";
	                                callback(true);
	                            },
	                            fail: res => {
	                                callback(null);
	                            }
	                        });
	                    }
	                    else {
	                        let wxinfo = wx.getSystemInfoSync();
	                        let allw = wxinfo.windowWidth;
	                        let allh = wxinfo.windowHeight;
	                        let btncol = (option && option["btncol"]) ? this.getColorStrFromInt(option["btncol"]) : "#ffffff";
	                        let btnw = (option && option["btnw"]) ? option["btnw"] : 300;
	                        let btnh = (option && option["btnh"]) ? option["btnh"] : 40;
	                        let btnx = (option && option["btnx"]) ? option["btnx"] : (allw - btnw) / 2;
	                        let btny = (option && option["btny"]) ? option["btny"] : allh - btnh - 20;
	                        const button = wx.createUserInfoButton({
	                            type: 'text',
	                            text: '授权用户信息',
	                            style: {
	                                left: btnx,
	                                top: btny,
	                                width: btnw,
	                                height: btnh,
	                                lineHeight: 40,
	                                backgroundColor: btncol,
	                                borderColor: "#000000",
	                                color: '#ff9900',
	                                textAlign: 'center',
	                                fontSize: 22,
	                                borderRadius: 4
	                            }
	                        });
	                        button.onTap((res) => {
	                            var userInfo = res.userInfo;
	                            var nickName = userInfo.nickName;
	                            var avatarUrl = userInfo.avatarUrl;
	                            var gender = userInfo.gender;
	                            var province = userInfo.province;
	                            var city = userInfo.city;
	                            PlayerDefault.player.Nick = nickName;
	                            PlayerDefault.player.head = avatarUrl;
	                            PlayerDefault.player.sex = gender == 1 ? "男" : gender == 2 ? "女" : "未知";
	                            callback(true);
	                            button.destroy();
	                        });
	                    }
	                },
	                fail(res) {
	                    callback(null);
	                }
	            });
	        }
	        else {
	            callback(null);
	        }
	    }
	    static getColorStrFromInt(col) {
	        let str = col.toString(16);
	        while (str.length < 6) {
	            str = "0" + str;
	        }
	        return "#" + str;
	    }
	    static canShowAD(cb) {
	        if (this.SDK == "4399") {
	            if (('h5api' in Laya.Browser.window)) {
	                try {
	                    if (Laya.Browser.window.h5api.canPlayAd()) {
	                        cb(true);
	                    }
	                }
	                catch (e) {
	                    console.error(e);
	                }
	            }
	        }
	        else if (this.SDK == "wx") {
	            var wx = window["wx"];
	            if (wx == null) {
	                console.error('找不到 wx API');
	                cb(false);
	                return;
	            }
	            if (window["wxAdID"] == null) {
	                console.error('找不到 wxAdID');
	                cb(false);
	                return;
	            }
	            let rewardedVideoAd = wx.createRewardedVideoAd({ adUnitId: window["wxAdID"] });
	            rewardedVideoAd.onLoad(() => {
	                console.log('激励视频 广告加载成功');
	                this.Data["广告准备"] = true;
	                cb(true);
	            });
	            rewardedVideoAd.onError(err => {
	                console.log(err);
	                cb(err.errCode + ":" + err.errMsg);
	            });
	            return;
	        }
	        cb(false);
	    }
	    static onShowAD(callback) {
	        if (this.SDK == "4399") {
	            Laya.Browser.window.h5api.playAd(data => {
	                if (data.code == 10000)
	                    console.log("开始播放广告");
	                else if (data.code == 10001) {
	                    console.log("广告结束");
	                    ConfigTidus.GFun_MM_addDelay(ConfigTidus.FrameRate / 2, callback, true);
	                }
	                else if (data.code == 10010) {
	                    console.log("广告异常");
	                    callback(data.message + "");
	                }
	            });
	        }
	        else if (this.SDK == "wx") {
	            var wx = window["wx"];
	            let rewardedVideoAd = wx.createRewardedVideoAd({ adUnitId: window["wxAdID"] });
	            rewardedVideoAd.onClose(res => {
	                if (res && res.isEnded || res === undefined) {
	                    callback(true);
	                }
	                else {
	                    callback("广告中断");
	                }
	            });
	            try {
	                rewardedVideoAd.show().then(() => console.log('激励视频 广告开始显示'));
	            }
	            catch (error) {
	                rewardedVideoAd.load().then(() => {
	                    callback("广告播放失败，请重试");
	                });
	            }
	        }
	        else {
	            callback('没有初始化MyPF的渠道');
	        }
	    }
	}
	MyPlatformManager.Data = {};
	MyPlatformManager.LocalFiles = {};

	class GT_Equip {
	    constructor(nid, data) {
	        this.Quality = 1;
	        this.Star = 1;
	        this.Plus = 0;
	        this.netid = Tool_Function.onForceConvertType(nid);
	        this.baseid = data["bid"];
	        if (GT_Equip.Data_equips == null || GT_Equip.Data_equips[this.baseid] == null) {
	            console.error("没有配置装备的数据：" + this.baseid);
	            return;
	        }
	        this.netChangeValue(data);
	    }
	    netChangeValue(data) {
	        if (data["强化"]) {
	            this.Plus = data["强化"];
	        }
	        if (data["star"] != null) {
	            this.Star = data["star"];
	        }
	        if (GT_Equip.Data_equips == null || GT_Equip.Data_equips[this.baseid] == null)
	            return;
	        let dic = GT_Equip.Data_equips[this.baseid];
	        if (dic["po"]) {
	            this.po = dic["po"];
	        }
	        if (this.po == "weapon") {
	            if (dic["补正"]) {
	                this.weaponBuzheng = ConfigTidus.GFun_ToolObj_copy(dic["补正"]);
	                if (this.Plus > 0 && dic["补正强化"]) {
	                    let lv = 1;
	                    while (lv <= this.Plus) {
	                        if (dic["补正强化"][lv]) {
	                            ConfigTidus.GFun_ToolObj_combo(this.weaponBuzheng, dic["补正强化"][lv], false);
	                        }
	                        lv++;
	                    }
	                }
	            }
	            if (dic["Type"]) {
	                this.weaponType = dic["Type"];
	            }
	        }
	        if (dic["属性"]) {
	            this.Values = new MyCheaterNum();
	            for (let vname in dic["属性"]) {
	                let vnum = dic['属性'][vname];
	                if (dic["属性强化"]) {
	                    let lv = 1;
	                    while (lv <= this.Plus) {
	                        if (dic["属性强化"][lv] && dic["属性强化"][lv][vname]) {
	                            vnum = dic["属性强化"][lv][vname];
	                        }
	                        lv++;
	                    }
	                }
	                this.Values.setValue(vname, vnum);
	            }
	        }
	    }
	    isNowEquipedByRole() {
	        for (let nid in ConfigTidus.GVal["PL"].player.roles) {
	            let role = ConfigTidus.GVal["PL"].player.roles[nid];
	            if (role.Equips) {
	                for (let po in role.Equips) {
	                    if (role.Equips[po] == this.netid)
	                        return role;
	                }
	            }
	        }
	        return null;
	    }
	}

	class GT_Ruin {
	    constructor(nid, data) {
	        this.Quality = 1;
	        this.Star = 1;
	        this.Lv = 1;
	        this.Exp = 0;
	        this.netid = Tool_Function.onForceConvertType(nid);
	        this.baseid = data["bid"];
	        if (GT_Ruin.Data_ruins == null || GT_Ruin.Data_ruins[this.baseid] == null) {
	            console.error("没有配置符文的数据：" + this.baseid);
	            return;
	        }
	        this.netChangeValue(data);
	    }
	    netChangeValue(data) {
	        if (data["lv"]) {
	            this.Lv = data["lv"];
	        }
	        if (data["exp"]) {
	            this.Exp = data["exp"];
	        }
	        if (data["star"] != null) {
	            this.Star = data["star"];
	        }
	        if (data["自定义"] != null) {
	            this.custom = data["自定义"];
	        }
	        if (GT_Ruin.Data_ruins == null || GT_Ruin.Data_ruins[this.baseid] == null)
	            return;
	        let dic = GT_Ruin.Data_ruins[this.baseid];
	        if (dic["类型"]) {
	            this.Type = dic["类型"];
	        }
	        if (dic["name"])
	            this.Name = dic["name"];
	        if (dic["品质"])
	            this.Quality = dic["品质"];
	        if (dic["属性"]) {
	            this.Values = new MyCheaterNum();
	            for (let vname in dic["属性"]) {
	                let vlevel = this.getRankOfValueName(vname);
	                let dataup = GT_Ruin.Data_ruins["属性值"];
	                if (dataup[vname])
	                    dataup = dataup[vname];
	                else
	                    dataup = dataup["其他"];
	                let vnum = Tool_Function.onForceConvertType(dataup[vlevel] * this.Lv);
	                this.Values.setValue(vname, vnum);
	            }
	        }
	    }
	    getRankOfValueName(vname) {
	        let dic = GT_Ruin.Data_ruins[this.baseid];
	        if (dic && dic["属性"]) {
	            return dic['属性'][vname];
	        }
	        return null;
	    }
	    isNowEquipedByRole() {
	        for (let nid in ConfigTidus.GVal["PL"].player.roles) {
	            let role = ConfigTidus.GVal["PL"].player.roles[nid];
	            if (role.Ruins) {
	                for (let po in role.Ruins) {
	                    if (role.Ruins[po] == this.netid)
	                        return role;
	                }
	            }
	        }
	        return null;
	    }
	}

	class PlayerDefault {
	    constructor() {
	        this.gold = 0;
	        this.money = 0;
	        this.isRegist = false;
	        PlayerDefault.player = this;
	        var dicLogin = LSOManager.get("账号密码");
	        if (dicLogin) {
	            this.LoginName = dicLogin["账号"];
	            this.LoginPass = dicLogin["密码"];
	        }
	        ConfigTidus.GVal["NetMessageModel"].instance.addMessageListener(ConfigTidus.GVal["NetMessageModel"].CMD_PlayerInfo, this.netChangeValue.bind(this), false);
	    }
	    onSaveLoginInfo() {
	        var dicLogin = { "账号": PlayerDefault.player.LoginName, "密码": PlayerDefault.player.LoginPass, "ver": 1 };
	        LSOManager.put("账号密码", dicLogin);
	    }
	    initF(fun) {
	        if (this.DeviceInfo && this.DeviceInfo["设备号"])
	            this.DC = this.DeviceInfo["设备号"];
	        else
	            this.DC = "ffx";
	        fun();
	    }
	    getLoginData() {
	        var dic = {};
	        dic["账号"] = this.LoginName;
	        dic["密码"] = this.LoginPass;
	        dic["设备信息"] = this.DeviceInfo;
	        dic["SDK"] = MyPlatformManager.SDK;
	        dic["渠道"] = ConfigTidus.Platform;
	        return dic;
	    }
	    getBaseInfo() {
	        var dic = {};
	        dic["uid"] = this.ID;
	        dic["账号"] = this.LoginName;
	        dic["nick"] = this.Nick;
	        dic["dc"] = this.DC;
	        return dic;
	    }
	    netChangeValue(data) {
	        if (data.uid > 0) {
	            ConfigTidus.LogF("UID=" + data.uid);
	            this.ID = data.uid;
	        }
	        if (data.gold > 0) {
	            this.gold = data.gold;
	            ConfigTidus.LogF("gold=" + data.gold);
	            ConfigTidus.GFun_MEO_dispatch("玩家gold改变");
	        }
	        if (data.money > 0) {
	            this.money = data.money;
	            ConfigTidus.LogF("money=" + data.money);
	            ConfigTidus.GFun_MEO_dispatch("玩家money改变");
	        }
	        if (data.userInfo != null) {
	            if (data.userInfo["nick"])
	                data.nick = data.userInfo["nick"];
	            if (data.userInfo["age"])
	                data.age = data.userInfo["age"];
	            if (data.userInfo["head"])
	                data.head = data.userInfo["head"];
	        }
	        if (data.userInfo_change != null) {
	            if (data.userInfo_change["nick"])
	                data.nick = data.userInfo_change["nick"];
	            if (data.userInfo_change["age"])
	                data.age = data.userInfo_change["age"];
	            if (data.userInfo_change["head"])
	                data.head = data.userInfo_change["head"];
	        }
	        if (data.nick != null && data.nick.length > 0) {
	            ConfigTidus.LogF("nick=" + data.nick);
	            this.Nick = data.nick;
	        }
	        if (data.head != null) {
	            if (Tool_Function.isTypeOf(data.head, "int"))
	                this.head = data.head;
	            else if (data.head.length > 0)
	                this.head = data.head;
	        }
	        if (data.phone != null) {
	            this.Phone = data.phone;
	        }
	        if (data.gameProgress != null) {
	            this.gameProgress = data.gameProgress;
	        }
	        if (data.gameProgress_change != null) {
	            if (this.gameProgress == null)
	                this.gameProgress = {};
	            for (let key in data.gameProgress_change) {
	                this.gameProgress[key] = data.gameProgress_change[key];
	            }
	            ConfigTidus.GFun_MEO_dispatch("玩家gameProgress改变");
	        }
	        if (data.dicRole != null) {
	            this.roles = {};
	            for (let key in data.dicRole) {
	                let r = new GT_Role();
	                r.netID = data.dicRole[key]["nid"];
	                r.baseID = data.dicRole[key]["bid"];
	                this.roles[key] = r;
	            }
	        }
	        if (data.dicRole_change != null) {
	            if (this.roles == null)
	                this.roles = {};
	            for (let key in data.dicRole_change) {
	                if (data.dicRole_change[key] == "删除")
	                    delete this.roles[key];
	                else if (this.roles[key] == null) {
	                    let r = new GT_Role();
	                    r.netID = data.dicRole_change[key]["nid"];
	                    r.baseID = data.dicRole_change[key]["bid"];
	                    this.roles[key] = r;
	                }
	                else {
	                    let r = this.roles[key];
	                }
	                ConfigTidus.GFun_MEO_dispatch("玩家role改变", Tool_Function.onForceConvertType(key));
	            }
	        }
	        if (data.dicItem != null) {
	            this.items = data.dicItem;
	        }
	        if (data.dicItem_change != null) {
	            if (this.items == null)
	                this.items = {};
	            for (let key in data.dicItem_change) {
	                if (data.dicItem_change[key] == "删除")
	                    delete this.items[key];
	                else
	                    this.items[key] = data.dicItem_change[key];
	            }
	            ConfigTidus.GFun_MEO_dispatch("玩家items改变");
	        }
	        if (data.dicEquip) {
	            this.equips = {};
	            for (let type in data.dicEquip) {
	                this.equips[type] = {};
	                for (let nid in data.dicEquip[type]) {
	                    let one;
	                    if (type == "装备") {
	                        one = new GT_Equip(nid, data.dicEquip[type][nid]);
	                    }
	                    else {
	                        one = new GT_Ruin(nid, data.dicEquip[type][nid]);
	                    }
	                    this.equips[type][nid] = one;
	                }
	            }
	        }
	        if (data.dicEquip_change) {
	            if (this.equips == null)
	                this.equips = {};
	            let changedroleid = [];
	            for (let type in data.dicEquip_change) {
	                if (this.equips[type] == null)
	                    this.equips[type] = {};
	                for (let nid in data.dicEquip_change[type]) {
	                    let one = this.equips[type][nid];
	                    if (one == null) {
	                        if (type == "装备") {
	                            one = new GT_Equip(nid, data.dicEquip_change[type][nid]);
	                        }
	                        else {
	                            one = new GT_Ruin(nid, data.dicEquip_change[type][nid]);
	                        }
	                        this.equips[type][nid] = one;
	                    }
	                    else {
	                        one.netChangeValue(data.dicEquip_change[type][nid]);
	                        console.log("玩家[" + type + "]" + nid + "改变：" + JSON.stringify(data.dicEquip_change[type][nid]));
	                        let role = one.isNowEquipedByRole();
	                        if (role && changedroleid.indexOf(role.netID) == -1) {
	                            changedroleid.push(role.netID);
	                        }
	                    }
	                }
	            }
	            for (let i = 0; i < changedroleid.length; i++) {
	                ConfigTidus.GFun_MEO_dispatch("玩家role改变", changedroleid[i]);
	            }
	        }
	        if (data.dicEveryDay != null) {
	            this.dicEveryDay = data.dicEveryDay;
	        }
	        if (data.customData != null) {
	            this.customData = data.customData;
	        }
	        if (data.custom_change != null) {
	            if (this.customData == null)
	                this.customData = {};
	            ConfigTidus.GFun_ToolObj_combo(this.customData, data.custom_change, false, false, true);
	            ConfigTidus.GFun_MEO_dispatch("玩家customData改变");
	        }
	    }
	}
	PlayerDefault.havePermission = {};

	class Tool_ArrayUtils {
	    static removeAt(arr, index) {
	        arr.splice(index, 1);
	    }
	    static removeTar(arr, tar) {
	        if (arr == null)
	            return;
	        let i = arr.indexOf(tar);
	        if (i >= 0)
	            this.removeAt(arr, i);
	    }
	    static concatArray(a, b, checkRepeat = true) {
	        for (let i = 0; i < b.length; i++) {
	            if (checkRepeat) {
	                if (a.indexOf(b[i]) != -1)
	                    continue;
	            }
	            a.push(b[i]);
	        }
	    }
	    static copyArr(arr, onlyLowLevel = false) {
	        if (arr == null)
	            return null;
	        var out = [];
	        for (var i = 0; i < arr.length; i++) {
	            if (onlyLowLevel == false)
	                out[i] = ConfigTidus.GFun_ToolObj_copy(arr[i]);
	            else
	                out[i] = arr[i];
	        }
	        return out;
	    }
	    static isEqual(arr1, arr2) {
	        if (arr1 == null && arr2 == null)
	            return true;
	        if (arr1 == null || arr2 == null)
	            return false;
	        if (arr1.length != arr2.length)
	            return false;
	        for (var i = 0; i < arr1.length; i++) {
	            if (ConfigTidus.GFun_ToolObj_isEqual(arr1[i], arr2[i]) == false)
	                return false;
	        }
	        return true;
	    }
	    static onRemoveLoopNumber(arr) {
	        var dic = {};
	        for (var i = 0; i < arr.length; i++) {
	            if (arr[i] instanceof Number) {
	                if (dic[arr[i]] == true)
	                    arr.splice(i--, 1);
	                else
	                    dic[arr[i]] = true;
	            }
	        }
	    }
	    static getRadomOneFromArray(arr) {
	        let i = Tool_Function.onForceConvertType(Math.random() * arr.length);
	        return arr[i];
	    }
	    static onRadomSort(arr) {
	        var cloneArr = arr.slice();
	        arr.length = 0;
	        var i = cloneArr.length;
	        while (i) {
	            arr.push(cloneArr.splice(Tool_Function.onForceConvertType(Math.random() * i--), 1)[0]);
	        }
	    }
	    static onNumber_SmallToBig(a, b) {
	        if (a < b)
	            return -1;
	        else
	            return 1;
	    }
	    static onNumber_BigToSmall(a, b) {
	        if (a > b)
	            return -1;
	        else
	            return 1;
	    }
	    static onSerchAndRemove(arr, val) {
	        if (arr == null)
	            return false;
	        var po = arr.indexOf(val);
	        if (po == -1)
	            return false;
	        arr.splice(po, 1);
	        return true;
	    }
	    static insertInto(arr, index, val) {
	        if (index == -1 || index >= arr.length)
	            arr.push(val);
	        else {
	            arr.splice(index, 0, val);
	        }
	    }
	}

	class GT_Mail {
	    static onLogin() {
	        ConfigTidus.GFun_Net_SendCMD(ConfigTidus.GVal["NetMessageModel"].CMD_Mail, { "操作": "登录" }, false);
	    }
	    static onOpenOne(nid, del) {
	        if (del) {
	            var arrUnopen = PlayerDefault.player.dicMail["未读"];
	            for (let i = 0; i < arrUnopen.length; i++) {
	                let mail = arrUnopen[i];
	                if (mail["id"] == nid) {
	                    Tool_ArrayUtils.removeAt(arrUnopen, i);
	                    break;
	                }
	            }
	        }
	        ConfigTidus.GFun_MEO_dispatch("刷新邮件");
	        ConfigTidus.GFun_Net_SendCMD(ConfigTidus.GVal["NetMessageModel"].CMD_Mail, { "操作": "打开", "mid": nid, "删除": del }, false);
	    }
	    static getCMD(dic) {
	        if (dic == null || dic["结果"] == false) {
	            ConfigTidus.LogF("收到邮件指令错误：" + (dic ? dic["失败原因"] : "null"));
	            return;
	        }
	        if (dic["data"]) {
	            PlayerDefault.player.dicMail = dic["data"];
	            ConfigTidus.GFun_MEO_dispatch("刷新邮件");
	        }
	        else if (dic["收到"]) {
	            if (PlayerDefault.player.dicMail == null)
	                PlayerDefault.player.dicMail = { "未读": [] };
	            PlayerDefault.player.dicMail["未读"].push(dic["收到"]);
	            ConfigTidus.GFun_MEO_dispatch("收到新邮件");
	            ConfigTidus.LogF("收到新邮件！");
	        }
	        else {
	            ConfigTidus.GFun_MEO_dispatch("刷新邮件");
	        }
	    }
	}

	class MyKeyboardManager {
	    constructor(s) {
	        this.Dic = {};
	        this.Funs_Down = {};
	        this.Funs_Up = {};
	        this.Dic_stopEvent = {};
	        this.pause = false;
	        this.noKeyEvent = false;
	        if (MyKeyboardManager.Arr_instance == null)
	            MyKeyboardManager.Arr_instance = [];
	        MyKeyboardManager.Arr_instance.push(this);
	        this.S = s;
	        this.Funs_Down["Every"] = [];
	        this.Funs_Up["Every"] = [];
	        if (this.S) {
	            this.S.on(Laya.Event.KEY_DOWN, this, this.keyDown);
	            this.S.on(Laya.Event.KEY_UP, this, this.keyUP);
	        }
	    }
	    onForceDown(key) {
	        if (this.pause)
	            return;
	        if (this.Dic[key] == null || this.Dic[key] == false) {
	            this.Dic[key] = true;
	            var arr = this.Funs_Down["Every"];
	            for (let i = 0; i < arr.length; i++) {
	                Tool_Function.onRunFunction(arr[i], key);
	            }
	            if (this.Funs_Down[key] == null)
	                return;
	            arr = this.Funs_Down[key];
	            for (let i = 0; i < arr.length; i++) {
	                Tool_Function.onRunFunction(arr[i]);
	            }
	        }
	    }
	    onForceUp(key) {
	        if (this.pause)
	            return;
	        if (this.Dic[key] == null || this.Dic[key] == true) {
	            this.Dic[key] = false;
	            var arr = this.Funs_Up["Every"];
	            for (let i = 0; i < arr.length; i++) {
	                Tool_Function.onRunFunction(arr[i], key);
	            }
	            if (this.Funs_Up[key] == null)
	                return;
	            arr = this.Funs_Up[key];
	            for (let i = 0; i < arr.length; i++) {
	                Tool_Function.onRunFunction(arr[i]);
	            }
	        }
	    }
	    onUpAll() {
	        for (var key in this.Dic) {
	            this.Dic[key] = false;
	        }
	    }
	    keyDown(e) {
	        if (this.noKeyEvent || e == null)
	            return;
	        var key = e.keyCode;
	        if (this.Dic_stopEvent && this.Dic_stopEvent[key] == true) {
	            e.stopPropagation();
	        }
	        if (this.DicChangekeys && this.DicChangekeys[key])
	            key = this.DicChangekeys[key];
	        this.onForceDown(key);
	    }
	    keyUP(e) {
	        if (this.noKeyEvent || e == null)
	            return;
	        var key = e.keyCode;
	        if (this.Dic_stopEvent && this.Dic_stopEvent[key] == true) {
	            e.stopPropagation();
	        }
	        if (this.DicChangekeys && this.DicChangekeys[key])
	            key = this.DicChangekeys[key];
	        this.onForceUp(key);
	    }
	    isDown(key) {
	        if (this.Dic[key] == null)
	            return false;
	        return this.Dic[key];
	    }
	    addChangeKey(keyCode, key) {
	        if (this.DicChangekeys == null)
	            this.DicChangekeys = {};
	        this.DicChangekeys[keyCode] = key;
	    }
	    addFunctionListener(key, f, type = "down") {
	        if (f == null) {
	            console.log("keybord错误的参数");
	            return;
	        }
	        var dic;
	        if (type == "down")
	            dic = this.Funs_Down;
	        else if (type == "up")
	            dic = this.Funs_Up;
	        else {
	            console.log("keybord错误的参数");
	            return;
	        }
	        if (key == null) {
	            var arr = dic["Every"];
	            if (arr.indexOf(f) == -1)
	                arr.push(f);
	            return;
	        }
	        if (Tool_Function.isTypeOf(key, "str"))
	            key = Laya.Keyboard[key.toUpperCase()];
	        if (dic[key] == null)
	            dic[key] = [];
	        arr = dic[key];
	        if (arr.indexOf(f) == -1)
	            arr.push(f);
	    }
	    removeFunctionListener(key, f, type = "down") {
	        if (f == null) {
	            console.log("keybord错误的参数");
	            return;
	        }
	        var dic;
	        if (type == "down")
	            dic = this.Funs_Down;
	        else if (type == "up")
	            dic = this.Funs_Up;
	        else {
	            console.log("错误的参数");
	            return;
	        }
	        if (key == null) {
	            var arr = dic["Every"];
	            var index = arr.indexOf(f);
	            if (index != -1)
	                arr.splice(index, 1);
	            return;
	        }
	        if (Tool_Function.isTypeOf(key, "str"))
	            key = Laya.Keyboard[key.toUpperCase()];
	        if (dic[key] == null)
	            return;
	        arr = dic[key];
	        index = arr.indexOf(f);
	        if (index != -1)
	            arr.splice(index, 1);
	    }
	    addComboKeysListener(keys, cb) {
	        if (this.Funs_ComboKeys == null)
	            this.Funs_ComboKeys = {};
	        var lastKey = keys[keys.length - 1];
	        if (this.Funs_ComboKeys[lastKey] == null) {
	            this.Funs_ComboKeys[lastKey] = [];
	            this.addFunctionListener(lastKey, () => {
	                for (var i = 0; i < this.Funs_ComboKeys[lastKey].length; i++) {
	                    var keys = this.Funs_ComboKeys[lastKey][i][0];
	                    var f = this.Funs_ComboKeys[lastKey][i][1];
	                    var suc = true;
	                    for (var j = 0; j < keys.length - 1; j++) {
	                        if (keys[j] == "ctrl") {
	                            if (this.isDown(17) == false) {
	                                suc = false;
	                                break;
	                            }
	                        }
	                        else {
	                            var needKey = typeof keys[j] === "string" ? keys[j] : String.fromCharCode(keys[j]);
	                            if (this.isDown(keys[j]) == false) {
	                                suc = false;
	                                break;
	                            }
	                        }
	                    }
	                    if (suc)
	                        Tool_Function.onRunFunction(f);
	                }
	            });
	        }
	        this.Funs_ComboKeys[lastKey].push([keys, cb]);
	    }
	    stopEventMop(key, val = true) {
	        this.Dic_stopEvent[key] = val;
	    }
	    destroyF() {
	        if (this.S) {
	            this.S.off(Laya.Event.KEY_DOWN, this, this.keyDown);
	            this.S.off(Laya.Event.KEY_UP, this, this.keyUP);
	        }
	        if (MyKeyboardManager.Arr_instance) {
	            let index = MyKeyboardManager.Arr_instance.indexOf(this);
	            if (index >= 0) {
	                MyKeyboardManager.Arr_instance.splice(index, 1);
	            }
	        }
	    }
	}
	MyKeyboardManager.KEY_Back = "返回";

	class MyBusy {
	    constructor() {
	        this.Dic = {};
	    }
	    get busy() {
	        for (let key in this.Dic) {
	            if (this.Dic[key] == true)
	                return true;
	        }
	        return false;
	    }
	    set busy(value) {
	        this.setBusy("default", value);
	    }
	    getBusy(type = "default") {
	        return this.Dic[type] == true;
	    }
	    setBusy(type, value) {
	        let old = this.busy;
	        if (type == null)
	            type = "default";
	        this.Dic[type] = value;
	        if (this.ACs && this.busy != old) {
	            for (let i = 0; i < this.ACs.length; i++) {
	                this.ACs[i].setBusy(this.busy);
	            }
	        }
	    }
	    addAC(ac) {
	        if (this.ACs == null)
	            this.ACs = [];
	        this.ACs.push(ac);
	    }
	}

	class LoadingView extends Sprite {
	    constructor(id = 0) {
	        super();
	        this.per = 0;
	        var picUrl;
	        LayerManager.getInstence().LayerTop.addChild(this);
	        if (ConfigTidus.stageW > ConfigTidus.stageH)
	            this.sprBack = ConfigTidus.GVal["MySourceManager"].getInstance().getObjFromSwf("SWF_Default", "spr_LoadingBig");
	        else
	            this.sprBack = ConfigTidus.GVal["MySourceManager"].getInstance().getObjFromSwf("SWF_Default", "spr_LoadingBig竖");
	        if (this.sprBack != null) {
	            this.addChild(this.sprBack);
	            this.ac = new MyViewAllCompsController(this.sprBack);
	            if (LoadingView.NeedLoadBackPic && id != 0) {
	                picUrl = "res/LoadBack_" + id + ".jpg";
	                let imgtouming = this.sprBack.getChildAt(0);
	                this.picBack = new Sprite();
	                this.picBack.addChild(imgtouming);
	                if (this.sprBack.getChildByName("b") != null)
	                    this.sprBack.addChildAt(this.picBack, 1);
	                else
	                    this.sprBack.addChildAt(this.picBack, 0);
	                this.picBack.addChild(new Laya.Image(picUrl));
	            }
	        }
	        else {
	            return;
	        }
	        if (this.sprBack) {
	            Tool_ViewFit.onWindowFitScreen(this.sprBack, 4.2);
	        }
	        this.setNowper();
	    }
	    noAutoProgress() {
	    }
	    setPer(val) {
	        this.setNowper(val);
	    }
	    setNowper(val = null) {
	        if (this.ac) {
	            this.ac.setPBRatio("pb_1", val);
	            this.ac.setPBRatio("pb_进度", val);
	        }
	    }
	    destroyF() {
	        this.ac = ConfigTidus.GFun_ToolObj_destroyOne(this.ac);
	        if (this.picBack) {
	            this.picBack.removeFromParent();
	            this.picBack = null;
	        }
	        super.destroyF();
	    }
	}
	LoadingView.NeedLoadBackPic = true;

	class MySourceManagerOne {
	    constructor() {
	        this.Busy = false;
	        this.Arr_waite = [];
	        this.countAddTime = 0;
	        this.SM = MySourceManager.instance;
	    }
	    static onNewOne(mso) {
	        for (var i = 0; i < MySourceManagerOne.waiteClear.length; i++) {
	            MySourceManagerOne.waiteClear[i].onRemoveSources(mso.Arr_source);
	        }
	    }
	    static onClearWaite() {
	        while (MySourceManagerOne.waiteClear.length > 0) {
	            MySourceManagerOne.waiteClear[0].onRealDestroyF();
	            MySourceManagerOne.waiteClear.shift();
	        }
	    }
	    addSource(arr, f, needLoadingMC, _autoClear = true, ...arg) {
	        if (this.Busy) {
	            this.Arr_waite.push([arr, f, needLoadingMC, _autoClear]);
	            return;
	        }
	        this.Busy = true;
	        this.autoClearLoadView = _autoClear;
	        if (this.autoClearLoadView == true) {
	            this.Fun = f;
	        }
	        var j;
	        for (var i = 0; i < arr.length; i++) {
	            if (this.Arr_source && this.countAddTime > 0) {
	                var re = false;
	                for (j = 0; j < this.Arr_source.length; j++) {
	                    if (ConfigTidus.GFun_ToolObj_isEqual(arr[i], this.Arr_source[j]) == true) {
	                        arr.splice(i, 1);
	                        re = true;
	                        break;
	                    }
	                }
	                if (re == true) {
	                    i--;
	                    continue;
	                }
	            }
	            for (j = i + 1; j < arr.length; j++) {
	                if (ConfigTidus.GFun_ToolObj_isEqual(arr[i], arr[j]) == true) {
	                    arr.splice(j, 1);
	                }
	            }
	        }
	        if (this.Arr_source == null)
	            this.Arr_source = arr.concat();
	        else
	            this.Arr_source = this.Arr_source.concat(arr);
	        if (this.countAddTime == 0) {
	            arr = this.Arr_source;
	        }
	        this.countAddTime++;
	        this.SM.addTexture(arr, this.addOver.bind(this), this.loadingProgress.bind(this));
	        MySourceManagerOne.onNewOne(this);
	        if (MySourceManagerOne.FunGetLoadView) {
	            if (arg.length == 0)
	                this.LView = MySourceManagerOne.FunGetLoadView(arr.length, needLoadingMC);
	            else
	                this.LView = Tool_Function.onRunFunction(MySourceManagerOne.FunGetLoadView, [arr.length, needLoadingMC].concat(arg));
	        }
	        if (this.LView == null) {
	            if (typeof needLoadingMC == "number") {
	                this.LView = new LoadingView(needLoadingMC);
	            }
	            else if (needLoadingMC == true) {
	                this.LView = new LoadingView(0);
	            }
	        }
	    }
	    loadingProgress(per) {
	        if (this.LView) {
	            this.LView.setNowper();
	        }
	        if (this.FunLoading)
	            this.FunLoading(per);
	    }
	    addOver() {
	        if (this.autoClearLoadView == false)
	            return;
	        if (this.SM == null)
	            return;
	        if (this.LView) {
	            this.LView.destroyF();
	            this.LView = null;
	        }
	        if (this.Fun) {
	            Tool_Function.onRunFunction(this.Fun);
	        }
	        this.Fun = null;
	        this.Busy = false;
	        if (this.Arr_waite.length > 0) {
	            var arr = this.Arr_waite[0];
	            this.Arr_waite.shift();
	            this.addSource(arr[0], arr[1], arr[2], arr[3]);
	        }
	        MySourceManagerOne.onClearWaite();
	    }
	    onRemoveLoadView() {
	        this.autoClearLoadView = true;
	        this.addOver();
	    }
	    deleteSourceFromArray(arr) {
	        arr.forEach((item, index, array) => {
	            for (var i = 0; i < this.Arr_source.length; i++) {
	                if (this.Arr_source[i][0] == item || this.Arr_source[i][0] == item[0]) {
	                    this.Arr_source.splice(i--, 1);
	                }
	            }
	        });
	    }
	    getSwf(name) {
	        return this.SM.getSwf(name);
	    }
	    setSwfAutoSmooth(name) {
	    }
	    getTexture(picname) {
	        return this.SM.getTexture(picname);
	    }
	    getTextureFromSWF(swfName, name) {
	        return this.SM.getTextureFromSWF(swfName, name);
	    }
	    getImgFromSwf(swfName, imgname) {
	        return this.SM.getImgFromSwf(swfName, imgname);
	    }
	    getS9FromSwf(swf, s9name) {
	        return this.SM.getS9FromSwf(swf, s9name);
	    }
	    getSprFromSwf(swfName, sprname) {
	        return this.SM.getSprFromSwf(swfName, sprname);
	    }
	    getMcFromSwf(swfName, mcname, canChangeClothes = false) {
	        return this.SM.getMcFromSwf(swfName, mcname, canChangeClothes);
	    }
	    getMMC(swf, mcname) {
	        let mc = this.getMcFromSwf(swf, mcname);
	        if (mc == null)
	            return null;
	        let mmc = new MyMC(mc);
	        if (mc instanceof MyZMovieClip) {
	            mmc.gotoAndStop(mcname);
	        }
	        return mmc;
	    }
	    getObjFromSwf(swfName, objName) {
	        if (objName.indexOf("img_") == 0)
	            return this.getImgFromSwf(swfName, objName);
	        if (objName.indexOf("spr_") == 0)
	            return this.getSprFromSwf(swfName, objName);
	        if (objName.indexOf("mc_") == 0)
	            return this.getMcFromSwf(swfName, objName);
	        if (objName.indexOf("s9_") == 0)
	            return this.getS9FromSwf(swfName, objName);
	        return null;
	    }
	    getJson(jName) {
	        return this.SM.getJson(jName);
	    }
	    getTXT(jName) {
	        return this.SM.getTxt(jName);
	    }
	    onRemoveSources(source) {
	        if (this.Arr_source == null || source == null)
	            return;
	        var arrNoRepeat = [];
	        this.Arr_source.forEach((item, index, arr) => {
	            if (item == null)
	                return;
	            var re = false;
	            source.forEach((item2, index2, arr2) => {
	                if (Tool_ArrayUtils.isEqual(item, item2) == true)
	                    re = true;
	            });
	            if (re == false) {
	                this.Arr_source[index] = null;
	                arrNoRepeat.push(item);
	            }
	        });
	        if (arrNoRepeat.length > 0) {
	            MySourceManager.instance.removeTextures(arrNoRepeat);
	        }
	    }
	    destroyF() {
	        this.Fun = ConfigTidus.GFun_ToolObj_destroyOne(this.Fun);
	        if (this.LView) {
	            this.LView.destroyF();
	            this.LView = null;
	        }
	        if (this.SM) {
	            MySourceManagerOne.waiteClear.push(this);
	            this.SM = null;
	        }
	    }
	    onRealDestroyF() {
	        if (this.Arr_source && this.Arr_source.length > 0) {
	            MySourceManager.instance.removeTextures(this.Arr_source);
	        }
	    }
	}
	MySourceManagerOne.waiteClear = [];

	class MySceneSprite extends Sprite {
	    constructor() {
	        super();
	        this.mso = new MySourceManagerOne();
	        this.smo = new SoundManagerMy();
	        this.mmo = new MainManager();
	        this.mkm = new MyKeyboardManager(Laya.stage);
	        this.mbusy = new MyBusy();
	        this.isAlerting = false;
	        this.mkm.addFunctionListener(null, this.onKeyboardF.bind(this));
	    }
	    get busy() { return this.mbusy.busy; }
	    set busy(v) { this.mbusy.busy = v; }
	    initSprView(swf, url, spr = null) {
	        if (spr == null) {
	            this.sprView = this.mso.getSprFromSwf(swf, url);
	            this.addChild(this.sprView);
	            Tool_ViewFit.onViewFitF(this.sprView);
	        }
	        else {
	            this.sprView = spr;
	            Tool_SpriteUtils.onAddchild_ChangeParent(this, spr);
	        }
	        this.ac = new MyViewAllCompsController(this.sprView);
	        this.mbusy.addAC(this.ac);
	    }
	    destroyF() {
	        this.mso = ConfigTidus.GFun_ToolObj_destroyOne(this.mso);
	        this.smo = ConfigTidus.GFun_ToolObj_destroyOne(this.smo);
	        this.mmo = ConfigTidus.GFun_ToolObj_destroyOne(this.mmo);
	        this.mkm = ConfigTidus.GFun_ToolObj_destroyOne(this.mkm);
	        this.ac = ConfigTidus.GFun_ToolObj_destroyOne(this.ac);
	        super.destroyF();
	    }
	    onKeyBackF() {
	        return false;
	    }
	    onKeyboardF(code) {
	        if (code == Laya.Keyboard.BACKSPACE) {
	            this.onKeyBackF();
	        }
	    }
	    alertForLeave() {
	        if (this.isAlerting)
	            return;
	        this.isAlerting = true;
	        let fun = Laya.Handler.create(this, realLeave);
	        AlertWindow.showF("是否离开游戏？", null, true, fun, true, fun);
	        function realLeave(real) {
	            this.isAlerting = false;
	            if (real)
	                ConfigTidus.onCloseProgram();
	        }
	    }
	    destroyByOtherScene() {
	        this.destroyF();
	    }
	    getClickTarByLP(lp) {
	        for (let i = this.sprView.numChildren - 1; i >= 0; i--) {
	            let one = this.sprView.getChildAt(i);
	            if (one.name == null || one.name.length == 0)
	                continue;
	            if (lp.x > one.x && lp.x < one.x + one.width && lp.y > one.y && lp.y < one.y + one.height)
	                return one;
	        }
	        return null;
	    }
	}

	class GT_UserGuideMask extends Sprite {
	    constructor(data, v) {
	        super();
	        this.mso = new MySourceManagerOne();
	        this.rad = 80;
	        this.busy = false;
	        this.Arr_datas = data;
	        if (v instanceof MySceneSprite)
	            this.View = v;
	        else {
	            this.View = v.owner;
	            this.Script = v;
	        }
	        Laya.stage.addChild(this);
	        this.guideContainer = new Sprite();
	        this.guideContainer.cacheAs = "bitmap";
	        this.addChild(this.guideContainer);
	        var maskArea = new Sprite();
	        maskArea.alpha = 0.5;
	        maskArea.graphics.drawRect(0, 0, ConfigTidus.stageRealW, ConfigTidus.stageRealH, "#000000");
	        this.guideContainer.addChild(maskArea);
	        this.interactionArea = new Sprite();
	        this.interactionArea.blendMode = "destination-out";
	        this.guideContainer.addChild(this.interactionArea);
	        this.mme = new MyMouseManager(this);
	        this.mme.addClickHandler(this.onClickF.bind(this));
	        this.onShowNextData();
	    }
	    static checkGuide(v, type) {
	        if (this.SDataDic == null)
	            return;
	        var step = PlayerDefault.player.gameProgress["指引"]["step"];
	        if (this.SDataDic[step] == null || this.SDataDic[step][type] == null)
	            return;
	        let data = this.SDataDic[step][type];
	        if (Tool_Function.isTypeOf(data, "arr") == false)
	            data = [data];
	        new GT_UserGuideMask(data, v);
	    }
	    destroyF() {
	        this.fullSpr = ConfigTidus.GFun_ToolObj_destroyOne(this.fullSpr);
	        this.mcHandClick = ConfigTidus.GFun_ToolObj_destroyOne(this.mcHandClick);
	        this.mme = ConfigTidus.GFun_ToolObj_destroyOne(this.mme);
	        this.ac = ConfigTidus.GFun_ToolObj_destroyOne(this.ac);
	        this.sprTips = ConfigTidus.GFun_ToolObj_destroyOne(this.sprTips);
	        this.mso = ConfigTidus.GFun_ToolObj_destroyOne(this.mso);
	        super.destroyF();
	        if (this.View) {
	            this.View.busy = false;
	            this.View = null;
	        }
	    }
	    onShowNextData() {
	        if (this.Arr_datas.length == 0) {
	            this.destroyF();
	            return;
	        }
	        this.fullSpr = ConfigTidus.GFun_ToolObj_destroyOne(this.fullSpr);
	        let nowdata = this.Arr_datas.shift();
	        this.View.busy = true;
	        if (nowdata["全屏img"] != null) {
	            this.guideContainer.visible = false;
	            this.fullSpr = new MySceneSprite();
	            this.fullSpr.initSprView(nowdata["全屏img"]["swf"], nowdata["全屏img"]["url"]);
	            this.addChild(this.fullSpr);
	            this.fullSpr.ac.Fun_mbc_click = (btn) => { this.onSureClick(); };
	            return;
	        }
	        this.guideContainer.visible = true;
	        if (nowdata["btn"]) {
	            let tso = new TidusScriptOne();
	            let btn = tso.doScriptOnce(nowdata["btn"], { "v": this.View, "s": this.Script });
	            this.onShowWithButton(nowdata["message"], btn, nowdata["fun"]);
	        }
	        else {
	            this.onShowF(nowdata["message"], nowdata["x"], nowdata["y"], nowdata["fun"]);
	        }
	    }
	    onClickF(e) {
	        if (this.busy)
	            return;
	        if (this.lastClickTime == ConfigTidus.GVal["MMMain"].countFrameTotals)
	            return;
	        if (e == null || this.nowPo == null) {
	            console.error("！！！！！");
	        }
	        if (Tool_Function.onGetLengthBetweenPoint(e, this.nowPo) > 100)
	            return;
	        this.lastClickTime = ConfigTidus.GVal["MMMain"].countFrameTotals;
	        this.onSureClick();
	    }
	    onSureClick() {
	        if (this.Arr_message && this.Arr_message.length > 0) {
	            this.showNextMessage();
	            return;
	        }
	        if (this.FunClick) {
	            this.View.busy = false;
	            if (Tool_Function.isTypeOf(this.FunClick, "str")) {
	                let tso = new TidusScriptOne();
	                tso.doScriptOnce(this.FunClick, { "v": this.View, "s": this.Script });
	            }
	            else {
	                this.FunClick();
	            }
	        }
	        this.onShowNextData();
	    }
	    onShowWithButton(message, btn, cb = null) {
	        let midpo = new Laya.Point(btn.width / 2, btn.height / 2);
	        btn.localToGlobal(midpo, false);
	        this.onShowF(message, midpo.x, midpo.y, cb);
	    }
	    onShowF(message, _x, _y, cb = null) {
	        if (GT_UserGuideMask.SWFName && message && MySourceManager.getInstance().getSwf(GT_UserGuideMask.SWFName) == null) {
	            let source = [[GT_UserGuideMask.SWFName, "swf"]];
	            LoadingSmall.showF();
	            this.mso.addSource(source, () => {
	                LoadingSmall.removeF();
	                if (MySourceManager.getInstance().getSwf(GT_UserGuideMask.SWFName) == null) {
	                    console.error("没有新手指引需要的swf：" + GT_UserGuideMask.SWFName);
	                    GT_UserGuideMask.SWFName = null;
	                }
	                this.onShowF(message, _x, _y, cb);
	            }, false);
	            return;
	        }
	        this.FunClick = cb;
	        this.nowPo = { "x": _x, "y": _y };
	        let typex = "右";
	        let typey = "下";
	        if (_x < ConfigTidus.stageW / 2)
	            typex = "左";
	        if (_y < 150 + ConfigTidus.stageH / 2)
	            typey = "上";
	        if (this.nowDir != typex + typey && GT_UserGuideMask.SWFName && message) {
	            this.nowDir = typex + typey;
	            this.ac = ConfigTidus.GFun_ToolObj_destroyOne(this.ac);
	            this.sprTips = ConfigTidus.GFun_ToolObj_destroyOne(this.sprTips);
	            this.sprTips = this.mso.getSprFromSwf(GT_UserGuideMask.SWFName, "spr_V" + this.nowDir);
	            if (this.sprTips) {
	                this.addChild(this.sprTips);
	                this.ac = new MyViewAllCompsController(this.sprTips);
	            }
	        }
	        else if (GT_UserGuideMask.SWFName == null || message == null) {
	            this.ac = ConfigTidus.GFun_ToolObj_destroyOne(this.ac);
	            this.sprTips = ConfigTidus.GFun_ToolObj_destroyOne(this.sprTips);
	        }
	        if (this.sprTips) {
	            if (ConfigTidus.stageW > ConfigTidus.stageH)
	                this.sprTips.x = _x - this.rad / 2;
	            this.sprTips.y = _y - this.rad;
	            this.Arr_message = [];
	            if (Tool_Function.isTypeOf(message, "str"))
	                this.Arr_message = [message];
	            else
	                this.Arr_message = Tool_ArrayUtils.copyArr(message);
	        }
	        else {
	            this.Arr_message = null;
	        }
	        this.interactionArea.graphics.clear();
	        this.interactionArea.graphics.drawCircle(_x, _y, this.rad, "#000000");
	        if (this.mcHandClick == null) {
	            this.mcHandClick = this.mso.getMcFromSwf(GT_UserGuideMask.SWFName, "mc_手指动画");
	            if (this.mcHandClick) {
	                this.addChild(this.mcHandClick);
	                this.mcHandClick.loop = true;
	                this.mcHandClick.play();
	            }
	        }
	        if (this.mcHandClick) {
	            this.mcHandClick.x = _x;
	            this.mcHandClick.y = _y;
	            this.mcHandClick.visible = false;
	        }
	        this.showNextMessage();
	    }
	    showNextMessage() {
	        if (this.Arr_message == null || this.Arr_message.length == 0) {
	            if (this.mcHandClick)
	                this.mcHandClick.visible = true;
	        }
	        else {
	            let str = this.Arr_message.shift();
	            if (str == null) {
	                this.sprTips.visible = false;
	            }
	            else {
	                this.sprTips.visible = true;
	                this.ac.setText("tx_info", str);
	            }
	            if (this.Arr_message.length == 0) {
	                if (this.mcHandClick)
	                    this.mcHandClick.visible = true;
	            }
	        }
	    }
	}
	GT_UserGuideMask.SWFName = "SWF_NewGuide";

	class IconModel extends Sprite {
	    constructor(spr, tar = null, showType = 0) {
	        super();
	        this.sprBack = spr;
	        this.ac = new MyViewAllCompsController(this.sprBack);
	        this.showType = showType;
	        Tool_SpriteUtils.onAddchild_ChangeParent(this, this.sprBack);
	        this.setTarget(tar);
	    }
	    destroyF() {
	        this.img = ConfigTidus.GFun_ToolObj_destroyOne(this.img);
	        this.imgRec = ConfigTidus.GFun_ToolObj_destroyOne(this.imgRec);
	        this.ac = ConfigTidus.GFun_ToolObj_destroyOne(this.ac);
	        this.sprBack = ConfigTidus.GFun_ToolObj_destroyOne(this.sprBack);
	        this.mme = ConfigTidus.GFun_ToolObj_destroyOne(this.mme);
	        this.target = null;
	        this.funClick = ConfigTidus.GFun_ToolObj_destroyOne(this.funClick);
	        super.destroyF();
	    }
	    setTarget(tar) {
	        this.target = tar;
	        this.img = ConfigTidus.GFun_ToolObj_destroyOne(this.img);
	        this.imgRec = ConfigTidus.GFun_ToolObj_destroyOne(this.imgRec);
	        if (this.target) {
	            this.sprBack.visible = true;
	            if (this.showType == IconModel.ShowType_allNoNum) {
	                this.ac.setText("tx_name", this.target.Name);
	            }
	            else {
	                if (this.target.num < 0) {
	                    this.ac.setText("tx_name", this.target.Name);
	                    this.ac.setNum("num_num", "");
	                }
	                else if (this.showType == IconModel.ShowType_nameNoNum_numNoX) {
	                    this.ac.setText("tx_name", this.target.Name);
	                    this.ac.setNum("num_num", this.target.num);
	                }
	                else if (this.showType == IconModel.ShowType_nameNoNum_numHaveX) {
	                    this.ac.setText("tx_name", this.target.Name);
	                    this.ac.setNum("num_num", "x" + this.target.num);
	                }
	                else if (this.showType == IconModel.ShowType_nameHaveNum_numNoX) {
	                    this.ac.setText("tx_name", this.target.Name + " x" + this.target.num);
	                    this.ac.setNum("num_num", this.target.num);
	                }
	                else if (this.showType == IconModel.ShowType_nameHaveNum_numHaveX) {
	                    this.ac.setText("tx_name", this.target.Name + " x" + this.target.num);
	                    this.ac.setNum("num_num", "x" + this.target.num);
	                }
	            }
	            this.img = this.target.getIcon();
	            let sprIcon = this.sprBack.getChildByName("icon");
	            let bg = this.sprBack.getChildByName("bg");
	            if (this.img) {
	                if (sprIcon) {
	                    if (sprIcon instanceof Image$1)
	                        this.sprBack.addChildWithChild(this.img, "icon", true, false);
	                    else
	                        sprIcon.addChild(this.img);
	                }
	                else if (bg)
	                    this.sprBack.addChildWithChild(this.img, "bg", true, false);
	                else
	                    this.sprBack.addChildAt(this.img, 0);
	            }
	            this.imgRec = Tool_Function.onRunFunction(IconModel.Fun_createRec, this.target);
	            if (this.imgRec) {
	                if (this.img) {
	                    this.imgRec.x = this.img.x;
	                    this.imgRec.y = this.img.y;
	                    this.img.parent.addChildAt(this.imgRec, this.img.parent.getChildIndex(this.img) + 1);
	                }
	                else if (sprIcon) {
	                    if (sprIcon instanceof Image$1)
	                        this.sprBack.addChildWithChild(this.imgRec, "icon", true, false);
	                    else
	                        sprIcon.addChild(this.imgRec);
	                }
	                else if (bg)
	                    this.sprBack.addChildWithChild(this.imgRec, "bg", true, false);
	                else
	                    this.sprBack.addChild(this.imgRec);
	            }
	            if (IconModel.Fun_show) {
	                Tool_Function.onRunFunction(IconModel.Fun_show, this.ac, this.target);
	            }
	        }
	        else {
	            this.sprBack.visible = false;
	        }
	    }
	    addClick(fun) {
	        if (this.mme == null) {
	            this.mme = new MyMouseManager(this);
	            this.mme.addClickHandler(this.onClick.bind(this));
	            if (this.target == null) {
	            }
	        }
	        this.funClick = fun;
	    }
	    onClick(p) {
	        if (this.funClick) {
	            Tool_Function.onRunFunction(this.funClick, this);
	        }
	        else {
	            ConfigTidus.GFun_MEO_dispatch("点击图标", this);
	        }
	    }
	}
	IconModel.ShowType_nameNoNum_numNoX = 0;
	IconModel.ShowType_nameNoNum_numHaveX = 1;
	IconModel.ShowType_nameHaveNum_numNoX = 2;
	IconModel.ShowType_nameHaveNum_numHaveX = 3;
	IconModel.ShowType_allNoNum = 10;

	class TidusScript_classInterface {
	    static LSOManager_get(name) { return LSOManager.get(name); }
	    static LSOManager_put(name, val) { LSOManager.put(name, val); }
	    static get todayDate() { return new Date(); }
	    static get NetMessageModel() { return ConfigTidus.GVal["NetMessageModel"].instance; }
	    static getNetMessageModelClass() { return ConfigTidus.GVal["NetMessageModel"]; }
	    static newMyMouseEvent(tar) { return new MyMouseManager(tar); }
	    static newSlideDefaultOne(spr) { var one = new MySlideMC_defaultOne(); one.initF(spr); return one; }
	    static newTmpMoviclip(mc) { var one = new TmpMovieClip(mc); return one; }
	    static newIconModel(spr, tar = null, showType = 0) { return new IconModel(spr, tar, showType); }
	    static newGameObject(info) { return new GT_GameObject(info); }
	    static newTmpMC(mc, cb = null, type = "nor") {
	        if ((mc instanceof SwfMovieClip) == false) {
	            mc = MySourceManager.getInstance().getMcFromSwf(mc["swf"], mc["url"]);
	        }
	        return new TmpMovieClip(mc, cb, type);
	    }
	    static newTmpTweenMove(mc, endx, endy, sec, ease, cb) {
	        let t = new Tween(mc, sec, ease);
	        if (endx == null)
	            endx = mc.x;
	        if (endy == null)
	            endy = mc.y;
	        t.moveTo(endx, endy);
	        return new TmpTween(t, cb);
	    }
	    static newTmpTweenScale(mc, ends, sec, ease, cb) {
	        let t = new Tween(mc, sec, ease);
	        if (ends == null)
	            ends = mc.scaleX;
	        t.scaleTo(ends);
	        return new TmpTween(t, cb);
	    }
	    static check_Guilde(v, type) {
	        GT_UserGuideMask.checkGuide(v, type);
	    }
	    static get MyPFManager() { return MyPlatformManager; }
	    static get GTMail() { return GT_Mail; }
	    static get GTHurt() { return GT_Hurt; }
	    static get MgsSocket() { return ConfigTidus.GVal["MgsSocket"]; }
	}

	class TidusScript_component {
	    constructor() {
	        this.needEnter = true;
	        this.pause = false;
	        this.needMouse = false;
	        this.stopMouseEventMop = false;
	        this.__valuesDic = {};
	        this.haveScript = false;
	        this.isDestroy = false;
	    }
	    static parseCode(script, cache) {
	        if (script == null || script.length == 0)
	            return null;
	        if (script.includes("try{") || script.includes("try {")) {
	            if (script.includes("catch(")) {
	                console.error("不支持try...catch..语法", null);
	                return null;
	            }
	        }
	        if (script.includes(" is ") || script.includes(" instanceof ")) {
	            console.error("不支持is语法，请用ToolFunction.isTypeOf代替", null);
	            return null;
	        }
	        if (script.indexOf(" int(") > 0 || script.indexOf("=int(") > 0 || script.indexOf(" Number(") > 0 || script.indexOf("=Number(") > 0) {
	            console.error("不支持强转语法，请用ToolFunction.onForceChangeType代替", null);
	            return null;
	        }
	        script = Tool_StringBuild.replaceSTR(script, "this.", "");
	        script = Tool_StringBuild.replaceSTR(script, "let ", "var ");
	        if (script.indexOf("class") == 0) {
	            if (script.includes("private funtion") || script.includes("public funtion") || script.includes("protected funtion")) {
	                script = "package{" + script + "}";
	            }
	            else {
	                script = "export " + script;
	            }
	        }
	        if (script.indexOf("export class") != -1) {
	            script = script.substr(script.indexOf("export class"));
	            script = "package {\n" + script + "}";
	            script = Tool_StringBuild.replaceSTR(script, "public ", "private ");
	            script = Tool_StringBuild.replaceSTR(script, "protected ", "private ");
	            script = Tool_StringBuild.replaceSTR(script, ".bind(this)", "");
	            script = Tool_StringBuild.replaceSTR(script, "export class", "public class");
	            script = Tool_StringBuild.replaceSTR(script, "Laya.", "");
	            let ipub = 0;
	            while (true) {
	                ipub = script.indexOf("private ", ipub);
	                if (ipub == -1)
	                    break;
	                if (script.indexOf("{", ipub) < script.indexOf(";", ipub) && script.indexOf(")", ipub) < script.indexOf("{", ipub)) {
	                    script = Tool_StringBuild.onInsertF(script, ipub + 8, "function ");
	                }
	                else {
	                    script = Tool_StringBuild.onInsertF(script, ipub + 8, "var ");
	                }
	                ipub++;
	            }
	        }
	        var arrFunNames = [];
	        let ifun = 0;
	        let ikuohao;
	        while (true) {
	            ifun = script.indexOf("function ", ifun);
	            if (ifun == -1)
	                break;
	            ikuohao = script.indexOf("(", ifun);
	            let fname = Tool_StringBuild.getStringBetweenTwoPlace(script, ifun + 9, "(");
	            arrFunNames.push(fname);
	            ifun++;
	        }
	        while (arrFunNames.length > 0) {
	            let fname = arrFunNames.pop();
	            script = Tool_StringBuild.replaceSTR(script, "this." + fname, fname);
	        }
	        if (script.indexOf("package") != 0) {
	            console.error("传入的脚本不是class类型！");
	            return null;
	        }
	        var cname = Tool_StringBuild.getStringBetweenTwoPlace(script, "public class ", "{");
	        if (cname.indexOf(" extends") != -1)
	            cname = cname.substr(0, cname.indexOf(" extends"));
	        console.log("得到class脚本：" + cname);
	        var dicScript = {};
	        var i = 0;
	        var j;
	        var k;
	        var m;
	        script = Tool_StringBuild.replaceSTR(script, "public var ", "private var ");
	        script = Tool_StringBuild.replaceSTR(script, "public function ", "private function ");
	        script = Tool_StringBuild.replaceSTR(script, "protected function ", "private function ");
	        while (true) {
	            i = script.indexOf("private var ", i);
	            if (i == -1)
	                break;
	            j = script.indexOf(":", i);
	            var vname = script.substring(i + 12, j);
	            i += 12;
	            k = script.indexOf(";", i);
	            m = script.lastIndexOf("=", k);
	            if (m > i) {
	                var strStartValue = Tool_StringBuild.onRemoveEnterSpace(script.substring(m + 1, k));
	                if (strStartValue.indexOf("new ") == -1) {
	                    try {
	                        dicScript[vname] = { "type": "变量", "value": JSON.parse(strStartValue) };
	                    }
	                    catch (error) {
	                        console.error("初始值[" + strStartValue + "]无法解析！");
	                        return null;
	                    }
	                }
	                else {
	                    k = strStartValue.indexOf("(");
	                    m = strStartValue.lastIndexOf(")");
	                    var classname = Tool_StringBuild.getStringBetweenTwoPlace(strStartValue, strStartValue.indexOf("new ") + 4, k);
	                    var values = Tool_StringBuild.getStringBetweenTwoPlace(strStartValue, k + 1, ")");
	                    dicScript[vname] = { "type": "变量", "class": classname, "startvalue": values };
	                }
	            }
	            else {
	                dicScript[vname] = { "type": "变量" };
	            }
	        }
	        i = 0;
	        while (i >= 0) {
	            i = script.indexOf("private function ", i);
	            if (i == -1)
	                break;
	            j = script.indexOf("(", i);
	            var fname = script.substring(i + 17, j);
	            i += 8;
	            var i2 = script.indexOf("private function ", i);
	            if (i2 == -1) {
	                j = script.lastIndexOf("}");
	                j = script.lastIndexOf("}", j - 1);
	                j = script.lastIndexOf("}", j - 1);
	                var fstr = Tool_StringBuild.getStringBetweenTwoPlace(script, i, j + 1);
	            }
	            else {
	                j = script.lastIndexOf("}", i2);
	                fstr = Tool_StringBuild.getStringBetweenTwoPlace(script, i, j + 1);
	            }
	            fstr = this.rebuildFunctionStr(fstr);
	            dicScript[fname] = { "type": "方法", "value": fstr };
	            i = i2;
	        }
	        if (cache) {
	            if (this.DicClass == null)
	                this.DicClass = {};
	            this.DicClass[cname] = dicScript;
	        }
	        return dicScript;
	    }
	    static rebuildFunctionStr(fstr) {
	        var i = 0;
	        var j;
	        var k;
	        var m;
	        j = fstr.indexOf("(");
	        while (true) {
	            j = fstr.indexOf(":", j + 1);
	            k = fstr.indexOf(")");
	            if (j == -1 || j > k)
	                break;
	            m = fstr.indexOf(",", j);
	            if (m == -1 || m > k) {
	                fstr = Tool_StringBuild.removeStrFromStr(fstr, j, k - j, null);
	            }
	            else {
	                fstr = Tool_StringBuild.removeStrFromStr(fstr, j, m - j, null);
	            }
	        }
	        j = fstr.indexOf(":", k);
	        i = fstr.indexOf("{");
	        if (j != -1 && j < i) {
	            fstr = Tool_StringBuild.removeStrFromStr(fstr, j, i - j, null);
	        }
	        j = 0;
	        var n;
	        while (true) {
	            j = fstr.indexOf("var ", j);
	            if (j == -1)
	                break;
	            k = fstr.indexOf(":", j);
	            m = fstr.indexOf(";", j);
	            if (k != -1 && k < m) {
	                n = Tool_StringBuild.minIndexBehind(fstr, ["=", " in "], j);
	                if (n != -1 && n < m) {
	                    if (n < k) {
	                    }
	                    else {
	                        fstr = Tool_StringBuild.removeStrFromStr(fstr, k, n - k, null);
	                    }
	                }
	                else {
	                    fstr = Tool_StringBuild.removeStrFromStr(fstr, k, m - k, null);
	                }
	            }
	            j += 3;
	        }
	        j = 0;
	        while (true) {
	            j = fstr.indexOf("as ", j);
	            if (j == -1)
	                break;
	            k = Tool_StringBuild.minIndexBehind(fstr, [";", ")", ","], j);
	            if (fstr.charAt(k) == ")") {
	                m = fstr.lastIndexOf("(", k);
	                fstr = Tool_StringBuild.removeStrFromStr(fstr, j, k - j + 1, null);
	                fstr = Tool_StringBuild.removeStrFromStr(fstr, m, 1, null);
	            }
	            else {
	                fstr = Tool_StringBuild.removeStrFromStr(fstr, j, k - j, null);
	            }
	            j += 3;
	        }
	        j = 0;
	        while (true) {
	            j = fstr.indexOf("new ", j);
	            if (j == -1)
	                break;
	            k = fstr.indexOf("(", j);
	            m = fstr.indexOf(");", j);
	            var classname = Tool_StringBuild.getStringBetweenTwoPlace(fstr, j + 3, k);
	            var values = Tool_StringBuild.getStringBetweenTwoPlace(fstr, k + 1, m);
	            if (values == null || values.length == 0)
	                fstr = Tool_StringBuild.removeStrFromStr(fstr, j, m - j, "newClassByName(\"" + classname + "\"");
	            else
	                fstr = Tool_StringBuild.removeStrFromStr(fstr, j, m - j, "newClassByName(\"" + classname + "\"," + values + "");
	        }
	        i = 0;
	        var countnewvalue = 0;
	        while (true) {
	            i = fstr.indexOf("newClassByName(", i);
	            if (i == -1)
	                break;
	            j = fstr.lastIndexOf(";", i);
	            k = fstr.lastIndexOf("=", i);
	            if (j == -1 || j < k)
	                j = k;
	            k = fstr.lastIndexOf("(", i);
	            if (k == -1) {
	                i += 5;
	                continue;
	            }
	            if (j < k) {
	                j = fstr.indexOf(")", i + "newClassByName(".length + 2);
	                middlestr = Tool_StringBuild.getStringBetweenTwoPlace(fstr, i, j + 1);
	                var replaceVname = "____newv" + countnewvalue;
	                fstr = Tool_StringBuild.removeStrFromStr(fstr, i, j - i, replaceVname);
	                k = fstr.lastIndexOf(";", k - 1);
	                if (k == -1)
	                    k = fstr.indexOf("{");
	                fstr = Tool_StringBuild.insertInto(fstr, "var " + replaceVname + "=" + middlestr + ";", k + 1);
	                i = 0;
	                countnewvalue++;
	                continue;
	            }
	            i += 5;
	        }
	        i = 0;
	        var countforin = 0;
	        while (true) {
	            i = fstr.indexOf("for(", i);
	            if (i == -1)
	                break;
	            j = fstr.indexOf(")", i);
	            k = fstr.indexOf(" in ", i);
	            if (k != -1 && k < j) {
	                var dicname = Tool_StringBuild.getStringBetweenTwoPlace(fstr, k + 3, j);
	                var keyname = Tool_StringBuild.getStringBetweenTwoPlace(fstr, i + 4, k);
	                if (keyname.indexOf(" ") != -1)
	                    keyname = keyname.substr(keyname.indexOf(" ") + 1);
	                var iname = "__i" + countforin;
	                var arrname = "__keys" + countforin;
	                countforin++;
	                fstr = Tool_StringBuild.removeStrFromStr(fstr, i, j - i + 2, "for(var " + iname + "=0;" + iname + "<" + arrname + ".length;" + iname + "++){\n var " + keyname + "=" + arrname + "[" + iname + "];");
	                var strGetarr = "var " + arrname + " = Tool_ObjUtils.getKeysOfObject(" + dicname + ");";
	                fstr = Tool_StringBuild.onInsertF(fstr, i, strGetarr);
	                i += strGetarr.length + 10;
	            }
	            else {
	                i += 10;
	            }
	        }
	        var dicChecked = {};
	        i = 0;
	        while (true) {
	            i = fstr.indexOf("]", i);
	            if (i == -1)
	                break;
	            j = fstr.lastIndexOf("[", i);
	            while (dicChecked[j] == true) {
	                j = fstr.lastIndexOf("[", j - 1);
	            }
	            if (j == -1)
	                throw new Error("解析报错，[]组不匹配");
	            dicChecked[j] = true;
	            var middlestr = fstr.substring(j + 1, i);
	            if (middlestr.indexOf(".") != -1 || middlestr.indexOf("[") != -1) {
	                var replaceVname = "____v" + countnewvalue;
	                k = fstr.lastIndexOf(";", j);
	                m = fstr.lastIndexOf("{", j);
	                if (k == -1)
	                    k = m;
	                else if (m != -1 && k < m)
	                    k = m;
	                fstr = Tool_StringBuild.removeStrFromStr(fstr, j + 1, i - j - 1, replaceVname);
	                fstr = Tool_StringBuild.insertInto(fstr, "var " + replaceVname + "=" + middlestr + ";", k + 1);
	                dicChecked = {};
	                i = 0;
	                countnewvalue++;
	                continue;
	            }
	            i++;
	        }
	        i = 0;
	        while (true) {
	            i = fstr.indexOf(")[");
	            if (i == -1)
	                i = fstr.indexOf(").");
	            if (i == -1)
	                break;
	            j = fstr.lastIndexOf("(", i);
	            k = Tool_StringBuild.minIndexBefore(fstr, ["+", "-", "*", "/", "{", "(", "[", "=", ";", "return "], j - 1);
	            if (k != -1) {
	                var replaceVname;
	                if (fstr.charAt(k) == "r") {
	                    middlestr = Tool_StringBuild.getStringBetweenTwoPlace(fstr, k + 7, i + 1);
	                    replaceVname = "____fv" + countnewvalue;
	                    fstr = Tool_StringBuild.removeStrFromStr(fstr, k + 7, i - k - 6, replaceVname);
	                }
	                else {
	                    middlestr = Tool_StringBuild.getStringBetweenTwoPlace(fstr, k + 1, i + 1);
	                    replaceVname = "____fv" + countnewvalue;
	                    fstr = Tool_StringBuild.removeStrFromStr(fstr, k + 1, i - k, replaceVname);
	                }
	                k = Tool_StringBuild.minIndexBefore(fstr, [";", "{"], k);
	                fstr = Tool_StringBuild.insertInto(fstr, "var " + replaceVname + "=" + middlestr + ";", k + 1);
	                i = 0;
	                countnewvalue++;
	                continue;
	            }
	            i += 3;
	        }
	        return fstr;
	    }
	    static registCustomClass(_name, classTarget) {
	        if (this.Dic_customClass == null)
	            this.Dic_customClass = {};
	        this.Dic_customClass[_name] = classTarget;
	    }
	    static runEnterForOwner(one) {
	        if (one.tidusscriptM) {
	            for (let i = 0; i < one.tidusscriptM.length; i++) {
	                if (one.tidusscriptM[i].getValue("needEnter") == false)
	                    one.tidusscriptM[i].____getEvent("enter");
	            }
	        }
	    }
	    static findOne(tar, cls) {
	        if (tar == null || tar.tidusscriptM == null)
	            return null;
	        for (let i = 0; i < tar.tidusscriptM.length; i++) {
	            let one = tar.tidusscriptM[i];
	            if (cls instanceof Function) {
	                if (one instanceof cls)
	                    return one;
	            }
	            else {
	                if (Tool_Function.getLastClassName(one) == cls)
	                    return one;
	            }
	        }
	        return null;
	    }
	    destroyF() {
	        this.isDestroy = true;
	        if (this.mmo) {
	            this.mmo.destroyF();
	            this.mmo = null;
	        }
	        if (this.mm) {
	            this.mm.destroyF();
	            this.mm = null;
	        }
	        if (this.tso) {
	            if (this.owner) {
	                Tool_ArrayUtils.removeTar(this.owner.tidusscriptM, this);
	            }
	            this.tso.addToGlobalObject("owner", null);
	            if (this.__valuesDic) {
	                var global = this.tso.vm.getGlobalObject();
	                for (let key in this.__valuesDic) {
	                    let one = global[key];
	                    if (one)
	                        ConfigTidus.GFun_ToolObj_destroyOne(one);
	                    delete global[key];
	                }
	                this.__valuesDic = null;
	            }
	            this.tso = null;
	        }
	    }
	    init(tar, script, info = null) {
	        this.owner = tar;
	        if (this.owner.tidusscriptM == null)
	            this.owner.tidusscriptM = [];
	        this.owner.tidusscriptM.push(this);
	        if (TidusScriptOne.FunRebuidStr == null)
	            TidusScriptOne.FunRebuidStr = TidusScript_component.rebuildFunctionStr;
	        this.tso = new TidusScriptOne();
	        this.tso.addToGlobalObject("owner", this.owner);
	        this.tso.addToGlobalObject("getClassByName", this.getClassByName.bind(this));
	        this.tso.addToGlobalObject("newClassByName", this.newClassByName.bind(this));
	        this.tso.addToGlobalObject("getGlobalValue", this.getGlobalValue.bind(this));
	        this.tso.addToGlobalObject("addEventListener", this.addEventListener.bind(this));
	        this.tso.addToGlobalObject("dispatchEvent", this.dispatchEvent.bind(this));
	        this.tso.addToGlobalObject("addDelayFunction", this.addDelayFunction.bind(this));
	        this.tso.addToGlobalObject("JSON", JSON);
	        this.tso.addToGlobalObject("isNaN", isNaN);
	        this.tso.addToGlobalObject("Config", ConfigTidus);
	        this.tso.addToGlobalObject("trace", console.log);
	        this.tso.addToGlobalObject("console", console);
	        this.tso.addToGlobalObject("Tool_Function", Tool_Function);
	        this.tso.addToGlobalObject("Tool_Object", ConfigTidus.GVal["Tool_Object"]);
	        this.tso.addToGlobalObject("TidusScript_component", TidusScript_component);
	        this.tso.addToGlobalObject("TidusScript_classInterface", TidusScript_classInterface);
	        if (TidusScript_component.Dic_customClass) {
	            for (var customname in TidusScript_component.Dic_customClass) {
	                this.tso.addToGlobalObject(customname, TidusScript_component.Dic_customClass[customname]);
	            }
	        }
	        if (script == null || script.length == 0) {
	            var lastname = Tool_Function.getLastClassName(this);
	            if (TidusScript_component.DicClass && TidusScript_component.DicClass[lastname])
	                script = TidusScript_component.DicClass[lastname];
	        }
	        if (script) {
	            if (Tool_Function.isTypeOf(script, "str")) {
	                if (script.indexOf("package") == 0 || script.indexOf("export class") != -1 || script.indexOf("class") == 0) {
	                    var dicScript = TidusScript_component.parseCode(script, true);
	                }
	                else if (script.indexOf("function") == 0) {
	                    this.tso.addFunction(TidusScript_component.rebuildFunctionStr(script));
	                }
	                else if (TidusScript_component.DicClass && TidusScript_component.DicClass[script]) {
	                    dicScript = TidusScript_component.DicClass[script];
	                }
	                else {
	                    let resourceScript = ConfigTidus.GVal["MySourceManager"].getInstance().getTxt(script);
	                    if (resourceScript) {
	                        TidusScript_component.parseCode(resourceScript, true);
	                        if (TidusScript_component.DicClass && TidusScript_component.DicClass[script])
	                            dicScript = TidusScript_component.DicClass[script];
	                    }
	                    if (dicScript == null && script.length > 50)
	                        console.error("无法初始化的脚本：" + script);
	                }
	            }
	            else {
	                dicScript = script;
	            }
	            if (dicScript) {
	                var strFunctions = "";
	                for (var key in dicScript) {
	                    if (dicScript[key]["type"] == "方法") {
	                        strFunctions += dicScript[key]["value"];
	                    }
	                    else if (dicScript[key]["class"] != null) {
	                        if (dicScript[key]["startvalue"] == null || dicScript[key]["startvalue"].length == 0) {
	                            this.tso.addToGlobalObject(key, this.newClassByName(dicScript[key]["class"]));
	                        }
	                        else {
	                            let startvaluearr = dicScript[key]["startvalue"].split(",");
	                            for (let i = 0; i < startvaluearr.length; i++) {
	                                startvaluearr[i] = JSON.parse(startvaluearr[i]);
	                            }
	                            if (startvaluearr.length == 1) {
	                                this.tso.addToGlobalObject(key, this.newClassByName(dicScript[key]["class"], startvaluearr[0]));
	                            }
	                            else if (startvaluearr.length == 2) {
	                                this.tso.addToGlobalObject(key, this.newClassByName(dicScript[key]["class"], startvaluearr[0], startvaluearr[1]));
	                            }
	                            else if (startvaluearr.length == 3) {
	                                this.tso.addToGlobalObject(key, this.newClassByName(dicScript[key]["class"], startvaluearr[0], startvaluearr[1], startvaluearr[2]));
	                            }
	                            else {
	                                console.error("暂不支持过长的方法外变量初始构造参数！");
	                            }
	                        }
	                        this.__valuesDic[key] = true;
	                    }
	                    else {
	                        this.tso.addToGlobalObject(key, dicScript[key]["value"]);
	                        this.__valuesDic[key] = true;
	                    }
	                }
	                if (strFunctions.length > 0)
	                    this.tso.addFunction(strFunctions);
	                this.haveScript = true;
	                this.setValue("haveScript", true);
	            }
	        }
	        if (info) {
	            for (var vname in info) {
	                this.setValue(vname, info[vname]);
	            }
	        }
	        this.____getEvent("init");
	        if (this.owner == null)
	            return;
	        if (this.getValue("needEnter")) {
	            this.mmo = new MainManager();
	            this.mmo.addEnterFrame(this.enterF.bind(this));
	        }
	        if (this.getValue("needMouse") == false) {
	            if (this.tso.vm.getGlobalObject()["onMouseDownF"] != null || this.tso.vm.getGlobalObject()["mouseup"] != null || this.tso.vm.getGlobalObject()["mousemove"] != null
	                || this.tso.vm.getGlobalObject()["onMouseClickF"] != null) {
	                this.needMouse = true;
	            }
	        }
	        if (this.getValue("needMouse") == true) {
	            this.owner.touchable = true;
	            this.mm = new MyMouseManager(this.owner);
	            this.mm.addClickHandler((p) => {
	                this.____getEvent("click", { "lp": this.mm.nowLocalXY, "gp": this.mm.worldXY });
	            });
	        }
	    }
	    enterF() {
	        this.____getEvent("enter");
	    }
	    getGlobalValue(vname) {
	        return this.tso.vm.getGlobalObject()[vname];
	    }
	    getClassByName(cname) {
	        if (TidusScript_component.Dic_customClass && TidusScript_component.Dic_customClass[cname])
	            return TidusScript_component.Dic_customClass[cname];
	        if (this.tso.vm.getGlobalObject()[cname])
	            return this.tso.vm.getGlobalObject()[cname];
	        return null;
	    }
	    newClassByName(cname, ...arg) {
	        var c = this.getClassByName(cname);
	        if (c) {
	            if (arg.length == 0)
	                return new c();
	            else if (arg.length == 1)
	                return new c(arg[0]);
	            else if (arg.length == 2)
	                return new c(arg[0], arg[1]);
	            else if (arg.length == 3)
	                return new c(arg[0], arg[1], arg[2]);
	            else if (arg.length == 4)
	                return new c(arg[0], arg[1], arg[2], arg[3]);
	            else if (arg.length == 5)
	                return new c(arg[0], arg[1], arg[2], arg[3], arg[4]);
	        }
	        console.error("脚本中NEW CLASS ： " + cname + "  得到Null！");
	        return null;
	    }
	    ____getEvent(type, val = null) {
	        if (this.pause)
	            return;
	        if (type == "init") {
	            this.runFun("onInit");
	        }
	        else if (type == "enter") {
	            this.runFun("onEnterF");
	        }
	        else if (type == "hit") {
	            this.runFun("hit", val);
	        }
	        else if (type == "mousedown") {
	            this.runFun("onMouseDownF", val);
	        }
	        else if (type == "mouseup") {
	            this.runFun("onMouseUpF", val);
	        }
	        else if (type == "mousemove") {
	            this.runFun("onMouseMoveF", val);
	        }
	        else if (type == "click") {
	            this.runFun("onMouseClickF", val);
	        }
	    }
	    runFun(funName, ...arg) {
	        if (funName instanceof Function)
	            funName = funName.name;
	        if (this.haveScriptFunction(funName)) {
	            try {
	                if (arg.length == 0) {
	                    return this.tso.runFunction(funName, null);
	                }
	                else if (arg.length == 1) {
	                    return this.tso.runFunction(funName, null, arg[0]);
	                }
	                else if (arg.length == 2) {
	                    return this.tso.runFunction(funName, null, arg[0], arg[1]);
	                }
	                else if (arg.length == 3) {
	                    return this.tso.runFunction(funName, null, arg[0], arg[1], arg[2]);
	                }
	                else if (arg.length == 4) {
	                    return this.tso.runFunction(funName, null, arg[0], arg[1], arg[2], arg[3]);
	                }
	            }
	            catch (error) {
	                console.error("执行脚本方法[" + funName + "]报错：" + error);
	            }
	        }
	        else {
	            try {
	                var f = this[funName];
	            }
	            catch (error) {
	                return;
	            }
	            if (f == null)
	                return;
	            f = f.bind(this);
	            if (arg.length == 0) {
	                return f();
	            }
	            else if (arg.length == 1) {
	                return f(arg[0]);
	            }
	            else if (arg.length == 2) {
	                return f(arg[0], arg[1]);
	            }
	            else if (arg.length == 3) {
	                return f(arg[0], arg[1], arg[2]);
	            }
	            else if (arg.length == 4) {
	                return f(arg[0], arg[1], arg[2], arg[3]);
	            }
	        }
	    }
	    haveScriptFunction(fName) {
	        if (this.tso && this.tso.vm.getGlobalObject()[fName]) {
	            return true;
	        }
	        return false;
	    }
	    getValue(vname) {
	        if (this.tso == null) {
	            return null;
	        }
	        if (this.haveScript) {
	            let v = this.tso.vm.getGlobalObject()[vname];
	            return v;
	        }
	        else {
	            return this[vname];
	        }
	    }
	    setValue(vname, val) {
	        if (this.tso == null)
	            return null;
	        if (this.haveScript) {
	            let v = this.tso.vm.getGlobalObject()[vname];
	            this.tso.addToGlobalObject(vname, val);
	        }
	        else {
	            this[vname] = val;
	        }
	    }
	    addEventListener(ename, fun, once = false) {
	        if (this.mmo == null)
	            this.mmo = new MainManager();
	        this.mmo.EventManager.addCustomEventListener(ename, fun, once);
	    }
	    dispatchEvent(ename, val = null, cb = null) {
	        if (cb) {
	            this.addEventListener(ename + ":::::Back", cb, true);
	        }
	        MainManager.instance.EventManager.onCustomEventAct(ename, val);
	    }
	    addDelayFunction(frame, fun, val = null) {
	        return MainManager.instance.addDelayFunction(frame, fun, val);
	    }
	    onInit() {
	    }
	    onEnterF() {
	    }
	}

	class GT_Skill {
	    constructor() {
	        this.Lv = 0;
	        this.isAtk = false;
	        this.isPure = false;
	    }
	    static createNewSkill(bid, tar, netValue, data = null, save = true) {
	        if (tar == null)
	            return null;
	        if (this.SDataDic == null && data == null)
	            throw new Error("还未给GT_Skill.SDataDic赋值！！");
	        if (data == null)
	            data = this.SDataDic[bid];
	        if (data == null) {
	            ConfigTidus.LogF("没有skill数据，bid==" + bid);
	            return;
	        }
	        let s;
	        if (this.FunNewSkill) {
	            s = this.FunNewSkill(bid, tar, netValue);
	        }
	        else {
	            s = new GT_Skill();
	        }
	        s.baseID = bid;
	        s.role = tar;
	        if (save) {
	            if (tar.SkillData == null)
	                tar.SkillData = {};
	            tar.SkillData[bid] = s;
	        }
	        s.Name = data["name"];
	        s.customData = ConfigTidus.GFun_ToolObj_copy(data["自定义"]);
	        s.cost = data["cost"];
	        if (data["cd"] != null && Tool_Function.isTypeOf(data["cd"], "int"))
	            s.CD = { "time": data["cd"] };
	        else
	            s.CD = data["cd"];
	        s.animate = data["美术"];
	        s.IconData = data["图标"];
	        if (s.customData && s.customData["数量"])
	            s.haveNum = s.customData["数量"];
	        if (data["攻击"]) {
	            s.isAtk = true;
	            s.attackData = data["攻击"];
	        }
	        if (data["辅助"]) {
	            s.isPure = true;
	            s.pureData = data["辅助"];
	        }
	        if (data["被动"]) {
	            s.passitiveData = data["被动"];
	        }
	        if (netValue) {
	            if (netValue["lv"] != null)
	                s.Lv = netValue["lv"];
	            if (netValue["自定义"] != null) {
	                if (s.customData == null)
	                    s.customData = netValue["自定义"];
	                else
	                    ConfigTidus.GFun_ToolObj_combo(s.customData, netValue["自定义"], false, false, false);
	            }
	        }
	        s.value = new MyCheaterNum();
	        if (data["属性"]) {
	            for (let vname in data["属性"]) {
	                s.value.setValue(vname, data["属性"][vname]);
	            }
	        }
	        s.onChangedLv(data["升级"]);
	        s.strIntroduce = data["描述"];
	        s.Type = data["分类"];
	        return s;
	    }
	    get Name() { return this._Name; }
	    set Name(v) { this._Name = v; }
	    get Introduce() {
	        var data = GT_Skill.SDataDic[this.baseID];
	        if (data) {
	            let str = data["介绍"];
	            str = GT_Skill.getPureStringWithRealValue(str, this.value);
	            return str;
	        }
	        return "";
	    }
	    onChangedLv(dicGrow) {
	        if (dicGrow == null || dicGrow["属性"] == null)
	            return;
	        dicGrow = dicGrow["属性"];
	        for (let key in dicGrow) {
	            let start = this.value.getValue(key);
	            if (start == null) {
	                console.error("技能" + this.Name + "中，升级时找不到基础属性：" + key);
	                continue;
	            }
	            if (Tool_Function.isTypeOf(start, "str") && start.includes("%")) {
	                start = Tool_Function.onForceConvertType(start.substr(0, start.length - 1));
	            }
	            var newnum = 0;
	            let str = dicGrow[key];
	            let midlv = Tool_Function.onForceConvertType(str.slice(1, str.indexOf("级")));
	            str = str.substr(str.indexOf("级") + 1);
	            let num = Tool_Function.onForceConvertType(str.substr(1));
	            if (str.includes("*")) {
	                newnum = start;
	                for (let i = 1 + midlv; i <= this.Lv; i += midlv) {
	                    newnum = Tool_Function.onForceConvertType(newnum * num);
	                }
	            }
	            else {
	                if (str.charAt(0) == "+") {
	                    newnum = start + num * Tool_Function.onForceConvertType((this.Lv - 1) / midlv);
	                }
	                else {
	                    newnum = start - num * Tool_Function.onForceConvertType((this.Lv - 1) / midlv);
	                }
	            }
	            this.value.setValue(key, newnum);
	        }
	    }
	    initFight() {
	        if (!isNaN(this.haveNum) && this.haveNum > 0) {
	            this.nowNum = this.haveNum;
	        }
	        if (this.CD && this.CD["初始"] == true)
	            this.onResetCD();
	    }
	    onCreatePassitiveBuff() {
	        if (this.passitiveData != null && this.passitiveData["buff"] != null && this.role) {
	            let arr = this.passitiveData["buff"];
	            this.checkTargetBuff(this.role, arr);
	        }
	    }
	    checkCanUse() {
	        if (!isNaN(this.haveNum)) {
	            if (this.nowNum <= 0)
	                return GT_Skill.ReasonCant_num;
	        }
	        if (this.CD != null) {
	            if (this.countcd > 0)
	                return GT_Skill.ReasonCant_cd;
	        }
	        if (this.cost) {
	            for (let need in this.cost) {
	                let now = this.role.getValue(need);
	                let costnum = this.getRealCostNumOfCostType(need);
	                if (costnum > now)
	                    return GT_Skill.ReasonCant_cost;
	            }
	            if (this.role.buffEffectObj["沉默"] == true)
	                return false;
	        }
	        return true;
	    }
	    getRealCostNumOfCostType(need) {
	        if (Tool_Function.isTypeOf(this.cost[need], "num")) {
	            return this.cost[need];
	        }
	        else {
	            let str = this.cost[need];
	            if (str.includes("%") == false) {
	                console.error("cost中有未知的数据：" + str);
	                return 999999;
	            }
	            let per = Tool_Function.onForceConvertType(str.substr(0, str.length - 1));
	            let max = this.role.getValue(need + "Max");
	            return Tool_Function.onForceConvertType(max * per * 0.01);
	        }
	    }
	    onResetCD() {
	        if (this.CD != null) {
	            if (this.CD["type"] == "时间") {
	                this.countcd = this.CD["time"] * ConfigTidus.FrameRate;
	            }
	            else {
	                this.countcd = this.CD["time"];
	            }
	            this.onEventF("CD");
	        }
	    }
	    onActF() {
	        this.onResetCD();
	        if (this.cost) {
	            for (let need in this.cost) {
	                if (Tool_Function.isTypeOf(this.cost[need], "num")) {
	                    this.role.beHurt(need, this.cost[need], null);
	                }
	                else {
	                    let str = this.cost[need];
	                    let per = Tool_Function.onForceConvertType(str.substr(0, str.length - 1));
	                    let max = this.role.getValue(need + "Max");
	                    let costv = Tool_Function.onForceConvertType(max * per * 0.01);
	                    this.role.beHurt(need, costv, null);
	                }
	            }
	        }
	        if (!isNaN(this.haveNum)) {
	            this.nowNum--;
	        }
	        if (this.canThroughType != 0) {
	            this.canThrough = true;
	        }
	        else {
	            this.canThrough = false;
	        }
	        this.atkCountTime = 0;
	        this.hitTars = null;
	    }
	    addEventCallBack(ename, fun) {
	        if (this.Dic_callBack == null)
	            this.Dic_callBack = {};
	        if (this.Dic_callBack[ename] == null)
	            this.Dic_callBack[ename] = [];
	        this.Dic_callBack[ename].push(fun);
	    }
	    clearAllEventCB() {
	        this.Dic_callBack = null;
	    }
	    onEnterF() {
	        if (this.CD != null) {
	            if (this.CD["type"] == "事件") {
	            }
	            else {
	                this.onCDOne();
	            }
	        }
	    }
	    onCDOne() {
	        if (this.countcd > 0) {
	            this.countcd--;
	            this.onEventF("CD");
	        }
	    }
	    onAtkF(tars) {
	        if (tars == null || tars.length == 0)
	            return;
	        if (this.isAtk && this.attackData) {
	            let data = this.attackData;
	            if (data) {
	                let atkType = data["类型"] ? data["类型"] : GT_Role.getAttibuteType(data["属性"]);
	                let atk = 0;
	                let atkp = 0;
	                let breakdef = 0;
	                if (data["atk"] != null) {
	                    if (Tool_Function.isTypeOf(data["atk"], "str")) {
	                        atk = GT_Skill.getRealValueFromValue(data["atk"], this.value, { "role": this.role });
	                        atk = Tool_Function.onForceConvertType(atk);
	                    }
	                    else if (data["atk"] > 0) {
	                        atk = data["atk"];
	                    }
	                }
	                if (data["atkP"] != null) {
	                    if (Tool_Function.isTypeOf(data["atkP"], "str")) {
	                        atkp = GT_Skill.getRealValueFromValue(data["atkP"], this.value, { "role": this.role });
	                        atkp = Tool_Function.onForceConvertType(atkp);
	                    }
	                    else if (data["atkP"] > 0) {
	                        atkp = data["atkP"];
	                    }
	                }
	                if (atkp > 0) {
	                    let batk;
	                    if (data["类型"] == "物理")
	                        batk = this.role.getValue(GT_Role.VName_Phy);
	                    else if (data["类型"] == "魔法")
	                        batk = this.role.getValue(GT_Role.VName_Mag);
	                    else
	                        batk = this.role.getValue(GT_Role.VName_Atk);
	                    if (batk == 0)
	                        batk = this.role.getValue(GT_Role.VName_Atk);
	                    atk += Tool_Function.onForceConvertType(batk * atkp * 0.01);
	                }
	                if (data["破防"] != null) {
	                    if (Tool_Function.isTypeOf(data["破防"], "str")) {
	                        breakdef = GT_Skill.getRealValueFromValue(data["破防"], this.value, { "role": this.role });
	                    }
	                    else if (data["破防"] > 0) {
	                        breakdef = data["破防"];
	                    }
	                    if (breakdef > 0) {
	                        let peradd = this.role.getValue("破防效果");
	                        if (peradd != 0)
	                            breakdef += peradd * breakdef * 0.01;
	                        breakdef = Tool_Function.onForceConvertType(breakdef);
	                    }
	                }
	                let dicbuffdata = { "属性": data["属性"], "类型": atkType, "补正": data["补正"], "职业补正": data["职业补正"], "类型补正": data["类型补正"], "自定义补正": data["自定义补正"] };
	                let perBloodSuck = this.role.getValue(GT_Role.VName_Bloodsucking);
	                if (atkType == "物理")
	                    perBloodSuck += this.role.getValue(GT_Role.VName_PhyBloodsucking);
	                else if (atkType == "魔法")
	                    perBloodSuck += this.role.getValue(GT_Role.VName_MagBloodsucking);
	                let sucHP = 0;
	                for (let i = 0; i < tars.length; i++) {
	                    let tar = tars[i];
	                    let miss = GT_Hurt.onMiss(this.role, tar, this);
	                    if (miss) {
	                        console.log(tar.Name + "：闪避了" + this.role.Name + "的攻击：" + this.Name);
	                        tar.beHurt("闪避", 0, this.role, dicbuffdata);
	                        continue;
	                    }
	                    let cri = GT_Hurt.onCritcal(this.role, tar, this);
	                    this.role.checkBuff(GT_Buff.BuffTime_atkBefore, { "num": atk, "暴击": cri, "tar": tar });
	                    if (this.role.buffEffectTmpObj["num"] != null) {
	                        atk = this.role.buffEffectTmpObj["num"];
	                        delete this.role.buffEffectTmpObj["num"];
	                    }
	                    dicbuffdata["暴击"] = cri;
	                    atk = tar.beHurt(GT_Role.VName_HP, atk, this.role, dicbuffdata);
	                    if (this.hitTars = null) {
	                        if (this.hitTars[tar.netID] == null)
	                            this.hitTars[tar.netID] = { "time": 0, "hp": 0 };
	                        this.hitTars[tar.netID]["time"]++;
	                        this.hitTars[tar.netID]["hp"] += atk;
	                    }
	                    dicbuffdata["hp"] = atk;
	                    this.role.checkBuff(GT_Buff.BuffTime_atkRole, dicbuffdata);
	                    this.role.onEventF("造成伤害", atk);
	                    if (breakdef > 0)
	                        tar.beHurt("破坏值", breakdef, this.role, null);
	                    if (perBloodSuck > 0) {
	                        let _suckHp = atk * perBloodSuck * 0.01;
	                        sucHP += _suckHp;
	                    }
	                    this.checkTargetBuff(tar, data["buff"], dicbuffdata);
	                }
	                if (sucHP >= 1) {
	                    sucHP = Tool_Function.onForceConvertType(sucHP);
	                    this.role.bePure(GT_Role.VName_HP, sucHP);
	                }
	            }
	            else {
	                console.error(this.Name + ":攻击帧没有找到对应攻击数据");
	            }
	            this.atkCountTime++;
	        }
	        if (this.isPure && this.pureData) {
	            for (let i = 0; i < tars.length; i++) {
	                let tar = tars[i];
	                this.checkTargetBuff(tar, this.pureData["buff"]);
	                if (this.pureData["HP"] != null) {
	                    let pureHP;
	                    if (Tool_Function.isTypeOf(this.pureData["HP"], "num")) {
	                        pureHP = this.pureData["HP"];
	                    }
	                    else {
	                        let strPureHP = this.pureData["HP"];
	                        if (strPureHP.includes("%")) {
	                            strPureHP = Tool_StringBuild.replaceSTR(strPureHP, "%", " *role.HP");
	                        }
	                        pureHP = GT_Skill.getRealValueFromValue(strPureHP, this.value, { "role": this.role }, "Skill" + this.Name);
	                    }
	                    this.role.checkBuff(GT_Buff.BuffTime_pureBefore, { "num": pureHP, "tar": tar });
	                    if (this.role.buffEffectTmpObj["num"] != null) {
	                        pureHP = this.role.buffEffectTmpObj["num"];
	                        delete this.role.buffEffectTmpObj["num"];
	                    }
	                    let peradd = this.role.getValue("治疗加成");
	                    if (peradd != 0) {
	                        pureHP = Tool_Function.onForceConvertType(pureHP + pureHP * peradd * 0.01);
	                    }
	                    tar.bePure(GT_Role.VName_HP, pureHP);
	                    if (this.hitTars = null) {
	                        if (this.hitTars[tar.netID] == null)
	                            this.hitTars[tar.netID] = { "time": 0, "hp": 0, "mp": 0 };
	                        this.hitTars[tar.netID]["time"]++;
	                        this.hitTars[tar.netID]["hp"] += pureHP;
	                    }
	                    let dicbuffdata = {};
	                    dicbuffdata["hp"] = pureHP;
	                    this.role.checkBuff(GT_Buff.BuffTime_pureRole, dicbuffdata);
	                    this.role.onEventF("造成治疗", pureHP);
	                }
	                if (this.pureData["MP"] != null) {
	                    let pureHP;
	                    if (Tool_Function.isTypeOf(this.pureData["MP"], "num")) {
	                        pureHP = this.pureData["MP"];
	                    }
	                    else {
	                        let strPureHP = this.pureData["MP"];
	                        if (strPureHP.includes("%")) {
	                            strPureHP = Tool_StringBuild.replaceSTR(strPureHP, "%", " *role.MP");
	                        }
	                        pureHP = GT_Skill.getRealValueFromValue(strPureHP, this.value, { "role": this.role }, "Skill" + this.Name);
	                    }
	                    tar.bePure(GT_Role.VName_MP, pureHP);
	                    if (this.hitTars = null) {
	                        if (this.hitTars[tar.netID] == null)
	                            this.hitTars[tar.netID] = { "time": 0, "hp": 0, "mp": 0 };
	                        this.hitTars[tar.netID]["time"]++;
	                        this.hitTars[tar.netID]["mp"] += pureHP;
	                    }
	                    let dicbuffdata = {};
	                    dicbuffdata["mp"] = pureHP;
	                    this.role.checkBuff(GT_Buff.BuffTime_pureRole, dicbuffdata);
	                    this.role.onEventF("造成治疗", pureHP);
	                }
	                if (this.pureData["治疗异常"] != null) {
	                    tar.bePureBadBuff(this.pureData["治疗异常"]);
	                }
	            }
	        }
	    }
	    checkTargetBuff(tar, targetBuffs, buffdata = null) {
	        if (targetBuffs) {
	            if (targetBuffs instanceof Array == false) {
	                targetBuffs = [targetBuffs];
	            }
	            let arr = targetBuffs;
	            for (let i = 0; i < arr.length; i++) {
	                if (arr[i]["id"] != null) {
	                    if (arr[i]["生成条件"] && arr[i]["生成条件"] != "无") {
	                        let need = arr[i]["生成条件"];
	                        need = GT_Buff.getRealNeedCodeFromNeed(need);
	                        let re = GT_Skill.getRealValueFromValue(need, this.value, { "role": this.role, "dic": buffdata });
	                        if (re == false)
	                            continue;
	                    }
	                    GT_Buff.createNewBuff(arr[i]["id"], tar, this.value.getNowObject(), arr[i], this.role, this);
	                }
	            }
	        }
	    }
	    static changeTSO_quickRoleValueToFull(str) {
	        str = Tool_StringBuild.replaceSTR(str, "role.HP", 'role.getValue("HP")');
	        str = Tool_StringBuild.replaceSTR(str, "role.MP", 'role.getValue("MP")');
	        str = Tool_StringBuild.replaceSTR(str, "role.力量", 'role.getValue("力量")');
	        str = Tool_StringBuild.replaceSTR(str, "role.智力", 'role.getValue("智力")');
	        str = Tool_StringBuild.replaceSTR(str, "role.敏捷", 'role.getValue("敏捷")');
	        str = Tool_StringBuild.replaceSTR(str, "role.幸运", 'role.getValue("幸运")');
	        str = Tool_StringBuild.replaceSTR(str, "role.速度", 'role.getValue("速度")');
	        str = Tool_StringBuild.replaceSTR(str, "role.攻击", 'role.getValue("攻击")');
	        str = Tool_StringBuild.replaceSTR(str, "role.防御", 'role.getValue("防御")');
	        str = Tool_StringBuild.replaceSTR(str, "role.物攻", 'role.getValue("物攻")');
	        str = Tool_StringBuild.replaceSTR(str, "role.魔攻", 'role.getValue("魔攻")');
	        str = Tool_StringBuild.replaceSTR(str, "role.物防", 'role.getValue("物防")');
	        str = Tool_StringBuild.replaceSTR(str, "role.魔防", 'role.getValue("魔防")');
	        str = Tool_StringBuild.replaceSTR(str, "role.命中", 'role.getValue("命中")');
	        str = Tool_StringBuild.replaceSTR(str, "role.闪避", 'role.getValue("闪避")');
	        return str;
	    }
	    static getPureStringWithRealValue(str, _values, errorTitle = "") {
	        var value;
	        if (_values instanceof MyCheaterNum)
	            value = _values.getNowObject();
	        else
	            value = _values;
	        while (true) {
	            let i = str.indexOf("@");
	            if (i == -1)
	                break;
	            let vname;
	            for (let key in value) {
	                if (str.indexOf(key, i) == i + 1) {
	                    vname = key;
	                    break;
	                }
	            }
	            if (vname == null) {
	                console.error(errorTitle + "-->公式：" + str + "  没有@对应的属性，中断转换");
	                return str;
	            }
	            else {
	                if (vname.length + 1 == str.length)
	                    str = value[vname] + "";
	                else
	                    str = Tool_StringBuild.removeStrFromStr(str, i, vname.length + 1, value[vname] + " ");
	            }
	        }
	        str = this.changeTSO_quickRoleValueToFull(str);
	        return str;
	    }
	    static getTSO_byRealValueString(str) {
	        if (GT_Skill.Dic_tso == null)
	            GT_Skill.Dic_tso = {};
	        let tso;
	        if (GT_Skill.Dic_tso[str] == null) {
	            tso = new TidusScriptOne();
	            GT_Skill.Dic_tso[str] = tso;
	        }
	        else {
	            tso = GT_Skill.Dic_tso[str];
	            str = null;
	        }
	        return { "tso": tso, "str": str };
	    }
	    static getRealValueFromValue(str, _values, context, errorTitle = "") {
	        str = this.getPureStringWithRealValue(str, _values, errorTitle);
	        if (str.includes("@"))
	            return 0;
	        if (str.includes("+") || str.includes("-") || str.includes("*") || str.includes("/") || str.includes("role") || str.includes("tar")) {
	            let obj = this.getTSO_byRealValueString(str);
	            let tso = obj["tso"];
	            str = obj["str"];
	            return tso.doScriptOnce(str, context);
	        }
	        else {
	            return Tool_Function.onForceConvertType(str, "num");
	        }
	    }
	    onEventF(ename) {
	        if (this.Dic_callBack && this.Dic_callBack[ename]) {
	            let arr = this.Dic_callBack[ename];
	            for (let i = 0; i < arr.length; i++) {
	                arr[i]();
	            }
	        }
	    }
	    onStopF() {
	        if (this.isAtk && this.hitTars) {
	            this.role.checkBuff(GT_Buff.BuffTime_atkEnd, this.hitTars);
	        }
	        if (this.isPure && this.hitTars) {
	            this.role.checkBuff(GT_Buff.BuffTime_pureEnd, this.hitTars);
	        }
	    }
	    getIcon() {
	        if (this.IconData) {
	            return MySourceManager.getInstance().getObjFromSwf(this.IconData["swf"], this.IconData["url"]);
	        }
	        return null;
	    }
	}
	GT_Skill.ReasonCant_cd = "cd";
	GT_Skill.ReasonCant_cost = "cost";
	GT_Skill.ReasonCant_num = "num";

	class GT_Buff {
	    constructor() {
	        this.Lv = 1;
	        this.canActMulty = false;
	        this.isAct = false;
	        this.isBorned = true;
	    }
	    static createNewBuff(bid, tar, skillValues, buffData, fromRole = null, skill = null) {
	        if (tar == null)
	            return null;
	        if (this.SDataDic_buff == null)
	            throw new Error("还未给GT_Buff.SDataDic赋值！！");
	        if (this.SDataDic_target == null)
	            throw new Error("还未设置buff的目标数据sdata");
	        var data = this.SDataDic_buff[bid];
	        if (data == null) {
	            ConfigTidus.LogF("没有buff数据，bid==" + bid);
	            return;
	        }
	        if (data["叠加"] && data["叠加"]["type"] == "属性累加") {
	            for (let i = 0; i < tar.buffs.length; i++) {
	                let bff = tar.buffs[i];
	                if (bff.baseID == bid) {
	                    bff.values[data["叠加"]["type"]["属性"]] += skillValues[data["叠加"]["属性"]];
	                    if (data["持续"]) {
	                        if (Tool_Function.isTypeOf(data["持续"], "str")) {
	                            bff.duration = { "time": skillValues[data["持续"].substr(1)] };
	                        }
	                        else if (Tool_Function.isTypeOf(data["持续"], "int")) {
	                            bff.duration = { "time": data["持续"] };
	                        }
	                        else if (Tool_Function.isTypeOf(data["持续"]["time"], "str")) {
	                            bff.duration = ConfigTidus.GFun_ToolObj_copy(data["持续"]);
	                            bff.duration["time"] = skillValues[data["持续"]["time"].substr(1)];
	                        }
	                        else {
	                            bff.duration = data["持续"];
	                        }
	                    }
	                    return null;
	                }
	            }
	        }
	        else if (tar.buffs && skill) {
	            for (let i = 0; i < tar.buffs.length; i++) {
	                let bff = tar.buffs[i];
	                if (bff.baseID == bid && bff.fromSkill && bff.fromSkill == skill.Name) {
	                    return null;
	                }
	            }
	        }
	        if (data["类型"] == '异常' || buffData["类型"] == "异常") {
	            let peranti = tar.getAntiAttribute(bid);
	            peranti += tar.getValue("异常抗性");
	            if (peranti > 100000) {
	                return null;
	            }
	            let per = 1;
	            if (fromRole) {
	                if (this.Config_badneedLuc == true) {
	                    let luc1 = fromRole.getValue(GT_Role.VName_Lucky);
	                    let luc2 = tar.getValue(GT_Role.VName_Lucky);
	                    let midLuc = luc1 - luc2;
	                    per = Tool_Function.getPerValueInArray(midLuc, [-100, 0, 0, 0.5, 100, 1]);
	                }
	                let peradd = fromRole.getValue(bid + "附加率");
	                peradd += fromRole.getValue("异常附加率");
	                per += per * peradd * 0.01;
	            }
	            per -= peranti * 0.01;
	            if (Math.random() > per)
	                return null;
	        }
	        let buff = new GT_Buff();
	        buff.baseID = bid;
	        buff.role = tar;
	        if (skill)
	            buff.fromSkill = skill.Name;
	        buff.buffType = data["类型"];
	        if (tar.buffs == null)
	            tar.buffs = [];
	        tar.buffs.push(buff);
	        buff.Name = buffData["name"] ? buffData["name"] : bid;
	        buff.actTime = buffData["生效时间"] ? buffData["生效时间"] : data["生效时间"] ? data["生效时间"] : "生成";
	        buff.actNeed = buffData["生效条件"] ? buffData["生效条件"] : data["生效条件"] ? data["生效条件"] : null;
	        if (buff.actNeed) {
	            buff.actNeed = this.getRealNeedCodeFromNeed(buff.actNeed);
	        }
	        buff.actTargetType = buffData["目标"];
	        buff.CD = data["cd"];
	        if (buffData["效果"]) {
	            let obj = ConfigTidus.GFun_ToolObj_copy(data["效果"]);
	            obj["属性"] = buffData["效果"];
	            buff.setActEffect(obj);
	        }
	        else {
	            buff.setActEffect(data["效果"]);
	        }
	        buff.effectDuration = data["效果持续"];
	        let v = ConfigTidus.GFun_ToolObj_copy(data["属性"]);
	        if (skillValues) {
	            for (let vname in skillValues) {
	                if (v == null)
	                    v = {};
	                v[vname] = skillValues[vname];
	            }
	        }
	        buff.values = v;
	        if (data["持续"]) {
	            if (Tool_Function.isTypeOf(data["持续"], "str")) {
	                buff.duration = { "time": buff.values[data["持续"].substr(1)] };
	            }
	            else if (Tool_Function.isTypeOf(data["持续"], "int")) {
	                buff.duration = { "time": data["持续"] };
	            }
	            else if (Tool_Function.isTypeOf(data["持续"]["time"], "str")) {
	                buff.duration = ConfigTidus.GFun_ToolObj_copy(data["持续"]);
	                buff.duration["time"] = buff.values[data["持续"]["time"].substr(1)];
	            }
	            else {
	                buff.duration = data["持续"];
	            }
	            if (buff.duration["time"] < 0)
	                buff.duration = null;
	        }
	        buff.animate = data["美术"];
	        if (data["多次生效"] == true || buffData["多次生效"] == true)
	            buff.canActMulty = true;
	        buff.initF();
	        return buff;
	    }
	    static getRealNeedCodeFromNeed(need) {
	        if (Tool_Function.isTypeOf(need, "str")) {
	            need = GT_Buff.SDataDic_need[need]["val"];
	        }
	        else {
	            let changeprops = need["props"];
	            need = GT_Buff.SDataDic_need[need["id"]]["val"];
	            for (let key in changeprops) {
	                need = Tool_StringBuild.replaceSTR(need, "@" + key, "@" + changeprops[key]);
	            }
	        }
	        return need;
	    }
	    get Name() { return this._Name; }
	    set Name(v) { this._Name = v; }
	    get duration() { return this._duration; }
	    set duration(v) {
	        this._duration = v;
	        if (v == null)
	            return;
	        var time = v["time"];
	        if (Tool_Function.isTypeOf(time, "str")) {
	            let strtime = time;
	            if (strtime.charAt(0) == "@")
	                strtime = strtime.substr(1);
	            time = this.values[strtime];
	        }
	        if (v && v["type"] == "时间")
	            this.countDuration = time * ConfigTidus.FrameRate;
	        else if (v)
	            this.countDuration = time;
	    }
	    destroyF() {
	        if (this.role) {
	            let i = this.role.buffs.indexOf(this);
	            if (i >= 0) {
	                this.role.buffs[i] = null;
	            }
	            this.onDeact();
	        }
	        this.tsoNeed = ConfigTidus.GFun_ToolObj_destroyOne(this.tsoNeed);
	        this.typeAct = ConfigTidus.GFun_ToolObj_destroyOne(this.typeAct);
	        this.role = null;
	    }
	    cloneF(newrole) {
	        var buff = new GT_Buff();
	        buff.role = newrole;
	        buff.baseID = this.baseID;
	        buff.countcd = this.countcd;
	        buff.CD = this.CD;
	        buff.Lv = this.Lv;
	        buff.duration = this.duration;
	        buff.effectDuration = this.effectDuration;
	        buff.countDuration = this.countDuration;
	        buff.countEffectDuration = this.countEffectDuration;
	        buff.addType = this.addType;
	        buff.buffType = this.buffType;
	        buff.values = ConfigTidus.GFun_ToolObj_copy(this.values, false);
	        buff.actTime = this.actTime;
	        buff.actNeed = this.actNeed;
	        buff.actTargetType = this.actTargetType;
	        buff.canActMulty = this.canActMulty;
	        buff.isAct = this.isAct;
	        buff.typeAct = [];
	        buff.isBorned = this.isBorned;
	        if (this.typeAct) {
	            for (let i = 0; i < this.typeAct.length; i++) {
	                buff.typeAct[i] = this.typeAct[i].cloneF(buff);
	            }
	        }
	        return buff;
	    }
	    addEventCallBack(ename, fun) {
	        if (this.Dic_callBack == null)
	            this.Dic_callBack = {};
	        if (this.Dic_callBack[ename] == null)
	            this.Dic_callBack[ename] = [];
	        this.Dic_callBack[ename].push(fun);
	    }
	    clearAllEventCB() {
	        this.Dic_callBack = null;
	    }
	    onEventF(ename) {
	        if (this.Dic_callBack && this.Dic_callBack[ename]) {
	            let arr = this.Dic_callBack[ename];
	            for (let i = 0; i < arr.length; i++) {
	                arr[i]();
	            }
	        }
	    }
	    onDeact() {
	        if (this.isAct == true) {
	            this.isAct = false;
	            if (this.role == null)
	                return;
	            if (this.typeAct) {
	                for (let i = 0; i < this.typeAct.length; i++) {
	                    let one = this.typeAct[i];
	                    one.onDeact();
	                }
	            }
	            if (this.icon != null) {
	                this.role.removeBuffIcon(this.icon);
	                this.icon = ConfigTidus.GFun_ToolObj_destroyOne(this.icon);
	            }
	            if (this.mcLight != null) {
	                this.mcLight = ConfigTidus.GFun_ToolObj_destroyOne(this.mcLight);
	            }
	        }
	    }
	    setActEffect(arr) {
	        this.typeAct = [];
	        if (arr == null)
	            return;
	        if (arr instanceof Array) {
	            for (let i = 0; i < arr.length; i++) {
	                this.typeAct[i] = new buff_actOne(arr[i], this);
	            }
	        }
	        else {
	            this.typeAct[0] = new buff_actOne(arr, this);
	        }
	    }
	    initF() {
	        if (this.role == null)
	            return;
	        if (this.CD != null && this.CD["初始"] == true)
	            this.onResetCD();
	        this.checkF(GT_Buff.BuffTime_created, null);
	    }
	    onResetCD() {
	        if (this.CD != null) {
	            if (this.CD["type"] == "时间") {
	                this.countcd = this.CD["time"] * ConfigTidus.FrameRate;
	            }
	            else {
	                this.countcd = this.CD["time"];
	            }
	            this.onEventF("CD");
	        }
	    }
	    onTimer() {
	        if (this.CD != null) {
	            if (this.CD["type"] != "事件") {
	                this.onCDOne();
	            }
	        }
	        if (this.duration) {
	            if (this.duration["type"] == null || this.duration["type"] == "回合" || this.duration["type"] == "时间") {
	                if (this.isBorned) {
	                    this.isBorned = false;
	                }
	                else {
	                    this.countDuration--;
	                }
	                if (this.countDuration <= 0) {
	                    this.destroyF();
	                    return;
	                }
	            }
	        }
	        if (this.isAct && this.effectDuration && this.effectDuration["type"] != "满足条件") {
	            if (--this.countEffectDuration < 0) {
	                this.onDeact();
	            }
	        }
	    }
	    onCDOne() {
	        if (this.countcd > 0) {
	            this.countcd--;
	            this.onEventF("CD");
	        }
	    }
	    checkCanAct(time, dic) {
	        if (this.isAct)
	            return false;
	        if (time != this.actTime)
	            return false;
	        if (this.CD && this.countcd > 0)
	            return false;
	        if (this.checkNeed(dic) == false) {
	            return false;
	        }
	        return true;
	    }
	    checkF(time, dic) {
	        if (this.duration && this.duration["type"] == GT_Buff.BuffDuration_Event && this.duration["event"] == time) {
	            this.countDuration--;
	            if (this.countDuration <= 0) {
	                this.destroyF();
	                return;
	            }
	        }
	        if (this.CD != null && this.CD["type"] == "事件" && this.CD["事件"] == time) {
	            this.onCDOne();
	        }
	        if (this.isAct)
	            return;
	        if (time != this.actTime && this.actTime != "永久")
	            return;
	        if (this.CD && this.countcd > 0)
	            return;
	        if (this.checkNeed(dic) == false) {
	            if (this.isAct && this.effectDuration && this.effectDuration["type"] == "满足条件") {
	                this.onDeact();
	            }
	            return;
	        }
	        for (let i = 0; i < this.typeAct.length; i++) {
	            let one = this.typeAct[i];
	            if (this.isAct)
	                one.onDeact();
	            one.onAct(dic);
	        }
	        if (this.isAct == false) {
	            if (this.canActMulty == false)
	                this.isAct = true;
	            if (this.CD != null)
	                this.onResetCD();
	            if (this.animate && this.animate["生效图标"]) {
	                if (this.icon == null) {
	                    this.icon = MySourceManager.getInstance().getObjFromSwf(this.animate["生效图标"]["swf"], this.animate["生效图标"]["url"]);
	                    if (this.icon) {
	                        this.role.addBuffIcon(this.icon);
	                    }
	                }
	            }
	            if (this.animate && this.animate["生效特效"]) {
	                if (this.mcLight == null) {
	                    let mc = MySourceManager.getInstance().getMcFromSwf(this.animate["生效特效"]["swf"], this.animate["生效特效"]["url"]);
	                    if (mc) {
	                        this.mcLight = new MyMC(mc);
	                        if (this.animate["生效特效"]["y"] == "下") {
	                        }
	                        else if (this.animate["生效特效"]["y"] == "上") {
	                            this.mcLight.y = -this.role.roleH;
	                        }
	                        else {
	                            this.mcLight.y = -this.role.roleH / 2;
	                        }
	                        this.mcLight.loop = true;
	                        this.mcLight.play();
	                        if (this.role.fightRole)
	                            this.role.fightRole.addChild(this.mcLight);
	                    }
	                }
	            }
	            this.role.buffEffectTmpObj[this.baseID + "buff生效"] = true;
	        }
	        if (this.effectDuration && this.effectDuration["type"] != "满足条件") {
	            if (this.effectDuration["type"] == "时间") {
	                this.countEffectDuration = this.effectDuration["time"] * ConfigTidus.FrameRate;
	            }
	            else {
	                this.countEffectDuration = this.effectDuration["time"];
	            }
	        }
	        if (this.duration && this.duration["type"] == GT_Buff.BuffDuration_act) {
	            this.countDuration--;
	            if (this.countDuration <= 0) {
	                this.destroyF();
	            }
	        }
	    }
	    checkNeed(dic) {
	        if (this.actNeed == null)
	            return true;
	        if (this.checkOneNeed(dic, this.actNeed) == false)
	            return false;
	        return true;
	    }
	    checkOneNeed(dic, type) {
	        if (type == null || type == "无" || type == GT_Buff.BuffNeed_always)
	            return true;
	        if (this.tsoNeed == null) {
	            this.tsoNeed = new TidusScriptOne();
	            type = GT_Buff.changeFStr_at_to_value(type);
	            let str = GT_Skill.changeTSO_quickRoleValueToFull(type);
	            let code = "function a(){";
	            if (str.indexOf("return ") == -1)
	                code += "return " + str;
	            else
	                code += str;
	            if (code.charAt(code.length - 1) != ";")
	                code += ";";
	            code += "}";
	            this.tsoNeed.addFunction(TidusScript_component.rebuildFunctionStr(code));
	            this.tsoNeed.addToGlobalObject("role", this.role);
	            this.tsoNeed.addToGlobalObject("buff", this);
	            this.tsoNeed.addToGlobalObject("values", this.values);
	            this.tsoNeed.addToGlobalObject("TidusScript_classInterface", TidusScript_classInterface);
	        }
	        this.tsoNeed.addToGlobalObject("dic", dic);
	        let re = this.tsoNeed.runFunction("a", null);
	        return re;
	    }
	    static changeFStr_at_to_value(str) {
	        while (str.includes("@")) {
	            let i = str.indexOf("@");
	            let j = Tool_StringBuild.minIndexBehind(str, [" ", "]", ")", "&", "|", ">", "<", "=", "/", "*", "%"], i);
	            let oldstr;
	            if (j == -1) {
	                oldstr = str.substr(i + 1);
	            }
	            else {
	                oldstr = str.substr(i + 1, j - i);
	            }
	            str = Tool_StringBuild.replaceSTR(str, "@" + oldstr, "values." + oldstr);
	        }
	        return str;
	    }
	}
	GT_Buff.Config_badneedLuc = true;
	GT_Buff.BuffTime_created = "生成";
	GT_Buff.BuffTime_newRound = "新回合";
	GT_Buff.BuffTime_campRound = "阵营回合";
	GT_Buff.BuffTime_startFight = "进入战斗";
	GT_Buff.BuffTime_endFight = "离开战斗";
	GT_Buff.BuffTime_atkBefore = "攻击前";
	GT_Buff.BuffTime_atkRole = "攻击到";
	GT_Buff.BuffTime_atkEnd = "攻击后";
	GT_Buff.BuffTime_pureBefore = "治疗前";
	GT_Buff.BuffTime_pureRole = "治疗到";
	GT_Buff.BuffTime_pureEnd = "治疗后";
	GT_Buff.BuffTime_bePure = "被治疗";
	GT_Buff.BuffTime_beDamage = "被伤害";
	GT_Buff.BuffTime_afterMiss = "闪避后";
	GT_Buff.BuffTime_beHurtHP = "扣血后";
	GT_Buff.BuffTime_dead = "死亡";
	GT_Buff.BuffNeed_always = "必定";
	GT_Buff.BuffDuration_act = "生效次数";
	GT_Buff.BuffDuration_Event = "事件";
	GT_Buff.BuffAct_addValue = "改变属性";
	GT_Buff.BuffAct_cannotAct = "无法行动";
	GT_Buff.BuffAct_changeBuffEffect = "添加标识";
	GT_Buff.BuffAct_changeTmpBuffEffect = "添加临时标识";
	GT_Buff.BuffAct_changeEffectNum = "数值改变";
	GT_Buff.BuffAct_newBuff = "生成buff";
	GT_Buff.BuffAct_recover = "治疗";
	GT_Buff.BuffAct_damage = "伤害";
	GT_Buff.BuffAct_clearBuff = "驱散";
	GT_Buff.BuffAct_sheild = "生成护盾";
	GT_Buff.BuffAct_code = "代码";
	class buff_actOne {
	    constructor(info, buff) {
	        this.actType = info["type"];
	        this.data = info;
	        this.buff = buff;
	    }
	    cloneF(newbuffTar) {
	        var one = new buff_actOne(this.data, newbuffTar);
	        one.addValueEffect = ConfigTidus.GFun_ToolObj_copy(this.addValueEffect);
	        return one;
	    }
	    checkActTarget(dic) {
	        let roles = [];
	        if (this.buff.actTargetType == "技能目标") {
	            if (dic && dic["hit"]) {
	                for (let key in dic["hit"]) {
	                    let tarrole = this.buff.role.findOtherRoleByNID(key);
	                    if (tarrole && tarrole.nowHP > 0) {
	                        roles.push(tarrole);
	                    }
	                }
	            }
	        }
	        else if (this.buff.actTargetType == "我方全体") {
	            let arrroles = this.buff.role.findOtherRoleByCamp(this.buff.role.camp);
	            if (arrroles) {
	                for (let i = 0; i < arrroles.length; i++) {
	                    let tarrole = arrroles[i];
	                    if (tarrole && tarrole.nowHP > 0) {
	                        roles.push(tarrole);
	                    }
	                }
	            }
	        }
	        else if (this.buff.actTargetType == "敌方全体") {
	            let arrroles = this.buff.role.findOtherRoleByCamp(-1, this.buff.role.camp);
	            if (arrroles) {
	                for (let i = 0; i < arrroles.length; i++) {
	                    let tarrole = arrroles[i];
	                    if (tarrole && tarrole.nowHP > 0) {
	                        roles.push(tarrole);
	                    }
	                }
	            }
	        }
	        else if (this.buff.actTargetType == "自身") {
	            if (roles.indexOf(this.buff.role) == -1)
	                roles.push(this.buff.role);
	        }
	        else if (GT_Buff.SDataDic_target[this.buff.actTargetType] && GT_Buff.SDataDic_target[this.buff.actTargetType] != true) {
	            if (this.tsoTarget == null) {
	                this.tsoTarget = new TidusScriptOne();
	                let strCode = GT_Buff.changeFStr_at_to_value(GT_Buff.SDataDic_target[this.buff.actTargetType]);
	                let code = "function a(){return " + strCode + ";}";
	                this.tsoTarget.addFunction(TidusScript_component.rebuildFunctionStr(code));
	                this.tsoTarget.addToGlobalObject("role", this.buff.role);
	                this.tsoTarget.addToGlobalObject("buff", this.buff);
	                this.tsoTarget.addToGlobalObject("values", this.buff.values);
	            }
	            let camp1 = this.buff.role.findOtherRoleByCamp(this.buff.role.camp);
	            if (camp1) {
	                for (let i = 0; i < camp1.length; i++) {
	                    let _tar = camp1[i];
	                    if (this.tsoTarget.runFunction("a", { "tar": _tar }) == true) {
	                        roles.push(_tar);
	                    }
	                }
	            }
	            let camp2 = this.buff.role.findOtherRoleByCamp(null, this.buff.role.camp);
	            if (camp2) {
	                for (let i = 0; i < camp2.length; i++) {
	                    let _tar = camp2[i];
	                    if (this.tsoTarget.runFunction("a", { "tar": _tar }) == true) {
	                        roles.push(_tar);
	                    }
	                }
	            }
	        }
	        else {
	            console.error(this.buff.Name + "：还未支持的buff作用目标：" + this.buff.actTargetType);
	        }
	        return roles;
	    }
	    onAct(dic) {
	        if (this.actType == GT_Buff.BuffAct_addValue) {
	            this.onChangeValues(true);
	        }
	        else if (this.actType == GT_Buff.BuffAct_cannotAct) {
	            this.buff.role.buffEffectTmpObj[this.actType] = true;
	        }
	        else if (this.actType == GT_Buff.BuffAct_changeTmpBuffEffect) {
	            if (this.buff.values["标识value"] == null)
	                this.buff.role.buffEffectTmpObj[this.buff.values["标识name"]] = true;
	            else
	                this.buff.role.buffEffectTmpObj[this.buff.values["标识name"]] = this.buff.values["标识value"];
	        }
	        else if (this.actType == GT_Buff.BuffAct_changeBuffEffect) {
	            if (this.buff.values["标识value"] == null)
	                this.buff.role.buffEffectObj[this.buff.values["标识name"]] = true;
	            else
	                this.buff.role.buffEffectObj[this.buff.values["标识name"]] = this.buff.values["标识value"];
	        }
	        else if (this.actType == GT_Buff.BuffAct_changeEffectNum) {
	            let numname = "num";
	            if (this.data && this.data["数值name"])
	                numname = this.data["数值name"];
	            if (dic[numname] != null) {
	                if (this.buff.values["数值改变per"] != null) {
	                    this.buff.role.buffEffectTmpObj[numname] = Tool_Function.onForceConvertType(dic[numname] * (100 + this.buff.values["数值改变per"]) * 0.01);
	                }
	                else if (this.buff.values["数值改变num"] != null) {
	                    this.buff.role.buffEffectTmpObj[numname] = Tool_Function.onForceConvertType(dic[numname] + this.buff.values["数值改变num"]);
	                }
	                else if (this.buff.values["数值改变公式"] != null) {
	                    this.buff.role.buffEffectTmpObj[numname] = GT_Skill.getRealValueFromValue(this.buff.values["数值改变公式"], this.buff.values, { "role": this.buff.role, "buff": this.buff, "values": this.buff.values, "dic": dic });
	                }
	                dic[numname] = this.buff.role.buffEffectTmpObj[numname];
	            }
	        }
	        else if (this.actType == GT_Buff.BuffAct_newBuff) {
	            let newbuffid = this.data["newBuffID"];
	            let buffdata = this.data["newBuff独立数据"];
	            let role = this.checkActTarget(dic);
	            for (let i = 0; i < role.length; i++) {
	                this.addBuffToTar(role[i], newbuffid, this.buff.values, buffdata);
	            }
	        }
	        else if (this.actType == GT_Buff.BuffAct_recover) {
	            let role = this.checkActTarget(dic);
	            if (this.data["属性"]["治疗异常"]) {
	                let pureNum = this.data["属性"]["治疗异常"];
	                for (let j = 0; j < role.length; j++) {
	                    for (let i = 0; i < role[j].buffs.length; i++) {
	                        role[j].bePureBadBuff(pureNum);
	                    }
	                }
	            }
	            if (this.data["属性"]["HP"]) {
	                let pureNum = this.data["属性"]["HP"];
	                if (Tool_Function.isTypeOf(pureNum, "int") == false) {
	                    if (pureNum.includes("%")) {
	                        pureNum = Tool_StringBuild.replaceSTR(pureNum, "%", " *role.HP");
	                    }
	                    pureNum = GT_Skill.getRealValueFromValue(pureNum, this.buff.values, { "role": this.buff.role });
	                }
	                for (let j = 0; j < role.length; j++) {
	                    role[j].bePure("HP", pureNum);
	                }
	            }
	            if (this.data["属性"]["MP"]) {
	                let pureNum = this.data["属性"]["MP"];
	                if (Tool_Function.isTypeOf(pureNum, "int") == false) {
	                    if (pureNum.includes("%")) {
	                        pureNum = Tool_StringBuild.replaceSTR(pureNum, "%", " *role.MP");
	                    }
	                    pureNum = GT_Skill.getRealValueFromValue(pureNum, this.buff.values, { "role": this.buff.role });
	                }
	                for (let j = 0; j < role.length; j++) {
	                    role[j].bePure("MP", pureNum);
	                }
	            }
	        }
	        else if (this.actType == GT_Buff.BuffAct_damage) {
	            let role = this.checkActTarget(dic);
	            for (let key in this.buff.values) {
	                if (key.indexOf("伤害") != 0)
	                    continue;
	                let vname = key.substr(2);
	                let vnum = this.buff.values[vname];
	                if (vnum && Tool_Function.isTypeOf(vnum, "str")) {
	                    let str = vnum;
	                    if (str.includes("%")) {
	                        vnum = Tool_Function.onForceConvertType(str.substr(0, str.length - 1));
	                        vnum = Tool_Function.onForceConvertType(vnum * 0.01 * this.buff.role.getValue(vname + "Max"));
	                    }
	                    else
	                        vnum = Tool_Function.onForceConvertType(vnum);
	                }
	                if (vnum <= 0)
	                    continue;
	                for (let j = 0; j < role.length; j++) {
	                    role[j].beHurt(vname, vnum, this.buff);
	                }
	            }
	        }
	        else if (this.actType == GT_Buff.BuffAct_clearBuff) {
	            let role = this.checkActTarget(dic);
	            if (this.buff.values["驱散条件"] == null && this.buff.values["驱散数量"] == null) {
	                ConfigTidus.LogF("驱散时缺少参数");
	                return;
	            }
	            let countbuff = this.buff.values["驱散数量"];
	            for (let i = 0; i < role.length; i++) {
	                for (let j = 0; j < role[i].buffs.length; j++) {
	                    let tarbuff = role[i].buffs[j];
	                    if (tarbuff == null)
	                        continue;
	                    if (this.buff.values["驱散条件"]) {
	                        if (GT_Skill.getRealValueFromValue(this.buff.values["驱散条件"], this.buff.values, { "role": this.buff.role, "buff": this.buff, "tar": role }, "Buff" + this.buff.Name) == true) {
	                            tarbuff.destroyF();
	                        }
	                    }
	                    else if (tarbuff.buffType == "异常") {
	                        tarbuff.destroyF();
	                        if (--countbuff <= 0)
	                            return;
	                    }
	                }
	            }
	        }
	        else if (this.actType == GT_Buff.BuffAct_sheild) {
	            let num;
	            if (Tool_Function.isTypeOf(this.data["属性"]["护盾"], "str")) {
	                let tarname = this.data["属性"]["护盾"];
	                num = GT_Skill.getRealValueFromValue(tarname, this.buff.values, { "role": this.buff.role }, "Buff" + this.buff.Name);
	            }
	            else {
	                num = this.data["属性"]["护盾"];
	            }
	            if (this.data["独立"] == true) {
	                this.buff.role.sheilds[this.buff.fromSkill] = num;
	            }
	            else {
	                this.buff.role.sheilds["0"] += num;
	            }
	        }
	        else if (this.actType == GT_Buff.BuffAct_code) {
	            let code = this.data["code"];
	            if (this.tsoAct == null) {
	                let obj = GT_Skill.getTSO_byRealValueString(GT_Skill.getPureStringWithRealValue(this.data["code"], this.buff.values, "Buff" + this.buff.Name));
	                this.tsoAct = obj["tso"];
	                code = obj["str"];
	            }
	            this.tsoAct.doScriptOnce(code, { "role": this.buff.role, "buff": this.buff, "values": this.buff.values, "dic": dic });
	        }
	        else {
	            ConfigTidus.LogF("未知的buff生效方式：" + this.actType);
	            return;
	        }
	    }
	    addBuffToTar(tar, newbuffid, dic, buffdata) {
	        if (tar) {
	            let newbuff = GT_Buff.createNewBuff(newbuffid, tar, dic, buffdata);
	            if (newbuff == null || newbuff.role == null)
	                return;
	            if (this.data["关联"] == true) {
	                if (this.childBuffs == null)
	                    this.childBuffs = [];
	                this.childBuffs.push(newbuff);
	            }
	        }
	    }
	    onChangeValues(add) {
	        if (add == true) {
	            this.addValueEffect = {};
	            for (let vname in this.data["属性"]) {
	                var num;
	                if (Tool_Function.isTypeOf(this.data["属性"][vname], "str")) {
	                    let tarname = this.data["属性"][vname];
	                    if (tarname.includes(" ") == false && tarname.charAt(tarname.length - 1) == "%") {
	                        num = tarname.substr(0, num.length - 1);
	                        let nownum = this.buff.role.getValue(vname);
	                        num = nownum * Tool_Function.onForceConvertType(num) * 0.01;
	                    }
	                    else {
	                        num = GT_Skill.getRealValueFromValue(tarname, this.buff.values, { "role": this.buff.role }, "Buff" + this.buff.Name);
	                    }
	                }
	                else {
	                    num = this.data["属性"][vname];
	                }
	                if (isNaN(num) || num == 0) {
	                }
	                else {
	                    this.buff.role.addValue(vname, num);
	                    this.addValueEffect[vname] = num;
	                }
	            }
	        }
	        else if (this.addValueEffect) {
	            for (let vname in this.addValueEffect) {
	                this.buff.role.addValue(vname, -this.addValueEffect[vname]);
	            }
	            this.addValueEffect = null;
	        }
	    }
	    onDeact() {
	        if (this.actType == GT_Buff.BuffAct_addValue) {
	            this.onChangeValues(false);
	        }
	        else if (this.actType == GT_Buff.BuffAct_changeBuffEffect) {
	            delete this.buff.role.buffEffectObj[this.buff.values["标识name"]];
	        }
	        else if (this.actType == GT_Buff.BuffAct_sheild) {
	            if (this.data["独立"] == true) {
	                this.buff.role.sheilds[this.buff.fromSkill] = null;
	            }
	        }
	        if (this.childBuffs) {
	            for (let i = 0; i < this.childBuffs.length; i++) {
	                this.childBuffs[i].destroyF();
	            }
	            this.childBuffs = null;
	        }
	    }
	    destroyF() {
	        this.onDeact();
	    }
	}

	class GT_Role {
	    constructor() {
	        this.Quality = 1;
	        this.Rank = 1;
	        this.Star = 1;
	        this.Plus = 0;
	        this.Lv = 1;
	        this.exp = 0;
	        this.sp = 0;
	        this.pieces = 0;
	        this.sheilds = new Proxy({ "0": 0 }, {
	            get: function (obj, prop) {
	                return obj[prop];
	            },
	            set: (obj, key, value) => {
	                if (value <= 0 || value == null) {
	                    delete obj[key];
	                }
	                else {
	                    obj[key] = value;
	                }
	                this.onEventF("护盾");
	                return true;
	            }
	        });
	        this.buffEffectTmpObj = {};
	        this.buffEffectObj = {};
	        this.isWeak = false;
	        this._nowHP = null;
	        this.isFighting = false;
	        this.roleW = 0;
	        this.roleH = 0;
	        this.isDead = 0;
	        this.countEvent = 1;
	    }
	    get Name() { return this._Name; }
	    set Name(v) { this._Name = v; }
	    get nowHP() {
	        if (this._nowHP == null)
	            return -1;
	        let realnow = this.value.getValue("当前HP");
	        if (realnow != this._nowHP) {
	            throw new Error("HP数值错误！");
	        }
	        return realnow;
	    }
	    set nowHP(value) {
	        this._nowHP = value;
	        this.value.setValue("当前HP", value);
	    }
	    get nowMP() {
	        if (this._nowMP == null)
	            return -1;
	        let realnow = this.value.getValue("当前MP");
	        if (realnow != this._nowMP) {
	            throw new Error("MP数值错误！");
	        }
	        return realnow;
	    }
	    set nowMP(value) {
	        this._nowMP = value;
	        this.value.setValue("当前MP", value);
	    }
	    destroyF() {
	        this.roleMc = ConfigTidus.GFun_ToolObj_destroyOne(this.roleMc);
	        this.layerIcon = ConfigTidus.GFun_ToolObj_destroyOne(this.layerIcon);
	        this.fightRole = ConfigTidus.GFun_ToolObj_destroyOne(this.fightRole);
	    }
	    initBaseValueByData(dataLv1, dataLv50, dataAntiMag, dataAntiBad, datalv = 50) {
	        this.baseValue = {};
	        for (let key in dataLv1) {
	            let num0 = dataLv1[key];
	            let num1 = dataLv50[key];
	            let mid = (num1 - num0) / datalv;
	            this.baseValue[key] = Tool_Function.onForceConvertType(num0 + mid * this.Lv);
	        }
	        this.baseAttribute = {};
	        if (dataAntiMag) {
	            for (let key in dataAntiMag) {
	                if (dataAntiMag[key] != 0)
	                    this.baseAttribute["抗" + key] = dataAntiMag[key];
	            }
	        }
	        if (dataAntiBad) {
	            for (let key in dataAntiBad) {
	                if (dataAntiBad[key] != 0)
	                    this.baseAttribute["抗" + key] = dataAntiBad[key];
	            }
	        }
	    }
	    getAllResourceFromBaseData(basedata) {
	        if (basedata == null)
	            return null;
	        let arr = [];
	        if (basedata["美术"]) {
	            if (basedata["美术"]["形象"]) {
	                if (basedata["美术"]["形象"]["类型"] == "spine") {
	                    arr.push([basedata["美术"]["形象"]["spine"], "zmc"]);
	                }
	                else {
	                    arr.push([basedata["美术"]["形象"]["swf"], "swf"]);
	                }
	            }
	            if (basedata["美术"]["头像"]) {
	                arr.push([basedata["美术"]["头像"]["swf"], "swf"]);
	            }
	            if (basedata["美术"]["战斗头像"]) {
	                arr.push([basedata["美术"]["战斗头像"]["swf"], "swf"]);
	            }
	        }
	        if (basedata["技能"]) {
	            for (let sid in basedata["技能"]) {
	                let sdata = basedata["技能"][sid];
	                if (sdata["美术"] == null)
	                    continue;
	                if (sdata["美术"]["动作特效"])
	                    arr.push([sdata["美术"]["动作特效"]["swf"], "swf"]);
	                if (sdata["美术"]["命中特效"])
	                    arr.push([sdata["美术"]["命中特效"]["swf"], "swf"]);
	                if (sdata["美术"]["生效方式"] == "飞行") {
	                    if (sdata["美术"]["飞行mc"])
	                        arr.push([sdata["美术"]["飞行mc"]["swf"], "swf"]);
	                    if (sdata["美术"]["飞行消失mc"])
	                        arr.push([sdata["美术"]["飞行消失mc"]["swf"], "swf"]);
	                }
	            }
	        }
	        return arr;
	    }
	    getIcon() {
	        if (this.animate && this.animate["头像"]) {
	            return MySourceManager.getInstance().getObjFromSwf(this.animate["头像"]["swf"], this.animate["头像"]["url"]);
	        }
	        return null;
	    }
	    getFightIcon() {
	        if (this.animate && this.animate["战斗头像"]) {
	            return MySourceManager.getInstance().getObjFromSwf(this.animate["战斗头像"]["swf"], this.animate["战斗头像"]["url"]);
	        }
	        return null;
	    }
	    cloneF() {
	        var role = new GT_Role();
	        role.netID = this.netID;
	        role.baseID = this.baseID;
	        role.Name = this.Name;
	        role.Lv = this.Lv;
	        role.job = this.job;
	        role.type = this.type;
	        role.race = this.race;
	        role.sex = this.sex;
	        role.camp = this.camp;
	        role.isWeak = this.isWeak;
	        role.buffs = [];
	        role.nowHP = this.nowHP;
	        role.nowMP = this.nowMP;
	        if (this.buffs) {
	            for (let i = 0; i < this.buffs.length; i++) {
	                role.buffs[i] = this.buffs[i].cloneF(role);
	            }
	        }
	        role.buffEffectTmpObj = ConfigTidus.GFun_ToolObj_copy(this.buffEffectTmpObj);
	        role.value = new MyCheaterNum();
	        let arrVName = ["HP", "攻击", "速度", "物防", "魔防"];
	        for (let i = 0; i < arrVName.length; i++) {
	            role.value.setValue(arrVName[i], this.value.getValue(arrVName[i]));
	        }
	        role.baseValue = ConfigTidus.GFun_ToolObj_copy(this.baseValue, false);
	        role.baseAttribute = ConfigTidus.GFun_ToolObj_copy(this.baseAttribute, false);
	        role.netValue = ConfigTidus.GFun_ToolObj_copy(this.netValue, false);
	        role.Equips = this.Equips;
	        role.Ruins = this.Ruins;
	        return role;
	    }
	    initValueF() {
	        this.value = new MyCheaterNum();
	        this.attribute = new MyCheaterNum();
	        this.equipSkills = null;
	        if (this.baseValue) {
	            for (let key in this.baseValue) {
	                this.addValue(key, this.baseValue[key]);
	            }
	        }
	        if (this.netValue) {
	            for (let key in this.netValue) {
	                this.addValue(key, this.netValue[key]);
	            }
	        }
	        if (!isNaN(this.baseBP)) {
	            this.addValue("破坏值", this.baseBP);
	        }
	        if (this.baseAttribute) {
	            for (let key in this.baseAttribute) {
	                if (key.charAt(0) == "抗") {
	                    this.addAntiAttribute(key.substr(1), this.baseAttribute[key]);
	                }
	                else {
	                    this.addAttribute(key, this.baseAttribute[key]);
	                }
	            }
	        }
	        if (this.Equips) {
	            for (let key in this.Equips) {
	                let ep = this.Equips[key];
	                if (ep.Values) {
	                    for (let vname in ep.Values.Dic) {
	                        this.addValue(vname, ep.Values.getValue(vname));
	                    }
	                }
	                if (ep.passitiveData) {
	                    if (this.equipSkills == null)
	                        this.equipSkills = [];
	                    for (let strid in ep.passitiveData) {
	                        let sid = Tool_Function.onForceConvertType(strid);
	                        let s = GT_Skill.createNewSkill(sid, this, null, null, false);
	                        if (s) {
	                            this.equipSkills.push(s);
	                            s.onCreatePassitiveBuff();
	                        }
	                    }
	                }
	            }
	        }
	        if (this.nowSkills) {
	            for (let key in this.nowSkills) {
	                let s = this.SkillData[this.nowSkills[key]];
	                if (s)
	                    s.onCreatePassitiveBuff();
	            }
	        }
	        this.orgValues = {};
	        for (let key in this.value.Dic) {
	            this.orgValues[key] = this.value.getValue(key);
	        }
	    }
	    initFight() {
	        this.isDead = 0;
	        this.isWeak = false;
	        this.isFighting = false;
	        if (this._nowHP == null)
	            this.recoverHPtoMax();
	        if (this._nowMP == null)
	            this.recoverMPtoMax();
	        if (this.nowSkills) {
	            for (let key in this.nowSkills) {
	                let s = this.SkillData[this.nowSkills[key]];
	                if (s)
	                    s.initFight();
	            }
	        }
	        if (this.value == null)
	            this.value = new MyCheaterNum();
	        this.value.setValue(GT_Role.VName_Atb, 0);
	        this.value.setValue(GT_Role.VName_Anger, 0);
	        this.value.setValue(GT_Role.VName_Break, 0);
	        if (this.fightRole == null) {
	        }
	    }
	    recoverHPtoMax() {
	        this.nowHP = this.getValue(GT_Role.VName_HP);
	    }
	    recoverMPtoMax() {
	        this.nowMP = this.getValue(GT_Role.VName_MP);
	    }
	    endF() {
	        this.nowAction = null;
	        if (this.buffs) {
	            this.buffs = ConfigTidus.GFun_ToolObj_destroyOne(this.buffs);
	            this.buffs = [];
	        }
	        this.arrIcons = ConfigTidus.GFun_ToolObj_destroyOne(this.arrIcons);
	        this.layerIcon = ConfigTidus.GFun_ToolObj_destroyOne(this.layerIcon);
	        this.roleMc = ConfigTidus.GFun_ToolObj_destroyOne(this.roleMc);
	        ConfigTidus.GFun_ToolObj_clearObj(this.buffEffectTmpObj);
	    }
	    onBeginFight(dic) {
	        if (this.isFighting == true)
	            return;
	        this.isFighting = true;
	        this.checkBuff(GT_Buff.BuffTime_startFight, dic);
	    }
	    onEndFight() {
	        if (this.isFighting == false)
	            return;
	        this.isFighting = false;
	        this.checkBuff(GT_Buff.BuffTime_endFight, null);
	    }
	    changeRoleFlag(action, mcName) {
	        ConfigTidus.GFun_ToolObj_clearObj(this.buffEffectTmpObj);
	        if (this.nowAction && this.nowAction != action && action != null) {
	            this.nowAction.onStopF();
	        }
	        if (action)
	            this.nowAction = action;
	        if (mcName == null || this.animate == null || this.animate["形象"] == null || this.animate["形象"][mcName] == null)
	            return;
	        if (this.roleMc && this.roleMc.isSpine != true) {
	            this.roleMc = ConfigTidus.GFun_ToolObj_destroyOne(this.roleMc);
	        }
	        if (this.roleMc == null) {
	            var mc;
	            if (this.animate["形象"]["类型"] == "spine") {
	                mc = MySourceManager.getInstance().getMcFromSwf(this.animate["形象"]["spine"], this.animate["形象"][mcName]["url"]);
	            }
	            else {
	                mc = MySourceManager.getInstance().getMcFromSwf(this.animate["形象"]["swf"], this.animate["形象"][mcName]["url"]);
	            }
	            this.roleMc = new MyMC(mc);
	            this.fightRole.addChildAt(this.roleMc, 0);
	        }
	        else {
	            this.roleMc.gotoAndStop(this.animate["形象"][mcName]["url"]);
	        }
	    }
	    enterF_rolemc() {
	        if (this.roleMc) {
	            this.roleMc.enterPlayF();
	        }
	    }
	    enterF_buff() {
	        if (this.buffs) {
	            for (let i = 0; i < this.buffs.length; i++) {
	                if (this.buffs[i] == null) {
	                    Tool_ArrayUtils.removeAt(this.buffs, i--);
	                }
	                else {
	                    this.buffs[i].onTimer();
	                }
	            }
	        }
	    }
	    enterF_skill() {
	        if (this.nowSkills) {
	            for (let key in this.nowSkills) {
	                let s = this.SkillData[this.nowSkills[key]];
	                if (s)
	                    s.onEnterF();
	            }
	        }
	    }
	    addBuffIcon(icon) {
	        if (icon == null)
	            return;
	        if (this.layerIcon == null) {
	            this.layerIcon = new Sprite();
	            this.fightRole.addChild(this.layerIcon);
	            this.layerIcon.y = -this.roleH - 30;
	        }
	        if (this.arrIcons == null)
	            this.arrIcons = [];
	        icon.x = this.arrIcons.length * (30 + 2);
	        this.arrIcons.push(icon);
	        this.layerIcon.addChild(icon);
	        this.layerIcon.x = -((this.arrIcons.length - 1) * (30 + 2) + 30) / 2;
	    }
	    removeBuffIcon(icon) {
	        if (icon == null || this.arrIcons == null)
	            return;
	        let i = this.arrIcons.indexOf(icon);
	        if (i == -1)
	            return;
	        Tool_ArrayUtils.removeAt(this.arrIcons, i);
	        this.layerIcon.x = -((this.arrIcons.length - 1) * (30 + 2) + 30) / 2;
	    }
	    addValue(vname, num) {
	        if (isNaN(num) || num == 0)
	            return;
	        if (this.value == null)
	            this.value = new MyCheaterNum();
	        let old = this.getValue(vname);
	        this.value.setValue(vname, num + old);
	    }
	    getValue(vname) {
	        let num;
	        if (this.value) {
	            num = this.value.getValue(vname);
	        }
	        if (num == null)
	            return 0;
	        return num;
	    }
	    addAttribute(vname, num) {
	        if (this.attribute == null)
	            this.attribute = new MyCheaterNum();
	        let old = this.getAttribute(vname);
	        this.attribute.setValue(vname, num + old);
	    }
	    getAttribute(vname) {
	        let num;
	        if (this.attribute) {
	            num = this.attribute.getValue(vname);
	        }
	        if (num == null)
	            return 0;
	        return num;
	    }
	    addAntiAttribute(vname, num) {
	        if (this.attribute == null)
	            this.attribute = new MyCheaterNum();
	        vname = vname + "抗";
	        let old = this.getAttribute(vname);
	        this.attribute.setValue(vname, num + old);
	    }
	    getAntiAttribute(vname) {
	        vname = vname + "抗";
	        let num;
	        if (this.attribute) {
	            num = this.attribute.getValue(vname);
	        }
	        if (num == null)
	            return 0;
	        return num;
	    }
	    checkBuff(time, data) {
	        if (this.buffs != null) {
	            for (let i = 0; i < this.buffs.length; i++) {
	                if (this.buffs[i] == null) {
	                    Tool_ArrayUtils.removeAt(this.buffs, i--);
	                }
	                else {
	                    this.buffs[i].checkF(time, data);
	                }
	            }
	        }
	        if (this.nowSkills) {
	            for (let key in this.nowSkills) {
	                let s = this.SkillData[this.nowSkills[key]];
	                if (s && s.CD != null && s.CD["type"] == "事件" && s.CD["事件"] == time) {
	                    s.onCDOne();
	                }
	            }
	            ;
	        }
	    }
	    findBuffByBaseid(bid) {
	        if (this.buffs) {
	            let out;
	            for (let i = 0; i < this.buffs.length; i++) {
	                if (this.buffs[i] && this.buffs[i].baseID == bid) {
	                    if (out == null)
	                        out = [];
	                    out.push(this.buffs[i]);
	                }
	            }
	            return out;
	        }
	        return null;
	    }
	    bePure(type, num) {
	        if (type == GT_Role.VName_TeamMP) {
	            this.onEventF("获得团队MP", num);
	            return;
	        }
	        if (this.nowHP <= 0)
	            return;
	        if (num < 0)
	            return;
	        let now = this.getValue(type);
	        let max = this.getValue(type + "Max");
	        if (type == GT_Role.VName_HP) {
	            now = this.nowHP;
	            max = this.getValue("HP");
	        }
	        else if (type == GT_Role.VName_MP) {
	            now = this.nowMP;
	            max = this.getValue("MP");
	        }
	        if (now < max) {
	            this.checkBuff(GT_Buff.BuffTime_bePure + type, { "num": num });
	            if (this.buffEffectTmpObj["num"] != null) {
	                num = this.buffEffectTmpObj["num"];
	                delete this.buffEffectTmpObj["num"];
	            }
	            if (now + num > max) {
	                if (type == GT_Role.VName_HP) {
	                    this.nowHP = max;
	                }
	                else if (type == GT_Role.VName_MP) {
	                    this.nowMP = max;
	                }
	                else {
	                    this.addValue(type, max - now);
	                }
	            }
	            else {
	                if (type == GT_Role.VName_HP) {
	                    this.nowHP += num;
	                }
	                else if (type == GT_Role.VName_MP) {
	                    this.nowMP += num;
	                }
	                else {
	                    this.addValue(type, num);
	                }
	            }
	            if (type == GT_Role.VName_HP) {
	                this.onEventF("回血", num);
	            }
	            else {
	                this.onEventF("回" + type, num);
	            }
	        }
	    }
	    beHurt(valueName, num, from, data = null) {
	        if (valueName == GT_Role.VName_TeamMP) {
	            this.onEventF("消耗团队MP", num);
	            return 0;
	        }
	        if (valueName == "闪避") {
	            this.checkBuff(GT_Buff.BuffTime_afterMiss, { "from": from });
	            this.onEventF("闪避", {});
	            return 0;
	        }
	        if (this.getValue(GT_Role.VName_HP) <= 0)
	            return 0;
	        if (num < 0)
	            return 0;
	        if (valueName == GT_Role.VName_HP && (data == null || data["绝对"] != true)) {
	            if (data && data["职业补正"] && data["职业补正"][this.job])
	                num *= data["职业补正"][this.job];
	            if (data && data["类型补正"] && data["类型补正"][this.type])
	                num *= data["类型补正"][this.type];
	            if (data && data["自定义补正"] && this.customData) {
	                for (let key in data["自定义补正"]) {
	                    if (this.customData[key] == true)
	                        num *= data["自定义补正"][key];
	                }
	            }
	            if (data && data["属性"] && data["属性"] != "物理" && data["属性"] != "魔法" && data["属性"] != "通用") {
	                if (from instanceof GT_Role)
	                    num = GT_Hurt.onAttribuiltDef(num, data["属性"], from, this);
	                else
	                    num = GT_Hurt.onAttribuiltDef(num, data["属性"], from.role, this);
	            }
	            if (from instanceof GT_Role)
	                num = GT_Hurt.onDef(num, data, from, this);
	            if (num < 0)
	                return;
	            if (data && data["补正"]) {
	                num = num * data["补正"];
	            }
	            if (from instanceof GT_Role)
	                num = GT_Hurt.onRoleTypeDef(num, this.type, from.type);
	        }
	        num = Tool_Function.onForceConvertType(num);
	        if (data == null || data["即死"] != true) {
	            for (let key in this.sheilds) {
	                let snum = this.sheilds[key];
	                if (snum >= num) {
	                    this.sheilds[key] = snum - num;
	                    num = 0;
	                    break;
	                }
	                this.sheilds[key] = 0;
	                num -= snum;
	            }
	        }
	        if (num < 0)
	            return;
	        if (data == null || data["即死"] != true) {
	            this.checkBuff(GT_Buff.BuffTime_beDamage + valueName, { "num": num, "from": from });
	            if (this.buffEffectTmpObj["num"] != null) {
	                num = this.buffEffectTmpObj["num"];
	                delete this.buffEffectTmpObj["num"];
	            }
	            if (num < 0)
	                return;
	            num = Tool_Function.onForceConvertType(num);
	        }
	        if (valueName == GT_Role.VName_HP) {
	            this.nowHP -= num;
	            if (this.nowHP <= 0)
	                this.nowHP = 0;
	            this.checkBuff(GT_Buff.BuffTime_beHurtHP, { "num": num, "from": from });
	            if (this.nowHP <= 0) {
	                this.onDeadF();
	            }
	            else {
	                if (this.Toughness) {
	                }
	                else {
	                }
	            }
	            this.onEventF("扣血", { "num": num, "data": data, "from": from });
	        }
	        else if (valueName == GT_Role.VName_MP) {
	            this.nowMP -= num;
	            if (this.nowMP <= 0)
	                this.nowMP = 0;
	            this.onEventF("扣MP", num);
	        }
	        else {
	            this.addValue(valueName, -num);
	            this.onEventF("扣" + valueName, num);
	        }
	        return num;
	    }
	    onDeadF() {
	        this.checkBuff(GT_Buff.BuffTime_dead, null);
	        this.isDead = 1;
	        this.onEventF("死亡");
	    }
	    onEventF(type, val = null) {
	        if (this.Dic_event) {
	            for (let key in this.Dic_event) {
	                if (this.Dic_event[key] == null) {
	                    delete this.Dic_event[key];
	                    continue;
	                }
	                this.Dic_event[key](type, val);
	            }
	        }
	    }
	    addEventF(cb) {
	        if (this.Dic_event == null)
	            this.Dic_event = {};
	        this.Dic_event[this.countEvent] = cb;
	        return this.countEvent++;
	    }
	    removeEventF(i) {
	        if (this.Dic_event && i >= 0) {
	            this.Dic_event[i] = null;
	        }
	    }
	    clearAllCallBack() {
	        if (this.buffs) {
	            for (let i = 0; i < this.buffs.length; i++) {
	                if (this.buffs[i])
	                    this.buffs[i].clearAllEventCB();
	            }
	        }
	        if (this.nowSkills) {
	            for (let key in this.nowSkills) {
	                let s = this.SkillData[this.nowSkills[key]];
	                if (s)
	                    s.clearAllEventCB();
	            }
	            ;
	        }
	    }
	    checkDistanceFromTar(tar) {
	        return Tool_Function.onGetLengthBetweenPoint(this, tar);
	    }
	    findOtherRoleByNID(nid) {
	        return null;
	    }
	    findOtherRoleByCamp(camp, notcamp = -1) {
	        return null;
	    }
	    bePureBadBuff(tar) {
	        let canPureBadTypes;
	        let canPureBadNum = -1;
	        if (tar == true || tar == 100) {
	        }
	        else if (Tool_Function.isTypeOf(tar, "arr")) {
	            canPureBadTypes = tar;
	        }
	        else if (Tool_Function.isTypeOf(tar, "num")) {
	            canPureBadNum = tar;
	        }
	        else {
	            canPureBadTypes = tar["类型"];
	            canPureBadNum = tar["数量"];
	        }
	        let countbad = canPureBadNum;
	        for (let i = 0; i < this.buffs.length; i++) {
	            let tmpbuff = this.buffs[i];
	            if (tmpbuff == null)
	                continue;
	            if ((canPureBadTypes == null && tmpbuff.buffType == "异常" && tmpbuff.duration != null) || (canPureBadTypes != null && canPureBadTypes.indexOf(tmpbuff.baseID) != -1)) {
	                tmpbuff.destroyF();
	                if (countbad > 0) {
	                    countbad--;
	                    if (countbad == 0)
	                        break;
	                }
	            }
	        }
	    }
	    static getAttibuteType(type) {
	        if (type == "物理" || type == "魔法" || type == "通用" || type == null)
	            return type;
	        if (type == "毒" || type == "火" || type == "雷" || type == "冰" || type == "土" || type == "风" || type == "水" || type == "光" || type == "暗")
	            return "魔法";
	        return "物理";
	    }
	}
	GT_Role.VName_HP = "HP";
	GT_Role.VName_MP = "MP";
	GT_Role.VName_TeamMP = "团队MP";
	GT_Role.VName_Anger = "怒气";
	GT_Role.VName_AngerMax = "怒气Max";
	GT_Role.VName_Atb = "ATB";
	GT_Role.VName_AtbMax = "ATBMax";
	GT_Role.VName_Break = "破坏";
	GT_Role.VName_BreakMax = "破坏Max";
	GT_Role.VName_Weight = "重量";
	GT_Role.VName_Power = "力量";
	GT_Role.VName_Intelligence = "智力";
	GT_Role.VName_Spirit = "精力";
	GT_Role.VName_Agile = "敏捷";
	GT_Role.VName_Lucky = "幸运";
	GT_Role.VName_Speed = "速度";
	GT_Role.VName_Toughness = "韧性";
	GT_Role.VName_Atk = "攻击";
	GT_Role.VName_Def = "防御";
	GT_Role.VName_Phy = "物攻";
	GT_Role.VName_Mag = "魔攻";
	GT_Role.VName_Pde = "物防";
	GT_Role.VName_Mde = "魔防";
	GT_Role.VName_Hit = "命中";
	GT_Role.VName_Mis = "闪避";
	GT_Role.VName_Cri = "暴击率";
	GT_Role.VName_AntiCri = "抗暴击";
	GT_Role.VName_Crinum = "暴击值";
	GT_Role.VName_Area = "范围";
	GT_Role.VName_Stiff = "硬直";
	GT_Role.VName_AtkPer = "伤害加成";
	GT_Role.VName_PhyPer = "物伤加成";
	GT_Role.VName_MagPer = "魔伤加成";
	GT_Role.VName_AntiAtk = "伤害减免";
	GT_Role.VName_AntiPhy = "物理减免";
	GT_Role.VName_AntiMag = "魔法减免";
	GT_Role.VName_Bloodsucking = "吸血";
	GT_Role.VName_PhyBloodsucking = "物理吸血";
	GT_Role.VName_MagBloodsucking = "魔法吸血";
	GT_Role.VName_BadActpercent = "异常附加率";
	GT_Role.VName_BadAntipercent = "异常抗性";
	GT_Role.VName_AutoRecoverHP = "自回血";
	GT_Role.VName_AutoRecoverHPp = "自回血p";
	GT_Role.VName_AutoRecoverMP = "自回魔";
	GT_Role.AttributeName_PHY = "物理";
	GT_Role.AttributeName_MAG = "魔法";
	GT_Role.AttributeName_Posion = "毒";
	GT_Role.AttributeName_FIRE = "火";
	GT_Role.AttributeName_ICE = "冰";
	GT_Role.AttributeName_THUNDER = "雷";
	GT_Role.AttributeName_EARTH = "土";
	GT_Role.AttributeName_WIND = "风";
	GT_Role.AttributeName_WATER = "水";
	GT_Role.AttributeName_HOLY = "光";
	GT_Role.AttributeName_DARK = "暗";
	GT_Role.AttributeName_NONE = "无";
	GT_Role.AttributeName_dagger = "匕首";
	GT_Role.AttributeName_sword = "剑";
	GT_Role.AttributeName_claymore = "大剑";
	GT_Role.AttributeName_knife = "刀";
	GT_Role.AttributeName_Bigkatana = "大刀";
	GT_Role.AttributeName_katana = "武士刀";
	GT_Role.AttributeName_lance = "枪";
	GT_Role.AttributeName_arch = "弓";
	GT_Role.AttributeName_hammer = "锤";
	GT_Role.AttributeName_axe = "斧";
	GT_Role.AttributeName_staff = "杖";
	GT_Role.AttributeName_book = "书";
	GT_Role.AttributeName_lash = "鞭";
	GT_Role.AttributeName_gun = "枪械";
	GT_Role.AttributeName_bomb = "炸弹";
	GT_Role.AttributeName_handfeet = "拳脚";

	class MyDebug extends Laya.Sprite {
	    constructor() {
	        super();
	        Laya.stage.addChild(this);
	        this.TF = this.newTextfield(600, 600, "", null, 18, 0xff0000, "上", "左");
	        this.addChild(this.TF);
	        this.TF.overflow = "scroll";
	        this.mouseEnabled = false;
	    }
	    static show(str) {
	        if (this.ins == null)
	            this.ins = new MyDebug();
	        this.ins.TF.text = this.ins.TF.text + str + "\n";
	        this.ins.TF.scrollY = this.ins.TF.maxScrollY;
	        Laya.stage.addChild(this.ins);
	    }
	    newTextfield(w, h, text, font, size, col, vUD = "中", hLR = "中", italic = false, bold = false, multiline = false) {
	        var t = new TextField(w, h, text);
	        t.font = font;
	        t.fontSize = size;
	        if (typeof col === "number")
	            t.color = this.getColorStrFromInt(col);
	        else
	            t.color = col;
	        if (vUD == "上" || vUD == "up")
	            t.valign = "top";
	        else if (vUD == "下" || vUD == "down")
	            t.valign = "bottom";
	        else
	            t.valign = "middle";
	        if (hLR == "左" || hLR == "left")
	            t.align = "left";
	        else if (hLR == "右" || hLR == "right")
	            t.align = "right";
	        else
	            t.align = "center";
	        if (multiline)
	            t.wordWrap = true;
	        t.width = w;
	        t.height = h;
	        t.italic = italic;
	        t.bold = bold;
	        t.text = text;
	        t.autoScale = true;
	        return t;
	    }
	    getColorStrFromInt(col) {
	        let str = col.toString(16);
	        while (str.length < 6) {
	            str = "0" + str;
	        }
	        return "#" + str;
	    }
	}

	class ConfigTidus {
	    static initF() {
	        if (this.bgCol == -1) {
	            Laya.stage.bgColor = "none";
	        }
	        else {
	            let strcol = this.bgCol.toString(16);
	            while (strcol.length < 6) {
	                strcol = "0" + strcol;
	            }
	            Laya.stage.bgColor = "#" + strcol;
	        }
	        Laya.stage.frameRate = this.FrameRate == 60 ? Laya.Stage.FRAME_FAST : Laya.Stage.FRAME_SLOW;
	        console.log("屏幕大小：" + Laya.Browser.width + ":" + Laya.Browser.height);
	        Laya.stage.on(Laya.Event.RESIZE, this, this.onStageResize, [false]);
	        this.onStageResize(true);
	        if (Laya.Browser.onMiniGame) {
	        }
	        else {
	            Laya.stage.on(Laya.Event.FOCUS, this, (e) => {
	                this.onStageActiveF();
	            });
	            Laya.stage.on(Laya.Event.BLUR, this, () => {
	                this.onStageDeActiveF();
	            });
	        }
	        window["appEventListener"] = this.onASEvent.bind(this);
	    }
	    static onASEvent(data) {
	        if (typeof data === "string")
	            data = JSON.stringify(data);
	        this.LogF("收到APP发来的事件：" + JSON.stringify(data));
	        if (data["con"] == "进入后台") {
	            this.onStageDeActiveF();
	            return;
	        }
	        if (data["con"] == "回到前台") {
	            this.onStageActiveF();
	            return;
	        }
	        if (this.GFun_MEO_dispatch)
	            this.GFun_MEO_dispatch("APP事件", data);
	    }
	    static onStageActiveF() {
	        if (this.isActive == true)
	            return;
	        this.isActive = true;
	        if (PlayerDefault.player && PlayerDefault.player.initData && PlayerDefault.player.initData["app"] == true) { }
	        else {
	            Laya.stage.renderingEnabled = true;
	            Laya.updateTimer.resume();
	            Laya.physicsTimer.resume();
	        }
	        if (this.Arr_activeFuns) {
	            for (let i = 0; i < this.Arr_activeFuns.length; i++) {
	                this.Arr_activeFuns[i]();
	            }
	            this.Arr_activeFuns = null;
	        }
	    }
	    static onStageDeActiveF() {
	        if (this.isActive == false)
	            return;
	        this.isActive = false;
	        if (PlayerDefault.player && PlayerDefault.player.initData && PlayerDefault.player.initData["app"] == true)
	            return;
	        Laya.stage.renderingEnabled = false;
	        Laya.updateTimer.pause();
	        Laya.physicsTimer.pause();
	    }
	    static addActiveFun(cb) {
	        if (this.Arr_activeFuns == null)
	            this.Arr_activeFuns = [];
	        this.Arr_activeFuns.push(cb);
	    }
	    static onStageResize(first) {
	        if (this.deviceHeight == Laya.Browser.height && this.deviceWidth == Laya.Browser.width)
	            return;
	        this.deviceWidth = Laya.Browser.width;
	        this.deviceHeight = Laya.Browser.height;
	        var sx = this.deviceWidth / this.stageW;
	        var sy = this.deviceHeight / this.stageH;
	        if (first == true) {
	            if (sx < sy)
	                Laya.stage.scaleMode = Laya.Stage.SCALE_FIXED_HEIGHT;
	            else
	                Laya.stage.scaleMode = Laya.Stage.SCALE_FIXED_WIDTH;
	        }
	        var stageScale = sx > sy ? sx : sy;
	        var stageScaleInfo = { "屏幕w": this.deviceWidth / stageScale, "屏幕h": this.deviceHeight / stageScale };
	        console.log("屏幕宽" + this.deviceWidth + "屏幕高" + this.deviceHeight + "，缩放=" + stageScale + "，屏幕真实占用：" + JSON.stringify(stageScaleInfo));
	        this.stageScale = stageScale;
	        this.stageRealW = stageScaleInfo["屏幕w"];
	        this.stageRealH = stageScaleInfo["屏幕h"];
	        if (this.GVal["舞台尺寸事件"])
	            this.GVal["舞台尺寸事件"]();
	    }
	    static onCloseProgram() {
	        if (window["conch"]) {
	            window["conch"].exit();
	        }
	        else {
	            window.close();
	            if (this.FunClose)
	                this.FunClose();
	        }
	    }
	    static LogF(...arg) {
	        if (arg.length == 0)
	            return;
	        var str = "";
	        while (arg.length > 0) {
	            if (str.length > 0)
	                str += "  ";
	            str += JSON.stringify(arg[0]);
	            arg.shift();
	        }
	        console.log(str);
	        if (this.isDebug)
	            MyDebug.show(str);
	    }
	    static get GVal_GTRole() { return GT_Role; }
	    ;
	}
	ConfigTidus.FrameRate = 60;
	ConfigTidus.GRAVITY = -3.6;
	ConfigTidus.stageW = 750;
	ConfigTidus.stageH = 1500;
	ConfigTidus.bgCol = 0;
	ConfigTidus.iSWakongping = false;
	ConfigTidus.Platform = "default";
	ConfigTidus.Ver = "0.0";
	ConfigTidus.isActive = true;
	ConfigTidus.isDebug = false;
	ConfigTidus.GVal = {};

	class SceneManager {
	    constructor() {
	        window.addEventListener('resize', () => {
	        });
	    }
	    static initF() {
	        if (this._instance == null) {
	            this._instance = new SceneManager();
	        }
	    }
	    static get instance() {
	        if (this._instance == null) {
	            this._instance = new SceneManager();
	        }
	        return this._instance;
	    }
	    static get nowScene() {
	        return SceneManager._nowScene;
	    }
	    static set nowScene(value) {
	        if (this.nowScene)
	            this.nowScene.destroyF();
	        SceneManager._nowScene = value;
	    }
	}

	class Tool_Object {
	    static destroyF_One(obj) {
	        if (obj == null)
	            return null;
	        if (obj instanceof Array) {
	            for (var i = 0; i < obj.length; i++) {
	                Tool_Object.destroyF_One(obj[i]);
	            }
	            obj.length = 0;
	        }
	        else if (obj instanceof TidusScript_component) {
	            if (obj.isDestroy == false) {
	                obj.isDestroy = true;
	                obj.runFun("destroyF");
	                obj.isDestroy = false;
	            }
	        }
	        else if (typeof obj === "number") {
	            return 0;
	        }
	        else if (this.hasFunction(obj, "destroyF") == true) {
	            if (obj instanceof Sprite) {
	                if (obj.isDestroyed == false) {
	                    obj.isDestroyed = true;
	                    obj.destroyF();
	                    obj.isDestroyed = false;
	                }
	                this.destroyDisplayObj(obj);
	            }
	            else {
	                obj.destroyF();
	            }
	        }
	        else if (obj instanceof Laya.Handler) {
	            obj.clear();
	        }
	        else if (obj instanceof Laya.Sprite3D) {
	            obj.destroy();
	        }
	        else if (obj instanceof Sprite) {
	            this.destroyDisplayObj(obj);
	        }
	        else if (obj instanceof Laya.Text) {
	            obj.removeSelf();
	        }
	        else if (obj instanceof Laya.Sprite) {
	            obj.removeSelf();
	        }
	        else if (obj instanceof Object) {
	            for (let n in obj) {
	                if (n == "__proto__")
	                    continue;
	                let tmpobjone = obj[n];
	                delete obj[n];
	                this.destroyF_One(tmpobjone);
	            }
	        }
	        else if (obj instanceof Map) {
	            obj.forEach((val, key, map) => {
	                Tool_Object.destroyF_One(val);
	                Tool_Object.destroyF_One(key);
	            });
	            obj.clear();
	        }
	        return null;
	    }
	    static destroyDisplayObj(obj) {
	        if (obj != null && obj.isDestroyed == false) {
	            obj.isDestroyed = true;
	            obj.customVariable = Tool_Object.destroyF_One(obj.customVariable);
	            let isErrorChildren = false;
	            try {
	                obj.numChildren;
	            }
	            catch (error) {
	                isErrorChildren = true;
	            }
	            while (isErrorChildren == false && obj.numChildren > 0) {
	                try {
	                    var cone = obj.getChildAt(0);
	                }
	                catch (error) {
	                    break;
	                }
	                Tool_Object.destroyF_One(cone);
	                try {
	                    obj.numChildren;
	                }
	                catch (error) {
	                    isErrorChildren = true;
	                }
	            }
	            obj.removeFromParent(true);
	        }
	        if (obj && obj.tidusscriptM) {
	            let arr = obj.tidusscriptM;
	            obj.tidusscriptM = null;
	            Tool_Object.destroyF_One(arr);
	        }
	    }
	    static hasFunction(target, method) {
	        try {
	            return target[method] instanceof Function;
	        }
	        catch (e) {
	            return false;
	        }
	    }
	    static isEqual(obj1, obj2) {
	        if ((obj1 == null || obj1 == NaN) && (obj2 == null || obj2 == NaN))
	            return true;
	        if ((obj1 == null || obj1 == NaN) || (obj2 == null || obj2 == NaN))
	            return false;
	        if (obj1 instanceof Array)
	            return Tool_ArrayUtils.isEqual(obj1, obj2);
	        if (Tool_Function.isTypeOf(obj1, "str")) {
	            return obj1 == obj2;
	        }
	        if (Tool_Function.isTypeOf(obj1, "num") && Tool_Function.isTypeOf(obj2, "num")) {
	            let num1 = Tool_Function.onChangeInstance(obj1, "num");
	            let num2 = Tool_Function.onChangeInstance(obj2, "num");
	            if (Math.abs(num1 - num2) < 0.00001)
	                return true;
	            return false;
	        }
	        if (obj1 instanceof Object)
	            return Tool_Object.isEqualObj(obj1, obj2);
	        return obj1 == obj2;
	    }
	    static isEqualObj(dic1, dic2) {
	        var dicHave = {};
	        var haveWrong = false;
	        for (let n in dic1) {
	            dicHave[n] = true;
	            if (Tool_Object.isEqual(dic1[n], dic2[n]) == false) {
	                haveWrong = true;
	                break;
	            }
	        }
	        for (let n in dic2) {
	            if (dicHave[n] != true) {
	                haveWrong = true;
	                break;
	            }
	        }
	        if (haveWrong == true)
	            return false;
	        return true;
	    }
	    static CopyF(obj, noCopy = false) {
	        if (typeof obj === "string" || typeof obj === "number" || obj == null || typeof obj === "boolean")
	            return obj;
	        if (obj instanceof Array) {
	            if (noCopy)
	                return Tool_ArrayUtils.copyArr(obj);
	            return Tool_ArrayUtils.copyArr(obj);
	        }
	        if (obj instanceof Image$1) {
	            return Tool_ImgUtils.cloneImage(obj);
	        }
	        if (obj instanceof Sprite) {
	            return Tool_SpriteUtils.cloneSprite(obj);
	        }
	        if (obj instanceof Object) {
	            var out = {};
	            for (let key in obj) {
	                out[key] = this.CopyF(obj[key], noCopy);
	            }
	            return out;
	        }
	        return obj;
	    }
	    static onComboObject(obj, tar, copy, needAddNumber = true, comboChild = false, changeStrToDelete = true) {
	        if (obj == null || tar == null)
	            return;
	        for (let key in tar) {
	            if (tar[key] == null) {
	                continue;
	            }
	            if ((tar[key] == "-NULL-" || tar[key] == "删除") && changeStrToDelete == true) {
	                delete obj[key];
	                continue;
	            }
	            if (obj[key] == null) {
	                if (copy == false)
	                    obj[key] = tar[key];
	                else
	                    obj[key] = this.CopyF(tar[key]);
	                deleteNull(obj[key]);
	            }
	            else if (comboChild) {
	                if (Tool_Function.isTypeOf(obj[key], "obj") && Tool_Function.isTypeOf(tar[key], "obj")) {
	                    this.onComboObject(obj[key], tar[key], copy, needAddNumber, comboChild);
	                }
	                else if (typeof (obj[key]) != typeof (tar[key])) {
	                    obj[key] = tar[key];
	                    deleteNull(obj[key]);
	                }
	                else if (Tool_Function.isTypeOf(obj[key], "num") == false || Tool_Function.isTypeOf(tar[key], "num") == false) {
	                    if (copy == false)
	                        obj[key] = tar[key];
	                    else
	                        obj[key] = this.CopyF(tar[key]);
	                    deleteNull(obj[key]);
	                }
	                else {
	                    if (needAddNumber) {
	                        obj[key] += tar[key];
	                    }
	                    else {
	                        obj[key] = tar[key];
	                    }
	                }
	            }
	            else {
	                if (Tool_Function.isTypeOf(obj[key], "num") == false || Tool_Function.isTypeOf(tar[key], "num") == false) {
	                    if (copy == false)
	                        obj[key] = tar[key];
	                    else
	                        obj[key] = this.CopyF(tar[key]);
	                }
	                else {
	                    if (needAddNumber) {
	                        obj[key] += tar[key];
	                    }
	                    else {
	                        obj[key] = tar[key];
	                    }
	                }
	            }
	        }
	        function deleteNull(obj) {
	            if (changeStrToDelete == false)
	                return obj;
	            if (Tool_Function.isTypeOf(obj, "obj") == false)
	                return obj;
	            for (let n in obj) {
	                if (obj[n] == "-NULL-" || obj[n] == "删除")
	                    delete obj[n];
	                else if (Tool_Function.isTypeOf(obj[n], "obj")) {
	                    obj[n] = deleteNull(obj[n]);
	                }
	            }
	            return obj;
	        }
	    }
	    static onClearObj(obj) {
	        if (obj == null)
	            return;
	        for (var key in obj) {
	            delete obj[key];
	        }
	    }
	    static getLengthOfObject(obj) {
	        if (obj == null)
	            return 0;
	        var count = 0;
	        for (let key in obj) {
	            count++;
	        }
	        return count;
	    }
	    static isChildOfObject(tar, obj) {
	        if (obj) {
	            for (let key in obj) {
	                if (obj[key] == tar)
	                    return key;
	            }
	        }
	        return null;
	    }
	    static getRandomKeyFromObject(obj) {
	        let arr = [];
	        for (let key in obj) {
	            arr.push(key);
	        }
	        if (arr.length == 0)
	            return null;
	        var index = Tool_Function.onForceConvertType(Math.random() * arr.length);
	        return arr[index];
	    }
	    static getKeyOfValue(dic, value) {
	        for (let key in dic) {
	            if (dic[key] == value)
	                return key;
	        }
	        return null;
	    }
	    static getKeysOfObject(obj) {
	        if (obj == null)
	            return null;
	        var arr = [];
	        for (let key in obj) {
	            arr.push(key);
	        }
	        return arr;
	    }
	    static getKeysOfMap(obj) {
	        if (obj == null)
	            return null;
	        var arr = [];
	        obj.forEach((v, key, map) => {
	            arr.push(key);
	        });
	        return arr;
	    }
	}
	Tool_Object.Dic_HaveFun = new Object();

	class homeView extends Sprite {
	    constructor() {
	        super();
	        this.mso = new MySourceManagerOne();
	        this.swfname = "SWF_home";
	        this.defaultDrawingWidth = 30;
	        let source = [[this.swfname, "swf"]];
	        this.mso.addSource(source, this.init.bind(this), false);
	    }
	    init() {
	        this.sprView = this.mso.getSprFromSwf(this.swfname, "spr_View");
	        this.addChild(this.sprView);
	        if (!this.sprView) {
	            AlertWindow.showF(`${this.swfname}中没有sprView`, null);
	            return;
	        }
	        this.ac = new MyViewAllCompsController(this.sprView);
	        this.ac.Fun_mbc_click = clickBtn.bind(this);
	        let hanziWriter = window["HanziWriter"];
	        this.writeBox = document.createElement("div");
	        this.writeBox.setAttribute("id", "writeBox");
	        this.writeBox.style.position = "absolute";
	        this.writeBox.style.zIndex = 100 + "";
	        this.writeBox.style.left = (this.width - 400) / 2 + "px";
	        this.writeBox.style.top = (this.height - 300) / 2 + "px";
	        document.querySelector("#layaContainer").append(this.writeBox);
	        let writer = hanziWriter.create(this.writeBox, "润", {
	            width: 400,
	            height: 300,
	            drawingWidth: 30,
	            drawingColor: "#c5452e",
	            showCharacter: false,
	            padding: 5
	        });
	        writer.quiz();
	        function clickBtn(btn) {
	            console.log(btn);
	            this.writeBox.style.zIndex = -1 + "";
	            AlertWindow.showF("hello", null, true, onOK.bind(this));
	        }
	        function onOK() {
	            console.log(this.writeBox + "==");
	            this.writeBox.style.zIndex = 100 + "";
	        }
	    }
	}

	class Main {
	    constructor() {
	        Config.useRetinalCanvas = true;
	        if (window["Laya3D"])
	            Laya3D.init(GameConfig.width, GameConfig.height);
	        else
	            Laya.init(GameConfig.width, GameConfig.height, Laya["WebGL"]);
	        Laya["Physics"] && Laya["Physics"].enable();
	        Laya["DebugPanel"] && Laya["DebugPanel"].enable();
	        if (GameConfig.width < GameConfig.height)
	            Laya.stage.screenMode = "vertical";
	        else
	            Laya.stage.screenMode = "horizontal";
	        Laya.stage.screenMode = GameConfig.screenMode;
	        Laya.stage.alignV = GameConfig.alignV;
	        Laya.stage.alignH = GameConfig.alignH;
	        Laya.URL.exportSceneToJson = GameConfig.exportSceneToJson;
	        if (GameConfig.debug || Laya.Utils.getQueryString("debug") == "true")
	            Laya.enableDebugPanel();
	        if (GameConfig.physicsDebug && Laya["PhysicsDebugDraw"])
	            Laya["PhysicsDebugDraw"].enable();
	        if (GameConfig.stat)
	            Laya.Stat.show();
	        Laya.ResourceVersion.enable("version.json", Laya.Handler.create(this, this.onVersionLoaded), Laya.ResourceVersion.FILENAME_VERSION);
	    }
	    onVersionLoaded() {
	        Laya.AtlasInfoManager.enable("fileconfig.json", Laya.Handler.create(this, this.onConfigLoaded));
	    }
	    onConfigLoaded() {
	        MainManager.initF();
	        let source = [
	            ["SWF_Default", "swf"]
	        ];
	        MySourceManager.getInstance().addTexture(source, this.initF.bind(this));
	    }
	    initF() {
	        SceneManager.nowScene = new homeView();
	        LayerManager.LayerView.addChild(SceneManager.nowScene);
	    }
	}
	ConfigTidus.GFun_ToolObj_copy = Tool_Object.CopyF.bind(Tool_Object);
	ConfigTidus.GFun_ToolObj_isEqual = Tool_Object.isEqual.bind(Tool_Object);
	ConfigTidus.GFun_ToolObj_destroyOne = Tool_Object.destroyF_One.bind(Tool_Object);
	ConfigTidus.GFun_ToolObj_combo = Tool_Object.onComboObject.bind(Tool_Object);
	ConfigTidus.GFun_ToolObj_clearObj = Tool_Object.onClearObj.bind(Tool_Object);
	ConfigTidus.GFun_ToolView_winfit = Tool_ViewFit.onWindowFitScreen.bind(Tool_ViewFit);
	ConfigTidus.GFun_AlertWindow = AlertWindow.showF.bind(AlertWindow);
	ConfigTidus.GVal["MyMC"] = MyMC;
	ConfigTidus.GVal["SWF"] = SWF;
	ConfigTidus.GVal["Tool_Object"] = Tool_Object;
	ConfigTidus.GVal["MySourceManager"] = MySourceManager;
	ConfigTidus.GVal["PL"] = PlayerDefault;
	ConfigTidus.GVal["MgsSocket"] = MgsSocket;
	ConfigTidus.GVal["newAllComp"] = (tar, swfnum) => { return new MyViewAllCompsController(tar, swfnum); };
	new Main();

}());
